/* -*- mode:C++ ; compile-command: "g++ -I.. -g -c ezgcd.cc" -*- */
/*  Multivariate GCD for large data not covered by the heuristic GCD algo
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GIAC_EZGCD_H_
#define _GIAC_EZGCD_H_
#include "first.h"
#include "gausspol.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  template<class T>
  struct gen_unsigned {
    gen g;
    T u;
    gen_unsigned(const gen & myg,const T & myu): g(myg),u(myu) {};
    gen_unsigned(): g(0),u(0) {};
  };

  // warning, < is > so that monomial ordering is ok after back-conversion
  template <class T>
  inline bool operator < (const gen_unsigned<T> & gu1,const gen_unsigned<T> & gu2){
    return gu1.u > gu2.u;
  }

  // convert p to monomial represented by unsigned integers
  // using the rule [a1 a2 .. an] [deg1 ... degn ] -> 
  // (... (a1*deg2+a2)*deg3 +...)*degn+an
  template<class T>
  void convert(const polynome & p,const index_t & deg,std::vector< gen_unsigned<T> >  & v){
    typename std::vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
    v.clear();
    v.reserve(itend-it);
    gen_unsigned<T> gu;
    T u;
    index_t::const_iterator itit,ditbeg=deg.begin(),ditend=deg.end(),dit;
    for (;it!=itend;++it){
      u=0;
      itit=it->index.iptr->begin();
      for (dit=ditbeg;dit!=ditend;++itit,++dit)
	u=u*unsigned(*dit)+unsigned(*itit);
      gu.u=u;
      gu.g=it->value;
      v.push_back(gu);
    }
  }

  template<class T>
  void convert(const std::vector< gen_unsigned<T> > & v,const index_t & deg,polynome & p){
    typename std::vector< gen_unsigned<T> >::const_iterator it=v.begin(),itend=v.end();
    index_t::const_reverse_iterator ditbeg=deg.rbegin(),ditend=deg.rend(),dit;
    p.dim=ditend-ditbeg;
    p.coord.clear();
    p.coord.reserve(itend-it);
    T u;
    index_t i(p.dim);
    int k;
    for (;it!=itend;++it){
      u=it->u;
      for (k=p.dim-1,dit=ditbeg;dit!=ditend;++dit,--k){
	i[k]=u % unsigned(*dit);
	u = u/unsigned(*dit);
      }
      p.coord.push_back(monomial<gen>(it->g,i));
    }
  }

  class hash_function_unsigned_object {
  public:
    inline size_t operator () (size_t x) const { return x; }
    hash_function_unsigned_object() {};
  };

  template<class T>
  void smalladd(const std::vector< gen_unsigned<T> > & v1,const std::vector< gen_unsigned<T> > & v2,std::vector< gen_unsigned<T> > & v){
    typename std::vector< gen_unsigned<T> >::const_iterator it1=v1.begin(),it1end=v1.end(),it2=v2.begin(),it2end=v2.end();
    gen g;
    v.clear();
    v.reserve(it1end-it1+it2end-it2); // worst case
    for (;it1!=it1end && it2!=it2end;){
      if (it1->u==it2->u){
	g=it1->g+it2->g;
	if (!is_zero(g))
	  v.push_back(gen_unsigned<T>(g,it1->u));
	++it1;
	++it2;
      }
      else {
	if (it1->u>it2->u){
	  v.push_back(*it1);
	  ++it1;
	}
	else {
	  v.push_back(*it2);
	  ++it2;
	}
      }
    }
    for (;it1!=it1end;++it1)
      v.push_back(*it1);
    for (;it2!=it2end;++it2)
      v.push_back(*it2);
  }

  template<class T>
  void smallmult(const std::vector< gen_unsigned<T> > & v1,const std::vector< gen_unsigned<T> > & v2,std::vector< gen_unsigned<T> > & v){
#ifdef HASH_MAP_NAMESPACE
    typedef HASH_MAP_NAMESPACE::hash_map< T,gen,hash_function_unsigned_object > hash_prod ;
    // cout << "hash " << clock() << endl;
#else
    typedef std::map<T,gen> hash_prod;
    // cout << "small map" << endl;
#endif    
    hash_prod produit;
    typename hash_prod::iterator prod_it,prod_itend;
    typename std::vector< gen_unsigned<T> >::const_iterator it1=v1.begin(),it1end=v1.end(),it2beg=v2.begin(),it2,it2end=v2.end();
    gen g1,g;
    T u1,u;
    for (;it1!=it1end;++it1){
      g1=it1->g;
      u1=it1->u;
      for (it2=it2beg;it2!=it2end;++it2){
	u=u1+it2->u;
	g=g1*it2->g;
	prod_it=produit.find(u);
	if (prod_it==produit.end())
	  produit[u]=g;
	else {
	  gen * s=&prod_it->second;
	  if (s->type==_ZINT && g.type==_INT_ && *s->ref_count==1){
	    register int gval=g.val;
	    if (gval>0)
	      mpz_add_ui(*s->_ZINTptr,*s->_ZINTptr,gval);
	    else
	      mpz_sub_ui(*s->_ZINTptr,*s->_ZINTptr,-gval);
	  }
	  else
	    *s = *s + g;
	}
      }
    }
    gen_unsigned<T> gu;
    prod_it=produit.begin(),prod_itend=produit.end();
    v.clear();
    v.reserve(produit.size());
    for (;prod_it!=prod_itend;++prod_it){
      if (!is_zero(gu.g=prod_it->second)){
	gu.u=prod_it->first;
	v.push_back(gu);
      }
    }    
    // cout << "smallmult end " << clock() << endl;
    sort(v.begin(),v.end());
  }

  void change_dim(polynome & p,int dim);

  // Hensel quadratic lift
  // Lift the equality p(b)=qb*rb [where b is a vecteur like for peval
  // assumed to have p.dim-1 coordinates] to p=q*r mod (X-b)^deg
  // Assuming that lcoeff(q)=lcp, lcoeff(r)=lcp, lcoeff(p)=lcp^2
  // If you want to find factors of a poly P such that P(b)=Qb*Rb, 
  // if lcp is the leading coeff of P
  // then p=P*lcp, qb=Qb*lcp(b)/lcoeff(Qb), rb=Rb*lcp(b)/lcoeff(Rb)
  bool hensel_lift(const polynome & p, const polynome & lcp, const polynome & qb, const polynome & rb, const vecteur & b,polynome & q, polynome & r,bool linear_lift=true,double maxop=-1);

  bool find_good_eval(const polynome & F,const polynome & G,polynome & Fb,polynome & Gb,vecteur & b,bool debuglog=false,const gen & mod=zero);
  polynome peval_1(const polynome & p,const vecteur &v,const gen & mod);

  // max_gcddeg is used when ezgcd was not successfull to find
  // the gcd even with 2 evaluations leading to the same gcd degree
  // in this case ezgcd calls itself with a bound on the gcd degree
  // is_sqff is true if we know that F_orig or G_orig is squarefree
  // is_primitive is true if F_orig and G_orig is primitive
  bool ezgcd(const polynome & F_orig,const polynome & G_orig,polynome & GCD,bool is_sqff=false,bool is_primitive=false,int max_gcddeg=0,double maxop=-1);

  extern const std::string _ezgcd_s;
  gen _ezgcd(const gen & args);
  extern unary_function_ptr at_ezgcd;  

  extern const std::string _modgcd_s;
  gen _modgcd(const gen & args);
  extern unary_function_ptr at_modgcd;  

  extern const std::string _heugcd_s;
  gen _heugcd(const gen & args);
  extern unary_function_ptr at_heugcd;  

  extern const std::string _psrgcd_s;
  gen _psrgcd(const gen & args);
  extern unary_function_ptr at_psrgcd;  

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // NO_NAMESPACE_GIAC

#endif // _GIAC_EZGCD_H
