 /* -*- mode: C++; compile-command: "flex -Pgiac_yy -olex.yy.c input_lexer.ll && mv -f lex.yy.c input_lexer.cc && g++ -g -I.. -c input_lexer.cc" -*- */
/** @file input_lexer.ll
 *
 *  Lexical analyzer definition for reading expressions.
 *  Note Maple input should be processed replacing # with // and { } for set
 *  This file must be processed with flex. */

/*
 *  Early version modified from GiNaC by B. Parisse (C) 2001
 *  GiNaC Copyright (C) 1999-2000 Johannes Gutenberg University Mainz, Germany
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/*
 * The lexer will first check for static patterns and strings (defined below)
 * If a match is not found, it calls find_or_make_symbol
 * This function looks first if the string should be translated
 * (e.g. add a prefix from the export table)
 * then look in lexer_functions for a match, then look in sym_tab
 * if not found in sym_tab, a new identificateur is created & added in sym_tab
 * Functions in lexer_functions are added during the construction
 * of the corresponding unary_functions using lexer_functions_register
 */


/*
 *  Definitions
 */

%{
#include "first.h"
#include <iostream>
#include <stdexcept>

#include "input_lexer.h"
#include "gen.h"
#include "identificateur.h"
#include "usual.h"
#include "derive.h"
#include "series.h"
#include "intg.h"
#include "sym2poly.h"
#include "moyal.h"
#include "subst.h"
#include "vecteur.h"
#include "modpoly.h"
#include "lin.h"
#include "solve.h"
#include "ifactor.h"
#include "alg_ext.h"
#include "gauss.h"
#include "isom.h"
#include "plot.h"
#include "prog.h"
#include "rpn.h"
#include "ezgcd.h"
#include "tex.h"
#include "risch.h"
    
  using namespace std;
#ifndef NO_NAMESPACE_GIAC
  using namespace giac;
  namespace giac {
#endif // ndef NO_NAMESPACE_GIAC
    
#include "input_parser.h"
    sym_tab syms,lexer_functions;
    std::map<std::string,std::vector<std::string> > lexer_translator;
    // lexer_translator will be updated when export/with is called
    // To each string (w/o ::) in a given library, 
    // If it exists, we push_back the full string (with ::)
    // If not we create a vector with the full string
    // If a library is unexported we remove the corresponding entry in the 
    // vector and remove the entry if the vector is empty
    std::map<std::string,std::vector<std::string> > library_functions;
    // First string is the library name, second is the vector of function names
    vector<user_function> registered_lexer_functions; // User defined relations
    string parse_filename="// ";
    extern int maple_mode;
    int index_status=0; // 0 if [ -> T_VECT_DISPATCH, 1 if [ -> T_INDEX_BEGIN
    int opened_quote=0; // 1 if we are inside a quote
    int in_rpn=0; // 1 inside RPN expression
    int lexer_line_number=1;
    bool spread_formula=false; // beginning = and meaning of :
    bool initialisation_done=false;
    string comment_s;

    bool has_special_syntax(const string & s){
      sym_tab::const_iterator i = lexer_functions.find(s);
      if (i==lexer_functions.end())
	return false;
      return (i->second.subtype!=T_UNARY_OP);
    }
    
    bool lexer_functions_register(const unary_function_ptr & u,const string & s,int parser_token){
      sym_tab::const_iterator i = lexer_functions.find(s);
      if (i!=lexer_functions.end())
	return false;
      registered_lexer_functions.push_back(user_function(s,parser_token));
      lexer_functions[s] = gen(u);
      if (parser_token==1)
	lexer_functions[s].subtype=T_UNARY_OP;
      else
	lexer_functions[s].subtype=parser_token;
      // If s is a library function name (with ::), update the library
      int ss=s.size(),j=0;
      for (;j<ss-1;++j){
	if (s[j]==':' && s[j+1]==':')
	  break;
      }
      if (j<ss-1){
	string libname=s.substr(0,j);
	string funcname=s.substr(j+2,ss-j-2);
	std::map<std::string,std::vector<string> >::iterator it=library_functions.find(libname);
	if (it!=library_functions.end())
	  it->second.push_back(funcname);
	else
	  library_functions[libname]=vector<string>(1,funcname);
      }
      return true;
    }

    bool lexer_function_remove(const vector<user_function> & v){
      vector<user_function>::const_iterator it=v.begin(),itend=v.end();
      sym_tab::const_iterator i,iend;
      bool ok=true;
      for (;it!=itend;++it){
	i = lexer_functions.find(it->s);
	iend=lexer_functions.end();
	if (i==iend)
	  ok=false;
	else
	  lexer_functions.erase(it->s);
      }
      return ok;
    }

    int find_or_make_symbol(const string & s,gen & res){
      if (s.size()==1){
	switch (s[0]){
	case '+':
	  res=at_plus;
	  return T_UNARY_OP;
	case '-':
	  res=at_neg;
	  return T_UNARY_OP;
	case '*':
	  res=at_prod;
	  return T_UNARY_OP;
	case '/':
	  res=at_division;
	  return T_UNARY_OP;
	case '^':
	  res=at_pow;
	  return T_UNARY_OP;
	}
      }
      string ts(s);
      std::map<std::string,std::vector<string> >::const_iterator j=lexer_translator.find(s);
      if (j!=lexer_translator.end() && !j->second.empty())
	ts=j->second.back();
      sym_tab::const_iterator i = lexer_functions.find(ts);
      if (i!=lexer_functions.end()){
	if (i->second.subtype==T_TO)
	  res=plus_one;
	else
	  res = i->second;
	res.subtype=1;
	index_status=0;
	return i->second.subtype ;
      }
      i = syms.find(s);
      if (i == syms.end()) {
	// std::cerr << "lexer new" << s << endl;
	res = *(new identificateur(s));
	syms[s] = res;
      } else {
	// std::cerr << "lexer" << s << endl;
	res = i->second;
      }
      return T_SYMBOL;
    }

#ifndef NO_NAMESPACE_GIAC
  } // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
  

      // lex input function
  static int lexer_input(char *buf, int max_size);
#define YY_INPUT(buf, result, max_size) (result = lexer_input(buf, max_size))
%}

	/* Abbreviations */
D	[0-9]
E	[eE][eE]?[-+]?{D}+
A	[a-zA-Z~]
AN	[0-9a-zA-Z_.~]

%x comment
%x comment_hash
%x str
%x backquote
/*
 *  Lexical rules
 */

%%

[ 
\t\\]+			/* skip whitespace */
\n                ++lexer_line_number; //cerr << "Scanning line " << lexer_line_number << endl;
  /* Strings */
  /* \"[^\"]*\"        giac_yylval = string2gen( giac_yytext); return T_STRING; */
\"                BEGIN(str); comment_s="";
<str>\"\"         comment_s +='"';
<str>\"        {  index_status=1; BEGIN(INITIAL); 
                  giac_yylval=string2gen(comment_s,false); 
                  return T_STRING; }
<str>\n        comment_s+='\n'; ++lexer_line_number;
<str>\\[0-7]{1,3} {
                   /* octal escape sequence */
                   int result;
                   (void) sscanf( yytext + 1, "%o", &result );
                   comment_s += char(result & 0xff);
                   }
<str>\\[0-9]+      {
                   /* generate error - bad escape sequence; something
                    * like '\48' or '\0777777'
                    */
                   }
<str>\\n  comment_s += '\n';
<str>\\t  comment_s += '\t';
<str>\\r  comment_s += '\r';
<str>\\b  comment_s += '\b';
<str>\\f  comment_s += '\f';
<str>\\(.|\n)  comment_s += yytext[1];
<str>[^\\\n\"]+ comment_s += yytext;
`                  if (rpn_mode){ index_status=0; return T_ACCENTGRAVE; } else { BEGIN(backquote); comment_s=""; }
<backquote>\n      comment_s+='\n'; ++lexer_line_number;
<backquote>[^\n`]+       comment_s += yytext;
<backquote>`       {  index_status=1; BEGIN(INITIAL); 
     return find_or_make_symbol(comment_s,giac_yylval); }

"://"[^\n]*\n      index_status=0; ++lexer_line_number;
"//"[^\n]*\n      index_status=0; ++lexer_line_number;/* giac_yylval = string2gen('"'+string(giac_yytext).substr(2,string(giac_yytext).size()-3)+'"');   return T_COMMENT; */
"/*"              BEGIN(comment); comment_s="";

<comment>[^*\n]*        comment_s+=giac_yytext; /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   comment_s+=giac_yytext; /* eat up '*'s not followed by '/'s */
<comment>\n             comment_s += '\n'; ++lexer_line_number; cerr << "(Comment) scanning line " << lexer_line_number << endl;
<comment>"*"+"/"        BEGIN(INITIAL); index_status=0; /* giac_yylval = string2gen(comment_s,false); return T_COMMENT; */
"#++"[^*]*"++#"         index_status=0; /* giac_yylval = string2gen('"'+string(giac_yytext).substr(3,string(giac_yytext).size()-6)+'"'); return T_COMMENT; */
"#--"[^*]*"--#"         index_status=0; /* giac_yylval = string2gen('"'+string(giac_yytext).substr(3,string(giac_yytext).size()-6)+'"'); return T_COMMENT; */

"?"                     if (index_status) return T_INTERROGATION; else return T_HELP;
"_"                     return T_UNIT;
"'"                     if (opened_quote) { opened_quote=0; return T_QUOTE; } if (index_status && !in_rpn && maple_mode!= 1) return T_PRIME; opened_quote=1; return T_QUOTE;
";"			index_status=0; if (maple_mode==3) return TI_SEMI; giac_yylval=0; return T_SEMI;
""                  index_status=0; if (maple_mode==3) return T_SEMI; return TI_SEMI;
":"			if (spread_formula) return T_DEUXPOINTS; if ( maple_mode==3 ) { index_status=0; return TI_DEUXPOINTS; }  index_status=0; if (maple_mode>0) { giac_yylval=1; return T_SEMI; } else return T_DEUXPOINTS;
":;"                    giac_yylval=1; return T_SEMI;
"::"                    return T_DOUBLE_DEUX_POINTS;

			/* special values */
"i"			index_status=1; if (maple_mode > 0 ) { giac_yylval=i__IDNT_e; return T_SYMBOL; } else { giac_yylval = cst_i; return T_LITERAL;};
\xa1                     index_status=1; giac_yylval = cst_i; return T_LITERAL;
"I"                     index_status=1; if (maple_mode==0 || maple_mode==3 || rpn_mode) { giac_yylval=I__IDNT_e; return T_SYMBOL; } else { giac_yylval = cst_i; return T_LITERAL; };
"pi"			index_status=1; giac_yylval = cst_pi; return T_LITERAL;
"Pi"			index_status=1; giac_yylval = cst_pi; return T_LITERAL;
"PI"			index_status=1; giac_yylval = cst_pi; return T_LITERAL;
"infinity"		index_status=1; giac_yylval = unsigned_inf; return T_LITERAL;
"unsigned_inf"		index_status=1; giac_yylval = unsigned_inf; return T_LITERAL;
"plus_inf"		index_status=1; giac_yylval = plus_inf; return T_LITERAL;
"minus_inf"		index_status=1; giac_yylval = minus_inf; return T_LITERAL;
"undef"		        index_status=1; giac_yylval = undef; return T_LITERAL;
""                     return T_END_INPUT;

               /* integer values */
"DOM_int"                  index_status=0; giac_yylval = _INT_; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_INT"               index_status=0; giac_yylval = _ZINT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"integer"               index_status=0; giac_yylval = _ZINT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"posint"               index_status=0; giac_yylval = _POSINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"negint"               index_status=0; giac_yylval = _NEGINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"nonposint"               index_status=0; giac_yylval = _NONPOSINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"nonnegint"               index_status=0; giac_yylval = _NONNEGINT; giac_yylval.subtype=_INT_MAPLECONVERSION; return T_TYPE_ID;
"DOM_COMPLEX"               index_status=0; giac_yylval = _CPLX; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"complex"               index_status=0; giac_yylval = _CPLX; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_FLOAT"                index_status=0; giac_yylval = _DOUBLE_; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"float"                index_status=0; giac_yylval = _DOUBLE_; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_LIST"              index_status=0; giac_yylval = _VECT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_MATRIX"              index_status=0; giac_yylval = _VECT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"array"              index_status=0; giac_yylval = _VECT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_IDENT"            index_status=0; giac_yylval = _IDNT; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_SYMBOLIC"              index_status=0; giac_yylval = _SYMB; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"algebraic"              index_status=0; giac_yylval = _SYMB; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_RAT"              index_status=0; giac_yylval = _FRAC; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"rational"              index_status=0; giac_yylval = _FRAC; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"DOM_STRING"                index_status=0; giac_yylval = _STRNG; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"operator"              if (maple_mode==2){ giac_yylval = gen(at_user_operator,6); index_status=0; return T_UNARY_OP; }  index_status=0; giac_yylval = _FUNC; giac_yylval.subtype=_INT_TYPE; return T_TYPE_ID;
"newton_solver"         index_status=1; giac_yylval = _NEWTON_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"dnewton_solver"         index_status=1; giac_yylval = _DNEWTON_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"newtonj_solver"         index_status=1; giac_yylval = _NEWTONJ_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybridj_solver"         index_status=1; giac_yylval = _HYBRIDJ_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybridsj_solver"         index_status=1; giac_yylval = _HYBRIDSJ_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybrid_solver"         index_status=1; giac_yylval = _HYBRID_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"hybrids_solver"         index_status=1; giac_yylval = _HYBRIDS_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"secant_solver"         index_status=1; giac_yylval = _SECANT_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"steffenson_solver"         index_status=1; giac_yylval = _STEFFENSON_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"bisection_solver"         index_status=1; giac_yylval = _BISECTION_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"falsepos_solver"         index_status=1; giac_yylval = _FALSEPOS_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"brent_solver"         index_status=1; giac_yylval = _BRENT_SOLVER; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"golub_reinsch_decomp"         index_status=1; giac_yylval = _GOLUB_REINSCH_DECOMP; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"golub_reinsch_mod_decomp"         index_status=1; giac_yylval = _GOLUB_REINSCH_MOD_DECOMP; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"jacobi_decomp"         index_status=1; giac_yylval = _JACOBI_DECOMP; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"minor_det"         index_status=1; giac_yylval = _MINOR_DET; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"rational_det"         index_status=1; giac_yylval = _RATIONAL_DET; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"keep_pivot"         index_status=1; giac_yylval = _KEEP_PIVOT; giac_yylval.subtype=_INT_SOLVER;return T_NUMBER;
"rempli"         index_status=1; giac_yylval = _FILL_POLYGON ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant1"         index_status=1; giac_yylval = _QUADRANT1 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant2"         index_status=1; giac_yylval = _QUADRANT2 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant3"         index_status=1; giac_yylval = _QUADRANT3 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"quadrant4"         index_status=1; giac_yylval = _QUADRANT4 ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_losange"         index_status=1; giac_yylval = _POINT_LOSANGE ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_plus"         index_status=1; giac_yylval = _POINT_PLUS ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_carre"         index_status=1; giac_yylval = _POINT_CARRE  ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_triangle"         index_status=1; giac_yylval = _POINT_TRIANGLE ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_etoile"         index_status=1; giac_yylval = _POINT_ETOILE ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"point_point"         index_status=1; giac_yylval = _POINT_POINT ; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"black"         index_status=1; giac_yylval = _BLACK; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"gomme"         index_status=1; giac_yylval = _WHITE; /* was 49 */ giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"noir"         index_status=1; giac_yylval = _BLACK; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"red"         index_status=1; giac_yylval = _RED; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"rouge"         index_status=1; giac_yylval = _RED; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"green"         index_status=1; giac_yylval = _GREEN; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"vert"         index_status=1; giac_yylval = _GREEN; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"yellow"         index_status=1; giac_yylval = _YELLOW; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"jaune"         index_status=1; giac_yylval = _YELLOW; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"blue"         index_status=1; giac_yylval = _BLUE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"bleu"         index_status=1; giac_yylval = _BLUE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"magenta"         index_status=1; giac_yylval = _MAGENTA; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"cyan"         index_status=1; giac_yylval = _CYAN; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"white"         index_status=1; giac_yylval = _WHITE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"blanc"         index_status=1; giac_yylval = _WHITE; giac_yylval.subtype=_INT_COLOR ;return T_NUMBER;
"adaptive"         index_status=1; giac_yylval =_ADAPTIVE ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"axes"         index_status=1; giac_yylval =_AXES ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"color"         index_status=1; giac_yylval = _COLOR ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"filled"         index_status=1; giac_yylval = _FILLED; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"font"         index_status=1; giac_yylval =_FONT ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"labels"         index_status=1; giac_yylval = _LABELS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"legend"         index_status=1; giac_yylval = _LEGEND; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"linestyle"         index_status=1; giac_yylval = _LINESTYLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"resolution"         index_status=1; giac_yylval =_RESOLUTION ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"sample"         index_status=1; giac_yylval = _SAMPLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"scaling"         index_status=1; giac_yylval = _SCALING; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"style"         index_status=1; giac_yylval = _STYLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"symbol"         index_status=0; giac_yylval = _SYMBOL; giac_yylval.subtype=_INT_PLOT ;return T_TYPE_ID;
"symbolsize"         index_status=1; giac_yylval =_SYMBOLSIZE ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"thickness"         index_status=1; giac_yylval = _THICKNESS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"title"         index_status=1; giac_yylval = _TITLE; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"titlefont"         index_status=1; giac_yylval = _TITLEFONT; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"view"         index_status=1; giac_yylval = _VIEW; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"axesfont"         index_status=1; giac_yylval =_AXESFONT ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"coords"         index_status=1; giac_yylval = _COORDS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"labelfont"         index_status=1; giac_yylval = _LABELFONT; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"labeldirections"         index_status=1; giac_yylval = _LABELDIRECTIONS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"numpoints"         index_status=1; giac_yylval = _NUMPOINTS; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"tickmarks"         index_status=1; giac_yylval =_TICKMARKS ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"xtickmarks"         index_status=1; giac_yylval =_XTICKMARKS ; giac_yylval.subtype=_INT_PLOT ;return T_NUMBER;
"set"         index_status=0; giac_yylval =_SET__VECT ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"list"         if (maple_mode==3) { index_status=1; return find_or_make_symbol(giac_yytext,giac_yylval); } index_status=0; giac_yylval = _MAPLE_LIST ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"vector"         index_status=0; giac_yylval = _MATRIX__VECT ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"polynom"         index_status=0; giac_yylval =_POLY1__VECT ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_TYPE_ID;
"trig"         index_status=1; giac_yylval =_TRIG ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"expln"         index_status=1; giac_yylval =_EXPLN ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"parfrac"         index_status=1; giac_yylval =_PARFRAC ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"base"         index_status=1; giac_yylval =_BASE ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"confrac"         index_status=1; giac_yylval =_CONFRAC ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"fullparfrac"         index_status=1; giac_yylval =_FULLPARFRAC ; giac_yylval.subtype=_INT_MAPLECONVERSION ;return T_NUMBER;
"Delete"         index_status=1; giac_yylval =_DELETE_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Prefix"         index_status=1; giac_yylval =_PREFIX_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Postfix"         index_status=1; giac_yylval =_POSTFIX_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Binary"         index_status=1; giac_yylval =_BINARY_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;
"Nary"         index_status=1; giac_yylval =_NARY_OPERATOR ; giac_yylval.subtype=_INT_MUPADOPERATOR ;return T_NUMBER;


    /* vector/polynom/matrice delimiters */
"seq["              giac_yylval = _SEQ__VECT; return T_VECT_DISPATCH;
"set["              giac_yylval = _SET__VECT; return T_VECT_DISPATCH;
"rpn_func["         giac_yylval = _RPN_FUNC__VECT; return T_VECT_DISPATCH;
"group["            giac_yylval = _GROUP__VECT; return T_VECT_DISPATCH;
"line["             giac_yylval = _LINE__VECT; return T_VECT_DISPATCH;
"vector["           giac_yylval = _VECTOR__VECT; return T_VECT_DISPATCH;
"matrix["           giac_yylval = _MATRIX__VECT; return T_VECT_DISPATCH;
"pnt["              giac_yylval = _PNT__VECT; return T_VECT_DISPATCH;
"curve["            giac_yylval = _CURVE__VECT; return T_VECT_DISPATCH;
"halfline["         giac_yylval = _HALFLINE__VECT; return T_VECT_DISPATCH;
"poly1["            giac_yylval = _POLY1__VECT; return T_VECT_DISPATCH;
"assume["           giac_yylval = _ASSUME__VECT; return T_VECT_DISPATCH;
"spreadsheet["      giac_yylval = _SPREAD__VECT; return T_VECT_DISPATCH;
"folder["      giac_yylval = _FOLDER__VECT; return T_VECT_DISPATCH;
"<"                     index_status=0; return T_STRICTINF;
">"                     index_status=0; return T_STRICTSUP;
","                     index_status=0; return T_VIRGULE;
",,"                     index_status=0; return T_VIRGULE;
"("                     index_status=0; return T_BEGIN_PAR;
")"                     index_status=1; return T_END_PAR;
\[			if (index_status) { index_status=0; return T_INDEX_BEGIN; } else { giac_yylval = 0; return T_VECT_DISPATCH; } ;
\]			index_status=1; return T_VECT_END;
"%["                    index_status=0; return T_POLY1_BEGIN;
"%]"                    index_status=1; return T_POLY1_END;
"%%["                   index_status=0; return T_MATRICE_BEGIN;
"%%]"                   index_status=1; return T_MATRICE_END;
"%%%["                  index_status=0; return T_ASSUME_BEGIN;
"%%%]"                  index_status=1; return T_ASSUME_END;
    /* geometric delimiters */
"%("                    index_status=0; return T_GROUPE_BEGIN;
"%)"                    index_status=1; return T_GROUPE_END;
"%%("                   index_status=0; return T_LINE_BEGIN;
"%%)"                   index_status=1; return T_LINE_END;
"%%%("                  index_status=0; return T_VECTOR_BEGIN;
"%%%)"                  index_status=1; return T_VECTOR_END;
"%%%%("                 index_status=0; return T_CURVE_BEGIN;
"%%%%)"                 index_status=1; return T_CURVE_END;
    /* gen delimiters */
"{"                     index_status=0; if (rpn_mode) { giac_yylval=0; return T_VECT_DISPATCH; } if (maple_mode==3) return T_VECT_BEGIN; if (maple_mode > 0 ) return T_SET_BEGIN; else return T_BLOC_BEGIN;
"}"                     index_status=1; if (rpn_mode) return T_VECT_END; if (maple_mode==3) return T_VECT_END; if (maple_mode > 0) return T_SET_END; else return T_BLOC_END;
"%{"                    index_status=0; return T_SET_BEGIN;
"%}"                    index_status=1; return T_SET_END;
"%%{"                   index_status=0; return T_ROOTOF_BEGIN;
"%%}"                   index_status=1; return T_ROOTOF_END;
"%%%{"                  index_status=0; return T_SPOLY1_BEGIN;
"%%%}"                  index_status=1; return T_SPOLY1_END;
"<<"                    index_status=0; ++in_rpn; return T_RPN_BEGIN;
">>"                    index_status=0; --in_rpn; return T_RPN_END;

    /* Maple libraries names */
"linalg"                index_status=1; giac_yylval=_LINALG; giac_yylval.subtype=_INT_MAPLELIB; return T_MAPLELIB;
"numtheory"                index_status=1; giac_yylval=_NUMTHEORY; giac_yylval.subtype=_INT_MAPLELIB; return T_MAPLELIB;
"groebner"                index_status=1; giac_yylval=_GROEBNER; giac_yylval.subtype=_INT_MAPLELIB; return T_MAPLELIB;

    /* binary operators */
"->"                    index_status=0; return T_MAPSTO;
"-<"                    giac_yylval = gen(at_couleur,2); index_status=0; return T_INTERVAL;
"=="			index_status=0; giac_yylval=gen(at_same,2); return T_TEST_EQUAL;
"'=='"                  index_status=0; giac_yylval=gen(at_same,2); return T_QUOTED_BINARY;
"_equal"                  index_status=0; giac_yylval=gen(at_same,2); return T_QUOTED_BINARY;
"!="			index_status=0; giac_yylval=gen(at_different,2); return T_NOTEQ;
"'!='"                  index_status=0; giac_yylval=gen(at_different,2); return T_QUOTED_BINARY;
"<>"			index_status=0; giac_yylval=gen(at_different,2); return T_NOTEQ;
"'<>'"                  index_status=0; giac_yylval=gen(at_different,2); return T_QUOTED_BINARY;
"_unequal"                  index_status=0; giac_yylval=gen(at_different,2); return T_QUOTED_BINARY;
"'<='"                  index_status=0; giac_yylval=gen(at_inferieur_egal,2); return T_QUOTED_BINARY;
"_leequal"                  index_status=0; giac_yylval=gen(at_inferieur_egal,2); return T_QUOTED_BINARY;
"<="			index_status=0; giac_yylval=gen(at_inferieur_egal,2); return T_LESSEQ;
"'<'"                  index_status=0; giac_yylval=gen(at_inferieur_strict,2); return T_QUOTED_BINARY;
"_less"                  index_status=0; giac_yylval=gen(at_inferieur_strict,2); return T_QUOTED_BINARY;
"'>'"                  index_status=0; giac_yylval=gen(at_superieur_strict,2); return T_QUOTED_BINARY;
">="			index_status=0; giac_yylval=gen(at_superieur_egal,2); return T_GREATEREQ;
"'>='"                  index_status=0; giac_yylval=gen(at_superieur_egal,2); return T_QUOTED_BINARY;
"="                     spread_formula=!index_status; index_status=0; giac_yylval=gen(at_equal,2); return T_EQUAL;
"'='"                  index_status=0; giac_yylval=gen(at_equal,2); return T_QUOTED_BINARY;
"$"                     index_status=0; giac_yylval=gen(at_dollar,2); if (maple_mode>0) return T_DOLLAR_MAPLE; else return T_DOLLAR;
"%$"                   index_status=0; giac_yylval=gen(at_dollar,2); return T_DOLLAR_MAPLE;
"'$'"                  index_status=0; giac_yylval=gen(at_dollar,2); return T_QUOTED_BINARY;
"_seqgen"                  index_status=0; giac_yylval=gen(at_dollar,2); return T_QUOTED_BINARY;
":="			index_status=0; giac_yylval=gen(at_sto,2); return T_AFFECT;
"':='"                  index_status=0; giac_yylval=gen(at_sto,2); return T_QUOTED_BINARY;
"_assign"                  index_status=0; giac_yylval=gen(at_sto,2); return T_QUOTED_BINARY;
"=>"                    index_status=0; giac_yylval=gen(at_sto,2); return TI_STO;
"@"{D}+                   index_status=1; giac_yytext[0]='0'; giac_yylval = symb_double_deux_points(makevecteur(_IDNT_break,chartab2gen( giac_yytext))); return T_SYMBOL;
"@"                     if (maple_mode!=3) {index_status=0; giac_yylval=gen(at_compose,2); return T_COMPOSE; } BEGIN(comment_hash);
"@@"                     index_status=0; giac_yylval=gen(at_composepow,2); return T_POW;
"'@@'"                     index_status=0; giac_yylval=gen(at_composepow,2); return T_QUOTED_BINARY;
"_fnest"                     index_status=0; giac_yylval=gen(at_composepow,2); return T_QUOTED_BINARY;
"'@'"                  index_status=0; giac_yylval=gen(at_compose,2); return T_QUOTED_BINARY;
"_fconcat"                  index_status=0; giac_yylval=gen(at_compose,2); return T_QUOTED_BINARY;
"&&"                    index_status=0; giac_yylval=gen(at_and,2); return T_AND_OP;
"and"                   index_status=0; giac_yylval=gen(at_and,2); return T_AND_OP;
"AND"                   index_status=0; giac_yylval=gen(at_and,2); return T_AND_OP;
"'&&'"                  index_status=0; giac_yylval=gen(at_and,2); return T_QUOTED_BINARY;
"'and'"                 index_status=0; giac_yylval=gen(at_and,2); return T_QUOTED_BINARY;
"_and"                 index_status=0; giac_yylval=gen(at_and,2); return T_QUOTED_BINARY;
"|"                     index_status=0; giac_yylval=gen(at_tilocal,2); return T_PIPE;
"||"                    index_status=0; giac_yylval=gen(at_ou,2); return T_OR_OP;
"'||'"                  index_status=0; giac_yylval=gen(at_ou,2); return T_QUOTED_BINARY;
"or"                    index_status=0; giac_yylval=gen(at_ou,2); return T_OR_OP;
"'or'"                  index_status=0; giac_yylval=gen(at_ou,2); return T_QUOTED_BINARY;
"_or"                  index_status=0; giac_yylval=gen(at_ou,2); return T_QUOTED_BINARY;
"OR"                    index_status=0; giac_yylval=gen(at_ou,2); return T_OR_OP;
"xor"                    index_status=0; giac_yylval=gen(at_xor,2); return T_XOR_OP;
"_xor"                  index_status=0; giac_yylval=gen(at_xor,2); return T_QUOTED_BINARY;
"'xor'"                  index_status=0; giac_yylval=gen(at_xor,2); return T_QUOTED_BINARY;
"XOR"                    index_status=0; giac_yylval=gen(at_xor,2); return T_XOR_OP;
".."                    index_status=0; giac_yylval=gen(at_interval,2); return T_INTERVAL;
"'..'"                  index_status=0; giac_yylval=gen(at_interval,2); return T_QUOTED_BINARY;
"_range"                  index_status=0; giac_yylval=gen(at_interval,2); return T_QUOTED_BINARY;
"!"                     if (maple_mode || index_status) { giac_yylval=gen(at_factorial); return T_FACTORIAL; } else { index_status=0; giac_yylval=gen(at_not,1); return T_NOT; }

    /* standard functions */
"+"                     index_status=0; giac_yylval=gen(at_plus,2); return T_PLUS;
".+"                    index_status=0; giac_yylval=gen(at_plus,2); return T_PLUS;
"&"                     index_status=0; giac_yylval=gen(at_plus,2); return T_PLUS;
  /* "','"                   index_status=0; giac_yylval=gen(at_makevector,2); return T_QUOTED_BINARY; commented because of f('a','b') */
"'+'"                   index_status=0; giac_yylval=gen(at_plus,2); return T_QUOTED_BINARY;
"_plus"                   index_status=0; giac_yylval=gen(at_plus,2); return T_QUOTED_BINARY;
"-"                     index_status=0; giac_yylval=gen(at_binary_minus,2); return T_MOINS;
".-"                     index_status=0; giac_yylval=gen(at_binary_minus,2); return T_MOINS;
"'-'"                   index_status=0; giac_yylval=gen(at_binary_minus,2); return T_QUOTED_BINARY;
"_subtract"                   index_status=0; giac_yylval=gen(at_binary_minus,2); return T_QUOTED_BINARY;
"*"                     index_status=0; giac_yylval=gen(at_prod,2); return T_FOIS;
"&*"                     index_status=0; giac_yylval=gen(at_ampersand_times,2); return T_FOIS;
"&^"                     index_status=0; giac_yylval=gen(at_quote_pow,2); return T_POW;
".*"                     index_status=0; giac_yylval=gen(at_pointprod,2); return T_FOIS;
"'*'"                   index_status=0; giac_yylval=gen(at_prod,2); return T_QUOTED_BINARY;
"_mult"                   index_status=0; giac_yylval=gen(at_prod,2); return T_QUOTED_BINARY;
"/"                     index_status=0; giac_yylval=gen(at_division,2); return T_DIV;
"./"                     index_status=0; giac_yylval=gen(at_pointdivision,2); return T_DIV;
"'/'"                   index_status=0; giac_yylval=gen(at_division,2); return T_QUOTED_BINARY;
"_divide"                   index_status=0; giac_yylval=gen(at_division,2); return T_QUOTED_BINARY;
"%"                     index_status=0; if (maple_mode==3) { giac_yylval=gen(at_pourcent); return T_FACTORIAL; } if (maple_mode==1) { giac_yylval=symbolic(at_ans,vecteur(0)); return T_NUMBER; }  if (maple_mode) giac_yylval=gen(at_irem,2); else giac_yylval=0; return T_MOD;
"'%'"                  index_status=0; giac_yylval=gen(at_irem,2); return T_QUOTED_BINARY;
"mod"                   index_status=0; if (maple_mode==3) { giac_yylval=gen(at_irem,2); return T_UNARY_OP; } else { if (maple_mode) giac_yylval=gen(at_irem,2); else giac_yylval=0; return T_MOD; }
"'mod'"                  index_status=0; giac_yylval=gen(at_irem,2); return T_QUOTED_BINARY;
"_mod"                  index_status=0; giac_yylval=gen(at_irem,2); return T_QUOTED_BINARY;
"MOD"                   index_status=0; return T_MOD;
"^"                     index_status=0; giac_yylval=gen(at_pow,2); return T_POW;
"**"                     index_status=0; giac_yylval=gen(at_pow,2); return T_POW;
".^"                     index_status=0; giac_yylval=gen(at_pointpow,2); return T_POW;
"'^'"                   index_status=0; giac_yylval=gen(at_pow,2); return T_QUOTED_BINARY;
"_power"                   index_status=0; giac_yylval=gen(at_pow,2); return T_QUOTED_BINARY;
"SWAP"                  giac_yylval = gen(at_SWAP,0); index_status=0; return T_RPN_OP;
"DROP"                  giac_yylval = gen(at_DROP,0); index_status=0; return T_RPN_OP;
"DUP"                   giac_yylval = gen(at_DUP,0); index_status=0; return T_RPN_OP;
"ROLL"                  giac_yylval = gen(at_ROLL,0); index_status=0; return T_RPN_OP;
"ROLLD"                 giac_yylval = gen(at_ROLLD,0); index_status=0; return T_RPN_OP;
"PICK"                  giac_yylval = gen(at_PICK,0); index_status=0; return T_RPN_OP;
"Digits"                  giac_yylval = gen(at_Digits,0); index_status=0; return T_DIGITS;
"DIGITS"                  giac_yylval = gen(at_Digits,0); index_status=0; return T_DIGITS;
"abcuv"		giac_yylval = gen(at_abcuv,3); index_status=0; return T_UNARY_OP;
"abs"			giac_yylval = gen(at_abs,1); index_status=0; return T_UNARY_OP;
"abscisse"		giac_yylval = gen(at_abscisse,1); index_status=0; return T_UNARY_OP;
"acos"			giac_yylval = gen(at_acos,1); index_status=0; return T_UNARY_OP;
"acos2asin"               giac_yylval = gen(at_acos2asin,1); index_status=0; return T_UNARY_OP;
"acos2atan"               giac_yylval = gen(at_acos2atan,1); index_status=0; return T_UNARY_OP;
"arccos"		giac_yylval = gen(at_acos,1); index_status=0; return T_UNARY_OP;
"acosh"			giac_yylval = gen(at_acosh,1); index_status=0; return T_UNARY_OP;
"Airy_Ai"		giac_yylval = gen(at_Airy_Ai,1); index_status=0; return T_UNARY_OP;
"Airy_Bi"		giac_yylval = gen(at_Airy_Bi,1); index_status=0; return T_UNARY_OP;
"erfc"		giac_yylval = gen(at_erfc,1); index_status=0; return T_UNARY_OP;
"erf"		giac_yylval = gen(at_erf,1); index_status=0; return T_UNARY_OP;
"randNorm"		giac_yylval = gen(at_randNorm,1); index_status=0; return T_UNARY_OP;
"randnorm"		giac_yylval = gen(at_randNorm,1); index_status=0; return T_UNARY_OP;
"arccosh"		giac_yylval = gen(at_acosh,1); index_status=0; return T_UNARY_OP;
"affixe"		giac_yylval = gen(at_affixe,1); index_status=0; return T_UNARY_OP;
"alg"                   giac_yylval = gen(at_alg,0); index_status=0; return T_UNARY_OP;
"algvar"		giac_yylval = gen(at_algvar,1); index_status=0; return T_UNARY_OP;
"angle"		        giac_yylval = gen(at_angle,1); index_status=0; return T_UNARY_OP;
"angle_radian"		giac_yylval = gen(at_angle_radian,0); index_status=0; return T_DIGITS;
"ans"		        giac_yylval = gen(at_ans,1); index_status=0; return T_UNARY_OP;
"quest"		        giac_yylval = gen(at_quest,1); index_status=0; return T_UNARY_OP;
"append"		giac_yylval = gen(at_append,2); index_status=0; return T_UNARY_OP;
"approx_mode"		giac_yylval = gen(at_approx_mode,0); index_status=0; return T_DIGITS;
"arg"			giac_yylval = gen(at_arg,1); index_status=0; return T_UNARY_OP;
"args"			index_status=1; return T_ARGS;
"'args'"                index_status=0; giac_yylval=gen(at_args,0); return T_QUOTED_BINARY;
"asin"			giac_yylval = gen(at_asin,1); index_status=0; return T_UNARY_OP;
"asc"			giac_yylval = gen(at_asc,1); index_status=0; return T_UNARY_OP;
"assume"		giac_yylval = gen(at_assume,1); index_status=0; return T_UNARY_OP;
"posons"		giac_yylval = gen(at_assume,1); index_status=0; return T_UNARY_OP;
"arcsin"		giac_yylval = gen(at_asin,1); index_status=0; return T_UNARY_OP;
"asin2acos"               giac_yylval = gen(at_asin2acos,1); index_status=0; return T_UNARY_OP;
"asin2atan"               giac_yylval = gen(at_asin2atan,1); index_status=0; return T_UNARY_OP;
"asinh"			giac_yylval = gen(at_asinh,1); index_status=0; return T_UNARY_OP;
"arcsinh"		giac_yylval = gen(at_asinh,1); index_status=0; return T_UNARY_OP;
"atan"			giac_yylval = gen(at_atan,1); index_status=0; return T_UNARY_OP;
"atan2acos"               giac_yylval = gen(at_atan2acos,1); index_status=0; return T_UNARY_OP;
"atan2asin"               giac_yylval = gen(at_atan2asin,1); index_status=0; return T_UNARY_OP;
"atrig2ln"               giac_yylval = gen(at_atrig2ln,1); index_status=0; return T_UNARY_OP;
"at"			giac_yylval = gen(at_at,2); index_status=0; return T_UNARY_OP;
"archive"		giac_yylval = gen(at_archive,2); index_status=0; return T_UNARY_OP;
"arctan"		giac_yylval = gen(at_atan,1); index_status=0; return T_UNARY_OP;
"atanh"			giac_yylval = gen(at_atanh,1); index_status=0; return T_UNARY_OP;
"apply"			giac_yylval = gen(at_apply,2); index_status=0; return T_UNARY_OP;
"arctanh"		giac_yylval = gen(at_atanh,1); index_status=0; return T_UNARY_OP;
"as_function_of"        giac_yylval = gen(at_as_function_of,1); index_status=0; return T_UNARY_OP;
"a2q"                   giac_yylval = gen(at_a2q,2); index_status=0; return T_UNARY_OP;
"backquote"		giac_yylval = gen(at_backquote,1); index_status=0; return T_UNARY_OP;
"basis"		giac_yylval = gen(at_basis,1);index_status=0; return T_UNARY_OP;
"bissectrice"		giac_yylval = gen(at_bissectrice,2); index_status=0; return T_UNARY_OP;
"exbissectrice"		giac_yylval = gen(at_exbissectrice,2); index_status=0; return T_UNARY_OP;
"bloc"  		giac_yylval = gen(at_bloc,1); index_status=0; return T_UNARY_OP;
"begin"                 index_status=0; return T_BLOC_BEGIN;
"BEGIN"                 index_status=0; return T_BLOC_BEGIN;
"bernoulli"		giac_yylval = gen(at_bernoulli,1); index_status=0; return T_UNARY_OP;
"break"  		index_status=0; giac_yylval=gen(at_break,0); return T_BREAK;
"BREAK"  		index_status=0; giac_yylval=gen(at_break,0); return T_BREAK;
"breakpoint"		giac_yylval = gen(at_breakpoint,1); index_status=0; return T_UNARY_OP;
"by"                    index_status=0; return T_BY;
"case"                  index_status=0; return T_CASE;
"CASE"                  index_status=0; return T_CASE;
"catch"                 index_status=0; return T_CATCH;
"ceil"                  giac_yylval = gen(at_ceil,1); index_status=0; return T_UNARY_OP;
"cd"                    giac_yylval = gen(at_cd,1); index_status=0; return T_UNARY_OP;
"cell"                  giac_yylval = gen(at_cell,2); index_status=0; return T_UNARY_OP;
"centre"                giac_yylval = gen(at_centre,1); index_status=0; return T_UNARY_OP;
"char"                  giac_yylval = gen(at_char,1); index_status=0; return T_UNARY_OP;
"continue"  		index_status=0; return T_CONTINUE;
"chinrem"		giac_yylval = gen(at_chinrem,2); index_status=0; return T_UNARY_OP;
"cercle"                giac_yylval = gen(at_cercle,2); index_status=0; return T_UNARY_OP;
"circonscrit"		giac_yylval = gen(at_circonscrit,2); index_status=0; return T_UNARY_OP;
"click"		        giac_yylval = gen(at_click,1); index_status=0; return T_UNARY_OP;
"inputform"		giac_yylval = gen(at_inputform,1); index_status=0; return T_UNARY_OP;
"interactive"		giac_yylval = gen(at_interactive,1); index_status=0; return T_UNARY_OP;
"keyboard"		giac_yylval = gen(at_keyboard,1); index_status=0; return T_UNARY_OP;
"widget_size"		giac_yylval = gen(at_widget_size,1); index_status=0; return T_UNARY_OP;
"current_sheet"         giac_yylval = gen(at_current_sheet,0); index_status=0; return T_UNARY_OP;
"cas_setup"		giac_yylval = gen(at_cas_setup,4); index_status=0; return T_UNARY_OP;
"choosebox"		giac_yylval = gen(at_choosebox,1); index_status=0; return T_UNARY_OP;
"output"		giac_yylval = gen(at_output,1); index_status=0; return T_UNARY_OP;
"Text"                  giac_yylval = gen(at_Text,1); index_status=0; return T_RETURN;
"DropDown"                  giac_yylval = gen(at_DropDown,1); index_status=0; return T_RETURN;
"Popup"                  giac_yylval = gen(at_Popup,1); index_status=0; return T_RETURN;
"Request"                  giac_yylval = gen(at_Request,1); index_status=0; return T_RETURN;
"Title"                  giac_yylval = gen(at_Title,1); index_status=0; return T_RETURN;
"conique_reduite"       giac_yylval = gen(at_conique_reduite,1); index_status=0; return T_UNARY_OP;
"cholesky"			giac_yylval = gen(at_cholesky,1); index_status=0; return T_UNARY_OP;
"collect"		giac_yylval = gen(at_collect,1); index_status=0; return T_UNARY_OP;
"comb"		        giac_yylval = gen(at_comb,2); index_status=0; return T_UNARY_OP;
"comment"		giac_yylval = gen(at_comment,1); index_status=0; return T_UNARY_OP;
"complex_mode"		giac_yylval = gen(at_complex_mode,1); index_status=0; return T_DIGITS;
"complex_variables"	giac_yylval = gen(at_complex_variables,0); index_status=0; return T_DIGITS;
"concat"		giac_yylval = gen(at_concat,2); index_status=0; return T_UNARY_OP;
"conj"			giac_yylval = gen(at_conj,1); index_status=0; return T_UNARY_OP;
"cont"		        giac_yylval = gen(at_cont,1); index_status=0; return T_UNARY_OP;
"CONT"		        giac_yylval = gen(at_cont,1); index_status=0; return T_UNARY_OP;
"contains"              giac_yylval = gen(at_contains,2); index_status=0; return T_UNARY_OP;
"cos"			giac_yylval = gen(at_cos,1); index_status=0; return T_UNARY_OP;
"cosh"			giac_yylval = gen(at_cosh,1); index_status=0; return T_UNARY_OP;
"couleur"               giac_yylval = gen(at_couleur,2); index_status=0; return T_UNARY_OP;
"affichage"             giac_yylval = gen(at_couleur,2); index_status=0; return T_UNARY_OP;
"convert"               giac_yylval = gen(at_convert,2); index_status=0; return T_UNARY_OP;
"cross"                 giac_yylval = gen(at_cross,2); index_status=0; return T_UNARY_OP;
"curve"                 giac_yylval = gen(at_curve,2); index_status=0; return T_UNARY_OP;
"cyclotomic"		giac_yylval = gen(at_cyclotomic,1); index_status=0; return T_UNARY_OP;
"debug"		        giac_yylval = gen(at_debug,1); index_status=0; return T_UNARY_OP;
"DEBUG"		        giac_yylval = gen(at_debug,1); index_status=0; return T_UNARY_OP;
"default"               index_status=1; return T_DEFAULT;
"derive"		giac_yylval = gen(at_derive,2); index_status=0; return T_UNARY_OP;
"det"			giac_yylval = gen(at_det,1); index_status=0; return T_UNARY_OP;
"det_minor"		giac_yylval = gen(at_det_minor,1); index_status=0; return T_UNARY_OP;
"divis"			giac_yylval = gen(at_divis,1); index_status=0; return T_UNARY_OP;
"fxnd"			giac_yylval = gen(at_fxnd,1); index_status=0; return T_UNARY_OP;
"longueur"		giac_yylval = gen(at_longueur,1);index_status=0; return T_UNARY_OP;
"longueur2"		giac_yylval = gen(at_longueur2,1);index_status=0; return T_UNARY_OP;
"lu"		giac_yylval = gen(at_lu,2);index_status=0; return T_UNARY_OP;
"do"			index_status=0; giac_yylval=gen(at_for,4); return T_DO;
"DO"			index_status=0; giac_yylval=gen(at_for,4); return T_DO;
"dot"			giac_yylval = gen(at_prod,2); index_status=0; return T_UNARY_OP;
"diff"  		giac_yylval = gen(at_derive,2); index_status=0; return T_UNARY_OP;
"D"  		if (maple_mode==1 || maple_mode==2) { giac_yylval = gen(at_function_diff,1); index_status=1; return T_UNARY_OP;} else { index_status=1; return find_or_make_symbol(giac_yytext,giac_yylval); }
"function_diff"  		giac_yylval = gen(at_function_diff,1); index_status=0; return T_UNARY_OP;
"e"                     if (maple_mode==1 || maple_mode==2) { giac_yylval=e__IDNT_e_; }else giac_yylval=e__IDNT_e; return T_SYMBOL;
"e2r"			giac_yylval = gen(at_e2r,2); index_status=0; return T_UNARY_OP;
"egcd"			giac_yylval = gen(at_egcd,2); index_status=0; return T_UNARY_OP;
"egv"			giac_yylval = gen(at_egv,1); index_status=0; return T_UNARY_OP;
"egvl"			giac_yylval = gen(at_egvl,1); index_status=0; return T_UNARY_OP;
"element"               giac_yylval = gen(at_element,1); index_status=0; return T_UNARY_OP;
"ellipse"               giac_yylval = gen(at_ellipse,3); index_status=0; return T_UNARY_OP;
"else"			index_status=0; giac_yylval=gen(at_ifte,3); return T_ELSE;
"ELSE"			index_status=0; giac_yylval=gen(at_ifte,3); return T_ELSE;
"elif"                  index_status=0; return T_ELIF;
"end"			index_status=0; return T_BLOC_END;
"END"			index_status=0; return T_BLOC_END;
"end_for"               index_status=0; return T_BLOC_END;
"end_if"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end if"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end do"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end proc"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"end_while"             index_status=0; return T_BLOC_END;
"end_case"              index_status=0; return T_ENDCASE;
"end_proc"              index_status=0; return T_BLOC_END;
"epsilon"               giac_yylval = gen(at_epsilon,0); index_status=0; return T_DIGITS;
"equal"			giac_yylval = gen(at_equal,2); index_status=0; return T_UNARY_OP;
"equation"		giac_yylval = gen(at_equation,2); index_status=0; return T_UNARY_OP;
"error"		        index_status=0; return T_THROW;
"erase"                 giac_yylval = gen(at_erase,0); index_status=0; return T_UNARY_OP;
"ERROR"		        index_status=0; return T_THROW;
"esac"                  index_status=0; return T_ENDCASE;
"est_aligne"            giac_yylval = gen(at_est_aligne,3);index_status=0; return T_UNARY_OP;
"est_cocyclique"        giac_yylval = gen(at_est_cocyclique,4);index_status=0; return T_UNARY_OP;
"est_element"            giac_yylval = gen(at_est_element,2);index_status=0; return T_UNARY_OP;
"est_parallele"         giac_yylval = gen(at_est_parallele,2);index_status=0; return T_UNARY_OP;
"est_perpendiculaire"   giac_yylval = gen(at_est_perpendiculaire,2);index_status=0; return T_UNARY_OP;
"euler"			giac_yylval = gen(at_euler,1); index_status=0; return T_UNARY_OP;
"eval"			giac_yylval = gen(at_eval,1); index_status=0; return T_UNARY_OP;
"evalb"			giac_yylval = gen(at_evalb,1); index_status=0; return T_UNARY_OP;
"evalm"			giac_yylval = gen(at_evalm,1); index_status=0; return T_UNARY_OP;
"evalf"			giac_yylval = gen(at_evalf,1); index_status=0; return T_UNARY_OP;
"exp"			giac_yylval = gen(at_exp,1); index_status=0; return T_UNARY_OP;
"exp2pow" 		giac_yylval = gen(at_exp2pow,1); index_status=0; return T_UNARY_OP;
"expand"                if (maple_mode==3) giac_yylval=gen(at_partfrac); else giac_yylval = gen(at_expand,1); index_status=0; return T_UNARY_OP;
"export"		giac_yylval = gen(at_insmod,1); index_status=0; return T_UNARY_OP;
"ezgcd"			giac_yylval = gen(at_ezgcd,2); index_status=0; return T_UNARY_OP;
"heugcd"		giac_yylval = gen(at_heugcd,2); index_status=0; return T_UNARY_OP;
"modgcd"		giac_yylval = gen(at_modgcd,2); index_status=0; return T_UNARY_OP;
"psrgcd"		giac_yylval = gen(at_psrgcd,2); index_status=0; return T_UNARY_OP;
"false"                  index_status=0; giac_yylval = zero; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"FALSE"                  index_status=0; giac_yylval = zero; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"factor"		giac_yylval = gen(at_factor,1); index_status=0; return T_UNARY_OP;
"factors"		giac_yylval = gen(at_factors,1); index_status=0; return T_UNARY_OP;
"factorial"		giac_yylval = gen(at_factorial,1); index_status=0; return T_UNARY_OP;
"fdistrib"		giac_yylval = gen(at_expand,1); index_status=0; return T_UNARY_OP;
"findhelp"		giac_yylval = gen(at_findhelp,1); index_status=0; return T_UNARY_OP;
"for"			index_status=0; giac_yylval=gen(at_for,4); return T_FOR;
"FOR"			index_status=0; giac_yylval=gen(at_for,4); return T_FOR;
"from"			index_status=0; return T_FROM;
"feuille"               giac_yylval = gen(at_feuille,1); index_status=0; return T_UNARY_OP;
"fi"                    index_status=0; return T_BLOC_END;
"floor"                 giac_yylval = gen(at_floor,1); index_status=0; return T_UNARY_OP;
"fracmod"		giac_yylval = gen(at_fracmod,1); index_status=0; return T_UNARY_OP;
"fsolve"		giac_yylval = gen(at_fsolve,1); index_status=0; return T_UNARY_OP;
"fourier_an"		giac_yylval = gen(at_fourier_an,4); index_status=0; return T_UNARY_OP;
"fourier_bn"		giac_yylval = gen(at_fourier_bn,4); index_status=0; return T_UNARY_OP;
"fourier_cn"		giac_yylval = gen(at_fourier_cn,4); index_status=0; return T_UNARY_OP;
"gamma"			giac_yylval = gen(at_gamma,1); index_status=0; return T_UNARY_OP;
"gauss"			giac_yylval = gen(at_gauss,2); index_status=0; return T_UNARY_OP;
"gbasis"		giac_yylval = gen(at_gbasis,1); index_status=0; return T_UNARY_OP;
"gcd"			giac_yylval = gen(at_gcd,2); index_status=0; return T_UNARY_OP;
"grad"			giac_yylval = gen(at_grad,2); index_status=0; return T_UNARY_OP;
"latex"		giac_yylval = gen(at_latex,1); index_status=0; return T_UNARY_OP;
"TeX"		giac_yylval = gen(at_TeX,1); index_status=0; return T_UNARY_OP;
"global"                index_status=0; return T_GLOBAL;
"graph2tex"		giac_yylval = gen(at_graph2tex,1); index_status=0; return T_UNARY_OP;
"graph3d2tex"		giac_yylval = gen(at_graph3d2tex,1); index_status=0; return T_UNARY_OP;
"greduce"		giac_yylval = gen(at_greduce,1); index_status=0; return T_UNARY_OP;
"halftan"               giac_yylval = gen(at_halftan,1); index_status=0; return T_UNARY_OP;
"halftan_hyp2exp"       giac_yylval = gen(at_halftan_hyp2exp,1); index_status=0; return T_UNARY_OP;
"halt"		        giac_yylval = gen(at_halt,1); index_status=0; return T_UNARY_OP;
"HALT"		        giac_yylval = gen(at_halt,1); index_status=0; return T_UNARY_OP;
"has"                   giac_yylval = gen(at_has,2); index_status=0; return T_UNARY_OP;
"hauteur"               giac_yylval = gen(at_hauteur,2); index_status=0; return T_UNARY_OP;
"head"                  giac_yylval = gen(at_head,1); index_status=0; return T_UNARY_OP;
"hessenberg"            giac_yylval = gen(at_hessenberg,1); index_status=0; return T_UNARY_OP;
"homothetie"		giac_yylval = gen(at_homothetie,2); index_status=0; return T_UNARY_OP;
"horner"		giac_yylval = gen(at_horner,2); index_status=0; return T_UNARY_OP;
"hyp2exp"               giac_yylval = gen(at_hyp2exp,1); index_status=0; return T_UNARY_OP;
"hyperbole"             giac_yylval = gen(at_hyperbole,3); index_status=0; return T_UNARY_OP;
"iabcuv"		giac_yylval = gen(at_iabcuv,3); index_status=0; return T_UNARY_OP;
"ibasis"		giac_yylval = gen(at_ibasis,2);index_status=0; return T_UNARY_OP;
"ibpdv"		giac_yylval = gen(at_ibpdv,2);index_status=0; return T_UNARY_OP;
"ichinrem"		giac_yylval = gen(at_ichinrem,2); index_status=0; return T_UNARY_OP;
"id"			giac_yylval = gen(at_id,1); index_status=0; return T_UNARY_OP;
"idn"			giac_yylval = gen(at_idn,1); index_status=0; return T_UNARY_OP;
"idivis"			giac_yylval = gen(at_idivis,1); index_status=0; return T_UNARY_OP;
"iegcd"			giac_yylval = gen(at_iegcd,2); index_status=0; return T_UNARY_OP;
"if"      		index_status=0; giac_yylval=gen(at_ifte,3); if (maple_mode==3) return T_TI_IF; if (maple_mode!=0) return T_MUPMAP_IF; return T_IF;
"IF"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_RPN_IF;
"ifte"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_IFTE;
"when"      		index_status=0; giac_yylval=gen(at_when,3); return T_UNARY_OP;
"IFTE"      		index_status=0; giac_yylval=gen(at_ifte,3); return T_IFTE;
"'ifte'"                index_status=0; giac_yylval=gen(at_ifte,3); return T_QUOTED_BINARY;
"'if'"                index_status=0; giac_yylval=gen(at_ifte,3); return T_QUOTED_BINARY;
"_if"                index_status=0; giac_yylval=gen(at_ifte,3); return T_QUOTED_BINARY;
"ifactor"      		giac_yylval = gen(at_ifactor,1); index_status=0; return T_UNARY_OP;
"ifactors"      	if (maple_mode==1) giac_yylval = gen(at_maple_ifactors); else giac_yylval = gen(at_ifactors,1); index_status=0; return T_UNARY_OP;
"maple_ifactors"      	giac_yylval = gen(at_maple_ifactors);  index_status=0; return T_UNARY_OP;
"im"			giac_yylval = gen(at_im,1); index_status=0; return T_UNARY_OP;
"image"			giac_yylval = gen(at_image,1); index_status=0; return T_UNARY_OP;
"in"                    index_status=0; return T_IN;
"INTVX"			giac_yylval = gen(at_integrate,1); index_status=0; return T_UNARY_OP;
"interactive_plotode" 	giac_yylval = gen(at_interactive_plotode,4); index_status=0; return T_UNARY_OP;
"interactive_odeplot" 	giac_yylval = gen(at_interactive_odeplot,4); index_status=0; return T_UNARY_OP;
"integrate"		giac_yylval = gen(at_integrate,4); index_status=0; return T_UNARY_OP;
"inv"			giac_yylval = gen(at_inv,1); index_status=0; return T_UNARY_OP;
"inversion"		giac_yylval = gen(at_inversion,2); index_status=0; return T_UNARY_OP;
"inscrit"		giac_yylval = gen(at_inscrit,2); index_status=0; return T_UNARY_OP;
"exinscrit"		giac_yylval = gen(at_exinscrit,2); index_status=0; return T_UNARY_OP;
"insmod"		giac_yylval = gen(at_insmod,1); index_status=0; return T_UNARY_OP;
"inter"			giac_yylval = gen(at_inter,2); index_status=0; return T_UNARY_OP;
"intersect"             index_status=0; return T_INTERSECT;
"'intersect'"                  index_status=0; giac_yylval=gen(at_intersect,2); return T_QUOTED_BINARY;
"_intersect"                  index_status=0; giac_yylval=gen(at_intersect,2); return T_QUOTED_BINARY;
"iquo"			giac_yylval = gen(at_iquo,2); index_status=0; return T_UNARY_OP;
"iquorem"		giac_yylval = gen(at_iquorem,2); index_status=0; return T_UNARY_OP;
"irem"			giac_yylval = gen(at_irem,2); index_status=0; return T_UNARY_OP;
"is_prime"		giac_yylval = gen(at_is_prime,1); index_status=0; return T_UNARY_OP;
"simp2"		giac_yylval = gen(at_simp2,2); index_status=0; return T_UNARY_OP;
"isobarycentre"		giac_yylval = gen(at_isobarycentre,2); index_status=0; return T_UNARY_OP;
"isom"		        giac_yylval = gen(at_isom,1); index_status=0; return T_UNARY_OP;
"jacobi_symbol"		giac_yylval = gen(at_jacobi_symbol,2); index_status=0; return T_UNARY_OP;
"jordan"		giac_yylval = gen(at_jordan,1); index_status=0; return T_UNARY_OP;
"ker"			giac_yylval = gen(at_ker,1); index_status=0; return T_UNARY_OP;
"kill"		        giac_yylval = gen(at_kill,1); index_status=0; return T_UNARY_OP;
"KILL"		        giac_yylval = gen(at_kill,1); index_status=0; return T_UNARY_OP;
"lcm"			giac_yylval = gen(at_lcm,2); index_status=0; return T_UNARY_OP;
"lieu"			giac_yylval = gen(at_lieu,1); index_status=0; return T_UNARY_OP;
"lin"			giac_yylval = gen(at_lin,1); index_status=0; return T_UNARY_OP;
"linsolve"              giac_yylval = gen(at_linsolve,2); index_status=0; return T_UNARY_OP;
"legende"               giac_yylval = gen(at_legende,2); index_status=0; return T_UNARY_OP;
"legendre_symbol"		giac_yylval = gen(at_legendre_symbol,2); index_status=0; return T_UNARY_OP;
"ln"			giac_yylval = gen(at_ln,1); index_status=0; return T_UNARY_OP;
"local"			index_status=0; return T_LOCAL;
"localbloc"		index_status=0; return T_LOCALBLOC;
"LOCAL"			index_status=0; return T_LOCAL;
"log"			giac_yylval = gen(at_ln,1); index_status=1; return T_UNARY_OP; /* index_status=1 to accept log[] for a basis log */
"log10"			giac_yylval = gen(at_log10,1); index_status=0; return T_UNARY_OP;
"limit"			giac_yylval = gen(at_limit,2); index_status=0; return T_UNARY_OP;
"demi_droite"           giac_yylval = gen(at_demi_droite,2); index_status=0; return T_UNARY_OP;
"droite"                giac_yylval = gen(at_droite,2); index_status=0; return T_UNARY_OP;
"lncollect"			giac_yylval = gen(at_lncollect,1); index_status=0; return T_UNARY_OP;
"lvar"			giac_yylval = gen(at_lvar,1); index_status=0; return T_UNARY_OP;
"lname"			giac_yylval = gen(at_lname,1); index_status=0; return T_UNARY_OP;
"lsmod"		        giac_yylval = gen(at_lsmod,1); index_status=0; return T_UNARY_OP;
"max"			giac_yylval = gen(at_max,2); index_status=0; return T_UNARY_OP;
"makelist"              giac_yylval = gen(at_makelist,3); index_status=0; return T_UNARY_OP;
"makesuite"              giac_yylval = gen(at_makesuite,1); index_status=0; return T_UNARY_OP;
"makevector"              giac_yylval = gen(at_makevector,1); index_status=0; return T_UNARY_OP;
"matrix"              giac_yylval = gen(at_matrix,1); index_status=0; return T_UNARY_OP;
"makemat"               giac_yylval = gen(at_makemat,3); index_status=0; return T_UNARY_OP;
"map"			giac_yylval = gen(at_map,2); index_status=0; return T_UNARY_OP;
"maple2mupad"		giac_yylval = gen(at_maple2mupad,2); index_status=0; return T_UNARY_OP;
"maple2xcas"		giac_yylval = gen(at_maple2xcas,2); index_status=0; return T_UNARY_OP;
"mupad2maple"		giac_yylval = gen(at_mupad2maple,2); index_status=0; return T_UNARY_OP;
"mupad2xcas"		giac_yylval = gen(at_mupad2xcas,2); index_status=0; return T_UNARY_OP;
"maple_mode"		giac_yylval = gen(at_maple_mode,2); index_status=0; return T_UNARY_OP;
"milieu"                giac_yylval = gen(at_milieu,2); index_status=0; return T_UNARY_OP;
"mediane"		giac_yylval = gen(at_mediane,2); index_status=0; return T_UNARY_OP;
"mediatrice"		giac_yylval = gen(at_mediatrice,2); index_status=0; return T_UNARY_OP;
"member"			giac_yylval = gen(at_member,2); index_status=0; return T_UNARY_OP;
"min"			giac_yylval = gen(at_min,2); index_status=0; return T_UNARY_OP;
"mkisom"		giac_yylval = gen(at_mkisom,2); index_status=0; return T_UNARY_OP;
"minus"                 index_status=0; return T_MINUS;
"'minus'"                  index_status=0; giac_yylval=gen(at_minus,2); return T_QUOTED_BINARY;
"_minus"                  index_status=0; giac_yylval=gen(at_minus,2); return T_QUOTED_BINARY;
"mods"			giac_yylval = gen(at_mods,2); index_status=0; return T_UNARY_OP;
"neg"                giac_yylval = gen(at_neg,1); index_status=0; return T_UNARY_OP;
"newton"                giac_yylval = gen(at_newton,2); index_status=0; return T_UNARY_OP;
"next"		        index_status=0; return T_CONTINUE;
"NEXT"		        index_status=0; return T_CONTINUE;
"nextprime"		giac_yylval = gen(at_nextprime,1); index_status=0; return T_UNARY_OP;
"normal"		giac_yylval = gen(at_normal,1); index_status=0; return T_UNARY_OP;
"nodisp"                  giac_yylval = gen(at_nodisp,1); index_status=0; return T_UNARY_OP;
"not"		        giac_yylval = gen(at_not,1); if (maple_mode) return T_NOT;  index_status=0; return T_UNARY_OP;
"'not'"                  index_status=0; giac_yylval=gen(at_not,1); return T_QUOTED_BINARY;
"_not"                  index_status=0; giac_yylval=gen(at_not,1); return T_QUOTED_BINARY;
"NULL"                  index_status=1; return T_NULL;
"nop"                   giac_yylval = gen(at_nop,1); index_status=0; return T_UNARY_OP;
"normalf"		giac_yylval = gen(at_greduce,1); index_status=0; return T_UNARY_OP;
"od"                    index_status=0; return T_BLOC_END;
"odesolve"		giac_yylval = gen(at_odesolve,1); index_status=0; return T_UNARY_OP;
"of"			index_status=0; return T_OF;
"'of'"                  index_status=0; giac_yylval=gen(at_of,2); return T_QUOTED_BINARY;
"op"                    if (maple_mode==1) giac_yylval = gen(at_maple_op,1); else giac_yylval = gen(at_feuille,1); index_status=0; return T_UNARY_OP;
"option"                index_status=0; return T_OPTION;
"ordonnee"		giac_yylval = gen(at_ordonnee,1); index_status=0; return T_UNARY_OP;
"order_size"		giac_yylval = gen(at_order_size,1); index_status=0; return T_UNARY_OP;
"otherwise"             index_status=0; return T_DEFAULT;
"pa2b2"              giac_yylval = gen(at_pa2b2,1); index_status=0; return T_UNARY_OP;
"parabole"              giac_yylval = gen(at_parabole,2); index_status=0; return T_UNARY_OP;
"parallele"             giac_yylval = gen(at_parallele,2); index_status=0; return T_UNARY_OP;
"parameter"             giac_yylval = gen(at_parameter,3); index_status=0; return T_UNARY_OP;
"plotimplicit"             giac_yylval = gen(at_plotimplicit,2); index_status=0; return T_UNARY_OP;
"implicitplot"             giac_yylval = gen(at_implicitplot,2); index_status=0; return T_UNARY_OP;
"plotcontour"             giac_yylval = gen(at_plotcontour,3); index_status=0; return T_UNARY_OP;
"contourplot"             giac_yylval = gen(at_contourplot,3); index_status=0; return T_UNARY_OP;
"bitmap"             giac_yylval = gen(at_bitmap,3); index_status=0; return T_UNARY_OP;
"Pictsize"             giac_yylval = gen(at_Pictsize,0); index_status=0; return T_UNARY_OP;
"plotparam"             giac_yylval = gen(at_plotparam,2); index_status=0; return T_UNARY_OP;
"paramplot"             giac_yylval = gen(at_paramplot,2); index_status=0; return T_UNARY_OP;
"parameq"		giac_yylval = gen(at_parameq,2); index_status=0; return T_UNARY_OP;
"partfrac"		giac_yylval = gen(at_partfrac,1); index_status=0; return T_UNARY_OP;
"pcar"			giac_yylval = gen(at_pcar,1); index_status=0; return T_UNARY_OP;
"pcar_hessenberg"       giac_yylval = gen(at_pcar_hessenberg,1); index_status=0; return T_UNARY_OP;
"perm"		        giac_yylval = gen(at_perm,2); index_status=0; return T_UNARY_OP;
"perpendiculaire"       giac_yylval = gen(at_perpendiculaire,2); index_status=0; return T_UNARY_OP;
"proot"			giac_yylval = gen(at_proot,1); index_status=0; return T_UNARY_OP;
"propfrac"		giac_yylval = gen(at_propfrac,1); index_status=0; return T_UNARY_OP;
"psi"			giac_yylval = gen(at_psi,1); index_status=0; return T_UNARY_OP;
"Psi"			giac_yylval = gen(at_psi,2); index_status=0; return T_UNARY_OP;
"pcoeff"		giac_yylval = gen(at_pcoeff,1); index_status=0; return T_UNARY_OP;
"pcoef"		        giac_yylval = gen(at_pcoeff,1); index_status=0; return T_UNARY_OP;
"peval" 		giac_yylval = gen(at_peval,2); index_status=0; return T_UNARY_OP;
"plot" 		giac_yylval = gen(at_plot,2); index_status=0; return T_UNARY_OP;
"plot_style" 		giac_yylval = gen(at_plot_style,1); index_status=0; return T_UNARY_OP;
"plot3d" 		giac_yylval = gen(at_plot3d,3); index_status=0; return T_UNARY_OP;
"plotfunc" 		giac_yylval = gen(at_plotfunc,2); index_status=0; return T_UNARY_OP;
"funcplot" 		giac_yylval = gen(at_funcplot,2); index_status=0; return T_UNARY_OP;
"plotfunc2d" 		giac_yylval = gen(at_funcplot,2); index_status=0; return T_UNARY_OP;
"plotfield" 		giac_yylval = gen(at_plotfield,4); index_status=0; return T_UNARY_OP;
"fieldplot" 		giac_yylval = gen(at_fieldplot,4); index_status=0; return T_UNARY_OP;
"plotode" 		giac_yylval = gen(at_plotode,3); index_status=0; return T_UNARY_OP;
"plotseq" 		giac_yylval = gen(at_plotseq,3); index_status=0; return T_UNARY_OP;
"seqplot" 		giac_yylval = gen(at_seqplot,3); index_status=0; return T_UNARY_OP;
"qr"		giac_yylval = gen(at_qr,2);index_status=0; return T_UNARY_OP;
"odeplot" 		giac_yylval = gen(at_odeplot,3); index_status=0; return T_UNARY_OP;
"user_operator" 		giac_yylval = gen(at_user_operator,6); index_status=0; return T_UNARY_OP;
"Ox" 		        giac_yylval = _droite(makevecteur(zero,plus_one),0); return T_EXPRESSION;
"Oy" 		        giac_yylval = _droite(makevecteur(zero,cst_i),0); return T_EXPRESSION;
"pnt"                   giac_yylval = gen(at_pnt,3); index_status=0; return T_UNARY_OP;
"point"                 giac_yylval = gen(at_point,2); index_status=0; return T_UNARY_OP;
"powexpand" 		giac_yylval = gen(at_powexpand,1); index_status=0; return T_UNARY_OP;
"plotpolar"             giac_yylval = gen(at_plotpolar,2); index_status=0; return T_UNARY_OP;
"polarplot"             giac_yylval = gen(at_polarplot,2); index_status=0; return T_UNARY_OP;
"pixon"                 giac_yylval = gen(at_pixon,1); index_status=0; return T_UNARY_OP;
"pixoff"                giac_yylval = gen(at_pixoff,1); index_status=0; return T_UNARY_OP;
"powmod"                giac_yylval = gen(at_powmod,3); index_status=0; return T_UNARY_OP;
"prevprime"		giac_yylval = gen(at_prevprime,1); index_status=0; return T_UNARY_OP;
"prepend"		giac_yylval = gen(at_prepend,2); index_status=0; return T_UNARY_OP;
"print"                 giac_yylval = gen(at_print,1); index_status=0; return T_UNARY_OP;
"projection"		giac_yylval = gen(at_projection,2); index_status=0; return T_UNARY_OP;
"program"		index_status=0; return T_PROGRAM;
"proc"                  index_status=0; return T_PROC;
"purge"                 if (rpn_mode) {giac_yylval=gen(at_purge,0); index_status=0; return T_RPN_OP;} else {giac_yylval = gen(at_purge,1); index_status=0; return T_UNARY_OP;};
"unassign"                 if (rpn_mode) {giac_yylval=gen(at_purge,0); index_status=0; return T_RPN_OP;} else {giac_yylval = gen(at_purge,1); index_status=0; return T_UNARY_OP;};
"PURGE"                 if (rpn_mode) {giac_yylval=gen(at_purge,0); index_status=0; return T_RPN_OP;} else {giac_yylval = gen(at_purge,1); index_status=0; return T_UNARY_OP;};
"pwd"                   giac_yylval = gen(at_pwd,1); index_status=0; return T_UNARY_OP;
"quote"			giac_yylval = gen(at_quote,1); index_status=0; return T_UNARY_OP;
"unquote"		giac_yylval = gen(at_unquote,1); index_status=0; return T_UNARY_OP;
"unapply"		giac_yylval = gen(at_unapply,2); index_status=0; return T_UNARY_OP;
"quo"			giac_yylval = gen(at_quo,2); index_status=0; return T_UNARY_OP;
"quorem"		giac_yylval = gen(at_quorem,2); index_status=0; return T_UNARY_OP;
"q2a"                   giac_yylval = gen(at_q2a,2); index_status=0; return T_UNARY_OP;
"r2e"			giac_yylval = gen(at_r2e,2); index_status=0; return T_UNARY_OP;
"rand"			giac_yylval = gen(at_rand,1); index_status=0; return T_UNARY_OP;
"ranm"			giac_yylval = gen(at_ranm,1); index_status=0; return T_UNARY_OP;
"randseed"			giac_yylval = gen(at_srand,1); index_status=0; return T_RETURN;
"rayon"                 giac_yylval = gen(at_rayon,1); index_status=0; return T_UNARY_OP;
"rcl"			giac_yylval = gen(at_RCL,1); index_status=0; return T_UNARY_OP;
"RCL"			giac_yylval = gen(at_RCL,1); index_status=0; return T_UNARY_OP;
"re"			giac_yylval = gen(at_re,1); index_status=0; return T_UNARY_OP;
"read"			giac_yylval = gen(at_read,1); index_status=0; return T_RETURN;
"non_recursive_normal"		giac_yylval = gen(at_non_recursive_normal,1); index_status=0; return T_UNARY_OP;
"rem"			giac_yylval = gen(at_rem,2); index_status=0; return T_UNARY_OP;
"remove"		giac_yylval = gen(at_remove,2); index_status=0; return T_UNARY_OP;
"repeat"                index_status=0; return T_REPEAT;
"REPEAT"                index_status=0; return T_REPEAT;
"resultant"		giac_yylval = gen(at_resultant,2); index_status=0; return T_UNARY_OP;
"return"		giac_yylval = gen(at_return,1) ; index_status=0; return T_RETURN;
"RETURN"		giac_yylval = gen(at_return,1) ; index_status=0; return T_RETURN;
"'return'"              giac_yylval = gen(at_return,1) ; index_status=0; return T_QUOTED_BINARY;
"rdiv"		giac_yylval = gen(at_rdiv,2); index_status=0; return T_UNARY_OP;
"risch"		giac_yylval = gen(at_risch,4); index_status=0; return T_UNARY_OP;
"root"                giac_yylval = gen(at_maple_root,1); index_status=1; return T_UNARY_OP;
"rootof"                giac_yylval = gen(at_rootof,1); index_status=0; return T_UNARY_OP;
"romberg"               giac_yylval=  gen(at_romberg,4); index_status=0; return T_UNARY_OP;
"rotation"		giac_yylval = gen(at_rotation,2); index_status=0; return T_UNARY_OP;
"round"                 giac_yylval = gen(at_round,1); index_status=0; return T_UNARY_OP;
"rm_a_z"		giac_yylval = gen(at_rm_a_z,1); index_status=0; return T_UNARY_OP;
"rm_all_vars"		giac_yylval = gen(at_rm_all_vars,1); index_status=0; return T_UNARY_OP;
"rmbreakpoint"		giac_yylval = gen(at_rmbreakpoint,1); index_status=0; return T_UNARY_OP;
"rmmod"		        giac_yylval = gen(at_rmmod,1); index_status=0; return T_UNARY_OP;
"rmwatch"		giac_yylval = gen(at_rmwatch,1); index_status=0; return T_UNARY_OP;
"rpn"                   giac_yylval = gen(at_rpn,0); index_status=0; return T_UNARY_OP;
"rref"			giac_yylval = gen(at_rref,1); index_status=0; return T_UNARY_OP;
"same"                  giac_yylval = gen(at_same,1); index_status=0; return T_UNARY_OP;
"save_history"			giac_yylval = gen(at_save_history,1); index_status=0; return T_UNARY_OP;
"segment"               giac_yylval = gen(at_segment,2); index_status=0; return T_UNARY_OP;
"series"		giac_yylval = gen(at_series,3); index_status=0; return T_UNARY_OP;
"select"		giac_yylval = gen(at_select,2); index_status=0; return T_UNARY_OP;
"sign"			giac_yylval = gen(at_sign,1); index_status=0; return T_UNARY_OP;
"sum"                   giac_yylval = gen(at_sum,4); index_status=0; return T_UNARY_OP;
"signal"                giac_yylval = gen(at_signal,1); index_status=0; return T_UNARY_OP;
"sin"			giac_yylval = gen(at_sin,1); index_status=0; return T_UNARY_OP;
"sincos"               giac_yylval = gen(at_sincos,1); index_status=0; return T_UNARY_OP;
"sinh"			giac_yylval = gen(at_sinh,1); index_status=0; return T_UNARY_OP;
"similitude"		giac_yylval = gen(at_similitude,2); index_status=0; return T_UNARY_OP;
"simplify"		giac_yylval = gen(at_simplify,1); index_status=0; return T_UNARY_OP;
"size"			giac_yylval = gen(at_size,1); index_status=0; return T_UNARY_OP;
"solve"		        giac_yylval = gen(at_solve,2); index_status=0; return T_UNARY_OP;
"sommet"                giac_yylval = gen(at_sommet,1); index_status=0; return T_UNARY_OP;
"sort"		        giac_yylval = gen(at_sort,1); index_status=0; return T_UNARY_OP;
"smod"			giac_yylval = gen(at_smod,2); index_status=0; return T_UNARY_OP;
"sq"			giac_yylval = gen(at_sq,1); index_status=0; return T_UNARY_OP;
"sqrt"			giac_yylval = gen(at_sqrt,1); index_status=0; return T_UNARY_OP;
"sst"		        giac_yylval = gen(at_sst,1); index_status=0; return T_UNARY_OP;
"SST"		        giac_yylval = gen(at_sst,1); index_status=0; return T_UNARY_OP;
"sst_in"		giac_yylval = gen(at_sst_in,1); index_status=0; return T_UNARY_OP;
"SST_IN"		giac_yylval = gen(at_sst_in,1); index_status=0; return T_UNARY_OP;
"srand"			giac_yylval = gen(at_srand,1); index_status=0; return T_RETURN;
"START"                 index_status=0; return T_START;
"stack"                 index_status=0; return T_STACK;
"step"                  index_status=0; return T_BY;
"STEP"                  index_status=0; return T_BY;
"sto"			giac_yylval = gen(at_sto,0); index_status=0; return T_UNARY_OP;
"STO"			giac_yylval = gen(at_sto,0); index_status=0; return T_UNARY_OP;
"sturm"			giac_yylval = gen(at_sturm,2); index_status=0; return T_UNARY_OP;
"sturmab"		giac_yylval = gen(at_sturmab,4); index_status=0; return T_UNARY_OP;
"subs"			if (maple_mode==1) giac_yylval = gen(at_maple_subs,2); else giac_yylval = gen(at_subs,2); index_status=0; return T_UNARY_OP;
"subsop"		if (maple_mode==1) giac_yylval = gen(at_maple_subsop,2); else giac_yylval = gen(at_subsop,2); index_status=0; return T_UNARY_OP;
"subst"			giac_yylval = gen(at_subst,2); index_status=0; return T_UNARY_OP;
"svd"			giac_yylval = gen(at_svd,1); index_status=0; return T_UNARY_OP;
"symetrie"		giac_yylval = gen(at_symetrie,2); index_status=0; return T_UNARY_OP;
"switch"                index_status=0; return T_SWITCH;
"SWITCH"                index_status=0; return T_SWITCH;
"switch_axes"		giac_yylval = gen(at_switch_axes,1); index_status=0; return T_UNARY_OP;
"table"			giac_yylval = gen(at_table,1); index_status=0; return T_UNARY_OP;
"tablefunc"			giac_yylval = gen(at_tablefunc,4); index_status=0; return T_UNARY_OP;
"tableseq"			giac_yylval = gen(at_tableseq,4); index_status=0; return T_UNARY_OP;
"tail"                  giac_yylval = gen(at_tail,1); index_status=0; return T_UNARY_OP;
"tan"			giac_yylval = gen(at_tan,1); index_status=0; return T_UNARY_OP;
"tan2sincos"               giac_yylval = gen(at_tan2sincos,1); index_status=0; return T_UNARY_OP;
"tan2sincos2"               giac_yylval = gen(at_tan2sincos2,1); index_status=0; return T_UNARY_OP;
"tan2cossin2"               giac_yylval = gen(at_tan2cossin2,1); index_status=0; return T_UNARY_OP;
"tangent"               giac_yylval = gen(at_tangent,1); index_status=0; return T_UNARY_OP;
"tanh"			giac_yylval = gen(at_tanh,1); index_status=0; return T_UNARY_OP;
"tcollect"			giac_yylval = gen(at_tcollect,1); index_status=0; return T_UNARY_OP;
"texpand"		giac_yylval = gen(at_texpand,1); index_status=0; return T_UNARY_OP;
"then"		        index_status=0; giac_yylval=gen(at_ifte,3); return T_THEN;
"THEN"		        index_status=0; giac_yylval=gen(at_ifte,3); return T_THEN;
"throw"		        index_status=0; return T_THROW;
"tlin"			giac_yylval = gen(at_tlin,1); index_status=0; return T_UNARY_OP;
"to"                    giac_yylval=1; return T_TO;
"TO"                    giac_yylval=1; return T_TO;
"downto"                giac_yylval=-1; return T_TO;
"DOWNTO"                giac_yylval=-1; return T_TO;
"trace"			giac_yylval = gen(at_trace,1); index_status=0; return T_UNARY_OP;
"tran"			giac_yylval = gen(at_tran,1); index_status=0; return T_UNARY_OP;
"translation"		giac_yylval = gen(at_translation,2); index_status=0; return T_UNARY_OP;
"triangle"		giac_yylval = gen(at_triangle,3); index_status=0; return T_UNARY_OP;
"triangle_rectangle"	giac_yylval = gen(at_triangle_rectangle,3); index_status=0; return T_UNARY_OP;
"triangle_isocele"	giac_yylval = gen(at_triangle_isocele,2); index_status=0; return T_UNARY_OP;
"triangle_equilateral"	giac_yylval = gen(at_triangle_equilateral,1); index_status=0; return T_UNARY_OP;
"rectangle"		giac_yylval = gen(at_rectangle,3); index_status=0; return T_UNARY_OP;
"carre"		giac_yylval = gen(at_carre,2); index_status=0; return T_UNARY_OP;
"parallelogramme"		giac_yylval = gen(at_parallelogramme,3); index_status=0; return T_UNARY_OP;
"losange"		giac_yylval = gen(at_losange,2); index_status=0; return T_UNARY_OP;
"polygone"		giac_yylval = gen(at_polygone,2); index_status=0; return T_UNARY_OP;
"quadrilatere"		giac_yylval = gen(at_quadrilatere,4); index_status=0; return T_UNARY_OP;
"factor_xn"		giac_yylval = gen(at_factor_xn,1); index_status=0; return T_UNARY_OP;
"trig2exp"               giac_yylval = gen(at_trig2exp,1); index_status=0; return T_UNARY_OP;
"trigcos"               giac_yylval = gen(at_trigcos,1); index_status=0; return T_UNARY_OP;
"trigsin"               giac_yylval = gen(at_trigsin,1); index_status=0; return T_UNARY_OP;
"trigtan"               giac_yylval = gen(at_trigtan,1); index_status=0; return T_UNARY_OP;
"true"                  index_status=0; giac_yylval = plus_one; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"TRUE"                  index_status=0; giac_yylval = plus_one; giac_yylval.subtype=_INT_BOOLEAN; return T_NUMBER;
"try"                   index_status=0; return T_TRY;
"try_catch"             index_status=0; return T_TRY_CATCH;
"tsimplify"		giac_yylval = gen(at_tsimplify,1); index_status=0; return T_UNARY_OP;
"type"                  giac_yylval = gen(at_type,1); index_status=0; return T_UNARY_OP;
"unarchive"		giac_yylval = gen(at_unarchive,1); index_status=0; return T_UNARY_OP;
"union"                 index_status=0; return T_UNION;
"'union'"                  index_status=0; giac_yylval=gen(at_union,2); return T_QUOTED_BINARY;
"_union"                  index_status=0; giac_yylval=gen(at_union,2); return T_QUOTED_BINARY;
"until"                 index_status=0; return T_UNTIL;
"UNTIL"                 index_status=0; return T_UNTIL;
"virgule"               giac_yylval = gen(at_virgule,2); index_status=0; return T_UNARY_OP;
"VARS"                  giac_yylval = gen(at_VARS,0); index_status=0; return T_UNARY_OP;
"variables_are_files"		        giac_yylval = gen(at_variables_are_files,0); index_status=0; return T_DIGITS;
"version"                  giac_yylval = gen(at_version,0); index_status=0; return T_UNARY_OP;
"watch"		        giac_yylval = gen(at_watch,1); index_status=0; return T_UNARY_OP;
"while"                 index_status=0; giac_yylval=gen(at_for,4); if (maple_mode==3) return TI_WHILE; if (maple_mode!=0) return T_MUPMAP_WHILE; return T_WHILE;
"write"			giac_yylval = gen(at_write,1); index_status=0; return T_UNARY_OP;
"WHILE"                 index_status=0; giac_yylval=gen(at_for,4); return T_RPN_WHILE;
"xyztrange"             giac_yylval = gen(at_xyztrange,8); index_status=0; return T_UNARY_OP;
"zeta"			giac_yylval = gen(at_zeta,1); index_status=0; return T_UNARY_OP;
"expr"			giac_yylval = gen(at_expr,1); index_status=0; return T_UNARY_OP;
"string"	       giac_yylval = gen(at_string,1); index_status=0; return T_UNARY_OP;
"part"	       giac_yylval = gen(at_part,2); index_status=0; return T_UNARY_OP;
"Define"               giac_yylval=0; index_status=0; return TI_DEFINE;
":Prgm"                 giac_yylval=0; index_status=0; return TI_PRGM;
"Prgm"                 giac_yylval=0; index_status=0; return TI_PRGM;
"EndPrgm"              giac_yylval=0; index_status=0; return T_BLOC_END;
"Endprgm"              giac_yylval=0; index_status=0; return T_BLOC_END;
":Func"                 giac_yylval=0; index_status=0; return TI_PRGM;
"Func"                 giac_yylval=0; index_status=0; return TI_PRGM;
"func"                 giac_yylval=0; index_status=0; return TI_PRGM;
":func"                 giac_yylval=0; index_status=0; return TI_PRGM;
"EndFunc"              giac_yylval=0; index_status=0; return T_BLOC_END;
"endfunc"              giac_yylval=0; index_status=0; return T_BLOC_END;
"Endfunc"              giac_yylval=0; index_status=0; return T_BLOC_END;
"Local"                giac_yylval=0; index_status=0; return TI_LOCAL;
"If"      	       index_status=0; giac_yylval=gen(at_ifte,3); return T_TI_IF;
"Then"                 index_status=0; giac_yylval=gen(at_ifte,3); return T_THEN;
"Else"                 index_status=0; giac_yylval=gen(at_ifte,3); return T_ELSE;
"EndIf"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"Endif"                index_status=0; giac_yylval=gen(at_ifte,3); return T_BLOC_END;
"Return"	       giac_yylval = gen(at_return,1) ; index_status=0; return T_RETURN;
"ElseIf"               giac_yylval = gen(at_ifte,3) ;  index_status=0; return T_ELIF;
"Elseif"               giac_yylval = gen(at_ifte,3) ;  index_status=0; return T_ELIF;
"Exit"  	       index_status=0; giac_yylval=gen(at_breakpoint,0); return T_BREAK;
"Loop"                  index_status=0; giac_yylval=gen(at_for,0); return TI_LOOP;
"EndLoop"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endloop"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"For"                   index_status=0; giac_yylval=gen(at_for,0); return TI_FOR;
"EndFor"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endfor"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"While"                   index_status=0; giac_yylval=gen(at_for,0); return TI_WHILE;
"EndWhile"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endwhile"               index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Cycle"               index_status=0; giac_yylval=gen(at_for,0); return T_CONTINUE;
"Try"           index_status=0; giac_yylval=gen(at_for,0); return TI_TRY;
"EndTry"           index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Endtry"           index_status=0; giac_yylval=gen(at_for,0); return T_BLOC_END;
"Disp"	       giac_yylval = gen(at_print,1) ; index_status=0; return T_RETURN;
"Pause"	       giac_yylval = gen(at_Pause,1) ; index_status=0; return T_RETURN;
"DelVar"       giac_yylval = gen(at_DelVar,1) ; index_status=0; return T_RETURN;
"label"	       giac_yylval = gen(at_label,1) ; index_status=0; return T_RETURN;
"Lbl"	       giac_yylval = gen(at_label,1) ; index_status=0; return T_RETURN;
"goto"	       giac_yylval = gen(at_goto,1) ; index_status=0; return T_RETURN;
"Goto"	       giac_yylval = gen(at_goto,1) ; index_status=0; return T_RETURN;
"Dialog"       giac_yylval = gen(at_Dialog,1) ; index_status=0; return TI_DIALOG; 
"EndDlog"    giac_yylval = gen(at_Dialog,1) ; index_status=0; return T_BLOC_END;
"Row"	       giac_yylval = gen(at_Row,0) ; index_status=0; return T_DIGITS;
"Col"	       giac_yylval = gen(at_Col,0) ; index_status=0; return T_DIGITS;
"threads"      giac_yylval = gen(at_threads,0) ; index_status=0; return T_DIGITS;
"_hbar_"        giac_yylval = cst_hbar; index_status=0; return T_SYMBOL;
"_c_"        giac_yylval = cst_clightspeed; index_status=0; return T_SYMBOL;
"_g_"        giac_yylval = cst_ga; index_status=0; return T_SYMBOL;
"_IO_" giac_yylval = cst_IO; index_status=0; return T_SYMBOL; 
"_epsilonox_" giac_yylval = cst_epsilonox; index_status=0; return T_SYMBOL; 
"_epsilonsi_" giac_yylval = cst_epsilonsi; index_status=0; return T_SYMBOL; 
"_qepsilon0_" giac_yylval = cst_qepsilon0; index_status=0; return T_SYMBOL; 
"_epsilon0q_" giac_yylval = cst_epsilon0q; index_status=0; return T_SYMBOL; 
"_kq_" giac_yylval = cst_kq; index_status=0; return T_SYMBOL; 
"_c3_" giac_yylval = cst_c3; index_status=0; return T_SYMBOL; 
"_lambdac_" giac_yylval = cst_lambdac; index_status=0; return T_SYMBOL; 
"_f0_" giac_yylval = cst_f0; index_status=0; return T_SYMBOL; 
"_lambda0_" giac_yylval = cst_lambda0; index_status=0; return T_SYMBOL; 
"_muN_" giac_yylval = cst_muN; index_status=0; return T_SYMBOL; 
"_muB_" giac_yylval = cst_muB; index_status=0; return T_SYMBOL; 
"_a0_" giac_yylval = cst_a0; index_status=0; return T_SYMBOL; 
"_Rinfinity_" giac_yylval = cst_Rinfinity; index_status=0; return T_SYMBOL; 
"_Faraday_" giac_yylval = cst_Faraday; index_status=0; return T_SYMBOL; 
"_phi_" giac_yylval = cst_phi; index_status=0; return T_SYMBOL; 
"_alpha_" giac_yylval = cst_alpha; index_status=0; return T_SYMBOL; 
"_mpme_" giac_yylval = cst_mpme; index_status=0; return T_SYMBOL; 
"_mp_" giac_yylval = cst_mp; index_status=0; return T_SYMBOL; 
"_qme_" giac_yylval = cst_qme; index_status=0; return T_SYMBOL; 
"_me_" giac_yylval = cst_me; index_status=0; return T_SYMBOL; 
"_qe_" giac_yylval = cst_qe; index_status=0; return T_SYMBOL; 
"_h_" giac_yylval = cst_hPlanck; index_status=0; return T_SYMBOL; 
"_G_" giac_yylval = cst_G; index_status=0; return T_SYMBOL; 
"_mu0_" giac_yylval = cst_mu0; index_status=0; return T_SYMBOL; 
"_epsilon0_" giac_yylval = cst_epsilon0; index_status=0; return T_SYMBOL; 
"_sigma_" giac_yylval = cst_sigma; index_status=0; return T_SYMBOL; 
"_StdP_" giac_yylval = cst_StdP; index_status=0; return T_SYMBOL; 
"_StdT_" giac_yylval = cst_StdT; index_status=0; return T_SYMBOL; 
"_R_" giac_yylval = cst_Rydberg; index_status=0; return T_SYMBOL; 
"_Vm_" giac_yylval = cst_Vm; index_status=0; return T_SYMBOL; 
"_k_" giac_yylval = cst_kBoltzmann; index_status=0; return T_SYMBOL; 
"_NA_" giac_yylval = cst_NA; index_status=0; return T_SYMBOL; 
"_in"        giac_yylval = symbolic(at_unit,makevecteur(1,_in_unit)); index_status=0; return T_SYMBOL;
 
			/* numbers */
{D}+			|
"#"[0-7]+"o"		|
"#"[0-1]+"b"		|
"#"[0-9a-fA-F]+"h"	|
"#o"[0-7]+		|
"#b"[0-1]+		|
"#x"[0-9a-fA-F]+	|
"0o"[0-7]+		|
"0b"[0-1]+		|
"0x"[0-9a-fA-F]+	|
{D}+"."{D}*({E})?	|
{D}*"."{D}+({E})?	|
{D}+{E}			{ 
  index_status=1;
  int l=0;
  char ch,ch2;
  for (;ch=*(giac_yytext+l);++l){
    if (ch=='e' || ch=='E'){
      if ( (ch2=*(giac_yytext+l+1)) && (ch2=='e' || ch2=='E')){
        ++l;
        for (;ch=*(giac_yytext+l);++l)
          *(giac_yytext+l-1)=ch;
	*(giac_yytext+l-1)=0;
        --l;
        break;
      }
    }
  }
  if (l>2 && (giac_yytext[l-1]=='o' || giac_yytext[l-1]=='b' || giac_yytext[l-1]=='h') ){
    char base=giac_yytext[l-1];
    for (int i=l-1;i>1;--i){
      giac_yytext[i]=giac_yytext[i-1];
    }
    if (base=='h')
      base='x';
    giac_yytext[1]=base;
  }
  giac_yylval = chartab2gen(giac_yytext); return T_NUMBER; 
}

			/* symbols */
{A}{AN}*		{
  index_status=1; 
  return find_or_make_symbol(giac_yytext,giac_yylval);
}
"#"                     if (!maple_mode || maple_mode==3) { 
  // cerr << "hash" << endl;
  giac_yylval=gen(at_hash,1); return TI_HASH; 
} else BEGIN(comment_hash);
<comment_hash>[^*\n]*\n BEGIN(INITIAL); index_status=0; ++lexer_line_number;  /* comment_s=string(giac_yytext); giac_yylval=string2gen(comment_s.substr(0,comment_s.size()-1),false); return T_COMMENT; */
			/* everything else */
.			giac_yylval=string2gen(string(giac_yytext),false); return T_STRING;

%%


/*
 *  Routines
 */

// The string from which we will read
static std::string lexer_string;

// The current position within the string
static int curr_pos = 0;

// Input function that reads from string
static int lexer_input(char *buf, int max_size)
{
  int actual = lexer_string.length() - curr_pos;
  if (actual > max_size)
    actual = max_size;
  if (actual <= 0)
    return YY_NULL;
  lexer_string.copy(buf, actual, curr_pos);
  curr_pos += actual;
  return actual;
}

// EOF encountered, terminate the scanner
int giac_yywrap()
{
  return 1;
}

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

        // Set the input string
  void set_lexer_string(const std::string &s_orig){
    string s(s_orig);
    // stupid match of bracket then parenthesis
    int l=s.size(),nb=0,np=0;
    int i=0;
    for (;i<l;++i){
      switch (s[i]){
      case '(':
	++np;
	break;
      case ')':
	--np;
	break;
      case '[':
	++nb;
	break;
      case ']':
	--nb;
	break;
      }
    }
    while (np<0 && i>=0 && s[i-1]==')'){
      --i;
      ++np;
    }
    while (nb<0 && i>=0 && s[i-1]==']'){
      --i;
      ++nb;
    }
    s=s.substr(0,i);
    if (nb<0)
      cerr << "Too many ]" << endl;
    if (np<0)
      cerr << "Too many )" << endl;
    if (nb>0)
      s=s+string(nb,']');
    if (np>0)
      s=s+string(np,')');
    BEGIN(INITIAL); 
    index_status=0;
    opened_quote=0;
    in_rpn=0;
    lexer_line_number=1;
    first_error_line=0;
    l=s.size();
    for (;l;l--){
      if (s[l-1]!=' ')
	break;
    }
    /* if (l && ( (s[l-1]==';') || (s[l-1]==':')))
       l--; */
    string ss;
    for (int i=0;i<l;++i){
      if (s[i]=='.'){
	if ( i && (i<l-1) && (s[i-1]!=' ') && (s[i+1]=='.') ){
	  ss+= " ..";
	  ++i;
	}
	else
	  ss+='.';
      }
      else {
	if (maple_mode > 0 && maple_mode !=3){
	  if (s[i]=='#')
	    ss += "//";
	  else
	    ss += s[i];
	}
	else
	  ss+=s[i];
      }
    }
    lexer_string = ss+'';
    curr_pos = 0;
  }
    
  // current lexer_string
  std::string the_lexer_string(){
    return lexer_string;
  }

  // current lexer_string
  int the_lexer_pos(){
    return curr_pos;
  }
    
  // Add to the list of predefined symbols
  void set_lexer_symbols(const vecteur & l){
    if (initialisation_done && (l==list_one_letter__IDNT) )
      return;
    initialisation_done=true;
    const_iterateur it=l.begin(),itend=l.end();
    for (; it!=itend; ++it) {
      if (it->type!=_IDNT)
	continue;
      sym_tab::const_iterator i = syms.find(* (it->_IDNTptr->name));
      if (i==syms.end())
	syms[* (it->_IDNTptr->name)] = *it;
    }
  }


#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
