// -*- mode:C++ ; compile-command: "g++ -I.. -g -c intg.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <vector>
#include <cmath>
#include <cstdlib>
#include "sym2poly.h"
#include "usual.h"
#include "intg.h"
#include "subst.h"
#include "derive.h"
#include "lin.h"
#include "vecteur.h"
#include "gausspol.h"
#include "plot.h"
#include "prog.h"
#include "modpoly.h"
#include "series.h"
#include "tex.h"
#include "ifactor.h"
#include "risch.h"
#include "solve.h"
#ifdef HAVE_LIBGSL
#include <gsl/gsl_math.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_sf_psi.h>
#include <gsl/gsl_sf_zeta.h>
#include <gsl/gsl_odeiv.h>
#include <gsl/gsl_errno.h>
#endif

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  // Left redimension p to degree n, i.e. size n+1
  void lrdm(modpoly & p,int n){
    int s=p.size();
    if (n+1>s)
      p=mergevecteur(vecteur(n+1-s),p);
  }

  struct pf1 {
    vecteur num;
    vecteur den;
    vecteur fact;
    int mult; // den=cste*fact^mult
    pf1(const pf1 & a) : num(a.num), fact(a.fact), den(a.den), mult(a.mult) {}
    pf1(const vecteur &n, const vecteur & d, const vecteur & f,int m) : num(n), fact(f), den(d), mult(m) {};
    pf1(const polynome & n,const polynome & d,const polynome & f,int m): num(polynome2poly1(n)),den(polynome2poly1(d)),fact(polynome2poly1(f)),mult(m) {}
  };

  gen complex_subst(const gen & e,const vecteur & substin,const vecteur & substout,GIAC_CONTEXT){
    bool save_complex_mode=complex_mode(contextptr);
    complex_mode(true,contextptr);
    gen res=eval(subst(e,substin,substout),contextptr);
    complex_mode(save_complex_mode,contextptr);
    return res;
  }

  gen complex_subst(const gen & e,const gen & x,const gen & newx,GIAC_CONTEXT){
    bool save_complex_mode=complex_mode(contextptr);
    complex_mode(true,contextptr);
    gen res=eval(subst(e,x,newx),contextptr);
    complex_mode(save_complex_mode,contextptr);
    return res;
  }

  bool has_nop_var(const vecteur & v){
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (contains(*it,at_nop))
	return true;
    }
    return false;
  }

  gen nop_inv(const gen & e){
    return symbolic(at_nop,gen(symbolic(at_inv,e)));
  }
  gen nop_pow(const gen & e){
    if ( (e.type!=_VECT) || (e._VECTptr->size()!=2))
      return symbolic(at_pow,e);
    if ( (e._VECTptr->back().type!=_INT_) || (e._VECTptr->back().val>=0) || ( (e._VECTptr->front().type==_SYMB) && (e._VECTptr->front()._SYMBptr->sommet==at_exp) ) )
      return symbolic(at_pow,e);
    return nop_inv(symbolic(at_pow,makevecteur(e._VECTptr->front(),-e._VECTptr->back())));
  }

  gen sin_over_cos(const gen & e){
    return rdiv(symb_sin(e),symb_cos(e));
  }
  vecteur lvarxwithinv(const gen &e,const identificateur & x){
    vector<unary_function_ptr> vu(1,at_inv);
    vu.push_back(at_pow);
    vu.push_back(at_tan);
    vector< gen (*) (const gen &) > ve(1,nop_inv);
    ve.push_back(nop_pow);
    ve.push_back(sin_over_cos);
    gen ee=subst(e,vu,ve);
    vecteur v(lvarx(ee,x));
    return v; // to remove nop do a return *(eval(v)._VECTptr);
  }

  bool is_constant_wrt(const gen & e,const identificateur & x){
    if (e.type==_VECT){
      const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it){
	if (!is_constant_wrt(*it,x))
	  return false;
      }
      return true;
    }
    if (e==x)
      return false;
    if (e.type!=_SYMB)
      return true;
    return is_zero(derive(e,x));
  }

  // return true if e=a*x+b
  bool is_linear_wrt(const gen & e,const identificateur &x,gen & a,gen & b){
    a=derive(e,x);
    if (!is_constant_wrt(a,x))
      return false;
    b=ratnormal(e-a*x);
    return true;
  }

  // return true if e=a*x+b
  bool is_quadratic_wrt(const gen & e,const identificateur &x,gen & a,gen & b,gen & c){
    gen tmp=derive(e,x);
    if (!is_linear_wrt(tmp,x,a,b))
      return false;
    a=ratnormal(rdiv(a,plus_two));
    c=ratnormal(e-a*x*x-b*x);
    return true;
  }

  void de_VECTose_plus(const vecteur & arg,const identificateur & x,vecteur & non_constant,gen & plus_constant){
    non_constant.clear();
    plus_constant=zero;
    const_iterateur it=arg.begin(),itend=arg.end();
    for (;it!=itend;++it){
      if (is_constant_wrt(*it,x))
	plus_constant=plus_constant+(*it);
      else
	non_constant.push_back(*it);
    }
  }

  void de_VECTose_prod(const vecteur & arg,const identificateur & x,vecteur & non_constant,gen & prod_constant){
    non_constant.clear();
    prod_constant=plus_one;
    const_iterateur it=arg.begin(),itend=arg.end();
    for (;it!=itend;++it){
      if (is_constant_wrt(*it,x))
	prod_constant=prod_constant*(*it);
      else
	non_constant.push_back(*it);
    }
  }

  // applies linearity of f. + & neg are distributed as well as * with respect
  // to terms that are constant w.r.t. x
  // e is assumed to be a scalar
  gen linear_apply(const gen & e,const identificateur & x,gen & remains, GIAC_CONTEXT, gen (* f)(const gen &,const identificateur &,gen &,const context *)){
    if (is_constant_wrt(e,x) || (e==x) )
      return f(e,x,remains,contextptr);
    // e must be of type _SYMB
    assert(e.type==_SYMB);
    unary_function_ptr u(e._SYMBptr->sommet);
    gen arg(e._SYMBptr->feuille);
    gen res;
    if (u==at_neg){
      res=-linear_apply(arg,x,remains,contextptr,f);
      remains=-remains;
      return res;
    } // end at_neg
    if (u==at_plus){
      if (arg.type!=_VECT)
	return linear_apply(arg,x,remains,contextptr,f);
      const_iterateur it=arg._VECTptr->begin(),itend=arg._VECTptr->end();
      for (gen tmp;it!=itend;++it){
	res = res + linear_apply(*it,x,tmp,contextptr,f);
	remains =remains + tmp;
      }
      return res;
    } // end at_plus
    if (u==at_prod){
      if (arg.type!=_VECT)
	return linear_apply(arg,x,remains,contextptr,f);
      // find all constant terms in the product
      vecteur non_constant;
      gen prod_constant;
      de_VECTose_prod(*arg._VECTptr,x,non_constant,prod_constant);
      assert(!non_constant.empty()); // otherwise the product would be constant
      if (non_constant.size()==1)
	res = linear_apply(non_constant.front(),x,remains,contextptr,f);
      else
	res = f(symbolic(at_prod,non_constant),x,remains,contextptr);
      remains = prod_constant * remains;
      return prod_constant * res;
    } // end at_prod
    return f(e,x,remains,contextptr);
  }

  gen lnabs(const gen & x,GIAC_CONTEXT){
    bool _lnabs=do_lnabs;
    if (contextptr && contextptr->globalptr)
      _lnabs=contextptr->globalptr->do_lnabs;
    if (!complex_mode(contextptr) && _lnabs)
      return ln(abs(x,contextptr));
    else
      return ln(x);
  }

  // eval N at e=x*exp(i*dephasage*pi/n)/(X-e)+conj and integrate
  gen substconj(const gen & N,const gen & X,const gen & x,gen dephasage,GIAC_CONTEXT){
    gen e=x*exp(dephasage*cst_i);
    gen s=sin(dephasage,contextptr);
    gen c=cos(dephasage,contextptr);
    gen b=subst(N,X,e);
    gen res=re(b,contextptr)*ln(pow(X,2)+normal(-2*c*x)*X+x.squarenorm(contextptr)); 
    gen N2=normal(-2*im(b,contextptr)); // same
    res=res+N2*atan((X-c*x)/(s*x),contextptr);
    return res;
  }
  // integration of cyclotomic-type denominators
  bool integrate_deno_length_2(const polynome & num,const vecteur & v,const vecteur & l,const vecteur & lprime,gen & res,GIAC_CONTEXT){
    if (v.size()<2)
      return false;
    const_iterateur it=v.begin()+1,itend=v.end()-1;
    for (;it!=itend;++it){
      if (!is_zero(*it))
	break;
    }
    int n=v.size()-1,d=it-v.begin(),deg;
    gen X=l.front();
    if (X.type==_VECT)
      X=X._VECTptr->front();
    gen a=r2e(v.front(),lprime);
    gen b=r2e(v.back(),lprime);
    gen c=-b/a;
    if (d!=n){ // rescale and check cyclotomic
      gen tw=r2e(v/v.front(),lprime);
      if (tw.type!=_VECT)
	return false;
      vecteur w=*tw._VECTptr;
      vecteur w_copy=w;
      c=pow(r2e(*it/v.front(),lprime),inv(d));
      iterateur jt=w.begin()+1,jtend=w.end();
      for (int k=1;jt!=jtend;++jt,++k){
	*jt=normal(*jt * pow(c,-k));
      }
      deg=is_cyclotomic(w,epsilon(contextptr));
      if (!deg){
	w=w_copy;
	c=pow(r2e(-*it/v.front(),lprime),inv(d));
	jt=w.begin()+1,jtend=w.end();
	for (int k=1;jt!=jtend;++jt,++k){
	  *jt=normal(*jt * pow(c,-k));
	}
	deg=is_cyclotomic(w,epsilon(contextptr));
      }
      if (!deg)
	return false;
      // int(num/(a*X^n+b),X)=sum(x=rootof(-b/a),num*x/(-n*b)*ln(X-x))
      vecteur vprime=derivative(v),V,Vprime,d;
      egcd(v,vprime,0,V,Vprime,d);
      if (d.size()!=1)
	return false;
      gen dd=d.front();
      // 1/vprime=Vprime/d      
      gen N=normal(_quorem(makevecteur(r2e(num,l)*horner(r2e(Vprime,lprime),X),horner(r2e(v,lprime),X),X))[1]/r2e(dd,lprime));
      if (complex_mode(contextptr)){
	for (int i=1;i<deg;++i){
	  if (gcd(i,deg)!=1)
	    continue;
	  gen x=c*exp((2*i*cst_i*cst_pi)/deg);
	  res = res + subst(N,X,x)*ln(X-x);
	}
      }
      else {
	for (int i=1;i<=deg/2;++i){
	  if (gcd(i,deg)!=1)
	    continue;
	  res=res+substconj(N,X,c,2*i*cst_pi/deg,contextptr);
	}
      }
      return true;
    }
    else {
      // int(num/(a*X^n+b),X)=sum(x=rootof(-b/a),num*x/(-n*b)*ln(X-x))
      gen N=r2e(num,l)*X/(-n*r2e(b,l));
      if (complex_mode(contextptr)){
	c=pow(c,inv(n));
	for (int i=0;i<n;++i){
	  gen x=c*exp((2*i*cst_i*cst_pi)/n);
	  res = res + subst(N,X,x)*ln(X-x);
	}
	return true;
      }
      if (n%2){
	c=pow(c,inv(n));
	res=res+subst(N,X,c)*lnabs(X-c,contextptr);
	for (int i=1;i<=n/2;++i)
	  res = res + substconj(N,X,c,2*i*cst_pi/n,contextptr);
	return true;
      }
      if (is_positive(c,contextptr) ){
	c=pow(c,inv(n));
	res=res+subst(N,X,c)*lnabs(X-c,contextptr)+subst(N,X,-c)*lnabs(X+c,contextptr);
	for (int i=1;i<n/2;++i)
	  res=res+substconj(N,X,c,2*i*cst_pi/n,contextptr);
      }
      else {
	c=pow(-c,inv(n));
	for (int i=0;i<n/2;++i)
	  res=res+substconj(N,X,c,(2*i+1)*cst_pi/n,contextptr);
      }
      return true;
    }
  }

  // integration of a rational fraction
  gen integrate_rational(const gen & e, const identificateur & x, gen & remains_to_integrate,GIAC_CONTEXT){
    vecteur varx(lvarx(e,x));
    int varxs=varx.size();
    if (!varxs){
      remains_to_integrate=zero;
      return e*x;
    }
    if ( (varxs>1) || (varx.front()!=x) ) {
      remains_to_integrate = e;
      return zero;
    }
    vecteur l;
    l.push_back(x); // insure x is the main var
    l=vecteur(1,l);
    alg_lvar(e,l);
    int s=l.front()._VECTptr->size();
    if (!s){
      l.erase(l.begin());
      s=l.front()._VECTptr->size();
    }
    gen r=e2r(e,l);
    // cout << "Int " << r << endl;
    gen r_num,r_den;
    fxnd(r,r_num,r_den);
    if (r_num.type==_EXT){
      remains_to_integrate=e;
      return zero;
    }
    if (r_den.type!=_POLY){
      if (r_num.type==_POLY)
	return rdiv(r2e(r_num._POLYptr->integrate(),l),r2sym(r_den,l));
      else 
	return e*x;
    }
    polynome den(*r_den._POLYptr),num(s);
    if (r_num.type==_POLY)
      num=*r_num._POLYptr;
    else
      num=polynome(r_num,s);
    // check for a t=x^aa change of variable: a divides deg(num)+1-deg(den)
    // as well as all differences of degrees in the poly num and den
    int den_deg=den.lexsorted_degree(),num_deg=num.lexsorted_degree();
    int aa=num_deg+1-den_deg,precedent,actuel;
    vector< monomial<gen> > ::const_iterator num_it=num.coord.begin(),num_itend=num.coord.end();
    if (num_itend-num_it>1){
      precedent=num_it->index.iptr->front(); 
      ++num_it;
      for (;num_it!=num_itend;++num_it){
	actuel=num_it->index.iptr->front();
	aa=gcd(aa,actuel-precedent);
	precedent=actuel;
      }
    }
    num_it=den.coord.begin(),num_itend=den.coord.end();
    if (num_itend-num_it>1){
      precedent=num_it->index.iptr->front(); 
      ++num_it;
      for (;num_it!=num_itend;++num_it){
	actuel=num_it->index.iptr->front();
	aa=gcd(aa,actuel-precedent);
	precedent=actuel;
      }
    }
    if (!aa)
      aa=den_deg;
    if (aa>1){ // Apply
      int k=0;
      k=den_deg % aa;
      // shift num and den by x^k
      index_t k1(num.dim);
      k1.front()=k+1;
      num=(num.shift(k1)).dividedegrees(aa);
      index_t ka(num.dim);
      ka.front()=k+aa;
      den=gen(aa)*(den.shift(ka)).dividedegrees(aa);
      gen tmpres,tmprem;
      tmpres=integrate_rational(r2e(fraction(num,den),l),x,tmprem,contextptr);
      remains_to_integrate=complex_subst(tmprem,x,pow(x,aa),contextptr);
      return complex_subst(tmpres,x,pow(x,aa),contextptr);
    }
    polynome p_content(lgcd(den));
    factorization vden(sqff(den/p_content)); // first square-free factorization
    vector< pf<gen> > pfde_VECT;
    polynome ipnum(s),ipden(s),temp(s),tmp(s);
    partfrac(num,den,vden,pfde_VECT,ipnum,ipden);
    vector< pf<gen> >::iterator it=pfde_VECT.begin();
    vector< pf<gen> >::const_iterator itend=pfde_VECT.end();
    vector< pf<gen> > intde_VECT,rest,finalde_VECT;
    for (;it!=itend;++it){
      pf<gen> single(intreduce_pf(*it,intde_VECT));
      // factor(single.den,p_content,vden,false,withsqrt(contextptr),complex_mode(contextptr));
      factor(single.den,p_content,vden,false,false,false);
      partfrac(single.num,single.den,vden,finalde_VECT,temp,tmp);
    }
    it=finalde_VECT.begin();
    itend=finalde_VECT.end();
    gen lnpart(0),deuxaxplusb,sqrtdelta;
    polynome a(s),b(s),c(s);
    polynome d(s),E(s),lnpartden(s);
    polynome delta(s),atannum(s),alpha(s);
    vecteur lprime(l);
    assert(lprime.front().type==_VECT);
    lprime.front()=cdr_VECT(*(lprime.front()._VECTptr));
    bool uselog;
    vecteur vtmp;
    for (;it!=itend;++it){
      int deg=it->fact.lexsorted_degree();
      switch (deg) {
      case 1: // 1st order
	lnpart=lnpart+rdiv(r2e(it->num,l),r2e(firstcoeff(it->den),l))*lnabs(r2e(it->fact,l),contextptr);
	break; 
      case 2: // 2nd order
	findabcdelta(it->fact,a,b,c,delta);
	uselog=is_positive(delta);
	alpha=(it->den/it->fact).trunc1()*a*gen(2);
	findde(it->num,d,E);
	atannum=a*E*gen(2)-b*d;
	// ln part d/alpha*ln(fact)
	lnpartden=alpha;
	simplify(d,lnpartden);
	lnpart=lnpart+rdiv(r2e(d,lprime),r2e(lnpartden,lprime))*lnabs(r2e(it->fact,l),contextptr);
	// atan or _FUNCnd ln part
	deuxaxplusb=r2e(it->fact.derivative(),l);
	if (uselog){
	  sqrtdelta=normal(sqrt(r2e(delta,lprime)));
	  simplify(atannum,alpha);
	  lnpart=lnpart+rdiv(r2e(atannum,lprime),(r2e(alpha,lprime))*sqrtdelta)*lnabs(rdiv(deuxaxplusb-sqrtdelta,deuxaxplusb+sqrtdelta),contextptr);
	}
	else {
	  sqrtdelta=normal(sqrt(r2e(-delta,lprime)));
	  atannum=atannum*gen(2);
	  simplify(atannum,alpha);
	  gen tmpatan=atan(ratnormal(rdiv(deuxaxplusb,sqrtdelta)),contextptr);
	  if (!angle_radian(contextptr))
	    tmpatan=tmpatan*deg2rad_e;
	  lnpart=lnpart+rdiv(r2e(atannum,lprime),(r2e(alpha,lprime))*sqrtdelta)*tmpatan;
	}
	break; 
      default: // divide a*it->num =b*it->den.derivative()+c 
	it->num.TPseudoDivRem(it->den.derivative(),b,c,a);
	// remaining pf
	if (!c.coord.empty()){
	  vtmp=polynome2poly1(a*it->den,1);
	  if (!integrate_deno_length_2(c,vtmp,l,lprime,lnpart,contextptr))
	    rest.push_back(pf<gen>(c,a*it->den,it->fact,1));
	}
	// extract log part b/a*ln[fact]
	simplify(b,a);
	lnpart=lnpart+rdiv(r2e(b,l),r2e(a,l))*lnabs(r2e(it->fact,l),contextptr);
	break ;
      }
    }
    remains_to_integrate=r2sym(rest,l);
    return rdiv(r2e(ipnum.integrate(),l),r2e(ipden,l))+r2sym(intde_VECT,l)+lnpart;
  }

  // integration of e when linear operations have been applied
  gen xln_x(const gen & x,GIAC_CONTEXT){
    return x*ln(x)-x;
  }

  gen int_exp(const gen & x,GIAC_CONTEXT){
    return exp(x);
  }

  gen int_sinh(const gen & x,GIAC_CONTEXT){
    return cosh(x);
  }

  gen int_cosh(const gen & x,GIAC_CONTEXT){
    return sinh(x);
  }

  gen int_sin(const gen & x,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return -cos(x,contextptr);
    else
      return -cos(x,contextptr)*rad2deg_e;
  }

  gen int_cos(const gen & x,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return sin(x,contextptr);
    else
      return sin(x,contextptr)*rad2deg_e;
  }

  gen int_tan(const gen & x,GIAC_CONTEXT){
    gen g=-lnabs(cos(x,contextptr),contextptr);
    if (angle_radian(contextptr))
      return g;
    else
      return g*rad2deg_e;
  }

  gen int_tanh(const gen & x,GIAC_CONTEXT){
    return -ln(cosh(x));
  }

  gen int_asin(const gen & x,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return x*asin(x,contextptr)+sqrt(1-pow(x,2));
    else
      return x*asin(x,contextptr)*deg2rad_e+sqrt(1-pow(x,2));
  }

  gen int_acos(const gen & x,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return x*acos(x,contextptr)-sqrt(1-pow(x,2));
    else
      return x*acos(x,contextptr)*deg2rad_e-sqrt(1-pow(x,2));
  }

  gen int_atan(const gen & x,GIAC_CONTEXT){
    if (angle_radian(contextptr)) 
      return x*atan(x,contextptr)-rdiv(ln(pow(x,2)+1),plus_two);
    else
      x*atan(x,contextptr)*deg2rad_e-rdiv(ln(pow(x,2)+1),plus_two);
  }

  extern unary_function_ptr primitive_tab_op[];
  extern unary_function_ptr inverse_tab_op[];

  gen_op_context primitive_tab_primitive[]={giac::int_sin,giac::int_cos,giac::int_tan,giac::int_exp,giac::int_sinh,giac::int_cosh,giac::int_tanh,giac::int_asin,giac::int_acos,giac::int_atan,giac::xln_x};

  void solve_aPprime_plus_P(const gen & anum,const gen & aden,const vecteur & Q,vecteur & R,gen & Pden){
    // a P+P'=Q, a=anum/aden, on cherche P sous la forme R/Pden
    // On a (k+1)p_(k+1)+ anum/aden*p_k=q_k
    // Donc p_k=aden/anum*(q_k-(k+1)p_(k+1))
    // n=deg[Q], on a donc Pden=anum^(n+1), puis on cherche R=P*anum^(n+1)
    // on multiplie donc Q par anum^(n+1) S=Q*anum^(n+1)/a
    // on a aR+R'=aS
    // on a donc par ordre decr. r_(n+1)=0 
    // r_k= s_k - (k+1)*r_(k+1)/a
    // avec des divisions sans creation de denominateurs
    // par ex. P'+3P=x^2+5x+7 -> r_2=9, r_1=39, r_0=50, a=3, n=2, a^n=9
    R.clear();
    if (Q.empty()){
      Pden=plus_one;
      return;
    }
    int n=Q.size()-1;
    R.reserve(n+1);
    Pden=pow(anum,n);
    vecteur S;
    multvecteur(Pden*aden,Q,S);
    Pden=Pden*anum;
    const_iterateur it=S.begin(),itend=S.end();
    R.push_back(*it);
    ++it;
    for (int k=n-1;it!=itend;++it,--k){
      R.push_back(*it-rdiv(gen(k+1)*R.back()*aden,anum));
    }
    // should simplify R with Pden
  }

  void insure_real_deno(gen & n,gen & d){
    gen i=im(d,0),c=conj(d,0);
    if (!is_zero(i)){
      n=n*c;
      d=d*c;
    }
  }

  // try to rewrite fu(x), function of x as a fonction of u(x), if possible 
  // return fx(x) such that fu(x)=fx(u(x))
  // FIXME: should detect u=pow(.,inv(n)) with n integer
  bool is_rewritable_as_f_of(const gen & fu,const gen & u,gen & fx,const identificateur & x,GIAC_CONTEXT){
    gen a,b;
    if (is_linear_wrt(u,x,a,b)){
      fx=complex_subst(fu,x,rdiv(x-b,a),contextptr);
      return false;// true?
    }
    // try first if u is a linear expression of something else
    if (u.type==_SYMB){
      if (u._SYMBptr->sommet==at_neg){
	gen tmpu=u._SYMBptr->feuille,tmpfx;
	if (!is_rewritable_as_f_of(fu,tmpu,tmpfx,x,contextptr))
	  return false;
	fx=complex_subst(tmpfx,x,-x,contextptr);
	return true;
      }
      if (u._SYMBptr->sommet==at_pow){
	gen tmpu=u._SYMBptr->feuille,tmpfx;
	if (tmpu.type==_VECT && tmpu._VECTptr->size()==2){
	  gen expo=inv(tmpu._VECTptr->back());
	  tmpu=tmpu._VECTptr->front();
	  if (expo.type==_INT_){ 
	    if (is_linear_wrt(tmpu,x,a,b)){
	      fx=complex_subst(fu,x,rdiv(pow(x,expo)-b,a),contextptr);
	      return true;
	    }
	  }
	}
      }
      gen alpha;
      vecteur non_constant;
      if (u._SYMBptr->sommet==at_prod ){
	if (u._SYMBptr->feuille.type!=_VECT)
	  return is_rewritable_as_f_of(fu,u._SYMBptr->feuille,fx,x,contextptr);
	de_VECTose_prod(*u._SYMBptr->feuille._VECTptr,x,non_constant,alpha);
	assert(!non_constant.empty());
	if (!is_one(alpha)){
	  gen tmpu,tmpfx;
	  tmpu=_prod(non_constant);
	  if (!is_rewritable_as_f_of(fu,tmpu,tmpfx,x,contextptr))
	    return false;
	  fx=complex_subst(tmpfx,x,rdiv(x,alpha),contextptr);
	  return true;
	}
      }
      if (u._SYMBptr->sommet==at_plus){
	if (u._SYMBptr->feuille.type!=_VECT)
	  return is_rewritable_as_f_of(fu,u._SYMBptr->feuille,fx,x,contextptr);
	de_VECTose_plus(*u._SYMBptr->feuille._VECTptr,x,non_constant,alpha);
	assert(!non_constant.empty());
	if (!is_zero(alpha)){
	  gen tmpu,tmpfx;
	  tmpu=_plus(non_constant);
	  if (!is_rewritable_as_f_of(fu,tmpu,tmpfx,x,contextptr))
	    return false;
	  fx=complex_subst(tmpfx,x,x-alpha,contextptr);
	  return true;
	}
      }
    }
    return in_is_rewritable_as_f_of(fu,u,fx,x,contextptr);
  }

  bool in_is_rewritable_as_f_of(const gen & fu,const gen & u,gen & fx,const identificateur & x,GIAC_CONTEXT){
    if (fu.type==_VECT){
      vecteur res;
      const_iterateur it=fu._VECTptr->begin(),itend=fu._VECTptr->end();
      gen tmp;
      for (;it!=itend;++it){
	if (!is_rewritable_as_f_of(*it,u,tmp,x,contextptr))
	  return false;
	res.push_back(tmp);
      }
      fx=res;
      return true;
    }
    if (fu.type==_IDNT){
      if (fu!=x){
	fx=fu;
	return true;
      }
      return false;
    }
    if (fu.type!=_SYMB){
      fx=fu;
      return true;
    }
    // symbolic
    if (fu==u){
      fx=x;
      return true;
    }
    // de_VECTose
    unary_function_ptr s=fu._SYMBptr->sommet;
    gen f=fu._SYMBptr->feuille,tmpfx;
    if (in_is_rewritable_as_f_of(f,u,tmpfx,x,contextptr)){
      fx=symbolic(s,tmpfx);
      return true;
    }
    // try special treatment for integral powers
    int fexp,uexp;
    if ( (u.type!=_SYMB) || (s!=at_pow) || (f._VECTptr->back().type!=_INT_) )
      return false;
    fexp=f._VECTptr->back().val;
    if ( (u._SYMBptr->sommet==at_pow) && (u._SYMBptr->feuille._VECTptr->back().type==_INT_) && (u._SYMBptr->feuille._VECTptr->front()==f._VECTptr->front()) ){
      uexp=u._SYMBptr->feuille._VECTptr->back().val;
      if (fexp%uexp)
	return false;
      fx=pow(x,fexp/uexp);
      return true;
    }
    // trigonometric fcns to an even power
    f=f._VECTptr->front();
    if ( (fexp %2) || (f.type!=_SYMB) )
      return false;
    fexp=fexp/2;
    int ftrig=equalposcomp(primitive_tab_op,f._SYMBptr->sommet);
    if (!ftrig)
      return false;
    int utrig=equalposcomp(primitive_tab_op,u._SYMBptr->sommet);
    if (!utrig)
      return false;
    switch (ftrig){
    case 1: // sin
      switch (utrig){
      case 2: // sin^2=1-cos^2
	fx=pow(1-pow(x,2),fexp);
	return true;
      case 3: // sin^2=1-1/(tan^2+1)
	fx=pow(1-inv(pow(x,2)+1),fexp);
	return true;
      default:
	return false;
      }
    case 2: // cos
      switch (utrig){
      case 1: // cos^2=1-sin^2
	fx=pow(1-pow(x,2),fexp);
	return true;
      case 3: // cos^2=1/(tan^2+1)
	fx=pow(pow(x,2)+1,-fexp);
	return true;
      default:
	return false;
      }
    case 3: // tan
      switch (utrig){
      case 1: // tan^2=1/(1-sin^2)-1
	fx=pow(inv(1-pow(x,-2))-1,fexp);
	return true;
      case 2: // tan^2=1/cos^2-1
	fx=pow(pow(x,-2)-1,fexp);
	return true;
      default:
	return false;
      }
    }
    return false;
  }

  gen firstcoefftrunc(const gen & e){
    if (e.type==_FRAC)
      return fraction(firstcoefftrunc(e._FRACptr->num),firstcoefftrunc(e._FRACptr->den));
    if (e.type==_POLY)
      return firstcoeff(*e._POLYptr).trunc1();
    return e;
  }

  // special version of lvarx that does not remove cst powers
  vecteur lvarxpow(const gen &e,const identificateur & x){
    vecteur v(lvar(e));
    vecteur res;
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (contains(*it,x))
	res.push_back(*it);
    }
    // do lvar again to _VECTrim the first arg of ^
    return lvar(res);
  }

  gen invexptoexpneg(const gen& g){
    if (g.type==_SYMB && g._SYMBptr->sommet==at_exp)
      return exp(-g._SYMBptr->feuille);
    else
      return symb_inv(g);
  }

  gen integrate(const gen & e_orig,const identificateur & x,gen & remains_to_integrate,GIAC_CONTEXT){
    gen e(e_orig);
    if (is_constant_wrt(e,x))
      return e*x;
    if (e.type!=_SYMB) {
      remains_to_integrate=zero;
      if (e==x)
	return rdiv(pow(e,2),plus_two);
      else
	return e*x;
    }
    // Step 0: replace abs(var_dep_x) with sign*var_dep_x
    // and then sign() with a constant
    vecteur l1(lop(e,at_abs)),m1(lop(e,at_sign));
    l1=lvarx(l1,x); m1=lvarx(m1,x);
    if (!l1.empty() || !m1.empty()){
      vecteur l2,m2,l3,l4;
      const_iterateur it=l1.begin(),itend=l1.end();
      int i=0;
      for (;it!=itend;++it,++i){
	gen tmp=it->_SYMBptr->feuille;
	identificateur tmpi(" s"+print_INT_(i));
	l2.push_back(tmpi*tmp);
	l3.push_back(tmpi);
	l4.push_back(symbolic(at_sign,tmp));
      }
      it=m1.begin(),itend=m1.end();
      for (;it!=itend;++it,++i){
	l1.push_back(*it);
	gen tmp=it->_SYMBptr->feuille;
	identificateur tmpi(" s"+print_INT_(i));
	l2.push_back(tmpi);
	l3.push_back(tmpi);
	l4.push_back(*it);
      }      
      e=complex_subst(e,l1,l2,contextptr);
      gen res=integrate(e,x,remains_to_integrate,contextptr);
      remains_to_integrate=complex_subst(remains_to_integrate,l3,l4,contextptr);
      return complex_subst(res,l3,l4,contextptr);
    }
    // Step1: detection of some unary_op[linear fcn]
    unary_function_ptr u=e._SYMBptr->sommet;
    gen f=e._SYMBptr->feuille,a,b;
    // particular case for ^, _FUNCnd arg must be constant
    if ( (u==at_pow) && is_constant_wrt(f._VECTptr->back(),x) && is_linear_wrt(f._VECTptr->front(),x,a,b) ){
      b=f._VECTptr->back();
      if (is_minus_one(b))
	return rdiv(lnabs(f._VECTptr->front(),contextptr),a);
      return rdiv(pow(f._VECTptr->front(),b+plus_one),a*(b+plus_one));
    }
    // unary op only
    int s=equalposcomp(primitive_tab_op,u);
    if (s && is_linear_wrt(f,x,a,b) )
      return rdiv(primitive_tab_primitive[s-1](f,contextptr),a);

    // Step2: detection of f(u)*u' 
    vecteur v(1,x);
    gen fu,fx;
    rlvarx(e,x,v);
    vecteur rvar=v;
    int rvarsize=v.size();
    if (rvarsize<TRY_FU_UPRIME){ // otherwise no hope
      const_iterateur it=v.begin(),itend=v.end();
      ++it; // don't try x!
      for (;it!=itend;++it){
	gen df=derive(*it,x);
	fu=ratnormal(rdiv(e,df));
	if (is_undef(fu) && is_zero(ratnormal(df))){
	  // *it is constant -> find the value
	  gen newvar=subst(*it,x,zero);
	  gen newe=subst(e,*it,newvar);
	  return integrate(newe,x,remains_to_integrate,contextptr);
	}
	if (is_rewritable_as_f_of(fu,*it,fx,x,contextptr)){
	  gen tmprem;
	  gen tmpres=linear_integrate(fx,x,tmprem,contextptr);
	  remains_to_integrate=remains_to_integrate+complex_subst(tmprem,x,*it,contextptr)*df;
	  return complex_subst(tmpres,x,*it,contextptr);
	}
	if (it->type!=_SYMB)
	  continue;
	f=it->_SYMBptr->feuille;
	if ( (f.type==_VECT) && (!f._VECTptr->empty()) )
	  f=f._VECTptr->front();
	if (f.type!=_SYMB)
	  continue;
	if (is_linear_wrt(f,x,a,b))
	  continue;
	df=derive(f,x);
	fu=ratnormal(rdiv(e,df));
	if (is_rewritable_as_f_of(fu,f,fx,x,contextptr)){
	  gen tmprem;
	  gen tmpres=linear_integrate(fx,x,tmprem,contextptr);
	  remains_to_integrate=remains_to_integrate+complex_subst(tmprem,x,f,contextptr)*df;
	  return complex_subst(tmpres,x,f,contextptr);
	}	  
      }
    }
    // try with argument of the product or of a power
    v.clear();
    bool est_puissance;
    if (e._SYMBptr->sommet==at_pow){
      v=vecteur(1,e._SYMBptr->feuille._VECTptr->front());
      fu=pow(e._SYMBptr->feuille._VECTptr->front(),e._SYMBptr->feuille._VECTptr->back()-plus_one);
      est_puissance=true;
    }
    else {
      if ( (e._SYMBptr->sommet==at_prod) && (e._SYMBptr->feuille.type==_VECT))
	v=*e._SYMBptr->feuille._VECTptr;
      est_puissance=false;
    }
    const_iterateur vt=v.begin(),vtend=v.end();
    for (int i=0;(i<TRY_FU_UPRIME) && (vt!=vtend);++vt,++i){
      gen tmprem,u=linear_integrate(*vt,x,tmprem,contextptr);
      if (!is_zero(tmprem))
	continue;
      if (!est_puissance){
	vecteur vv(v);
	vv.erase(vv.begin()+i,vv.begin()+i+1);
	fu=symbolic(at_prod,vv);
      }
      if (is_rewritable_as_f_of(fu,u,fx,x,contextptr)){
	gen tmpres=linear_integrate(fx,x,tmprem,contextptr);
	remains_to_integrate=remains_to_integrate+complex_subst(tmprem,x,u,contextptr)*derive(u,x);
	return complex_subst(tmpres,x,u,contextptr);
      }
    }

    // Step3: rational fraction?
    if (rvarsize==1)
      return integrate_rational(e,x,remains_to_integrate,contextptr);

    // square roots
    if ( (rvarsize==2) && (rvar.back().type==_SYMB) && (rvar.back()._SYMBptr->sommet==at_pow) ){ // x and a power
      // subcase 1: power is a fraction of int
      // find rational parametrization if possible
      // subcase 2: 1st argument of power is linear, 2nd is constant && no inv
      gen argument=rvar.back()._SYMBptr->feuille._VECTptr->front();
      gen exposant=rvar.back()._SYMBptr->feuille._VECTptr->back();
      if ( (exposant.type==_FRAC) && (exposant._FRACptr->num.type==_INT_) && (exposant._FRACptr->den.type==_INT_) ){
	int n=exposant._FRACptr->num.val,d=exposant._FRACptr->den.val;
	gen a,b,c,tmprem,tmpres,tmpe;
	if (is_linear_wrt(argument,x,a,b)){
	  // argument=(ax+b)=t^d -> x=(t^d-a)/b and dx=d/a*t^(d-1)*dt
	  vecteur substin(makevecteur(argument,x));
	  vecteur substout(makevecteur(pow(x,d),rdiv(pow(x,d)-b,a)));
	  tmpe=complex_subst(e,substin,substout,contextptr)*pow(x,d-1);
	  tmpres=linear_integrate(tmpe,x,tmprem,contextptr);
	  gen fnc_inverse=pow(a*x+b,fraction(1,d));
	  remains_to_integrate=rdiv(d,a)*complex_subst(tmprem,x,fnc_inverse,contextptr);
	  return rdiv(d,a)*complex_subst(tmpres,x,fnc_inverse,contextptr);
	}
	vecteur tmpv(1,x);
	lvar(argument,tmpv);
	gen fr,fr_n,fr_d,ap,bp;
	fr=e2r(argument,tmpv);
	fxnd(fr,fr_n,fr_d);
	fr_n=r2e(fr_n,tmpv);
	fr_d=r2e(fr_d,tmpv);
	if (is_linear_wrt(fr_n,x,a,b) && is_linear_wrt(fr_d,x,ap,bp) ){
	  // argument=(a*x+b)/(ap*x+bp)=t^d
	  // -> x=(bp*t^d-b)/(a-ap*t^d) 
	  // -> dx= d*(b*ap-a*bp)*t^(d-1)/(a-ap*t^d)^2
	  vecteur substin(makevecteur(argument,x));
	  vecteur substout(makevecteur(pow(x,d),rdiv(bp*pow(x,d)-b,a-ap*pow(x,d))));
	  tmpe=complex_subst(e,substin,substout,contextptr)*rdiv(pow(x,d-1),pow(a-ap*pow(x,d),2));
	  tmpres=linear_integrate(tmpe,x,tmprem,contextptr);
	  gen fnc_inverse=pow(rdiv(a*x+b,ap*x+bp),fraction(1,d));
	  gen tmp=gen(d)*(a*bp-b*ap);
	  remains_to_integrate=tmp*complex_subst(tmprem,x,fnc_inverse,contextptr);
	  return tmp*complex_subst(tmpres,x,fnc_inverse,contextptr);
	}
	/*   ( * 	2nd order: dispatch for y=ax^2+bx+c	           * )
	     ( * 	a>0	->	x=[m^2-c]/[b-2*sqrt[a]*m]          * )
	     ( *			m=sqrt[y]-sqrt[a]*x	           * )
	     ( * 			dx/sqrt[y]=2*dm/[b-2*sqrt[a]*m]	   * )
	*/
	if (d==2 && is_constant_wrt(fr_d,x)){
	  // write e as alpha+beta*sqrt(argument)
	  identificateur tmpx(" x");
	  gen e1=complex_subst(e,sqrt(argument),tmpx,contextptr);
	  vecteur lv(1,tmpx);
	  lvar(e1,lv);
	  gen e2=e2r(e1,lv),num,den;
	  fxnd(e2,num,den);
	  den=r2e(den,lv);
	  num=r2e(num,lv);
	  // multiply denominator of e2 by conjugate 
	  gen pmini=tmpx*tmpx-argument;
	  gen C=_egcd(makevecteur(den,pmini,tmpx));
	  num=_rem(makevecteur(num*C[0],pmini,tmpx));
	  den=C[2];
	  gen alpha,beta;
	  if (!is_linear_wrt(num,tmpx,beta,alpha))
	    setsizeerr();
	  alpha=integrate_rational(alpha/den,x,remains_to_integrate,contextptr);
	  /* Instead we should factor argument in den 
	     FIXME in usual.cc diff of ln should expand * and / and rm abs
	     write y=argument, P=beta
	     we want to integrate P*sqrt(y)/den=(P*y)/den* y^(-1/2)
	     let den=y^l*D where D is prime with y
	     P/Dy^l = P_y/y^l + P_D/D <--> P = P_y*D + P_D*y^l,
	     find P_D and P_y by Bezout, find
	     g = Q*D+R*y^l then Pg = P*Q*D + P*R*y^l hence
	     P_D = P*R mod D/g  and  P_y = P*Q /g + [P*R div D] *y^l /g 
	  */
	  gen y=argument,P=beta,D=den; // P*y/den
	  C=_quorem(makevecteur(D,y,x));
	  int l=0;
	  if (is_zero(C[1])){ // P/(den/y)
	    D=C[0];
	    for (;;++l){
	      C=_quorem(makevecteur(D,y,x));
	      if (!is_zero(C[1]))
		break;
	      D=C[0];
	    }
	  }
	  else
	    P=P*y;
	  gen yl=pow(y,l);
	  C=_egcd(makevecteur(D,yl,x));
	  gen g=C[2],Q=C[0],R=C[1];
	  C=_quorem(makevecteur(P*R,D,x));
	  gen PD=C[1]/g;
	  gen Py=(P*Q+C[0]*yl)/g;
	  C=_quorem(makevecteur(Py,y,x));
	  /*
	    int[ Py/y^l*y^-1/2 ] = Q*y^[1/2-l] + int[ C*y^-1/2 ]
	    degre[Py]=n, degre[y]=k, find Q degre[Q]=n+1-k and C degre[C]=k-2
	    so that Py = Q'*y + Q*y'*[1/2-l] + C y^l   
	    to do this we represent Q by a n+2-k-vector, C by a k-1-vector
	    gluing Q and C we get a n+1-vector that must be solution of a
	    n+1*n+1 linear system. Now we build the matrix of this system
	    The n+2-k first columns are				    
	    y'[1/2-l]  ...  x^alpha*y'*[1/2-l]+alpha*x^[alpha-1]*y ...    
	    The k-1 last columns are 
	    y^l  ...  x^beta*y^l 
	    Note 1: to avoid rational input in the matrix we multiply by 2
	    coef of Q and C are found in the reverse order
	    for l!=0 n is more precisely max[deg[Py],k[l+1]-2]
	    Note 2: at the end we integrate only (C+PD/D)*y^(-1/2)
	   */
	  gen tmpv=_e2r(makevecteur(Py,x));
	  if (tmpv.type!=_VECT) setsizeerr();
	  vecteur colP=*tmpv._VECTptr;
	  int n=colP.size()-1;
	  tmpv=_e2r(makevecteur(y,x));
	  if (tmpv.type!=_VECT) setsizeerr();
	  int k=tmpv._VECTptr->size()-1;
	  n=max(n,k*(l+1)-2);
	  if (n){
	    lrdm(colP,n);
	    gen yprime=(1-2*l)*derive(y,x);
	    matrice sys;
	    tmpv=_e2r(makevecteur(yprime,x));
	    if (tmpv.type!=_VECT) setsizeerr();
	    vecteur col0(*tmpv._VECTptr);
	    vecteur col(col0);
	    lrdm(col,n);
	    sys.push_back(col);
	    col0.push_back(zero);
	    tmpv=_e2r(makevecteur(2*y,x));
	    if (tmpv.type!=_VECT) setsizeerr();
	    vecteur col1(*tmpv._VECTptr);
	    for (int i=1;i<n+2-k;++i){
	      col=col0+gen(i)*col1;
	      lrdm(col,n);
	      col0.push_back(zero);
	      col1.push_back(zero);
	      sys.push_back(col);
	    }
	    tmpv=_e2r(makevecteur(2*yl,x));
	    if (tmpv.type!=_VECT) setsizeerr();
	    col0=*tmpv._VECTptr;
	    for (int i=0;i<k-1;++i){
	      col=col0;
	      lrdm(col,n);
	      sys.push_back(col);
	      col0.push_back(zero);
	    }
	    sys=mtran(sys);
	    col0=linsolve(sys,colP);
	    reverse(col0.begin(),col0.end()); // C at the beginning, Q at the end
	    C=2*horner(vecteur(col0.begin(),col0.begin()+k-1),x);
	    Q=2*horner(vecteur(col0.begin()+k-1,col0.end()),x);
	    alpha=alpha+Q*sqrt(y)/yl;
	  }
	  else
	    C=Py;
	  e=(C+PD/D)/sqrt(argument);
	  if (is_quadratic_wrt(argument,x,a,b,c)){
	    if (!is_positive(-a,contextptr)){
	      gen sqrta(sqrt(a));
	      identificateur id_m(" m");
	      gen m(id_m);
	      tmpe=eval(rdiv(complex_subst(e*sqrt(argument),argument,pow(m+sqrta*x,2),contextptr),b-plus_two*sqrta*m),contextptr);
	      tmpe=complex_subst(tmpe,x,rdiv(m*m-c,b-plus_two*sqrta*m),contextptr);
	      tmpres=linear_integrate(tmpe,id_m,tmprem,contextptr);
	      remains_to_integrate=remains_to_integrate+complex_subst(plus_two*tmprem,m,sqrt(argument)-sqrta*x,contextptr);
	      return alpha+complex_subst(plus_two*tmpres,m,sqrt(argument)-sqrta*x,contextptr);
	    }
	    else {
	      gen D;
	      identificateur tmpx(" x");
	      vecteur w=solve(tmpx*tmpx-(b*b-gen(4)*a*c),tmpx,0,contextptr); 
	      if (w.empty())
		setsizeerr();
	      D=w.back();
	      // D=normal(sqrt(b*b-gen(4)*a*c));  // was ratnormal...
	      /*
		( *	D=sqrt(b^2-4ac)                                    * )
		( * 	a<0 and D>0 ->	x=[D*2u/[1+u^2]-b]/2a		   * )
		( *			u=-[D-2*sqrt[-a]*sqrt[y]]/[2ax+b]   * )
		( *			dx/sqrt[y]=2*du/[sqrt[-a]*[1+u^2]] * )
	      */
	      gen sqrta(sqrt(-a));
	      identificateur id_u(" u");
	      gen u(id_u);
	      gen uasx=rdiv(plus_two*sqrta*sqrt(argument)-D,plus_two*a*x+b);
	      gen sqrty=rdiv(D-(plus_two*a*x+b)*u,plus_two*sqrta);
	      tmpe=eval(rdiv(complex_subst(e*sqrt(argument),argument,pow(sqrty,2),contextptr),1+u*u),contextptr);
	      tmpe=complex_subst(tmpe,x,rdiv(rdiv(plus_two*u*D,1+u*u)-b,plus_two*a),contextptr);
	      tmpres=linear_integrate(tmpe,id_u,tmprem,contextptr);
	      remains_to_integrate=remains_to_integrate+complex_subst(rdiv(plus_two,sqrta)*tmprem,u,uasx,contextptr);
	      return alpha+complex_subst(rdiv(plus_two,sqrta)*tmpres,u,uasx,contextptr);
	    }
	  } // end sqrt of quadratic
	  else {
	    remains_to_integrate=e;
	    return alpha;
	  }
	} // end if d==2
      } // end exposant=fraction of integers
    } // end recusive var size==2

    // detection of inv of trig or ln of a linear expression
    const_iterateur rvt=rvar.begin(),rvtend=rvar.end();
    for (;rvt!=rvtend;++rvt){
      if (rvt->type!=_SYMB)
	continue;
      int rvtt=equalposcomp(inverse_tab_op,rvt->_SYMBptr->sommet);
      if (!rvtt || rvtt==3 || rvtt==7) // exclude atan and atanh
	continue;
      rvtt=equalposcomp(primitive_tab_op,rvt->_SYMBptr->sommet);
      if (rvtt>7){
	unary_function_ptr inverse_sommet=primitive_tab_op[rvtt-8];
	gen feuille=rvt->_SYMBptr->feuille,a,b;
	if (!is_linear_wrt(feuille,x,a,b))
	  continue;
	// Additionaly check that e is polynomial wrt x
	identificateur tmpidnt(" t");
	gen tmpcheck=subst(e,*rvt,tmpidnt);
	vecteur vx2(rlvarx(tmpcheck,x));
	if ( vx2.size()>1)
	  continue;
	if (vx2.size()){
	  lvar(tmpcheck,vx2);
	  fraction ftemp=sym2r(tmpcheck,vx2);
	  if (ftemp.den.type==_POLY && ftemp.den._POLYptr->lexsorted_degree())
	    continue;
	}
	// make the change of var ln[ax+b]=t -> x=rdiv(exp(t)-b,a)
	gen tmprem,tmpres,tmpe,xt,dxt;
	xt=rdiv(symbolic(inverse_sommet,x)-b,a);
	dxt=derive(xt,x);
	vecteur substin(makevecteur(x,*rvt));
	vecteur substout(makevecteur(xt,x));
	tmpe=complex_subst(e,substin,substout,contextptr)*dxt;
	tmpres=linear_integrate(tmpe,x,tmprem,contextptr);
	remains_to_integrate=complex_subst(rdiv(tmprem,dxt),x,*rvt,contextptr);
	tmpres=_texpand(tmpres,contextptr);
	return complex_subst(tmpres,substout,substin,contextptr);
      }
    }

    // integration by part?
    if ( (e._SYMBptr->sommet==at_prod) && (e._SYMBptr->feuille.type==_VECT)){
      const_iterateur ibp=e._SYMBptr->feuille._VECTptr->begin(),ibpend=e._SYMBptr->feuille._VECTptr->end();
      for (int j=0;ibp!=ibpend;++ibp,++j){
	int test;
	if (ibp->type!=_SYMB)
	  continue;
	if ( (ibp->_SYMBptr->sommet==at_pow) &&
	     (ibp->_SYMBptr->feuille._VECTptr->front().type==_SYMB) &&  
	     (ibp->_SYMBptr->feuille._VECTptr->back().type==_INT_) && 
	     (ibp->_SYMBptr->feuille._VECTptr->back().val>0) )
	  test=equalposcomp(inverse_tab_op,ibp->_SYMBptr->feuille._VECTptr->front()._SYMBptr->sommet);
	else
	  test=equalposcomp(inverse_tab_op,ibp->_SYMBptr->sommet);
	if (!test)
	  continue;
	vecteur ibpv(*e._SYMBptr->feuille._VECTptr);
	ibpv.erase(ibpv.begin()+j);
	gen ibpe=_prod(ibpv);
	vecteur algx;
	if (lvarx(ibpe,x)==vecteur(1,x)){
	  gen tmpres,tmprem,tmpprimitive,tmp;
	  tmpprimitive=integrate_rational(ibpe,x,tmp,contextptr);
	  if (is_zero(tmp) && lvarx(tmpprimitive,x)==vecteur(1,x)){
	    tmpres=linear_integrate(tmpprimitive*derive(*ibp,x),x,tmprem,contextptr);
	    remains_to_integrate=-tmprem;
	    return tmpprimitive*(*ibp)-tmpres;
	  }
	}
      }
    }
    else { // check for u'=1
      int test;
      if ( (e._SYMBptr->sommet==at_pow) && (e._SYMBptr->feuille._VECTptr->front().type==_SYMB) && (e._SYMBptr->feuille._VECTptr->back().type==_INT_) && (e._SYMBptr->feuille._VECTptr->back().val>0) )
	test=equalposcomp(inverse_tab_op,e._SYMBptr->feuille._VECTptr->front()._SYMBptr->sommet);
      else
	test=equalposcomp(inverse_tab_op,e._SYMBptr->sommet);
      if (test){
	gen tmpres,tmprem;
	tmpres=linear_integrate(x*derive(e,x),x,tmprem,contextptr);
	remains_to_integrate=-tmprem;
	return x*e-tmpres;
      }
    }

    // rewrite inv(exp)
    vector<unary_function_ptr> vsubstin(1,at_inv);
    vector<gen_op> vsubstout(1,invexptoexpneg);
    e=subst(e,vsubstin,vsubstout);
    // detection of denominator=independent of x
    v=lvarxwithinv(e,x);
    // search for nop (for nop[inv])
    if (!has_nop_var(v)){
      // exp linearization
      vecteur vexp;
      gen res;
      lin(e,vexp); // vexp = coeff, arg of exponential
      const_iterateur it=vexp.begin(),itend=vexp.end();
      for (;it!=itend;){
	// trig linearization
	vecteur vtrig;
	gen coeff=*it;
	++it; // it -> on the arg of the exp that must be linear
	gen rea,reb,reaxb=*it;
	++it;
	if (!is_linear_wrt(reaxb,x,rea,reb)){
	  remains_to_integrate = remains_to_integrate + coeff*exp(reaxb);
	  continue;
	}
	tlin(coeff,vtrig,contextptr); // vtrig = coeff , sin/cos(arg)/1
	const_iterateur jt=vtrig.begin(),jtend=vtrig.end();
	for (;jt!=jtend;){
	  // now check that each arg is linear and coeff polynomial
	  coeff=*jt;
	  ++jt;
	  gen ima,imb,imaxb=*jt;
	  ++jt;
	  int trig_type=0; // 0 for 1, 1 for sin, 2 for cos
	  if (imaxb.type==_SYMB){
	    if (imaxb._SYMBptr->sommet==at_sin)
	      trig_type=1;
	    if (imaxb._SYMBptr->sommet==at_cos)
	      trig_type=2;
	  }
	  // check polynomial
	  vecteur vx2(lvarxpow(coeff,x));
	  if ( (vx2.size()>1) || ( (!vx2.empty()) && (vx2.front()!=x))  || ( (imaxb.type==_SYMB) && !is_linear_wrt(imaxb._SYMBptr->feuille,x,ima,imb) ) ){
	    if (trig_type) imaxb=imaxb._SYMBptr->feuille;
	    gen tmp(plus_one);
	    if (trig_type==1)
	      tmp=sin(imaxb,contextptr);
	    if (trig_type==2)
	      tmp=cos(imaxb,contextptr);
	    remains_to_integrate = remains_to_integrate + coeff * exp(reaxb) * tmp;
	    continue;
	  }
	  // everything OK coeff*exp(rea*x+reb)* 1/cos/sin(ima*x+imb)
	  if (trig_type)
	    imaxb=imaxb._SYMBptr->feuille;
	  else {
	    if (is_zero(rea)){
	      gen tmprem;
	      res= res + exp(reb)*integrate_rational(coeff,x,tmprem,contextptr);
	      remains_to_integrate = remains_to_integrate+exp(reb)*tmprem;
	      continue;
	    }
	  }
	  bool coeff_is_real=false;
	  if (trig_type){
	    gen imcoeff=im(coeff,contextptr);
	    rewrite_with_t_real(imcoeff,x,contextptr);
	    if (is_zero(imcoeff) && is_zero(im(rea,contextptr)) && is_zero(im(ima,contextptr)))
	      coeff_is_real=true;
	  }
	  // find vars of coeff,rea,reb,ima,imb
	  vecteur les_var(1,x); // insure x is the main var
	  lvar(makevecteur(coeff,rea,ima),les_var);
	  int les_vars=les_var.size();
	  gen in_coeff,in_coeffnum,in_coeffden,in_rea,in_ima,in_anum,in_aden;
	  in_coeff=e2r(coeff,les_var);
	  fxnd(in_coeff,in_coeffnum,in_coeffden);
	  vecteur in_coeffnumv;
	  if (in_coeffnum.type==_POLY)
	    in_coeffnumv=polynome2poly1(*in_coeffnum._POLYptr,1);
	  else
	    in_coeffnumv.push_back(in_coeffnum);
	  in_coeffden=firstcoefftrunc(in_coeffden);
	  in_rea=firstcoefftrunc(e2r(rea,les_var));
	  in_ima=firstcoefftrunc(e2r(ima,les_var));
	  vecteur resnum;
	  gen resden,resplus;
	  fxnd(in_rea+cst_i*in_ima,in_anum,in_aden);
	  solve_aPprime_plus_P(in_anum,in_aden,in_coeffnumv,resnum,resden);
	  resplus=rdiv(r2e(poly12polynome(resnum,1,les_vars),les_var),r2e(poly12polynome(vecteur(1,resden*in_coeffden),1,les_vars),les_var));
	  if (!trig_type){
	    res = res + resplus*exp(reaxb);
	    continue;
	  }
	  if (coeff_is_real){
 	    gen resre=re(resplus,contextptr);
	    rewrite_with_t_real(resre,x,contextptr);
 	    gen resim=im(resplus,contextptr);
	    rewrite_with_t_real(resim,x,contextptr);
	    if (trig_type==1)
	      res = res + exp(reaxb)*(resim*cos(imaxb,contextptr)+resre*sin(imaxb,contextptr));
	    else
	      res = res + exp(reaxb)*(resre*cos(imaxb,contextptr)-resim*sin(imaxb,contextptr));
	    continue;
	  }
	  fxnd(in_rea-cst_i*in_ima,in_anum,in_aden);
	  solve_aPprime_plus_P(in_anum,in_aden,in_coeffnumv,resnum,resden);
	  gen resmoins=rdiv(r2e(poly12polynome(resnum,1,les_vars),les_var),r2e(poly12polynome(vecteur(1,resden*in_coeffden),1,les_vars),les_var));
	  if (trig_type==1)
	    res = res + exp(reaxb)*rdiv(resplus*exp(cst_i*imaxb)-resmoins*exp(-cst_i*imaxb),plus_two*cst_i);
	  else
	    res = res +  exp(reaxb)*rdiv(resplus*exp(cst_i*imaxb)+resmoins*exp(-cst_i*imaxb),plus_two);
	} // end for (jt)
      } // end for (it)
      gen tmp=remains_to_integrate;
      remains_to_integrate=0;
      return res+risch(tmp,x,remains_to_integrate,contextptr);
    } // end linearizable

    // trigonometric fraction (or exp _FRAC), rewrite all elemnts of rvar as
    // tan of the common half angle, i.e. tan([coeff_trig*x+b]/2)
    int trig_fraction=-1; 
    gen coeff_trig;
    vecteur var(lvarx(e,x));
    const_iterateur vart=var.begin(),vartend=var.end();
    for (;vart!=vartend;++vart){
      if (vart->type!=_SYMB){
	trig_fraction=false;
	continue;
      }
      int vartt=equalposcomp(primitive_tab_op,vart->_SYMBptr->sommet);
      if ( (!vartt) || (vartt>4) )
	trig_fraction=0;
      if (trig_fraction==-1){
          trig_fraction=vartt;
      }
      else {
          if (trig_fraction==4){
              if (vartt!=4)
                  trig_fraction=0;
          }
          else {
              if (vartt==4)
                  trig_fraction=0;
          }
      }
      if (trig_fraction ){ // trig of linear?
	gen a,b;
	if (!is_linear_wrt(vart->_SYMBptr->feuille,x,a,b)){
	  trig_fraction=false;
	  continue;
	}
	if (is_zero(coeff_trig))
	  coeff_trig=a;
	else {
	  gen quotient=ratnormal(rdiv(a,coeff_trig));
	  if (quotient.type==_INT_)
	    continue;
	  if ( (quotient.type==_FRAC) && (quotient._FRACptr->num.type==_INT_) && (quotient._FRACptr->den.type==_INT_) ){
	    coeff_trig=ratnormal(rdiv(coeff_trig,quotient._FRACptr->den));
	    continue;
	  }
	  if ( (quotient.type==_SYMB) && (quotient._SYMBptr->sommet==at_inv) && (quotient._SYMBptr->feuille.type==_INT_)){
	    coeff_trig=ratnormal(rdiv(coeff_trig,quotient._SYMBptr->feuille));
	    continue;
	  }
	  trig_fraction=false;
	}
      } // end if (trig_fraction)
    }
    if (trig_fraction){
      vart=var.begin();
      vecteur substout;
      gen a,b,coeff_cst;
      is_linear_wrt(vart->_SYMBptr->feuille,x,a,b);
      coeff_cst=ratnormal(rdiv(a,coeff_trig))*b;
      // express all angles in vart as n*(coeff_trig*x+coeff_cst)+angle=a*x+b, 
      // t=coeff_trig*x+coeff_cst
      for (;vart!=vartend;++vart){
	is_linear_wrt(vart->_SYMBptr->feuille,x,a,b);
	gen n=ratnormal(rdiv(a,coeff_trig));
	assert(n.type==_INT_);
	gen angle=ratnormal(b-n*coeff_cst);
	substout.push_back(symbolic(vart->_SYMBptr->sommet,n*x+angle));
      }
      gen f=complex_subst(e,var,substout,contextptr); // should be divided by coeff_trig
      f=_texpand(f,contextptr);
      gen tmprem,tmpres;
      if (trig_fraction==4){ // everything depends on exp(x)
          f=complex_subst(f,exp(x),x,contextptr)*inv(x);
          tmpres=linear_integrate(f,x,tmprem,contextptr);
          gen expx=exp(coeff_trig*x+coeff_cst);
          remains_to_integrate = inv(coeff_trig)*complex_subst(tmprem,x,expx,contextptr);
          return inv(coeff_trig)*complex_subst(tmpres,x,expx,contextptr);
      }
      f=halftan(f); // now everything depends on tan(x/2)
      // t=tan(x/2), dt=1/2(1+t^2)*dx
      f=complex_subst(f,tan(rdiv(x,plus_two),contextptr),x,contextptr)*inv(plus_one+pow(x,2));
      tmpres=linear_integrate(f,x,tmprem,contextptr);
      // Make the antiderivative continuous once x 
      // is replaced by tanxsur2, it requires looking at the atan()
      // First check for atan(x), we will apply atan(tan(...*x))=...*x
      identificateur at(" a");
      tmpres=complex_subst(tmpres,symbolic(at_atan,x),at,contextptr);
      // Then compute the residue for other atan
      vecteur tmpresv(lop(tmpres,at_atan));
      int tmpresatan=tmpresv.size();
      tmpresv=mergevecteur(tmpresv,lvarx(tmpres,x)); 
      int tmpresall=tmpresv.size();
      // Replace every variable with 0 except atan by 1 then by 0 
      gen res1=subst(tmpres,tmpresv,mergevecteur(vecteur(tmpresatan,1),vecteur(tmpresall-tmpresatan,0)),contextptr);
      gen res0=subst(tmpres,tmpresv,vecteur(tmpresall),contextptr);
      gen residu=ratnormal(res1-res0);
      gen xsur2=rdiv(coeff_trig*x+coeff_cst,plus_two);
      residu=residu*symbolic(at_floor,xsur2/cst_pi+plus_one_half)*cst_pi; // xsur2=pi/2 -> pi, xsur2=3*pi/2 -> 2*pi, etc..
      gen tanxsur2=tan(xsur2,contextptr);
      remains_to_integrate = rdiv(plus_two,coeff_trig)*complex_subst(tmprem,x,tanxsur2,contextptr);
      vecteur subst1(2),subst2(2);
      subst1[0]=x; subst2[0]=tanxsur2;
      subst1[1]=at; subst2[1]=xsur2;
      tmpres=complex_subst(tmpres,subst1,subst2,contextptr)+residu;
      return tmpres*rdiv(plus_two,coeff_trig);
    }


    // finish by calling the Risch algorithm
    return risch(e,x,remains_to_integrate,contextptr);
  }

  gen linear_integrate(const gen & e,const identificateur & x,gen & remains_to_integrate,GIAC_CONTEXT){
    return linear_apply(e,x,remains_to_integrate,contextptr,integrate);
  }

  gen integrate(const gen & e,const identificateur & x,GIAC_CONTEXT){
    if (e.type==_VECT){
      vecteur w;
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	w.push_back(integrate(*it,x,contextptr));
      return w;
    }
    gen remains_to_integrate;
    gen ee=rewrite_hyper(e);
    gen res=linear_integrate(ee,x,remains_to_integrate,contextptr);
    if (is_zero(remains_to_integrate))
      return res;
    else
      return res+symbolic(at_integrate,makevecteur(remains_to_integrate,x));
  }

  gen integrate0(const gen & e,const identificateur & x,gen & remains_to_integrate,GIAC_CONTEXT){
    if (e.type==_VECT){
      vecteur w;
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	w.push_back(integrate(*it,x,contextptr));
      return w;
    }
    gen ee=rewrite_hyper(e),tmprem;
    gen res=linear_integrate(ee,x,tmprem,contextptr);
    remains_to_integrate=remains_to_integrate+tmprem;
    return res;
  }

  gen integrate(const gen & e,const gen & f,GIAC_CONTEXT){
    assert(f.type==_IDNT);
    return integrate(e,*f._IDNTptr,contextptr);
  }

  void adjust_int_sum_arg(vecteur & v,int & s){
    if ( (s==2) && (v[1].type==_SYMB) && (v[1]._SYMBptr->sommet==at_equal || v[1]._SYMBptr->sommet==at_same)){
      v.push_back(v[1]._SYMBptr->feuille._VECTptr->back());
      v[1]=v[1]._SYMBptr->feuille._VECTptr->front();
      if ( (v[2].type!=_SYMB) || (v[2]._SYMBptr->sommet!=at_interval) )
	settypeerr();
      v.push_back(v[2]._SYMBptr->feuille._VECTptr->back());
      v[2]=v[2]._SYMBptr->feuille._VECTptr->front();
      s=4;
    }
  }

  // "unary" version
  gen _integrate(const gen & args,GIAC_CONTEXT){
    vecteur v(gen2vecteur(args));
    if (v.size()==1)
      v.push_back(vx_var);
    int s=v.size();
    adjust_int_sum_arg(v,s);
    if ( (s==3) || (s==1) )
      toofewargs("integrate");
    if (s>4)
      toomanyargs("integrate");
    gen x=v[1];
    if (x.type!=_IDNT)
      settypeerr();
    bool quoted=*x._IDNTptr->quoted;
    *x._IDNTptr->quoted=true;
    v[0]=eval(v[0],contextptr);
    *x._IDNTptr->quoted=quoted;    
    if (approx_mode(contextptr) && (s==4))
      return rombergo(v[0],x,v[2],v[3],10);
    gen rem;
    gen primitive=integrate0( v[0],*x._IDNTptr,rem,contextptr);
    if (s==2){
      if (is_zero(rem))
	return primitive;
      return primitive + symbolic(at_integrate,makevecteur(rem,x));
    }
    // here s==4
    gen borne_inf=v[2];
    gen borne_sup=v[3];
    // FIXME should check for singularities
    gen res;
    if (is_greater(borne_sup,borne_inf,contextptr))
      res=limit(primitive,*x._IDNTptr,borne_sup,-1,contextptr)-limit(primitive,*x._IDNTptr,borne_inf,1,contextptr);
    else {
      if (is_greater(borne_inf,borne_sup,contextptr))
	res=limit(primitive,*x._IDNTptr,borne_sup,1,contextptr)-limit(primitive,*x._IDNTptr,borne_inf,-1,contextptr) ;
      else
	res=limit(primitive,*x._IDNTptr,borne_sup,0,contextptr)-limit(primitive,*x._IDNTptr,borne_inf,0,contextptr);
    }
    if (!is_zero(rem))
      res = res + symbolic(at_integrate,makevecteur(rem,x,v[2],v[3]));
    return res;
  }
  const string _integrate_s("integrate");
  string texprintasintegrate(const gen & g,const string & s_orig){
    string s("\\int ");
    if (g.type!=_VECT)
      return s+gen2tex(g);
    vecteur v(*g._VECTptr);
    int l(v.size());
    if (!l)
      return s;
    if (l==1)
      return s+gen2tex(v.front());
    if (l==2)
      return s+gen2tex(v.front())+"\\, d"+gen2tex(v.back());
    if (l==4)
      return s+"_{"+gen2tex(v[2])+"}^{"+gen2tex(v[3])+"}"+gen2tex(v.front())+"\\, d"+gen2tex(v[1]);
    return s;
  }
  unary_function_eval __integrate(&_integrate,_integrate_s,0,&texprintasintegrate);
  unary_function_ptr at_integrate (&__integrate,_QUOTE_ARGUMENTS);

  double rombergo(const gen & f,const gen & x, const gen & a, const gen & b, int n){
    //f est l'expression a integrer, x le nom de la variable, a et b les bornes
    // n si on veut faire 2^n subdivisions
    vector<double> T(n+1);
    //ligne du triangle de romberg avec T(n)=aire avec "pts du milieu" 
    //avec 2^n subdivisions
    double  h;
    gen at;
    //at sert a faire les substitutions c'est un gen = au debut a f((a+b)/2)
    //et en cours de prog egal a f(am)
    h=b.evalf()._DOUBLE_val-a.evalf()._DOUBLE_val;
    //h est la longueur de la subdivision
  
    //T[0] est l'aire du premier rectangle "pt milieu" f((a+b)/2)*(b-a)
    //puis T[j] = aire des rectangles "pt milieu" pour 2^j subdivisions
    double pui4;
    for (int j=0;j<=n;j++){
      //chaque fois que j augmente de 1 on double le nombre de subdivisions
    
      double ss;
      //ss est la somme provenant des valeurs de f aux points am ainsi rajoutes
      ss=0;
      gen am;
    
      am=a+gen(h/2);
      while (is_greater(b,am,0)){
	at=subst(f,x,am).evalf();
	ss=ss+at._DOUBLE_val;
	am=am+gen(h);
      }
      //T[j] est la nouvelle valeur de l'aire calculee avec les "pts  milieu"
      T[j]=ss*h;
    
      h=h/2;
    }
    //pui4 est la valeur de 4^k
    pui4=1;
    for (int j=1;j<=n;j++){ 
      pui4=pui4*4;
      for (int k=0;k<=n-j;k++){
	//on calcule T[k] en appliquant la formule de rec. de romberg
	//avec T[j] qui contient a chaque etape l'integrale par les pts milieu
	T[k]=(pui4*T[k+1]-T[k])/(pui4-1);
     
      }
      //on vient de remplir la kieme ligne on recommence avec j=j+1
      //on doit calculer les "pts du milieu" pour le nouv. j et le mettre ds T[j]
    }
 

    //c'est donc T[0] la meilleur approx de l'integrale
    return(T[0]);
  }

  double rombergt(const gen & f,const gen & x, const gen & a, const gen & b, int n){
    //f est l'expression a integrer, x le nom de la variable, a et b les bornes
    // n si on veut faire 2^n subdivisions
    vector<double> T(n+1);
    //ligne du triangle de romberg avec T(n)=trapezes avec 2^n subdivisions
    double  h;
    gen at;
    //at sert a faire les substitutions c'est un gen egal au debut a f(a)+f(b)
    //et en cours de prog egal a f(am)
    h=b.evalf()._DOUBLE_val-a.evalf()._DOUBLE_val;
    //h est la longueur de la subdivision
    at=subst(f,x,b).evalf()+subst(f,x,a).evalf();
    T[0]=at._DOUBLE_val*h/2;
    //T[0] est l'aire du premier trapeze (f(a)+f(b))*(b-a)/2
    //puis T[j] = aire des trapezes pour 2^j subdivisions
    double pui4;
    for (int j=1;j<=n;j++){
      //chaque fois que j augmente de 1 on double le nombre de subdivisions
      h=h/2;
      double ss;
      //ss est la somme provenant des valeurs de f aux points am ainsi rajoutes
      ss=0;
      gen am;
      am=a+gen(h);  
      while (is_greater(b,am,0)){
	at=subst(f,x,am).evalf();
	ss=ss+at._DOUBLE_val;
	am=am+gen(2*h);
      }
      //T[j] est la nouvelle valeur de l'aire des trapezes
      T[j]=T[j-1]/2+ss*h;
      //pui4 est la valeur de 4^k
      pui4=1;
      for (int k=j-1;k>=0;k--){
	pui4=pui4*4;
	//on calcule T[k] en appliquant la formule de rec. de romberg
	//avec T[j] qui contient a chaque etape l'integrale par les trapezes
	T[k]=(pui4*T[k+1]-T[k])/(pui4-1);
      }
      //on vient de remplir la kieme ligne on recommence avec j=j+1
      //on doit calculer les trapezes pour le nouveau j et le mettre ds T[j]
    }
    //c'est donc T[0] la meilleur approx de l'integrale
    return(T[0]);
  }

  gen romberg(const gen & f,const gen & x,const gen & a,const gen &b,const gen & eps,GIAC_CONTEXT,int nmax=10){
    // a, b and eps should be evalf-ed, and eps>0
    gen h=b-a;
    vecteur old_line,cur_line;
    old_line.push_back(evalf(h*(subst(f,x,a)+subst(f,x,b))/2,DEFAULT_EVAL_LEVEL,contextptr));
    if (is_inf(old_line[0])){
      old_line[0]=evalf(h*(subst(f,x,a+eps)+subst(f,x,b-eps))/2,DEFAULT_EVAL_LEVEL,contextptr);
      if (is_inf(old_line[0]))
	setsizeerr("Infinity at bounds");
    }
    int n=1;
    // At the i-th step of the loop compute the trapeze approx of the integral
    // and use old_line to compute cur_line
    for (int i=0;i<nmax;++i){
      cur_line.clear();
      // compute trapeze
      gen y=a+h/2,sum;
      for (int j=0;j<n;++j){
	sum=sum+evalf(subst(f,x,y),DEFAULT_EVAL_LEVEL,contextptr);
	y=y+h;
      }
      h=h/2;
      cur_line.push_back(old_line[0]/2+sum*h); 
      n = 2*n ;
      gen pui4=1;
      for (int j=0;j<=i;++j){
	pui4=4*pui4;
	cur_line.push_back((pui4*cur_line[j]-old_line[j])/(pui4-1));
      }
      if (i>nmax/2 && ck_is_greater(eps,abs(old_line[i]-cur_line[i+1],0),0))
	return (old_line[i]+cur_line[i+1])/2;
      old_line=cur_line;
    }
    return cur_line;
  }
  gen symb_romberg(const gen & a){
    return symbolic(at_romberg,a);
  }
  gen _romberg(const gen & args,GIAC_CONTEXT) {
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      return symbolic(at_romberg,args);
    const_iterateur it=args._VECTptr->begin(),itend=args._VECTptr->end();
    gen f=*it;
    ++it;
    gen x=*it,a,b;
    if (it<itend-1){
      ++it;
      a=*it;
      ++it;
      b=*it;
    }
    else {
      bool ok=false;
      if (x.is_symb_of_sommet(at_equal)){
	a=x._SYMBptr->feuille;
	if (a.type==_VECT && a._VECTptr->size()==2){
	  x=a._VECTptr->front();
	  a=a._VECTptr->back();
	  if (a.is_symb_of_sommet(at_interval)){
	    a=a._SYMBptr->feuille;
	    if (a.type==_VECT && a._VECTptr->size()==2){
	      b=a._VECTptr->back();
	      a=a._VECTptr->front();
	      ok=true;
	    }
	  }
	} 
      }
      if (!ok)
	setsizeerr();
    }
    if (is_inf(a) || is_inf(b)){ // change of variables x=tan(t), t=atan(x)
      gen tanx(tan(x,contextptr));
      f=subst(f,x,tanx)*(1+pow(tanx,2));
      a=atan(a,contextptr);
      b=atan(b,contextptr);
      return _romberg(makevecteur(f,x,a,b),contextptr);
    }
    a=a.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    b=b.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    ++it;
    gen eps(epsilon(contextptr));
    int n=10;
    if (it!=itend){
      eps=evalf(abs(*it,contextptr),DEFAULT_EVAL_LEVEL,contextptr);
      ++it;
      if (it!=itend && it->type==_INT_)
	n=it->val;
    }
    if (eps.type!=_DOUBLE_ || eps.type!=_REAL)
      eps=epsilon(contextptr);
    if ( x.type!=_IDNT || 
	 (a.type!=_DOUBLE_ && a.type!=_REAL) 
	 || (b.type!=_DOUBLE_ && b.type!=_REAL) 
	 )
      return symbolic(at_integrate,args);
    return romberg(f,x,a,b,eps,contextptr,n);
  }
  const string _romberg_s("romberg");
  unary_function_eval __romberg(&_romberg,_romberg_s);
  unary_function_ptr at_romberg (&__romberg);

/**********************************************************************
* Desc:		Solve P(x+1)-P(x)=Q(x)
* Algo:		degree of P=degree of Q+1, constant coeff 0
*		If P=Sigma a_k x^k then write linear system for a_k
*		Columns of the matrix of the system are lines
*		of the Pascal triangle without the first element
*		(since we must substract identity matrix to the triangle)
*		a_1 a_2 a_3 ... a_n+1		coeff of Q
*		1   1   1	1	X^0
*		0   2	3	n+1	X^1
*		0   0	3	...	X^2
*		0   0	0	n+1     X^n
**********************************************************************/
  vecteur solveP_x_plus_1_minus_P_x(const vecteur & Q){
      vecteur v(1,plus_one);
      matrice m;
      int n=Q.size();
      for (int i=0;i<n;++i){
          v=pascal_next_line(v);
          vecteur v_copy(v);
	  if (i!=n-1){
	    v_copy[i+1]=zero;
	    for (int j=0;j<n-i-2;j++)
	      v_copy.push_back(zero);
	  }
	  else
	    v_copy.pop_back();
          m.push_back(v_copy);
      }
      vecteur Q_copy(Q);
      reverse(Q_copy.begin(),Q_copy.end());
      m.push_back(Q_copy);
      m=mtran(m);
          // reduce matrix, solution P are diag coeff in reverse order
      m=mrref(m);
      vecteur res(n+1);
      for (int i=0;i<n;++i)
          res[n-i-1]=rdiv(m[i][n],m[i][i]);
      return res;
      
  }

  // true if v2[x]=v1[x-n], false otherwise
  bool is_shift_of(const vecteur & v1,const vecteur &v2,int & n){
    int s1=v1.size(),s2=v2.size();
    if (s1!=s2)
      return false;
    if (s1<2)
      setsizeerr();
    gen e=v1[1]-v2[1];
    if (e.type!=_INT_)
      return false;
    if (e.val % (s1-1))
      return false;
    n=e.val / (s1-1);
    return v1==taylor(v2,n);
  }

  bool rational_sum(const gen & e,const gen & x,gen & res,gen& remains_to_sum,GIAC_CONTEXT){
    // first detect rational fraction
    vecteur v(lvarxpow(e,*x._IDNTptr));
    if (v.empty()){
      res = e*x;
      remains_to_sum = 0;
      return true;
    }
    if ( (v.size()!=1) || (v.front()!=x) ){
      remains_to_sum=e;
      return false;
    }
    lvar(e,v);
    gen r=e2r(e,v),r_num,r_den;
    fxnd(r,r_num,r_den);
    if (r_num.type==_EXT){
      remains_to_sum=e;
      return false;
    }
    if ((r_den.type!=_POLY) || (!r_den._POLYptr->lexsorted_degree())){ // polynomial w.r.t. x
      vecteur Q;
      if (r_num.type!=_POLY){
	res=r_num*v.front()/r2e(r,v);
	return true;
      }
      Q=polynome2poly1(*r_num._POLYptr);
      vecteur P(solveP_x_plus_1_minus_P_x(Q));
      r_num=poly12polynome(P);
      r=rdiv(r_num,r_den);
      res=r2e(r,v);
      return true;
    }
    // rational fraction wrt x
    int s=r_den._POLYptr->dim;
    polynome den(*r_den._POLYptr),num(s);
    if (r_num.type==_POLY)
      num=*r_num._POLYptr;
    else
      num=polynome(r_num,s);
    polynome p_content(s);
    // partial fraction de_VECTosition
    factorization vden;
    factor(den,p_content,vden,false,false,false); // withsqrt=false complex_mode=false
    vector< pf<gen> > pfde_VECT;
    polynome ipnum(s),ipden(s);
    partfrac(num,den,vden,pfde_VECT,ipnum,ipden);
    // discrete antiderivative of integral part
    vecteur Q(polynome2poly1(ipnum));
    vecteur P(solveP_x_plus_1_minus_P_x(Q));
    ipnum=poly12polynome(P);
    r=rdiv(ipnum,ipden);
    res=r2e(r,v);
    // detect integral shifted denominators
    vector<pf1> shiftfree_pfde_VECT;
    vector< pf<gen> >::iterator it=pfde_VECT.begin();
    vector< pf<gen> >::const_iterator itend=pfde_VECT.end();
    for (;it!=itend;++it){
      vecteur it_fact(polynome2poly1(it->fact));
      vector<pf1>::iterator jt=shiftfree_pfde_VECT.begin();
      vector<pf1>::const_iterator jtend=shiftfree_pfde_VECT.end();
      int k;
      for (;jt!=jtend;++jt){
	if (is_shift_of(it_fact,jt->fact,k))
	  break;
      }
      if (jt==jtend)
	shiftfree_pfde_VECT.push_back(pf1(it->num,it->den,it->fact,it->mult));
      else { // it_fact is the shift of jt->fact
	vecteur it_num(polynome2poly1(it->num)),it_den(polynome2poly1(it->den));
	// check which one has the highest multiplicity
	if (jt->mult<it->mult){ // we must swap to keep highest mult in *jt
	  swap(jt->num,it_num);
	  swap(jt->den,it_den);
	  swap(jt->fact,it_fact);
	  swap(jt->mult,it->mult);
	  k=-k;
	}
	// do the shift (this will modify jt->num and the result res)
	int decal;
	if (k<0){
	  decal=1;
	  k=-k;
	}
	else
	  decal=-1;
	for (int j=0;j<k;++j){
	  if (decal>0)
	    res=res-rdiv(r2e(poly12polynome(it_num),v),r2e(poly12polynome(it_den),v));
	  it_num=taylor(it_num,decal);
	  it_den=taylor(it_den,decal);
	  if (decal<0)
	    res=res+rdiv(r2e(poly12polynome(it_num),v),r2e(poly12polynome(it_den),v));
	}
	modpoly constante=jt->den/it_den;
	gen const_den;
	lcmdeno(constante,const_den);
	// should check if constante is a fraction
	jt->num=constante*it_num+const_den*jt->num;
	jt->den=const_den*jt->den;
      } // end else
    } // end for (;it!=itend;++it)
    // now add psi parts for every element of the shiftfree de_VECTosition
    vector<pf1>::iterator jt=shiftfree_pfde_VECT.begin();
    vector<pf1>::const_iterator jtend=shiftfree_pfde_VECT.end();
    for (;jt!=jtend;++jt){
      if (jt->fact.size()!=2){ // add to remains_to_sum
	remains_to_sum=remains_to_sum+rdiv(r2e(poly12polynome(jt->num),v),r2e(poly12polynome(jt->den),v));
      }
      else {
	gen racine=-rdiv(jt->fact.back(),jt->fact.front());
	vecteur dec=taylor(jt->num,racine);
	vecteur vv=cdr_VECT(v);
	gen coeff(plus_one);
	int decal=jt->mult-dec.size();
	for (int i=0;i<jt->mult;++i){
	  if (i>=decal)
	    res=res+r2e(rdiv(dec[i-decal],coeff),vv)*psi(x-racine,i,contextptr);
	  coeff=gen(-i-1)*coeff;
	}
      }
    } // end for(;jt!=jtend;++jt)
    return true; // end non constant denominator
  } // end rational fraction or polynomial

  polynome taylor(const polynome & P,const gen & g){
    vecteur v(polynome2poly1(P,1));
    v=taylor(v,g);
    return poly12polynome(v,1,P.dim);
  }

  // Write a fraction A/B as E[P]/P*Q/E[R] where E[P]=subst(P,x,x+1)
  // and Q and all positive shifts of R are prime together
  void AB2PQR(const polynome & A,const polynome & B,polynome & P,polynome & Q, polynome & R){
    // First find integer roots of resultant of A(x),B(x+t) with respect to x
    int s=A.dim;
    polynome a(A.untrunc1()); // add the t parameter
    polynome b(B.untrunc1());
    // exchange var 1 (parameter t) and 2 (x variable)
    index_t i=transposition(0,1,s+1);
    a.reorder(i);
    b.reorder(i);
    // now translate b by t
    vecteur bb(polynome2poly1(b,1));
    polynome t(monomial<gen>(plus_one,1,1,s));
    bb=taylor(bb,t);
    b=poly12polynome(bb,1,s+1);
    polynome pres=Tresultant<gen>(a,b);
    pres=pres.trunc1();
    // Make the list of the positive integer roots k in t of the resultant
    vecteur racines(iroots(pres));
    // The algorithm begins with P0=1 Q0=A R0=B
    P=polynome(monomial<gen>(plus_one,s));
    Q=A;
    R=B;
    int d=racines.size();
    for (int i=0;i<d;++i){
      gen k=racines[i];
      if (k.type!=_INT_ || k.val<=0)
	continue;
      // Then compute Pi Qi Ri so that E[Pi]/Pi*Qi/Ri=A/B
      // for each positive integer root k 
      // gcd[Qi,E^k[Ri]]=Y!=1 then Qi=Y*Q_{i+1}, Ri=E^-k[Y]*R_{i+1}
      // hence Qi/Ri=Q_{i+1}/R_{i+1}* Y/E^[-k]Y 
      polynome Y=gcd(Q,taylor(R,k));
      Q=Q/Y;
      polynome Yk=taylor(Y,-k);
      R=R/Yk;
      // therefore P_{i+1}=Pi*E^[-k]Y*...*E^[-1]Y 
      for (int j=-k.val;j<0;++j){
	P=P*Yk;
	Yk=taylor(Yk,plus_one);
      }
    }
    // At the end R=E^[-1] R_i
    R=taylor(R,minus_one);
  }

  // Solve P = Q E[Y] - R Y for Y
  // return true if there is a solution Y (solution is more precisely Y/deno)
  bool gosper(const polynome & P,const polynome & Q,const polynome & R,polynome & Y,gen & deno){
    // First find degree of Y
    // if q>r then y=p-q, if q<r then y=p-r, (if y<0 return false)
    // if q==r then y=p-q or p (p only if same leading coeff in Q,R)
    int p=P.lexsorted_degree(),q=Q.lexsorted_degree(),r=R.lexsorted_degree(),y;
    vecteur vP(polynome2poly1(P,1)),vQ(polynome2poly1(Q,1)),vR(polynome2poly1(R,1));
    gen qq=Tfirstcoeff<gen>(Q),rr=Tfirstcoeff<gen>(R);
    if (q==r && qq==rr){ // cancellation
      ++p;
      y=p-max(q,r);
      if (q>0){
	vecteur vq=polynome2poly1(Q),vr=polynome2poly1(R);
	gen ydeg=(vr[q-1]-vq[q-1])/qq;
	if (ydeg.type==_INT_ && ydeg.val>y){
	  y=ydeg.val;
	  p=y+q-1;
	}
      }
    }
    else
      y=p-max(q,r);
    if (y<0)
      return false;
    // Then solve a linear system with p+1 equations and y+1 unknowns
    // (p+1 rows, y+1 columns)
    // built the matrix of the system column by column
    // the column i is (X+1)^i*Q-X^i*R
    vecteur v(1,plus_one); // this will contain (X+1)^i using pascal_next_line
    vecteur w(v); // this is X^i
    matrice m;
    for (int i=0;i<=y;++i){
      vecteur current=v*vQ-w*vR;
      // adjust current size to p
      lrdm(current,p);
      m.push_back(current);
      v=pascal_next_line(v);
      w.push_back(zero);
    }
    reverse(m.begin(),m.end()); // higher coeff at the beginning
    // last column is P
    lrdm(vP,p);
    m.push_back(vP);
    m=mtran(m);
    m=mrref(m);
    vecteur res(y+1);
    for (int i=0;i<=y;++i){
      if (is_zero(m[i][i]))
	return false;
      res[i]=m[i][y+1]/m[i][i];
    }
    lcmdeno(res,deno);
    Y=poly12polynome(res,1,P.dim);
    return p==y || is_zero(m[y+1]);
    // Or alternatively do a Rothstein-Trager like method if Q non constant
    // Let P = Q U + R V with deg(V)<deg(Q)
    // then Q(E(Y)-U)=R(Y+V)
    // hence Q divides Y+V
    // If we know that deg(Y)<deg(Q) then check that Y=-V is solution
    // Otherwise let Y+V=Qy
    // Then QE(Qy-V) -R(Qy-V)=P=QU+RV
    // hence E(Qy-V)-Ry=U
    // we are reduced to solve E(Q)y-Ry=U+E(V) with deg(y)=deg(Y)-deg(Q)
  }

  gen inner_sum(const gen & e,const identificateur & x,gen & remains_to_sum,GIAC_CONTEXT){
    gen res;
    if (rational_sum(e,x,res,remains_to_sum,contextptr))
      return res;
    // Check for hypergeometric e
    gen ratio=simplify(subst(e,x,x+1)/e);
    vecteur v(lvarx(makevecteur(ratio,x),x));
    if ( (v.size()!=1) || (v.front()!=x) ){
      ratio=simplify(_texpand(ratio,contextptr));
      v=lvarx(makevecteur(ratio,x),x);
      if ( (v.size()!=1) || (v.front()!=x) ){
	remains_to_sum=e;
	return zero;
      }
    }
    lvar(ratio,v);
    int s=v.size();
    gen f=e2r(ratio,v);
    polynome A(s),B(s);
    if (f.type==_FRAC){
      A=gen2poly(f._FRACptr->num,s);
      B=gen2poly(f._FRACptr->den,s);
    }
    else {
      A=gen2poly(f,s);
      B=gen2poly(plus_one,s);
    }
    polynome P(s),Q(s),R(s),Y(s);
    AB2PQR(A,B,P,Q,R); // A/B as E[P]/P*Q/E[R]
    gen deno;
    if (!gosper(P,Q,R,Y,deno)){
      remains_to_sum=e;
      return zero;
    }
    remains_to_sum=zero;
    gen facteur=r2e(Y*R,v)/r2e(P,v)/r2e(deno,vecteur(v.begin()+1,v.end()));
    return simplify(e*facteur);
  }

  // discrete antiderivative
  gen sum(const gen & e,const gen & x,gen & remains_to_sum,GIAC_CONTEXT){
    if (x.type!=_IDNT)
      setsizeerr();
    return linear_apply(e,*x._IDNTptr,remains_to_sum,contextptr,inner_sum); 
  }
  
  // discrete antiderivative evaluated
  gen sum_loop(const gen & e,const gen & x,int i,int j,GIAC_CONTEXT){
    gen f(e),res;
    if (i>j){
      int tmp=j;
      j=i-1;
      i=tmp+1;
      f=-e;
    }
    for (;i<=j;++i){
      res=res+subst(f,x,i).eval(DEFAULT_EVAL_LEVEL,contextptr);
    }
    return res;
  }

  gen sum(const gen & e,const gen & x,const gen & a,const gen &b,GIAC_CONTEXT){
    if ( (a.type==_INT_) && (b.type==_INT_) && (absint(b.val-a.val)<100) )
      return sum_loop(e,x,a.val,b.val,contextptr);
    gen remains_to_sum;
    gen res=sum(e,x,remains_to_sum,contextptr);
    res=subst(res,x,b+1)-subst(res,x,a);
    if (is_zero(remains_to_sum))
      return res;
    if ( (a.type==_INT_) && (b.type==_INT_) && (absint(b.val-a.val)<100000) )
      return res+sum_loop(remains_to_sum,x,a.val,b.val,contextptr);
    return symbolic(at_sum,gen(makevecteur(e,x,a,b),_SEQ__VECT));
    setsizeerr();
  }
  
  gen prodsum(const gen & g,bool isprod){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur v=*g._VECTptr;
    int s=v.size();
    int debut=1,fin=s;
    if (v[0].type==_VECT && g.subtype==_SEQ__VECT && s>1 && v[1].type==_INT_){
      debut=max(1,v[1].val);
      if (s>2 && v[2].type==_INT_)
	fin=v[2].val;
      v=*v[0]._VECTptr;
      s=v.size();
      fin=min(s,fin);
    }
    gen res;
    if (isprod){
      res=plus_one;
      for (--debut;debut<fin;++debut){
	res=matrix_apply(res,v[debut],prod);
      }
    }
    else {
      for (--debut;debut<fin;++debut){
	res=matrix_apply(res,v[debut],somme);
      }
    }
    return res;
  }

  // type=0 for seq, 1 for prod, 2 for sum
  gen seqprod(const gen & g,int type,GIAC_CONTEXT){
    vecteur v(gen2vecteur(g));
    if (v.size()==1)
      v=gen2vecteur(eval(g,contextptr));
    if (v.size()<4){
      if (type==0)
	return _dollar(g);
      if (type==1)
	return prodsum(v,true);	
      if (type==2)
	return prodsum(v,false);
      toofewargs("");
    }
    // v[1]=eval(v[1]);
    v[2]=eval(v[2],contextptr);
    v[3]=eval(v[3],contextptr);
    int step=1;
    gen tmp;
    if (v.size()==5 && (tmp=eval(v[4],contextptr)).type==_INT_)
      step=tmp.val;
    if (v[3].type!=_INT_ || v[2].type!=_INT_){
      if (type==1)
	return prodsum(v,true);
      if (type==2)
	return prodsum(v,false);
      setsizeerr();
    }
    int debut=v[2].val,fin=v[3].val;
    vecteur res;
    for (;debut<=fin;debut+=step){
      tmp=quotesubst(v[0],v[1],debut);
      res.push_back(eval(tmp,contextptr));
    }
    if (type==1)
      return _prod(res);
    if (type==2)
      return _plus(res);
    return res;// return gen(res,_SEQ__VECT);
  }

  bool maple_sum_product_unquote(vecteur & v,GIAC_CONTEXT){
    bool res=false;
    int s=v.size();
    if (s<2)
      setsizeerr();
    if (v[0].is_symb_of_sommet(at_quote))
      v[0]=v[0]._SYMBptr->feuille;
    if (v[1].type!=_IDNT){
      if (v[1].is_symb_of_sommet(at_equal) && v[1]._SYMBptr->feuille.type==_VECT){
	res=true;
	vecteur tmp =*v[1]._SYMBptr->feuille._VECTptr;
	if (tmp.size()==2){
	  if (tmp[0].is_symb_of_sommet(at_quote))
	    tmp[0]=tmp[0]._SYMBptr->feuille;
	  v[1]=symbolic(at_equal,gen(makevecteur(tmp[0],eval(tmp[1],contextptr)),_SEQ__VECT));
	}
      }
      else
	v[1]=eval(v[1],contextptr);
    }
    for (int i=2;i<s;++i)
      v[i]=eval(v[i],contextptr);
    return res;
  }

  gen _sum(const gen & args,GIAC_CONTEXT) {
    if (args.type==_VECT && args.subtype!=_SEQ__VECT)
      return prodsum(args.eval(DEFAULT_EVAL_LEVEL,contextptr),false);
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      return prodsum(args.eval(DEFAULT_EVAL_LEVEL,contextptr),false);
    vecteur v(*args._VECTptr);
    maple_sum_product_unquote(v,contextptr);
    int s=v.size();
    adjust_int_sum_arg(v,s);
    if (v[1].type==_INT_){
      v[0]=eval(v[0],contextptr);
      return prodsum(v,false);
    }
    if (s==4) {
      if (is_positive(v[2]-v[3]-1,contextptr))
	return -_sum(gen(makevecteur(v[0],v[1],v[3]+1,v[2]-1),_SEQ__VECT),contextptr);
      if (is_strictly_positive(-v[2],contextptr) && is_positive(-v[3],contextptr)){
	gen tmp=quotesubst(v[0],v[1],-v[1]);
	return _sum(gen(makevecteur(tmp,v[1],-v[3],-v[2])),contextptr);
      }
      if (v[2].type==_INT_ && v[3].type==_INT_)
	return seqprod(v,2,contextptr);
    }
    const_iterateur it=v.begin(),itend=v.end();
    gen f=*it;
    ++it;
    gen x=*it;
    if (x.type==_IDNT){ 
      // quote x for evaluation of f
      if (contextptr && contextptr->quoted_global_vars){
	contextptr->quoted_global_vars->push_back(x);
	f=eval(f,contextptr);
	contextptr->quoted_global_vars->pop_back();
      }
      else {
	bool savequote=*it->_IDNTptr->quoted;
	*it->_IDNTptr->quoted=true;
	f=eval(f,contextptr);    
	*it->_IDNTptr->quoted=savequote;
      }
    }
    ++it;
    if (it==itend){
      gen rem,res;
      res=sum(f,x,rem,contextptr);
      if (is_zero(rem))
	return res;
      else
	return res+symbolic(at_sum,makevecteur(rem,x));
    }
    gen a=*it;
    ++it;
    if (it==itend)
      return prodsum(gen(v).eval(DEFAULT_EVAL_LEVEL,contextptr),false);
    gen b=*it;
    ++it;
    if ( (x.type!=_IDNT) )
      return prodsum(gen(v).eval(DEFAULT_EVAL_LEVEL,contextptr),false);
    return sum(f,x,a,b,contextptr);
  }
  const string _sum_s("sum");
  unary_function_eval __sum(&_sum,_sum_s);
  unary_function_ptr at_sum (&__sum,_QUOTE_ARGUMENTS);

  gen bernoulli(const gen & x){
    if ( (x.type!=_INT_)  || (x.val<0))// Should add bernoulli polynomials 
      setsizeerr();
    int n=x.val;
    if (!n)
      return plus_one;
    if (n==1)
      return minus_one_half;
    if (n%2)
      return zero;
    gen a(plus_one);
    gen b(rdiv(1-n,plus_two));
    for (int i =2; i< n-1; i+=2){
      a=iquo( (a*gen(n+3-i)*gen(n+2-i)),((i-1)*i));
      b=b+a* bernoulli(i);
    }
    return rdiv(-b,n+1);    
  }
  gen _bernoulli(const gen & args) {
    return apply(args,bernoulli);
  }
  const string _bernoulli_s("bernoulli");
  unary_function_unary __bernoulli(&_bernoulli,_bernoulli_s);
  unary_function_ptr at_bernoulli (&__bernoulli);

  vecteur double2vecteur(const double * y,int dim){
    vecteur ye;
    ye.reserve(dim);
    for (int i=0;i<dim;++i)
      ye.push_back(y[i]);
    return ye;
  }

#ifdef HAVE_LIBGSL
  struct odesolve_param {
    gen odesolve_t;
    vecteur odesolve_f,odesolve_ft,odesolve_y;
    matrice odesolve_fy;
    gsl_odeiv_system odesolve_system;
  };
  int gsl_odesolve_function(double t, const double y[], double dydt[], void * params){
    // _VECTute
    odesolve_param * par =(odesolve_param *) params;
    gen res=subst(par->odesolve_f,par->odesolve_t,t);
    vecteur vtmp=double2vecteur(y,par->odesolve_system.dimension);
    res=subst(res,par->odesolve_y,vtmp);
    res=res.evalf();
    // store result
    if ( (res.type!=_VECT) || (res._VECTptr->size()!=par->odesolve_system.dimension))
      setsizeerr();
    const_iterateur it=res._VECTptr->begin(),itend=res._VECTptr->end();
    for (double * dydt_it=dydt;it!=itend;++it,++dydt_it){
      if (it->type==_DOUBLE_)
	*dydt_it=it->_DOUBLE_val;
      else
	setsizeerr();
    }
    return 0;
  }
  int gsl_odesolve_jacobian (double t, const double y[], double * dfdy, double dfdt[], void * params){
    // _VECTute
    odesolve_param * par =(odesolve_param *) params;
    vecteur yv(double2vecteur(y,par->odesolve_system.dimension));
    gen res=subst(par->odesolve_ft,par->odesolve_t,t);
    res=subst(res,par->odesolve_y,yv).evalf();
    // store result
    if ( (res.type!=_VECT) || (res._VECTptr->size()!=par->odesolve_system.dimension))
      setsizeerr();
    const_iterateur it=res._VECTptr->begin(),itend=res._VECTptr->end();
    for (double * dfdt_it=dfdt;it!=itend;++it,++dfdt_it){
      if (it->type==_DOUBLE_)
	*dfdt_it=it->_DOUBLE_val;
      else
	setsizeerr();
    }
    res=subst(par->odesolve_fy,par->odesolve_t,t);
    res=subst(res,par->odesolve_y,yv).evalf();
    // store result
    if ( (res.type!=_VECT) || (res._VECTptr->size()!=par->odesolve_system.dimension))
      setsizeerr();
    it=res._VECTptr->begin();
    itend=res._VECTptr->end();
    for (double * dfdy_it=dfdy;it!=itend;++it){
      if (it->type!=_VECT)
	setsizeerr();
      const_iterateur jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
      for (;jt!=jtend;++jt,++dfdy_it){
	if (jt->type==_DOUBLE_)
	  *dfdy_it=it->_DOUBLE_val;
	else
	  setsizeerr();
      }
    }    
    return 0;
  }
  // solve dy/dt=f(t,y) with initial value y(t0)=y0 to final value t1
  // returns by default y[t1] or a vector of [t,y[t]]
  // if return_curve is true stop as soon as y is outside ymin,ymax
  // f is eitheir a prog (t,y) -> f(t,y) or a _VECT [f(t,y) t y]
  gen odesolve(const gen & t0orig,const gen & t1orig,const gen & f,const gen & y0orig,bool return_curve,double * ymin,double * ymax,GIAC_CONTEXT){
    gen t0_e=t0orig.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    gen t1_e=t1orig.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    if ( (t0_e.type!=_DOUBLE_)|| (t1_e.type!=_DOUBLE_))
      setsizeerr();
    double t0=t0_e._DOUBLE_val;
    double t1=t1_e._DOUBLE_val;
    bool time_reverse=(t1<t0);
    if (time_reverse){
      t0=-t0;
      t1=-t1;
    }
    double t=t0;
    gen y0=y0orig.evalf(DEFAULT_EVAL_LEVEL,contextptr);
    if (y0.type!=_VECT)
      y0=vecteur(1,y0);
    vecteur y0v=*y0._VECTptr;
    int dim=y0v.size();
    double tstep;
    if (dim==2)
      tstep=(gnuplot_xmax-gnuplot_xmin)/100;
    else {
      if (return_curve)
	tstep=fabs(t0)/100;
      else
	tstep=(t1-t0)/100;
    }
    double * y=new double[dim];
    for (int i=0;i<dim;i++){
      if (y0v[i].type!=_DOUBLE_)
	setsizeerr();
      y[i]=y0v[i]._DOUBLE_val;
    }
    identificateur t_id(" t");
    vecteur yv;
    gen tmp;
    if (f.type==_VECT){
      vecteur tmpv(*f._VECTptr);
      if (tmpv.size()!=3)
	setsizeerr();
      tmp=tmpv[0];
      if (tmpv[1].type!=_IDNT)
	setsizeerr();
      t_id=*tmpv[1]._IDNTptr;
      if (tmpv[2].type!=_VECT){
	yv=vecteur(1,tmpv[2]);
      }
      else
	yv=*tmpv[2]._VECTptr;
      if (yv.size()!=dim)
	setdimerr();
    }
    else {
      for (int i=0;i<dim;++i)
	yv.push_back(identificateur(" y"+print_INT_(i)));
      tmp=f(gen(makevecteur(t_id,yv),_SEQ__VECT),contextptr);
    }
    odesolve_param * par=new odesolve_param;
    par->odesolve_t=t_id;
    par->odesolve_y=yv;
    if (time_reverse)
      tmp=-subst(tmp,t_id,-t_id);
    if (tmp.type!=_VECT) 
      par->odesolve_f=vecteur(1,tmp);
    else
      par->odesolve_f=*tmp._VECTptr;
    if (par->odesolve_f.size()!=dim)
      setdimerr();
    par->odesolve_fy=*derive(par->odesolve_f,yv)._VECTptr;
    par->odesolve_ft=*derive(par->odesolve_f,t_id)._VECTptr;
    par->odesolve_system.function=gsl_odesolve_function;
    par->odesolve_system.dimension=dim;
    par->odesolve_system.jacobian=gsl_odesolve_jacobian;
    par->odesolve_system.params=par;
    // GSL call
    const gsl_odeiv_step_type * T = gsl_odeiv_step_rk8pd;    
    gsl_odeiv_step * s   = gsl_odeiv_step_alloc (T, dim);
    gsl_odeiv_control * c = gsl_odeiv_control_y_new (1e-7, 1e-7);
    gsl_odeiv_evolve * e = gsl_odeiv_evolve_alloc (dim);
    double h;
    if (return_curve){
      h=fabs(t);
      if (h<1e-4)
	h=1e-4;
    }
    else
      h=(t1-t)/1e4;
    double oldt=t0;
    vecteur resv;
    bool do_while=true;
    while (do_while && (t < t1) ) {
      if (h>tstep)
	h=tstep;
      int status = gsl_odeiv_evolve_apply (e, c, s,
					   &par->odesolve_system,
					   &t, t1, &h,
					   y);
      if (status != GSL_SUCCESS)
	setsizeerr("RK8 evolve not successfull");
      if (return_curve)  {
	if ( (t-oldt)> tstep){
	  oldt=t;
	  if (time_reverse)
	    resv.push_back(makevecteur(-t,double2vecteur(y,dim)));
	  else
	    resv.push_back(makevecteur(t,double2vecteur(y,dim)));
	}
	for (int i=0;i<dim;++i){
	  // cerr << y[i] << endl;
	  if ( (y[i]<ymin[i]) || (y[i]>ymax[i]))
	    do_while=false;
	}
      }
    }
    gsl_odeiv_evolve_free(e);
    gsl_odeiv_control_free(c);
    gsl_odeiv_step_free(s);
    delete par;
    if (return_curve)
      return resv;
    else
      return double2vecteur(y,dim);
  }
  // note that params is not used
#else // HAVE_LIBGSL
  gen odesolve(const gen & t0orig,const gen & torig,const gen & f,const gen & y0orig,bool return_curve,double * ymin,double * ymax,GIAC_CONTEXT){
    settypeerr("GSL not available");
  }
#endif
  // odesolve(t0..t1,f,y0) or odesolve(f(t,y),t,y,t0,y0,t1)
  gen _odesolve(const gen & args,GIAC_CONTEXT) {
    if ( (args.type!=_VECT) || (args._VECTptr->size()<3 ) )
      return symbolic(at_odesolve,args);
    vecteur v(*args._VECTptr);
    if (v.size()==6)
      return odesolve(v[3],v[5],makevecteur(v[0],v[1],v[2]),v[4],contextptr);
    if (v.size()==4){
      if ( v[1].type!=_VECT || v[1]._VECTptr->size()!=2 || v[2].type!=_VECT || v[2]._VECTptr->size()!=2)
	setsizeerr();
      return odesolve(v[2]._VECTptr->front(),v[2]._VECTptr->back(),makevecteur(v[0],v[1]._VECTptr->front(),v[1]._VECTptr->back()),v[3],contextptr);
    }
    gen t0t=v[0];
    if ( (t0t.type!=_SYMB) || (t0t._SYMBptr->sommet!=at_interval) )
      setsizeerr();
    gen t0=t0t._SYMBptr->feuille._VECTptr->front(); 
    gen t1=t0t._SYMBptr->feuille._VECTptr->back(); 
    gen f=v[1];
    gen y0=v[2];
    return odesolve(t0,t1,f,y0,contextptr);
  }
  const string _odesolve_s("odesolve");
  unary_function_eval __odesolve(&_odesolve,_odesolve_s);
  unary_function_ptr at_odesolve (&__odesolve);

  // args=[u'*v,u] or [[F,u'*v],u] -> [F+u*v,-u*v']
  // a third argument would be the integration var
  // if u=cste returns F+integrate(u'*v,x)
  gen _ibpdv(const gen & args,GIAC_CONTEXT) {
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      return symbolic(at_ibpdv,args);
    vecteur & w=*args._VECTptr;
    gen x(vx_var);
    if (w.size()>=3)
      x=w[2];
    gen u(w[1]),v,uprimev,F;
    if (w.front().type==_VECT){
      vecteur & ww=*w.front()._VECTptr;
      if (ww.size()!=2)
	setsizeerr();
      F=ww.front();
      uprimev=ww.back();
    }
    else 
      uprimev=w.front();
    gen uprime(derive(u,x));
    if (is_zero(uprime))
      return integrate(uprimev,x,contextptr)+F;
    v=normal(rdiv(uprimev,derive(u,x)));
    return makevecteur(F+u*v,normal(-u*derive(v,x)));
  }
  const string _ibpdv_s("ibpdv");
  unary_function_eval __ibpdv(&_ibpdv,_ibpdv_s);
  unary_function_ptr at_ibpdv (&__ibpdv);

  gen fourier_an(const gen & f,const gen & x,const gen & T,const gen & n,const gen & a,GIAC_CONTEXT){
    gen primi,iT=inv(T);
    gen omega=2*cst_pi*iT;
    primi=integrate(f*cos(omega*n*x,contextptr),x,contextptr);
    gen an;
    an=iT*(subst(primi,x,a+T)-subst(primi,x,a));
    if (n!=0) 
      {an=2*an;}
    return normal(an);
  }
  gen _fourier_an(const gen & args,GIAC_CONTEXT){

    if (args.type!=_VECT) setsizeerr();
    vecteur v(*args._VECTptr);
    if (v.size()==4) v.push_back(0);
    if (v.size()!=5) setsizeerr();
    gen f=v[0],x=v[1],T=v[2],n=v[3],a=v[4];
    return fourier_an(f,x,T,n,a,contextptr);
  }
  const string _fourier_an_s("fourier_an");
  unary_function_eval __fourier_an(&_fourier_an,_fourier_an_s);
  unary_function_ptr at_fourier_an (&__fourier_an);


   gen fourier_bn(const gen & f,const gen & x,const gen & T,const gen & n,const gen & a,GIAC_CONTEXT){
    gen primi,iT=inv(T);
    gen omega=2*cst_pi*iT;
    primi=integrate(f*sin(omega*n*x,contextptr),x,contextptr);
    gen bn;
    bn=2*iT*(subst(primi,x,a+T)-subst(primi,x,a));
return normal(bn);
  }

  gen _fourier_bn(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT) setsizeerr();
    vecteur v(*args._VECTptr);
    if (v.size()==4) v.push_back(0);
    if (v.size()!=5) setsizeerr();
    gen f=v[0],x=v[1],T=v[2],n=v[3],a=v[4];
    return fourier_bn(f,x,T,n,a,contextptr);
  } 
  const string _fourier_bn_s("fourier_bn");
  unary_function_eval __fourier_bn(&_fourier_bn,_fourier_bn_s);
  unary_function_ptr at_fourier_bn (&__fourier_bn);

   gen fourier_cn(const gen & f,const gen & x,const gen & T,const gen & n,const gen & a,GIAC_CONTEXT){
     gen primi,iT=inv(T);
     gen omega=2*cst_pi*iT;
     primi=integrate(f*exp(-cst_i*omega*n*x),x,contextptr);
     gen cn;
     cn=normal(iT*(subst(primi,x,a+T)-subst(primi,x,a)));
     return cn;
   }

  gen _fourier_cn(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT) setsizeerr();
    vecteur v(*args._VECTptr);
    if (v.size()==4) v.push_back(0);
    if (v.size()!=5) setsizeerr();
    gen f=v[0],x=v[1],T=v[2],n=v[3],a=v[4];
    return fourier_cn(f,x,T,n,a,contextptr);
  } 

  const string _fourier_cn_s("fourier_cn");
  unary_function_eval __fourier_cn(&_fourier_cn,_fourier_cn_s);
  unary_function_ptr at_fourier_cn (&__fourier_cn);

  

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

