// -*- mode:C++ ; compile-command: "g++ -I.. -g -c quater.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2001 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "quater.h"
#include "unary.h"
#include "sym2poly.h"
#include "usual.h"
#include "intg.h"
#include "subst.h"
#include "derive.h"
#include "lin.h"
#include "vecteur.h"
#include "gausspol.h"
#include "plot.h"
#include "prog.h"
#include "modpoly.h"
#include "series.h"
#include "tex.h"
#include "ifactor.h"
#include "risch.h"
#include "solve.h"
using namespace std;

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  const string _quaternion_s("quaternion");
  gen _quaternion(const gen & args){
    if (args.type!=_VECT)
      return quaternion(args);
    vecteur v(*args._VECTptr);
    if (v.size()==1)
      return quaternion(v.front());
    if (v.size()!=4)
      setsizeerr("Quaternion has 1 or 4 arguments");
    return quaternion(v[0],v[1],v[2],v[3]);
  }
  unary_function_unary __quaternion(&giac::_quaternion,_quaternion_s);
  unary_function_ptr at_quaternion (&__quaternion,0,true); // auto-register
  
  string quaternion::print() const {
    return _quaternion_s+"("+r.print()+","+i.print()+","+j.print()+","+k.print()+")";
    this->dbgprint();
  }

  quaternion::quaternion(const gen & g){
    if (g.type==_USER){
      const quaternion  * q =dynamic_cast<const quaternion *>(g._USERptr);
      if (q)
	q->copy_to(this);
    }
    else {
      r=g;
      i=zero;
      j=zero;
      k=zero;
    }
  };

  const string _galois_field_s("GF");
  // Is the polynomial v irreducible and primitive modulo p?
  bool is_irreducible_primitive(const modpoly & v,const gen & p){
    int m=v.size()-1;
    if (m<2)
      setsizeerr("irreducibility: degree too short"+gen(v).print());
    gen gpm=pow(p,m);
    if (gpm.type!=_INT_)
      setsizeerr("Characteristic or degree too large p^m="+gpm.print());
    int pm=gpm.val;
    environment * env=new environment;
    env->modulo=p;
    env->moduloon=true;
    // Irreducible: v must be prime with x^(p^k)-x, for k<=m/2
    for (int k=1;k<=m/2;k++){
      gen pk(pow(p,k));
      vecteur test(pk.val+1),g;
      test[0]=1;
      test[pk.val-1]=-1;
      gcdmodpoly(test,v,env,g);
      if (!is_one(g)){
	delete env;
	return false;
      }
    }
    // Primi: must not divide x^[(p^m-1)/d]-1 for any prime divisor d of p^m-1
    gen tmp=pm-1;
    vecteur vp(pfacprem(tmp));
    int ntest=vp.size();
    for (int i=0;i<ntest;i+=2){
      int pm_d=(pm-1)/vp[i].val;
      vecteur test(pm_d+1);
      test[0]=1;
      test[pm_d]=-1;
      if (is_zero(operator_mod(test,v,env))){
	delete env;
	return false;
      }
    }
    delete env;
    return true;
  }

  vecteur find_irreducible_primitive(int p,int m){
    // Now test all possible coeffs for test[k] until it's irreducible
    int pm=int(std::pow(double(p),double(m)));
    for (int k=0;k<pm;k++){
      vecteur test(m+1);
      test[0]=1;
      // decompose k in base p
      for (int j=1,k1=k;j<=m;++j){
	test[j]=k1%p;
	k1/=p;
      }
      if (is_irreducible_primitive(test,p))
	return test;
    }
    setsizeerr("No irreducible primitive polynomial found");
  }
  gen _galois_field(const gen & args){
    if (is_integer(args)){ // must be a power of a prime
      gen pm=abs(args);
      vecteur u(pfacprem(pm));
      if (u.size()!=2)
	setsizeerr("Not a power of a prime");
      return galois_field(u);
    }
    if (args.type!=_VECT)
      return galois_field(args);
    vecteur v(*args._VECTptr);
    int s=v.size();
    if (s==2)
      return galois_field(args);
    if (s==3){
      v.push_back(undef);
      ++s;
    }
    if (s!=4)
      setsizeerr("galois_field has 1 or 4 arguments (charac p, irred poly P, var name x, value as a poly of x or as a vector)");
    vecteur a,P;
    gen & x= v[2];
    if (!is_undef(v[3]) && v[3].type!=_VECT)
      v[3]=_e2r(makevecteur(v[3],x));
    if (v[1].type!=_VECT)
      v[1]=_e2r(makevecteur(v[1],x));
    if (v[1].type!=_VECT)
      setsizeerr();
    if (!is_irreducible_primitive(*v[1]._VECTptr,v[0]))
      setsizeerr("Not irreducible or not primitive polynomial"+args.print());
    return galois_field(v[0],gen(*v[1]._VECTptr,_POLY1__VECT),v[2],v[3]);
  }
  unary_function_unary __galois_field(&giac::_galois_field,_galois_field_s);
  unary_function_ptr at_galois_field (&__galois_field,0,true); // auto-register
  
  string galois_field::print() const {
    return _galois_field_s+"("+p.print()+","+r2e(P,x).print()+","+x.print()+","+r2e(a,x).print()+")";
    this->dbgprint(); // not reached, it's for the debugger
  }

  galois_field::galois_field(const gen & g){
    if (g.type==_USER){
      const galois_field  * q =dynamic_cast<const galois_field *>(g._USERptr);
      if (q)
	q->copy_to(this);
      else
	setsizeerr();
    }
    else {
      if (g.type!=_VECT || g._VECTptr->size()!=2 || g._VECTptr->front().type!=_INT_ || g._VECTptr->back().type!=_INT_)
	setsizeerr("Expecting characteristic p, integer m");
      int p0=max(absint(g._VECTptr->front().val),2);
      int m0=max(absint(g._VECTptr->back().val),2);
      p=p0;
      P=find_irreducible_primitive(p0,m0);
      x=vx_var;
      a=undef;
    }
  };

  galois_field::galois_field(const galois_field & q):p(q.p),P(q.P),x(q.x),a(q.a) { 
    if (!is_undef(a)){
      a = smod(a,p);
      if (a.type!=_VECT)
	a=gen(vecteur(1,a),_POLY1__VECT);
    }
  }

  gen galois_field::operator + (const gen & g) const { 
    if (is_integer(g))
      return galois_field(p,P,x,a+g);
    if (g.type!=_USER)
      return sym_add(*this,g); // symbolic(at_plus,makevecteur(g,*this));
    if (galois_field * gptr=dynamic_cast<galois_field *>(g._USERptr)){
      if (gptr->p!=p || gptr->P!=P)
	setsizeerr();
      if (a.type==_VECT && gptr->a.type==_VECT){
	vecteur res;
	environment * env=new environment;
	env->modulo=p;
	env->moduloon=true;
	addmodpoly(*a._VECTptr,*gptr->a._VECTptr,env,res);
	delete env;
	return galois_field(p,P,x,res);
      }
      return galois_field(p,P,x,a+gptr->a);
    }
    else
      setsizeerr();
  }

  gen galois_field::operator - (const gen & g) const { 
    if (is_integer(g))
      return galois_field(p,P,x,a-g);
    if (g.type!=_USER)
      return sym_add(*this,-g); // symbolic(at_plus,makevecteur(-g,*this));
    if (galois_field * gptr=dynamic_cast<galois_field *>(g._USERptr)){
      if (gptr->p!=p || gptr->P!=P)
	setsizeerr();
      if (a.type==_VECT && gptr->a.type==_VECT){
	vecteur res;
	environment * env=new environment;
	env->modulo=p;
	env->moduloon=true;
	submodpoly(*a._VECTptr,*gptr->a._VECTptr,env,res);
	delete env;
	return galois_field(p,P,x,res);
      }
      return galois_field(p,P,x,a-gptr->a);
    }
    else
      setsizeerr();
  }

  gen galois_field::operator - () const { 
    return galois_field(p,P,x,-a);
  }

  gen galois_field::operator * (const gen & g) const { 
    if (is_integer(g))
      return galois_field(p,P,x,g*a);
    if (g.type!=_USER)
      return sym_mult(*this,g); // symbolic(at_prod,makevecteur(g,*this));
    if (galois_field * gptr=dynamic_cast<galois_field *>(g._USERptr)){
      if (gptr->p!=p || gptr->P!=P || P.type!=_VECT)
	setsizeerr();
      if (a.type==_VECT && gptr->a.type==_VECT){
	vecteur res;
	environment * env=new environment;
	env->modulo=p;
	env->moduloon=true;
	mulmodpoly(*a._VECTptr,*gptr->a._VECTptr,env,res);
	res=operator_mod(res,*P._VECTptr,env),
	delete env;
	return galois_field(p,P,x,res);
      }
      return galois_field(p,P,x,a*gptr->a);
    }
    else
      setsizeerr();
  }

  gen galois_field::inv () const {
    if (a.type!=_VECT || P.type!=_VECT)
      setsizeerr();
    vecteur & A = *a._VECTptr;
    if (A.empty())
      return galois_field(p,P,x,undef);
    modpoly u,v,d;
    environment * env=new environment;
    env->modulo=p;
    env->moduloon=true;
    egcd(A,*P._VECTptr,env,u,v,d);
    delete env;
    // d should be [1]
    if (d!=vecteur(1,1))
      setsizeerr("GF inv internal bug");
    return galois_field(p,P,x,u);
  }

  bool galois_field::operator == (const gen & g) const {
    if (g.type!=_USER)
      return a==vecteur(1,g);
    if (galois_field * gptr=dynamic_cast<galois_field *>(g._USERptr)){
      if (gptr->p!=p || gptr->P!=P)
	return false;
      return gptr->a==a;
    }
    return false;
  }

  bool galois_field::is_zero () const {
    return a.type==_VECT && ( a._VECTptr->empty() || (a._VECTptr->size()==1 && a._VECTptr->front()==0) );
  }

  bool galois_field::is_one () const {
    return a.type==_VECT && a._VECTptr->size()==1 && a._VECTptr->front()==1;
  }

  bool galois_field::is_minus_one () const {
    return a.type==_VECT && a._VECTptr->size()==1 && smod(a._VECTptr->front(),p)==-1;
  }

  gen galois_field::operator () (const gen & g,GIAC_CONTEXT) const {
    if (is_undef(a)){
      if (g.type==_VECT)
	return galois_field(p,P,x,g);
      else
	return galois_field(p,P,x,_e2r(makevecteur(g,x)));
    }
    return *this;
  }

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
