// -*- mode:C++ ; compile-command: "g++ -I.. -g -c solve.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2001 B. Parisse, R. De Graeve
 *  Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include "gen.h"
#include "solve.h"
#include "modpoly.h"
#include "unary.h"
#include "symbolic.h"
#include "usual.h"
#include "sym2poly.h"
#include "subst.h"
#include "derive.h"
#include "plot.h"
#include "prog.h"
#include "series.h"
#include "alg_ext.h"
#include "intg.h"
#include "rpn.h"
#include "lin.h"
#include "misc.h"
#ifdef HAVE_LIBGSL
#include <gsl/gsl_roots.h>
#include <gsl/gsl_multiroots.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_vector.h>
#endif

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  int NEWTON_DEFAULT_ITERATION=20;

  int intvar_counter=0;
  int realvar_counter=0;

  void set_merge(vecteur & v,const vecteur & w){
    const_iterateur it=w.begin(),itend=w.end();
    for (;it!=itend;++it)
      if (!equalposcomp(v,*it))
	v.push_back(*it);
  }

  gen one_tour(GIAC_CONTEXT){
    if (angle_radian(contextptr)) 
      return cst_two_pi;
    else
      return 360;
  }
  gen one_half_tour(GIAC_CONTEXT){
    if (angle_radian(contextptr)) 
      return cst_pi;
    else
      return 180;
  }
  gen isolate_exp(const gen & e,int isolate_mode,GIAC_CONTEXT){
    if (isolate_mode &1)
      return ln(e);
    if (e.type!=_VECT){
      if (is_strictly_positive(-e,contextptr))
	return vecteur(0);
      else
	return ln(e);
    }
    // check in real mode for negative ln
    const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
    vecteur res;
    for (;it!=itend;++it){
      if (!is_strictly_positive(-*it,contextptr))
	res.push_back(ln(*it));
    }
    return res;
  }
  gen isolate_ln(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return simplify(exp(e));
  }
  gen isolate_sin(const gen & e,int isolate_mode,GIAC_CONTEXT){
    gen asine=asin(e,contextptr);
    if (!(isolate_mode & 2))
      return makevecteur(asine,one_half_tour(contextptr)-asine);
    identificateur * x=new identificateur(string("n_")+print_INT_(intvar_counter));
    ++intvar_counter;
    return makevecteur(asine+(*x)*one_tour(contextptr),one_half_tour(contextptr)-asine+(*x)*one_tour(contextptr));
  }
  gen isolate_cos(const gen & e,int isolate_mode,GIAC_CONTEXT){
    gen acose=acos(e,contextptr);
    if (!(isolate_mode & 2))
      return makevecteur(acose,-acose);
    identificateur * x=new identificateur(string("n_")+print_INT_(intvar_counter));
    ++intvar_counter;
    return makevecteur(acose+(*x)*one_tour(contextptr),-acose+(*x)*one_tour(contextptr));
  }
  gen isolate_tan(const gen & e,int isolate_mode,GIAC_CONTEXT){
    if (!(isolate_mode & 2))
      return atan(e,contextptr);
    identificateur * x=new identificateur(string("n_")+print_INT_(intvar_counter));
    ++intvar_counter;
    return atan(e,contextptr)+(*x)*one_half_tour(contextptr);
  }
  gen isolate_asin(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return sin(e,contextptr);
  }
  gen isolate_acos(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return cos(e,contextptr);
  }
  gen isolate_atan(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return tan(e,contextptr);
  }

  gen isolate_asinh(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return sinh(e);
  }
  gen isolate_acosh(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return cosh(e);
  }
  gen isolate_atanh(const gen & e,int isolate_mode,GIAC_CONTEXT){
    return tanh(e);
  }

  gen isolate_sinh(const gen & e,int isolate_mode,GIAC_CONTEXT){
    gen asine= asinh(e);
    if (!(isolate_mode & 2))
      return asine;
    identificateur * x=new identificateur(string("n_")+print_INT_(intvar_counter));
    ++intvar_counter;
    return makevecteur(asine+(*x)*one_tour(contextptr)*cst_i,(one_half_tour(contextptr)+(*x)*one_tour(contextptr))*cst_i-asine);
  }
  gen isolate_cosh(const gen & e,int isolate_mode,GIAC_CONTEXT){
    gen acose=acosh(e);
    if (!(isolate_mode & 2))
      return makevecteur(acose,-acose);
    identificateur * x=new identificateur(string("n_")+print_INT_(intvar_counter));
    ++intvar_counter;
    return makevecteur(acose+(*x)*one_tour(contextptr)*cst_i,-acose+(*x)*one_tour(contextptr)*cst_i);
  }
  gen isolate_tanh(const gen & e,int isolate_mode,GIAC_CONTEXT){
    if (!(isolate_mode & 2))
      return atanh(e);
    identificateur * x=new identificateur(string("n_")+print_INT_(intvar_counter));
    ++intvar_counter;
    return atanh(e)+(*x)*one_half_tour(contextptr)*cst_i;
  }

  gen (* isolate_fcns[] ) (const gen &,int,GIAC_CONTEXT) = { isolate_exp,isolate_ln,isolate_sin,isolate_cos,isolate_tan,isolate_asin,isolate_acos,isolate_atan,isolate_sinh,isolate_cosh,isolate_tanh,isolate_asinh,isolate_acosh,isolate_atanh};

  vecteur find_excluded(const gen & g,GIAC_CONTEXT){
    if (g.type!=_IDNT)
      return vecteur(0);
    gen g2=g._IDNTptr->eval(DEFAULT_EVAL_LEVEL,contextptr);
    if ((g2.type==_VECT) && (g2.subtype==_ASSUME__VECT)){
      vecteur v=*g2._VECTptr;
      if ( (v.size()==3) && (v.front()==_DOUBLE_) && (v[2].type==_VECT)){
	return *v[2]._VECTptr;;
      }
    }
    return vecteur(0);
  }

  vecteur find_singularities(const gen & e,const identificateur & x,GIAC_CONTEXT){
    vecteur lv(lvarxpow(e,x));
    vecteur res;
    vecteur l(lvar(e));
    gen p=e2r(e,l),n,d;
    vecteur pv=gen2vecteur(p);
    const_iterateur jt=pv.begin(),jtend=pv.end();
    for (;jt!=jtend;++jt){
      fxnd(*jt,n,d);
      if (d.type==_POLY)
	res=solve(r2e(d,l),x,0,contextptr);
    }
    const_iterateur it=lv.begin(),itend=lv.end();
    for (;it!=itend;++it){
      if (it->type!=_SYMB)
	continue;
      unary_function_ptr & u=it->_SYMBptr->sommet;
      gen & f=it->_SYMBptr->feuille;
      res=mergevecteur(res,find_singularities(f,x,contextptr));
      if (u==at_ln)
	res=mergevecteur(res,solve(f,x,0,contextptr));
      if (u==at_tan)
	res=mergevecteur(res,solve(cos(f,contextptr),x,0,contextptr));
    }
    return *_sort(res,contextptr)._VECTptr;
  }

  void solve_ckrange(const identificateur & x,vecteur & v,GIAC_CONTEXT){
    vecteur w,excluded(find_excluded(x,contextptr));
    find_range(x,w,contextptr);
    if (w.size()!=1 || w.front().type!=_VECT)
      return;
    w=*w.front()._VECTptr;
    if (w.size()!=2)
      return;
    gen l(w.front()),m(w.back());
    vecteur newv;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (equalposcomp(excluded,*it))
	continue;
      gen sol(evalf(*it,DEFAULT_EVAL_LEVEL,contextptr));
      if (sol.type==_DOUBLE_ && !complex_mode(contextptr) && !is_zero(im(sol,contextptr)))
	continue;
      if (sol.type!=_DOUBLE_){ // check for trig solutions
	newv.push_back(*it);
	vecteur lv(lvar(sol));
	if (lv.size()!=1 || l==minus_inf || m==plus_inf)
	  continue;
	gen n(lv.front()),a,b;
	// check linearity
	if (n.type!=_IDNT || n.print().substr(0,2)!="n_" || !is_linear_wrt(*it,*n._IDNTptr,a,b))
	  continue;
	newv.pop_back();
	a=normal(a);
	b=normal(b);
	if (!is_positive(a,contextptr))
	  std::swap<gen>(l,m);
	int n0(_ceil(evalf((l-b)/a,DEFAULT_EVAL_LEVEL,contextptr),contextptr).val),n1(_floor(evalf((m-b)/a,DEFAULT_EVAL_LEVEL,contextptr),contextptr).val);
	for (;n0<=n1;++n0)
	  newv.push_back(n0*a+b);
      }
      else {
	if (is_strictly_greater(l,sol,contextptr))
	  continue;
	if (is_strictly_greater(sol,m,contextptr))
	  continue;
	newv.push_back(*it);
      }
    }
    v=newv;
  }

  // Helper for the solver, make a translation using x^(n-1) coeff
  // and find gcd of deg, return true if non-trivial gcd found
  bool translate_gcddeg(const vecteur & v,vecteur & v_translated, gen & x_translation,int & gcddeg){
    int s=v.size();
    if (s<4)
      return false;
    x_translation=-v[1]/((s-1)*v[0]);
    v_translated=taylor(v,x_translation,0);
    gcddeg=0;
    for (int i=1;i<s;++i){
      if (!is_zero(v_translated[i]))
	gcddeg=gcd(gcddeg,i);
    }
    if (gcddeg<=1)
      return false;
    int newdeg=(s-1)/gcddeg+1;
    // compress v_translated, keep only terms with index multiple of gcddeg
    for (int i=1;i<newdeg;++i){
      v_translated[i]=v_translated[i*gcddeg];
    }
    v_translated=vecteur(v_translated.begin(),v_translated.begin()+newdeg);
    return true;
  }

  // inner solver
  void in_solve(const gen & e,const identificateur & x,vecteur &v,int isolate_mode,GIAC_CONTEXT){
    bool complexmode=isolate_mode & 1;
    vecteur lv(lvarx(e,x));
    int s=lv.size();
    if (!s)
      return;
    if (s>1)
      throw(std::runtime_error("Unable to isolate "+x.print()+" in "+e.print()));
    gen xvar(lv.front());
    if (xvar!=x){ // xvar must be a unary function of x
      if (xvar.type!=_SYMB)
	settypeerr();
      if (xvar._SYMBptr->feuille.type==_VECT)
	throw(std::runtime_error("Unable to isolate "+x.print()+" in "+xvar.print()));
      int pos=equalposcomp(solve_fcns_v,xvar._SYMBptr->sommet);
      if (!pos)
	throw(std::runtime_error("Unable to isolate function "+xvar._SYMBptr->sommet.ptr->print()));
      // solve with respect to xvar
      identificateur localt(" t");
      // ck_parameter_t();
      gen new_e=subst(e,xvar,localt);
      vecteur new_v=solve(new_e,localt,isolate_mode,contextptr);
      const_iterateur it=new_v.begin(),itend=new_v.end();
      for (;it!=itend;++it){
	gen res=isolate_fcns[pos-1](*it,isolate_mode,contextptr);
	if (res.type!=_VECT)
	  set_merge(v,solve(xvar._SYMBptr->feuille-res,x,isolate_mode,contextptr));
	else {
	  const_iterateur it=res._VECTptr->begin(),itend=res._VECTptr->end();
	  for (;it!=itend;++it)
	    set_merge(v,solve(xvar._SYMBptr->feuille-*it,x,isolate_mode,contextptr));
	}
      }
      solve_ckrange(x,v,contextptr);
      return;
    } // end xvar!=x
    // rewrite e as a univariate polynomial, first add other vars to x
    vecteur newv;
    lvar(e,lv);
    int lv_size=lv.size();
    gen num,den,f;
    f=e2r(e,lv);
    fxnd(f,num,den);
    if (num.type!=_POLY)
      return;
    vecteur w=polynome2poly1(*num._POLYptr,1);
    lv.erase(lv.begin()); // remove x from lv (CDR_VECT)
    int deg;
    vecteur w_translated;
    gen delta_x;
    if (translate_gcddeg(w,w_translated,delta_x,deg)){
      // composite polynomials
      gen invdeg=inv(deg);
      gen newe=symb_horner(*r2sym(w_translated,lv)._VECTptr,x);
      delta_x=-r2sym(delta_x,lv);
      vecteur vtmp;
      in_solve(newe,x,vtmp,isolate_mode,contextptr);
      vecteur unitroot(1,plus_one);
      if (complexmode){
	for (int k=1;k<deg;++k)
	  unitroot.push_back(exp(2*k*cst_pi/deg*cst_i));
      }
      const_iterateur it=vtmp.begin(),itend=vtmp.end();
      for (;it!=itend;++it){
	gen tmp=pow(*it,invdeg);
	if (complexmode){
	  const_iterateur jt=unitroot.begin(),jtend=unitroot.end();
	  for (;jt!=jtend;++jt)
	    newv.push_back(delta_x + (*jt) * tmp);
	}
	else {
	  if (deg%2)
	    newv.push_back(delta_x + tmp);
	  else {
	    if (!is_strictly_positive(-*it,contextptr)){
	      newv.push_back(delta_x + tmp);
	      newv.push_back(delta_x - tmp);
	    }
	  }
	}
      }
      solve_ckrange(x,newv,contextptr);
      v=mergevecteur(v,newv);
      return;
    }
    // if degree(w)=0, 1 or 2 solve it, otherwise error (should return ext)
    int d=w.size()-1;
    if (!d)
      return;
    if (d==1){
      gen tmp=rdiv(-r2sym(w.back(),lv),r2sym(w.front(),lv));
      if (!complexmode && !is_real(tmp,contextptr))
	return;
      newv.push_back(tmp);
      solve_ckrange(x,newv,contextptr);
      v=mergevecteur(v,newv);
      return;
    }
    if (d>2){
      if (has_num_coeff(w)){
	if (complexmode)
	  newv=proot(w,epsilon(contextptr));
	else
	  newv=real_proot(w,epsilon(contextptr),contextptr);
	solve_ckrange(x,newv,contextptr);
	v=mergevecteur(v,newv);
	return;
      }
      int n=is_cyclotomic(w,epsilon(contextptr));
      if (!n){
	cerr << "Warning! Algebraic extension not implemented yet for poly " << r2sym(w,lv) << endl;
	if (has_num_coeff(evalf(w))){
	  if (complexmode)
	    newv=proot(w,epsilon(contextptr));
	  else
	    newv=real_proot(w,epsilon(contextptr),contextptr);
	  solve_ckrange(x,newv,contextptr);
	  v=mergevecteur(v,newv);
	  return;
	}
	return;
      }
      if (complexmode){
	for (int j=1;j<=n/2;++j){
	  if (gcd(j,n)==1){
	    if (n%2){
	      newv.push_back(exp(rdiv(gen(2*j)*cst_pi*cst_i,n)));
	      newv.push_back(exp(rdiv(gen(-2*j)*cst_pi*cst_i,n)));
	    }
	    else {
	      newv.push_back(exp(rdiv(gen(j)*cst_pi*cst_i,n/2)));
	      newv.push_back(exp(rdiv(gen(-j)*cst_pi*cst_i,n/2)));
	    }
	  }
	}
      }
      solve_ckrange(x,newv,contextptr);
      v=mergevecteur(v,newv);
      return ;
    }
    gen b_over_2=rdiv(w[1],plus_two);
    if (b_over_2.type!=_FRAC){
      gen a=r2sym(w.front(),lv);
      gen minus_b_over_2=r2sym(-b_over_2,lv);
      gen delta_prime=pow(minus_b_over_2,2)-r2sym(w.front()*w.back(),lv);
      if (!complexmode && is_positive(-delta_prime,contextptr))
	return;
      newv.push_back(rdiv(minus_b_over_2+sqrt(delta_prime),a));
      newv.push_back(rdiv(minus_b_over_2-sqrt(delta_prime),a));
    }
    else {
      gen two_a=r2sym(plus_two*w.front(),lv);
      gen minus_b=r2sym(-w[1],lv);
      gen delta=r2sym(w[1]*w[1]-gen(4)*w.front()*w.back(),lv);
      if (!complexmode && is_positive(-delta,contextptr))
	return;
      newv.push_back(rdiv(minus_b+sqrt(delta),two_a));
      newv.push_back(rdiv(minus_b-sqrt(delta),two_a));
    }
    solve_ckrange(x,newv,contextptr);
    v=mergevecteur(v,newv);
  }

  // v assumed to represent an irreducible dense 1-d poly
  vecteur solve(const vecteur & v,bool complexmode,GIAC_CONTEXT){
    vecteur res;
    int d=v.size()-1;
    if (d<1)
      return res;
    if (d==1){
      res.push_back(rdiv(-v.back(),v.front()));
      return res;
    }
    if (!is_one(v.front())){
      // if v is not monic, set Y=a*X
      gen a(v.front()),puissance(plus_one);
      vecteur w;
      w.reserve(d+1);
      for (int i=0;i<=d;++i,puissance=puissance*a)
	w.push_back(v[i]*puissance);
      return divvecteur(solve(divvecteur(w,a),complex_mode(contextptr),contextptr),a);
    }
    // should call sym2rxroot for extensions of extensions
    vecteur tmp(2,zero);
    tmp.front()=plus_one;
    if (d==2){
      gen b(v[1]),c(v[2]);
      gen bprime(rdiv(b,plus_two));
      if (!has_denominator(bprime)){
	gen delta(bprime*bprime-c);
	if (!complexmode && is_positive(-delta,contextptr))
	  return res;
	vecteur w(3,zero);
	w.front()=plus_one;
	w.back()=-delta;
	tmp.back()=-bprime;
	res.push_back(algebraic_EXTension(tmp,w));
	tmp.front()=minus_one;
	tmp.back()=-bprime;
	res.push_back(algebraic_EXTension(tmp,w));	
      }
      else {
	if (!complexmode && is_positive(4*c-b*b,contextptr))
	  return res;
	tmp.back()=zero;
	res.push_back(algebraic_EXTension(tmp,v));
	tmp.front()=minus_one;
	tmp.back()=-b;
	res.push_back(algebraic_EXTension(tmp,v));
      }
      return res;
    }
    // should return a list of d algebraic extension with order number
    res.push_back(algebraic_EXTension(tmp,v));
    return res;
  }

  // works for continuous functions only
  vecteur solve_inequation(const gen & e0,const identificateur & x,int direction,GIAC_CONTEXT){
    gen e(e0._SYMBptr->feuille._VECTptr->front()-e0._SYMBptr->feuille._VECTptr->back());
    vecteur veq,singu;
    singu=find_singularities(e,x,contextptr);
    veq=mergevecteur(solve(e,x,0,contextptr),singu);
    vecteur range,excluded(find_excluded(x,contextptr));
    excluded=mergevecteur(excluded,singu);
    find_range(x,range,contextptr); 
    // From the lower bound of range to the higher bound
    // find the sign 
    if (range.size()!=1 || range.front().type!=_VECT)
      return vecteur(0);
    range=mergevecteur(*range.front()._VECTptr,excluded);
    range=mergevecteur(range,veq);
    range=*_sort(range,contextptr)._VECTptr;
    vecteur res;
    int s=range.size();
    if (s<2)
      setsizeerr();
    if (s==2 && range[0]==minus_inf && range[1]==plus_inf)
      return vecteur(1,x);
    for (int i=0;i<s-1;++i){
      gen l=range[i],m=range[i+1];
      if (l==m)
	continue;
      gen test;
      if (l==minus_inf)
	test=m-1;
      else {
	if (m==plus_inf)
	  test=l+1;
	else
	  test=(l+m)/2;
      }
      test=eval(subst(e0,x,test),contextptr);
      if (test!=1)
	continue;
      gen symb_sup,symb_inf;
      if (equalposcomp(excluded,l) ||
	  ( !(direction %2) && equalposcomp(veq,l)) )
	symb_inf=symb_superieur_strict(x,l);
      else
	symb_inf=symb_superieur_egal(x,l);
      if (equalposcomp(excluded,m) ||
	  ( !(direction %2) && equalposcomp(veq,m)) )
	symb_sup=symb_inferieur_strict(x,m);
      else
	symb_sup=symb_inferieur_egal(x,m);
      if (l==minus_inf)
	res.push_back(symb_sup);
      else {
	if (m==plus_inf)
	  res.push_back(symb_inf);
	else
	  res.push_back(symbolic(at_and,makevecteur(symb_inf,symb_sup))); 
      }
    }
    return res;
  }
  
  // detect product and powers
  void solve(const gen & e,const identificateur & x,vecteur &v,int isolate_mode,GIAC_CONTEXT){
    if (is_zero(e)){
      v.push_back(x);
      return;
    }
    switch (e.type){
    case _IDNT:
      if (*e._IDNTptr==x && !equalposcomp(find_excluded(x,contextptr),zero))
	addtolvar(zero,v);
      return;
    case _SYMB:
      if ( e._SYMBptr->sommet==at_pow && ck_is_strictly_positive(e._SYMBptr->feuille._VECTptr->back(),contextptr) ){
	vecteur tmpv;
	solve(e._SYMBptr->feuille._VECTptr->front(),x,tmpv,isolate_mode,contextptr);
	int ncopy=1;
	// make copies of the answer (maple_mode==1 compatibility)
	if (maple_mode==1 && e._SYMBptr->feuille._VECTptr->back().type==_INT_)
	  ncopy=e._SYMBptr->feuille._VECTptr->back().val;
	const_iterateur it=tmpv.begin(),itend=tmpv.end();
	for (;it!=itend;++it){
	  for (int i=0;i<ncopy;++i)
	    v.push_back(*it);
	}
	return;
      }
      if (e._SYMBptr->sommet==at_prod){
	const_iterateur it=e._SYMBptr->feuille._VECTptr->begin(),itend=e._SYMBptr->feuille._VECTptr->end();
	for (;it!=itend;++it)
	  solve(*it,x,v,isolate_mode,contextptr);
	return;
      }
      if (e._SYMBptr->sommet==at_neg){
	solve(e._SYMBptr->feuille,x,v,isolate_mode,contextptr);
	return;
      }
      if (e._SYMBptr->sommet==at_inv || (e._SYMBptr->sommet==at_pow && ck_is_positive(-e._SYMBptr->feuille._VECTptr->back(),contextptr)))
	return;
      in_solve(e,x,v,isolate_mode,contextptr);
      break;
    default:
      return;
    }
  }

  // find the arguments of sqrt inside expression e
  vecteur lvarfracpow(const gen & e){
    vecteur l0=lop(e,at_pow),l;
    const_iterateur it=l0.begin(),itend=l0.end();
    for (;it!=itend;++it){
      vecteur & arg=*it->_SYMBptr->feuille._VECTptr;
      gen g=arg[1],expnum,expden;
      if (g.type==_FRAC){
	expnum=g._FRACptr->num;
	expden=g._FRACptr->den;
      }
      else {
	if ( (g.type!=_SYMB) || (g._SYMBptr->sommet!=at_prod) )
	  continue;
	gen & arg1=g._SYMBptr->feuille;
	if (arg1.type!=_VECT)
	  continue;
	vecteur & v=*arg1._VECTptr;
	if ( (v.size()!=2) || (v[1].type!=_SYMB) || (v[1]._SYMBptr->sommet==at_inv) )
	  continue;
	expnum=v[0];
	expden=v[1]._SYMBptr->feuille;
      }
      if (expden.type!=_INT_)
	continue;
      l.push_back(arg[0]);
      l.push_back(expden.val);
      l.push_back(*it);
    }
    return l;
  }

  vecteur lvarfracpow(const gen & g,const identificateur & x){
    vecteur l0=lvarfracpow(g),l;
    const_iterateur it=l0.begin(),itend=l0.end();
    for (;it!=itend;++it){
      if (!is_zero(derive(*it,x))){
	l.push_back(*it);
	++it;
	l.push_back(*it);
	++it;
	l.push_back(*it);
      }
      else
	it+=2;
    }
    return l;
  }

  void solve_fracpow(const gen & e,const identificateur & x,vecteur equations,const vecteur & listvars,vecteur & fullres,int isolate_mode,GIAC_CONTEXT){
    if (e.type==_IDNT){
      if (*e._IDNTptr==x && !equalposcomp(find_excluded(x,contextptr),zero)){
	addtolvar(zero,fullres);
	return;
      }
    }
    if (e.type==_SYMB){
      if ( (e._SYMBptr->sommet==at_pow) && (ck_is_positive(e._SYMBptr->feuille._VECTptr->back(),contextptr)) ){
	solve_fracpow(e._SYMBptr->feuille._VECTptr->front(),x,equations,listvars,fullres,isolate_mode,contextptr);
	return;
      }
      if (e._SYMBptr->sommet==at_prod){
	const_iterateur it=e._SYMBptr->feuille._VECTptr->begin(),itend=e._SYMBptr->feuille._VECTptr->end();
	for (;it!=itend;++it)
	  solve_fracpow(*it,x,equations,listvars,fullres,isolate_mode,contextptr);
	return;
      }
      if (e._SYMBptr->sommet==at_neg){
	solve_fracpow(e._SYMBptr->feuille,x,equations,listvars,fullres,isolate_mode,contextptr);
	return;
      }
    } // end if (e.type==_SYMB)
      // new code with resultant in all var except the first one
    gen expr(e);
    int s=listvars.size();
    for (int i=1;i<s;++i){
      if (!is_zero(derive(expr,listvars[i])))
	expr=_resultant(makevecteur(expr,equations[i-1],listvars[i]));
    }
    expr=ratfactor(expr,false);
    if (is_zero(derive(expr,x)))
      return;
    solve(expr,x,fullres,isolate_mode,contextptr);
    return;
    // old code with Groebner basis
    equations.push_back(e);      
    vecteur res=gsolve(equations,listvars,complex_mode(contextptr),contextptr);
    iterateur it=res.begin(),itend=res.end();
    for (;it!=itend;++it)
      *it=(*it)[0];
    _purge(vecteur(listvars.begin()+1,listvars.end()),contextptr);
    if (listvars[0].type==_IDNT){
      fullres=mergevecteur(res,fullres);
      return;
    }
    // recursive call to solve composevar=*it with respect to x
    for (it=res.begin();it!=itend;++it){
      fullres=mergevecteur(fullres,solve(*it-listvars[0],x,isolate_mode,contextptr));
    }
  }

  vecteur solve(const gen & e,const identificateur & x,int isolate_mode,GIAC_CONTEXT){
    bool complexmode=isolate_mode & 1;
    if (approx_mode(contextptr))
      return makevecteur(_fsolve(gen(makevecteur(e,x),_SEQ__VECT),contextptr));
    // should rewrite e in terms of a minimal number of variables
    // first factorization of e
    gen expr(e);
    if (e.type==_SYMB){ 
      if (e._SYMBptr->sommet==at_inferieur_strict)
	return solve_inequation(e,x,-2,contextptr);
      if (e._SYMBptr->sommet==at_inferieur_egal)
	return solve_inequation(e,x,-1,contextptr);
      if (e._SYMBptr->sommet==at_superieur_strict)
	return solve_inequation(e,x,2,contextptr);
      if (e._SYMBptr->sommet==at_superieur_egal)
	return solve_inequation(e,x,1,contextptr);
      if (e._SYMBptr->sommet==at_equal ||e._SYMBptr->sommet==at_same)
	expr = e._SYMBptr->feuille._VECTptr->front()-e._SYMBptr->feuille._VECTptr->back();
    }
    // Checking for abs
    vecteur la;
    if (!complexmode)
      la=lop(expr,at_abs);
    const_iterateur itla=la.begin(),itlaend=la.end();
    for (;itla!=itlaend;++itla){
      gen g=itla->_SYMBptr->feuille;
      if (is_zero(derive(g,x)))
	continue;
      vecteur res;
      gen ee=subst(expr,*itla,g);
      vecteur v1=solve(ee,x,isolate_mode,contextptr);
      const_iterateur it=v1.begin(),itend=v1.end();
      for (;it!=itend;++it){
	gen g1=subst(g,x,*it);
	if (abs(g1,contextptr)==g1) 
	  res.push_back(*it);
      }
      ee=subst(expr,*itla,-g);
      v1=solve(ee,x,isolate_mode,contextptr);
      it=v1.begin(); itend=v1.end();
      for (;it!=itend;++it){
	gen g1=subst(g,x,*it);
	if (abs(g1,contextptr)==-g1) 
	  res.push_back(*it);
      }
      return res;
    }
    vecteur lv(lvarx(expr,x));
    int s=lv.size();
    if (s>1)
      expr=halftan_hyp2exp(expr);
    // Checking for fractional power
    // Remark: algebraic extension could also be solved using resultant
    vecteur ls(lvarfracpow(expr,x));
    if (!ls.empty()){ // Use auxiliary variables
      int s=ls.size()/3;
      vecteur substin,substout,equations,listvars(lvarx(e,x));
      // remove ls from listvars, add aux var instead
      for (int i=0;i<s;++i){
	gen lsvar=ls[3*i+2];
	int j=equalposcomp(listvars,ls);
	if (j)
	  listvars.erase(listvars.begin()+j-1);
      }
      if (listvars.size()!=1)
	setsizeerr("unable to isolate "+gen(listvars).print());
      for (int i=0;i<s;++i){
	gen lsvar=ls[3*i+2];
	substin.push_back(lsvar);
	gen tmp("c_"+print_INT_(i));
	if (!(ls[3*i+1].val %2))
	  assumesymbolic(symb_superieur_strict(tmp,0),0,contextptr); 
	listvars.push_back(tmp);
	substout.push_back(tmp);
	equations.push_back(pow(tmp,ls[3*i+1])-ls[3*i]);
      }
      gen expr1=subst(expr,substin,substout);
      expr1=ratfactor(expr1,false);
      vecteur fullres;
      solve_fracpow(expr1,x,equations,listvars,fullres,isolate_mode,contextptr);
      // Check that expr at x=fullres is 0
      vecteur res;
      const_iterateur it=fullres.begin(),itend=fullres.end();
      for (;it!=itend;++it){
	if (is_zero(limit(expr,x,*it,0,contextptr)))
	  res.push_back(*it);
      }
      return res;
    }
    lv=lvarx(expr,x);
    if (lv.size()>1){
      gen tmp=simplify(expr);
      lv=lvarx(tmp,x);
      if (lv.size()<2)
	expr=tmp;
    }
    // -> exp/ln
    expr=pow2expln(expr,x);
    expr=ratfactor(expr,false);
    lv=lvarx(expr,x);
    s=lv.size();
    vecteur v;
    if (!s){
      if (is_zero(expr))
	v.push_back(x);
      return v;
    }
    solve(expr,x,v,isolate_mode,contextptr);
    return v;
  }

  gen remove_and(const gen & g,const unary_function_ptr & u){
    if (g.type==_VECT){
      vecteur res;
      const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
      for (;it!=itend;++it){
	gen tmp=remove_and(*it,u);
	if (tmp.type!=_VECT){
	  tmp=remove_and(*it,at_and);
	  res.push_back(tmp);
	}
	else
	  res=mergevecteur(res,*tmp._VECTptr);
      }
      return res;
    }
    if (!g.is_symb_of_sommet(u))
      return g;
    return remove_and(g._SYMBptr->feuille,u);
  }

  vecteur solve(const gen & e,const gen & x,int isolate_mode,GIAC_CONTEXT){
    bool complexmode=isolate_mode & 1;
    vecteur res;
    if (x.type!=_IDNT){
      if ( (x.type==_VECT) && (e.type==_VECT) )
	return gsolve(*e._VECTptr,*x._VECTptr,complexmode,contextptr);
      identificateur xx("x");
      res=solve(subst(e,x,xx),xx,isolate_mode,contextptr);
      return res;
    }
    if (e.type==_VECT){
      const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
      res.reserve(itend-it);
      for (;it!=itend;++it)
	res.push_back(solve(*it,*x._IDNTptr,isolate_mode,contextptr));
    }
    else
      res=solve(e,*x._IDNTptr,isolate_mode,contextptr);
    return res;
  }

  gen symb_solution(const gen & g,const gen & var){
    if (var.type!=_VECT){
      if (var.type==_IDNT && !lvarx(g,*var._IDNTptr).empty())
	return g;
      else
	return symbolic(at_equal,makevecteur(var,g));
    }
    vecteur v=*var._VECTptr;
    if (g.type!=_VECT || g._VECTptr->size()!=v.size())
      setsizeerr();
    iterateur it=v.begin(),itend=v.end(),jt=g._VECTptr->begin();
    for (;it!=itend;++it,++jt)
      *it=symbolic(at_equal,makevecteur(*it,*jt));
    if (maple_mode==3)
      return symbolic(at_and,v);
    else
      return v;
  }

  vecteur solvepreprocess(const gen & args,GIAC_CONTEXT){
    gen g(args);
    if (g.type==_VECT && !g._VECTptr->empty() & g._VECTptr->front().is_symb_of_sommet(at_and)){
      vecteur v(*g._VECTptr);
      v.front()=remove_and(v.front(),at_and);
      g=gen(v,g.subtype);
    }
    return plotpreprocess(g,contextptr);
  }

  gen solvepostprocess(const gen & g,const gen & x){
    if (g.type!=_VECT)
      return g;
    vecteur res=*g._VECTptr;
    // convert solution to an expression
    iterateur it=res.begin(),itend=res.end();
    if (it==itend)
      return res;
    if (x.type==_VECT || maple_mode==3){
      for (;it!=itend;++it)
	*it=symb_solution(*it,x);
    }
    if (maple_mode==3)
      return symbolic(at_ou,res);
    if (maple_mode==2)
      return gen(res,_SET__VECT);
    return gen(res,_SEQ__VECT);
  }

  gen _solve(const gen & args,GIAC_CONTEXT){
    int isolate_mode=complex_mode(contextptr) | (all_trig_sol(contextptr) << 1);
    vecteur v(solvepreprocess(args,contextptr));
    int s=v.size();
    if (s>2)
      toomanyargs("solve");
    vecteur res=solve(v.front(),v.back(),isolate_mode,contextptr);
    if (!maple_mode)
      return res;
    return solvepostprocess(res,v[1]);
  }
  const string _solve_s("solve");
  unary_function_eval __solve(&_solve,_solve_s);
  unary_function_ptr at_solve (&__solve,_QUOTE_ARGUMENTS);

  double nan(){
    double x=0.0;
    return 0.0/x;
  }
#ifdef HAVE_LIBGSL
  // p should point a vector with elements the expression f(x) and x
  // OR with f(x), f'(x) and x
  double my_f (double x0, void * p) {
    gen & params = * ((gen *)p) ;
#ifdef DEBUG_SUPPORT
    assert( (params.type==_VECT) && (params._VECTptr->size()>=2));
#endif	// DEBUG_SUPPORT
    gen & f=params._VECTptr->front();
    gen & x=params._VECTptr->back();
    gen res=evalf(subst(f,x,x0));
    if (res.type!=_DOUBLE_)
      return nan();
    else
      return res._DOUBLE_val;
  }

  double my_df (double x0, void * p) {
    gen & params = * ((gen *)p) ;
#ifdef DEBUG_SUPPORT
    assert( (params.type==_VECT) && (params._VECTptr->size()==3));
#endif	// DEBUG_SUPPORT
    gen & f=(*params._VECTptr)[1];
    gen & x=params._VECTptr->back();
    gen res=evalf(subst(f,x,x0));
    if (res.type!=_DOUBLE_)
      return nan();
    else
      return res._DOUBLE_val;
  }

  void my_fdf (double x0, void * p,double * fx,double * dfx) {
    gen & params = * ((gen *)p) ;
#ifdef DEBUG_SUPPORT
    assert( (params.type==_VECT) && (params._VECTptr->size()==3));
#endif	// DEBUG_SUPPORT
    gen & f=params._VECTptr->front();
    gen & df=(*params._VECTptr)[1];
    gen & x=params._VECTptr->back();
    gen res=evalf(subst(f,x,x0));
    if (res.type!=_DOUBLE_)
      *fx=nan();
    else
      *fx=res._DOUBLE_val;
    res=evalf(subst(df,x,x0));
    if (res.type!=_DOUBLE_)
      *dfx=nan();
    else
      *dfx=res._DOUBLE_val;
  }

  int my_F (const gsl_vector * x0, void * p,gsl_vector * F) {
    gen & params = * ((gen *)p) ;
#ifdef DEBUG_SUPPORT
    assert( (params.type==_VECT) && (params._VECTptr->size()>=2));
#endif	// DEBUG_SUPPORT
    gen & f=params._VECTptr->front();
    gen & x=params._VECTptr->back();
    gen res=evalf(subst(f,x,gsl_vector2vecteur(x0)));
    if (res.type!=_VECT)
      return !GSL_SUCCESS;
    return vecteur2gsl_vector(*res._VECTptr,F);
  }

  int my_dF (const gsl_vector *x0, void * p,gsl_matrix * J) {
    gen & params = * ((gen *)p) ;
#ifdef DEBUG_SUPPORT
    assert( (params.type==_VECT) && (params._VECTptr->size()==3));
#endif	// DEBUG_SUPPORT
    gen & f=(*params._VECTptr)[1];
    gen & x=params._VECTptr->back();
    gen res=evalf(subst(f,x,gsl_vector2vecteur(x0)));
    if (res.type!=_VECT)
      return !GSL_SUCCESS;
    else
      return matrice2gsl_matrix(*res._VECTptr,J);
  }

  int my_FdF (const gsl_vector * x0, void * p,gsl_vector * fx,gsl_matrix * dfx) {
    gen & params = * ((gen *)p) ;
#ifdef DEBUG_SUPPORT
    assert( (params.type==_VECT) && (params._VECTptr->size()==3));
#endif	// DEBUG_SUPPORT
    gen & f=params._VECTptr->front();
    gen & df=(*params._VECTptr)[1];
    gen & x=params._VECTptr->back();
    gen g0=gsl_vector2vecteur(x0);
    gen res=evalf(subst(f,x,g0));
    if (res.type!=_VECT)
      return !GSL_SUCCESS;
    int ires=vecteur2gsl_vector(*res._VECTptr,fx);
    if (ires!=GSL_SUCCESS)
      return !GSL_SUCCESS;
    res=evalf(subst(df,x,g0));
    if (res.type!=_VECT)
      return !GSL_SUCCESS;
    return matrice2gsl_matrix(*res._VECTptr,dfx);
  }

  gen msolve(const gen & f,const vecteur & vars,const vecteur & g,int method,double eps){
    vecteur guess(g);
    bool with_derivative;
    int dim=vars.size();
    switch (method){
    case _NEWTONJ_SOLVER: case _HYBRIDSJ_SOLVER: case _HYBRIDJ_SOLVER:
      with_derivative=true;
      break;
    case _DNEWTON_SOLVER: case _HYBRIDS_SOLVER: case _HYBRID_SOLVER:
      with_derivative=false;
      break;
    }
    if (with_derivative){
      gen params(makevecteur(f,derive(f,vars),vars));
      gsl_multiroot_function_fdf FDF;
      FDF.f=&my_F;
      FDF.df=&my_dF;
      FDF.fdf=&my_FdF;
      FDF.n=dim;
      FDF.params=&params;
      const gsl_multiroot_fdfsolver_type * T;
      switch (method){
      case _NEWTONJ_SOLVER: 
	T=gsl_multiroot_fdfsolver_gnewton;
	break;
      case _HYBRIDSJ_SOLVER:
	T=gsl_multiroot_fdfsolver_hybridsj;
	break;
      case _HYBRIDJ_SOLVER:
	T=gsl_multiroot_fdfsolver_hybridj;
	break;
      }
      gsl_multiroot_fdfsolver * s= gsl_multiroot_fdfsolver_alloc (T, dim);
      gsl_vector * X=vecteur2gsl_vector(guess);
      gsl_multiroot_fdfsolver_set (s, &FDF, X);
      int maxiter=SOLVER_MAX_ITERATE,res;
      vecteur oldguess;
      for (;maxiter;--maxiter){
	oldguess=guess;
	res=gsl_multiroot_fdfsolver_iterate(s);
	if ( (res==GSL_EBADFUNC) || (res==GSL_ENOPROG) )
	  break;
	guess=gsl_vector2vecteur(gsl_multiroot_fdfsolver_root(s));
	if (is_strictly_greater(eps,abs(guess-oldguess,0),0))
	  break;
      }
      gsl_multiroot_fdfsolver_free(s);
      if ( (res==GSL_EBADFUNC) || (res==GSL_ENOPROG) )
	setsizeerr("Not found");
      return guess;
    }
    else {
      gen params(makevecteur(f,vars));
      gsl_multiroot_function F;
      F.f=&my_F;
      F.n=dim;
      F.params=&params;
      const gsl_multiroot_fsolver_type * T;
      switch (method){
      case _DNEWTON_SOLVER: 
	T=gsl_multiroot_fsolver_dnewton;
	break;
      case _HYBRIDS_SOLVER:
	T=gsl_multiroot_fsolver_hybrids;
	break;
      case _HYBRID_SOLVER:
	T=gsl_multiroot_fsolver_hybrid;
	break;
      }
      gsl_multiroot_fsolver * s= gsl_multiroot_fsolver_alloc (T, dim);
      gsl_vector * X=vecteur2gsl_vector(guess);
      gsl_multiroot_fsolver_set (s, &F, X);
      int maxiter=SOLVER_MAX_ITERATE,res;
      vecteur oldguess;
      for (;maxiter;--maxiter){
	oldguess=guess;
	res=gsl_multiroot_fsolver_iterate(s);
	if ( (res==GSL_EBADFUNC) || (res==GSL_ENOPROG) )
	  break;
	guess=gsl_vector2vecteur(gsl_multiroot_fsolver_root(s));
	if (is_strictly_greater(eps,abs(guess-oldguess,0),0))
	  break;
      }
      gsl_multiroot_fsolver_free(s);
      if ( (res==GSL_EBADFUNC) || (res==GSL_ENOPROG) )
	setsizeerr("Not found");
      return guess;
    }
  }
#endif // HAVE_LIBGSL

  // fsolve(expr,var[,interval/guess,method])
  gen _fsolve(const gen & args,GIAC_CONTEXT){
    vecteur v(plotpreprocess(args,contextptr));
    double gsl_eps=epsilon(contextptr);
    int s=v.size();
    if (s<2)
      toofewargs("fsolve");
    gen v0=remove_equal(v[0]);
    gen gguess;
    if (s>=3)
      gguess=v[2];
    // check method
    int method=_NEWTON_SOLVER;
    //int method=0;
    if ( (s>=5) && (v[4].type==_DOUBLE_) )
      gsl_eps=v[4]._DOUBLE_val;
    if ( (s>=4) && (v[3].type==_INT_) )
      method=v[3].val;
    if (v[1].type==_VECT){
      int dim=v[1]._VECTptr->size();
      if (!dim)
	setsizeerr();
      if (s>=3){
	if (gguess.type!=_VECT)
	  setsizeerr();
	if (gguess._VECTptr->size()!=dim)
	  setsizeerr();
      }
      else {
	gguess=vecteur(dim);
	gguess[0]=(gnuplot_xmin+gnuplot_xmax)/2;
	if (dim>1)
	  gguess[1]=(gnuplot_ymin+gnuplot_ymax)/2;
	if (dim>2)
	  gguess[2]=(gnuplot_zmin+gnuplot_zmax)/2;
	if (dim>3)
	  gguess[3]=(gnuplot_tmin+gnuplot_tmax)/2;
      }
#ifdef HAVE_LIBGSL
      if (method!=_NEWTON_SOLVER)
	return msolve(v0,*v[1]._VECTptr,*gguess._VECTptr,method,gsl_eps);
#endif
    }
#ifdef HAVE_LIBGSL
    if (method!=_NEWTON_SOLVER){
      bool with_derivative;
      switch (method){
      case _BISECTION_SOLVER: case _FALSEPOS_SOLVER: case _BRENT_SOLVER:
	with_derivative=false;
	break;
      case _NEWTON_SOLVER: case _SECANT_SOLVER: case _STEFFENSON_SOLVER:
	with_derivative=true;
	break;
      }
      gen params;
      if (with_derivative){
	params= makevecteur(v0,derive(v0,v[1]),v[1]);
	double guess((gnuplot_xmin+gnuplot_xmax)/2),oldguess;
	if (s>=3){
	  gen g=evalf(gguess,DEFAULT_EVAL_LEVEL,contextptr);
	  if (gguess.type==_DOUBLE_)
	    guess=gguess._DOUBLE_val;
	}
	gsl_function_fdf FDF ;     
	FDF.f = &my_f ;
	FDF.df = &my_df ;
	FDF.fdf = &my_fdf ;
	FDF.params = &params ;
	gsl_root_fdfsolver * slv;
	switch (method){
	case _NEWTON_SOLVER:
	  slv=gsl_root_fdfsolver_alloc (gsl_root_fdfsolver_newton);
	  break;
	case _SECANT_SOLVER: 
	  slv=gsl_root_fdfsolver_alloc (gsl_root_fdfsolver_secant);
	  break;
	case _STEFFENSON_SOLVER:
	  slv=gsl_root_fdfsolver_alloc (gsl_root_fdfsolver_steffenson);
	  break;
	}
	if (!slv)
	  setsizeerr();
	gsl_root_fdfsolver_set(slv,&FDF,guess);
	int maxiter=SOLVER_MAX_ITERATE,res;
	for (;maxiter;--maxiter){
	  oldguess=guess;
	  res=gsl_root_fdfsolver_iterate(slv);
	  guess=gsl_root_fdfsolver_root(slv);
	  if ( (res==GSL_EBADFUNC) || (res==GSL_EZERODIV) )
	    break;
	  if (fabs(guess-oldguess)<gsl_eps)
	    break;
	}
	gsl_root_fdfsolver_free(slv);
	if (!maxiter)
	setsizeerr("Not found");
	if ( (res==GSL_EBADFUNC) || (res==GSL_EZERODIV) )
	  return undef;
	else
	  return guess;
      }
      else {
	params= makevecteur(v0,v[1]);
	double x_low,x_high;
	if (s>=3) {
	  vecteur w;
	  if (gguess.type==_VECT)
	    w=*gguess._VECTptr;
	  else {
	    if ( (gguess.type==_SYMB) && (gguess._SYMBptr->sommet==at_interval) )
	      w=*gguess._SYMBptr->feuille._VECTptr;
	  }
	  if (w.size()!=2)
	    settypeerr();
	  gen low=w[0].evalf(DEFAULT_EVAL_LEVEL,contextptr);
	  gen high=w[1].evalf(DEFAULT_EVAL_LEVEL,contextptr);
	  if ( (low.type!=_DOUBLE_) || (high.type!=_DOUBLE_) )
	  setsizeerr();
	  x_low=low._DOUBLE_val;
	  x_high=high._DOUBLE_val;
	}
	else {
	  x_low=gnuplot_xmin;
	  x_high=gnuplot_xmax;
	}
	if (x_low>x_high){
	  double tmp=x_low;
	  x_low=x_high;
	  x_high=tmp;
	}
	gsl_function F ;
	F.function=&my_f;
	F.params = &params ;
	gsl_root_fsolver * slv ;
	switch (method){
	case  _BISECTION_SOLVER:
	  slv=gsl_root_fsolver_alloc (gsl_root_fsolver_bisection);
	  break;
	case _FALSEPOS_SOLVER: 
	  slv=gsl_root_fsolver_alloc (gsl_root_fsolver_falsepos);
	  break;
	case _BRENT_SOLVER:
	  slv=gsl_root_fsolver_alloc (gsl_root_fsolver_brent);
	  break;
	}
	if (!slv)
	  setsizeerr();
	gsl_root_fsolver_set (slv,&F,x_low,x_high);
	int res;
	int maxiter=SOLVER_MAX_ITERATE;
	for (;maxiter && (x_high-x_low>gsl_eps);--maxiter){
	  res=gsl_root_fsolver_iterate(slv);
	  if (res==GSL_EBADFUNC)
	    break;
	  x_low=gsl_root_fsolver_x_lower(slv);
	  x_high= gsl_root_fsolver_x_upper(slv);
	}
	gsl_root_fsolver_free (slv);
	if (res==GSL_EBADFUNC)
	  return undef;
	return makevecteur(x_low,x_high);
      } // end if derivative
    }
#else // HAVE_LIBGSL
    if (method!=_NEWTON_SOLVER)
      setsizeerr("Not linked with GSL");
#endif    // HAVE_LIBGSL
    else  // newton method, call newton
      return newton(v0,v[1],gguess,NEWTON_DEFAULT_ITERATION,gsl_eps);
  } // end f_solve
  const string _fsolve_s("fsolve");
  unary_function_eval __fsolve(&_fsolve,_fsolve_s);
  unary_function_ptr at_fsolve (&__fsolve,_QUOTE_ARGUMENTS);

  vecteur sxa(const vecteur & sl_orig,const vecteur & x){
      vecteur sl(sl_orig);
      int d;
      d=x.size();
      int de;
      de=sl.size();
      for (int i=0;i<de;i++){
              //gen e:
              //e=sl[i];    
          if ( (sl[i].type==_SYMB) && ((*sl[i]._SYMBptr).sommet==at_equal || (*sl[i]._SYMBptr).sommet==at_same)){
              sl[i]=(*sl[i]._SYMBptr).feuille[0]-(*sl[i]._SYMBptr).feuille[1];
          }
      }
      vecteur A;
      for (int i=0;i<de;i++){
          vecteur li(d+1);
          gen lo=sl[i];
          for (int j=0;j<d;j++){
              lo=subst(lo,x[j],0);
              li[j]=derive(sl[i],x[j]);
          }
          li[d]=lo;
          A.push_back(li);
      }
      return(A);
  }

  vecteur linsolve(const vecteur & sl,const vecteur & x){
    vecteur A; 
    if (ckmatrix(sl)){
      int n=sl.size();
      A=mtran(sl);
      if (ckmatrix(x)){
	if (x.size()==1){
	  if (x.front()._VECTptr->size()!=n)
	    setdimerr();
	  A.push_back(-x.front());
	}
	else {
	  if (x.size()!=n)
	    setdimerr();
	  matrice xm=mtran(x);
	  if (xm.size()!=1)
	    setsizeerr();
	  A.push_back(-xm.front());
	}
      }
      else {
	if (x.size()!=n)
	  setdimerr();
	A.push_back(-x);
      }
      A=mtran(A);
      vecteur B=-mker(A);
      if (B.empty())
	return B;
      // The last element of B must have a non-zero last component
      vecteur Bend=*B.back()._VECTptr;
      gen last=Bend.back();
      if (is_zero(last))
	return vecteur(0);
      gen R=Bend/last;
      // The solution is sum(B[k]*Ck+Blast/last)
      int s=B.size();
      for (int k=0;k<s-1;k++)
	R=R+gen("C_"+print_INT_(k))*B[k];
      vecteur res=*R._VECTptr;
      res.pop_back();
      return res;
    }
    A=sxa(sl,x);
    vecteur B,R(x);
    gen rep;
    B=mrref(A);
    //cout<<B<<endl;
    int d=x.size();
    int de=sl.size();
    for (int i=0; i<de;i++){
      vecteur li(d+1);
      for(int k=0;k<d+1;k++){
	li[k]=B[i][k];
      }
      int j;
      j=i;
      while (li[j]==0 && j<d){
	j=j+1;
      }
      if (j==d && !is_zero(li[d])){
	return vecteur(0);
      } 
      else {
	if (j<d){
	  rep=-li[d];
	  for (int k=j+1;k<d;k++){
	    rep=rep-li[k]*x[k];
	  }
	  rep=rdiv(rep,li[j]);
	  R[j]=rep;
	}
      }
    }
    return R;
  }

  gen equal2diff(const gen & g){
    if ( (g.type==_SYMB) && (g._SYMBptr->sommet==at_equal || g._SYMBptr->sommet==at_same) ){
      vecteur & v=*g._SYMBptr->feuille._VECTptr;
      return v[0]-v[1];
    }
    else
      return g;
  }

  gen symb_linsolve(const gen & syst,const gen & vars){
    return symbolic(at_linsolve,makevecteur(syst,vars));
  }
 
  gen linsolve(const gen & syst,const gen & vars){
    if ((syst.type!=_VECT)||(vars.type!=_VECT))
      return symb_linsolve(syst,vars);
    return normal(linsolve(*syst._VECTptr,*vars._VECTptr));
  }
  
  gen _linsolve(const gen & args,GIAC_CONTEXT){
    vecteur v(plotpreprocess(args,contextptr));
    int s=v.size();
    if (s!=2)
      toomanyargs("linsolve");
    if (v[1].type==_IDNT)
      v[1]=eval(v[1],contextptr);
    gen syst=apply(v[0],equal2diff),vars=v[1];
    return linsolve(syst,v[1]);
  }
  const string _linsolve_s("linsolve");
  unary_function_eval __linsolve(&_linsolve,_linsolve_s);
  unary_function_ptr at_linsolve (&__linsolve,_QUOTE_ARGUMENTS);

  /*
  gen iter(const gen & f, const gen & x,const gen & arg,int maxiter,double eps, int & b){
    gen a=arg;
    complex<double> olda;
    complex<double> ad;
    b=0;
    ad=gen2complex_d(a);
    //cout<<"a"<<a<<endl;
    //cout<<"ad"<<ad<<endl;
    for (int j=1;j<=maxiter;j++){
      olda=ad;    
      // cout << f << " " << x << " " << a << endl;
      a=subst(f,x,a).evalf();
      // cout<<"a"<<a<<endl;
      //a=a.evalf();
      //ad=a._DOUBLE_val;
      ad=gen2complex_d(a);
      // cout<<"a"<<a<<endl;
      // cout<<"ad"<<ad<<endl;
      // cout<<"j"<<j<<"abs"<<abs(ad-olda)<<endl;
      if (eps>abs(ad-olda)) {
	b=1; return(a);
      }
    } 
    return(a); 
  }
  
  gen newtona(const gen & f, const gen & x, const gen & arg,int niter1, int niter2, double eps1,double eps2,double prefact1,double prefact2, int & b){
    if (x.type!=_IDNT)
      settypeerr("2nd arg must be an identifier");
    //cout<<a<<endl;
    gen a=arg;
    gen g1;
    gen g;
    g1=x-gen(prefact1)*rdiv(f,derive(f,x));
    // sym_sub(x,sym_mult(rdiv(9,10),rdiv(f,derive(f,x))));
    try {
      a= iter(g1,x,a,niter1,eps1,b);
      g=x-gen(prefact2)*rdiv(f,derive(f,x));
      a= iter(g,x,a,niter2,eps2,b); 
    }
    catch (std::runtime_error & err){
      b=0;
    }
    return a;
  }
 
  gen newton(const gen & f, const gen & x,const gen & guess,int niter1,int niter2,double eps1,double eps2,double prefact1,double prefact2){
    bool guess_first=is_undef(guess);
    for (int j=1;j<5;j++,niter2 *=2, niter1 *=2){ 
      gen a;
      int b;
      //on prend un dpart au hasard (a=x0=un _DOUBLE_)
      // a=gen(2.0);
      if (guess_first)
	a=j*4*(rand()/(RAND_MAX+1.0)-0.5);
      else {
	a=guess;
	guess_first=true;
      }
      // cout<<j<<"j"<<a<<endl; 
      gen e;
      e=newtona(f, x, a,niter1,niter2,eps1,eps2,prefact1,prefact2,b);
      if (b==1) return e;
      gen c;
      c=j*4*(rand()/(RAND_MAX+1.0)-0.5);
      // cout<<j<<"j"<<c<<endl;
      // g=x-gen(0.5)*rdiv(f,derive(f,x));
      gen ao(gen(0.0),c);
      // cout<<"ao"<<ao<<endl;
      gen e0= newtona(f, x, ao,niter1,niter2,eps1,eps2,prefact1,prefact2,b);
      if (b==1) 
	return(e0);
      gen a1(a,c);
      // cout<<j<<"j,a1"<<a1<<endl;
      e0= newtona(f, x, a1,niter1,niter2,eps1,eps2,prefact1,prefact2,b);
      if (b==1) 
	return(e0);
    }
    setsizeerr("nontrouve");
    return(0);
  }
  */

  gen newton_rand(int j){
    gen a=evalf(j*4*(gen(rand())/gen(RAND_MAX+1)-plus_one_half));
    if (j!=1)
      a=a+cst_i*evalf(j*4*(gen(rand())/gen(RAND_MAX+1)-plus_one_half));
    return a;
  }

  gen newton(const gen & f, const gen & x,const gen & guess,int niter,double eps1, double eps2){
    gen a,b,d,fa,fb,invdf=inv(derive(f,x)),epsg1(eps1),epsg2(eps2);
    bool guess_first=is_undef(guess);
    // Main loop with random initialization
    int j=1;
    for (;j<=5 ;j++,niter=2*niter){ 
      if (guess_first){
	if (f.type==_VECT){
	  int s=f._VECTptr->size();
	  vecteur v(s);
	  for (int i=0;i<s;++i)
	    v[i]=(newton_rand(j));
	  a=v;
	}
	else
	  a=newton_rand(j);
      }
      else {
	a=guess;
	guess_first=true;
      }
      fa=evalf(eval(subst(f,x,a)));
      // First loop to localize the solution with prefactor
      gen lambda(1);
      int k;
      try {
	for (k=0;k<niter;++k){
	  d=-evalf(eval(subst(invdf,x,a))*fa);
	  b=a+lambda*d;
	  if (is_positive(epsg1-_l2norm(d,0),0)){
	    a=b;
	    break;
	  }
	  fb=evalf(eval(subst(f,x,b)));
	  if (is_positive(_l2norm(fb,0)-_l2norm(fa,0),0)){
	    // Decrease prefactor and try again
	    lambda=evalf(plus_one_half)*lambda;
	  }
	  else {
	    // Save new value of a and increase the prefactor slightly
	    if (is_positive(lambda-0.9,0))
	      lambda=1;
	    else
	      lambda=evalf(gen(11)/gen(10))*lambda;
	    a=b;
	    fa=fb;
	  }
	}
	if (k==niter)
	  continue;
	// Second loop to improve precision (prefactor 1)
	for (k=0;k<niter;++k){
	  d=-evalf(subst(invdf,x,a)*subst(f,x,a));
	  a=a+d;
	  if (is_positive(epsg2-_l2norm(d,0),0))
	    break;
	}
	if (k!=niter)
	  break;
      } catch (std::runtime_error & e){
	break; // start with a new initial point
      }
    } // end for
    if (j>5)
      return undef;
    return a;
  }

  gen _newton(const gen & args,GIAC_CONTEXT){
    if (args.type!=_VECT)
      return newton(args,vx_var,undef,NEWTON_DEFAULT_ITERATION);
    vecteur & v=*args._VECTptr;
    int s=v.size();
    if (s<2)
      setsizeerr();
    if (s==2)
      return newton(v[0],v[1],undef,NEWTON_DEFAULT_ITERATION);
    int niter=NEWTON_DEFAULT_ITERATION;
    double eps=epsilon(contextptr);
    for (int j=3;j<s;++j){
      if (v[j].type==_INT_)
	niter=v[j].val;
      else {
	gen tmp=evalf_double(v[j]);
	if (tmp.type==_DOUBLE_)
	  eps=tmp._DOUBLE_val;
      }
    }
    gen res=newton(v[0],v[1],v[2],niter,1e-10,eps);
    cerr << res << endl;
    return res;
    toomanyargs("newton");
  }
  const string _newton_s("newton");
  unary_function_eval __newton(&_newton,_newton_s);
  unary_function_ptr at_newton (&__newton);
  
  bool has_num_coeff(const vecteur & v){
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (has_num_coeff(*it))
	return true;
    }
    return false;
  }
  
  bool has_num_coeff(const polynome & p){
    vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
    for (;it!=itend;++it){
      if (has_num_coeff(it->value))
	return true;
    }
    return false;
  }

  bool has_num_coeff(const gen & e){
    switch (e.type){
    case _ZINT: case _INT_: case _IDNT:
      return false;
    case _DOUBLE_:
      return true;
    case _CPLX:
      return (e._CPLXptr->type==_DOUBLE_) || ((e._CPLXptr+1)->type==_DOUBLE_);
    case _SYMB:
      return has_num_coeff(e._SYMBptr->feuille);
    case _VECT:
      return has_num_coeff(*e._VECTptr);
    case _POLY:
      return has_num_coeff(*e._POLYptr);
    case _FRAC:
      return has_num_coeff(e._FRACptr->num) || has_num_coeff(e._FRACptr->den);
    case _MOD:
      return false;
    default:
      setsizeerr("has_num_coeff");
    }
  }

  polynome spoly(const polynome & p,const polynome & q){
    if (p.coord.empty() || q.coord.empty())
      setsizeerr();
    index_t & pi = *p.coord.front().index.iptr;
    index_t & qi = *q.coord.front().index.iptr;
    index_t lcm= index_lcm(pi,qi);
    return p.shift(q.coord.front().value,lcm-pi)-q.shift(p.coord.front().value,lcm-qi);
  }

  polynome reduce(const polynome & p,vector<polynome>::const_iterator it,vector<polynome>::const_iterator itend){
    polynome q(p.dim),r(p.dim),res(p);
    gen a;
    for (;it!=itend;++it){
      TGroebnerDiv(res,*it,q,r);
      a=1;
      lcmdeno(r,a);
      res=r*a;
      ppz(res);
    }
    return res;
  }

  polynome reduce(const polynome & p,const vector<polynome> & v){
    vector<polynome>::const_iterator it=v.begin(),itend=v.end();
    return reduce(p,it,itend);
  }

  void reduce(vector<polynome> & res){
    if (res.empty())
      return;
    // reorder res
    sort(res.begin(),res.end(),tensor_is_greater<gen>);
    // reduce res
    int s=res.size();
    for (int i=0;i<s-1;){
      polynome & p=res[i];
      polynome pred=reduce(p,res.begin()+i+1,res.end());
      if (pred.coord.empty()){
	res.erase(res.begin()+i);
	--s;
	continue;
      }
      if (pred.coord.size()==p.coord.size() && pred*p.coord.front().value==p*pred.coord.front().value){
	++i;
	continue;
      }
      res[i]=pred;
      sort(res.begin()+i,res.end(),tensor_is_greater<gen>);
      i=0;
    }
  }

  vector<polynome> gbasis(const vector<polynome> & v){
    if (v.size()==1)
      return v;
    vector<polynome> res(v);
    reduce(res);
    bool notfound=true;
    for (;notfound;){
      // cerr << res << endl;
      notfound=false;
      vector<polynome>::const_iterator it=res.begin(),itend=res.end(),jt;
      vector<polynome> newres(res);
      for (;it!=itend;++it){
	for (jt=it+1;jt!=itend;++jt){
	  polynome toadd(reduce(spoly(*it,*jt),newres));
	  if (!toadd.coord.empty()){
	    notfound=true;
	    newres.push_back(toadd); // should be at the right place
	  }
	}
      }
      reduce(newres);
      res=newres;
    }
    return res;
  }

  gen remove_equal(const gen & f){
    if ( (f.type==_SYMB) && (f._SYMBptr->sommet==at_equal || f._SYMBptr->sommet==at_same ) ){
      vecteur & v=*f._SYMBptr->feuille._VECTptr;
      return v.front()-v.back();
    }
    if (f.type==_VECT)
      return apply(f,remove_equal);
    return f;
  }

  vecteur remove_equal(const_iterateur it,const_iterateur itend){
    vecteur conditions;
    conditions.reserve(itend-it);
    for (;it!=itend;++it){
	conditions.push_back(remove_equal(*it));
    }
    return conditions;
  }

  bool vecteur2vector_polynome(const vecteur & eq_in,const vecteur & l,vector<polynome> & eqp){
    // remove all denominators
    const_iterateur it=eq_in.begin(),itend=eq_in.end();
    for (;it!=itend;++it){
      gen n,d;
      fxnd(*it,n,d);
      if (n.type==_POLY){
	// should reordre n with total degree+revlex order here
	eqp.push_back(*n._POLYptr);
	continue;
      }
      if (!is_zero(n))
	return false;
    }
    return true;
  }

  vecteur gsolve(const vecteur & eq_orig,const vecteur & var_orig,bool complexmode,GIAC_CONTEXT){
    // replace variables in var_orig by true identificators
    vecteur var(var_orig);
    iterateur it=var.begin(),itend=var.end();
    int s=itend-it; // # of unknowns
    bool need_subst=false;
    identificateur tab_idnt[s];
    for (int i=0;it!=itend;++it,++i){
      if (it->type!=_IDNT){
	*it=tab_idnt[i]; 
	need_subst=true;
      }
    }
    vecteur eq(remove_equal(eq_orig.begin(),eq_orig.end()));
    if (need_subst)
      eq=subst(eq,var_orig,var);
    if (approx_mode(contextptr)){
#ifdef HAVE_LIBGSL
      return makevecteur(msolve(eq,var,multvecteur(zero,var),_HYBRID_SOLVER,epsilon(contextptr)));
#else
      return vecteur(1,undef);
#endif
    }
    // check rational
    for (it=var.begin();it!=itend;++it){
      if (it->type!=_IDNT) // should not occur!
	setsizeerr("Bad var "+it->print());
      vecteur l(rlvarx(eq,*it->_IDNTptr));
      if (l.size()>1)
	setsizeerr(gen(l).print()+" is not rational w.r.t. "+it->print());
    }
    vecteur l(1,var);
    alg_lvar(eq,l);
    // convert eq to polynomial
    vecteur eq_in(*e2r(eq,l)._VECTptr);
    vector<polynome> eqp;
    // remove all denominators
    it=eq_in.begin();
    itend=eq_in.end();
    for (;it!=itend;++it){
      gen n,d;
      fxnd(*it,n,d);
      if (n.type==_POLY){
	// should reordre n with total degree+revlex order here
	eqp.push_back(*n._POLYptr);
	continue;
      }
      if (!is_zero(n))
	return vecteur(0); // no solution since cst equation
    }
    vector<polynome> eqpr(gbasis(eqp));
    // should reorder eqpr with lex order here
    // solve from right to left
    reverse(eqpr.begin(),eqpr.end());
    vecteur sols(1,vecteur(0)); // sols=[ [] ]
    vector<polynome>::const_iterator jt=eqpr.begin(),jtend=eqpr.end();
    for (;jt!=jtend;++jt){
      // the # of found vars is the size of sols.front()
      if (sols.empty())
	break;
      vecteur newsols;
      gen g(r2e(*jt,l));
      const_iterateur st=sols.begin(),stend=sols.end();
      for (;st!=stend;++st){
	int foundvars=st->_VECTptr->size();
	vecteur current=*st->_VECTptr;
	gen curg=ratnormal(subst(g,vecteur(var.end()-foundvars,var.end()),*st));
	gen x;
	int xpos=0;
	// First search in current an identifier curg depends on
	for (;xpos<foundvars;++xpos){
	  x=current[xpos];
	  if (x==var[s-foundvars+xpos] && !is_zero(derive(curg,x)) )
	    break;
	}
	if (xpos==foundvars){
	  xpos=0;
	  // find next var g depends on 
	  for (;foundvars<s;++foundvars){
	    x=var[s-foundvars-1];
	    current.insert(current.begin(),x);
	    if (!is_zero(derive(curg,x)))
	      break;
	  }
	  if (s==foundvars){
	    if (is_zero(curg))
	      newsols.push_back(current);
	    continue;
	  }
	}
	// solve
	vecteur xsol(solve(curg,*x._IDNTptr,complexmode,contextptr));
	const_iterateur xt=xsol.begin(),xtend=xsol.end();
	for (;xt!=xtend;++xt){
	  current[xpos]=*xt;
	  newsols.push_back(current);
	}
      } // end for (;st!=stend;)
      sols=newsols;
    }
    // Add var at the beginning of each solution of sols if needed
    it=sols.begin(); 
    itend=sols.end();
    for (;it!=itend;++it){
      int ss=it->_VECTptr->size();
      if (ss<s)
	*it=mergevecteur(vecteur(var.begin(),var.begin()+s-ss),*it->_VECTptr);
    }
    if (need_subst)
      return subst(sols,var,var_orig);
    return sols;
  }

  // gbasis([Pi],[vars]) -> [Pi']
  gen _gbasis(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_gbasis,args);
    vecteur & v = *args._VECTptr;
    if (v.size()<2)
      toofewargs("gbasis");
    if ( (v[0].type!=_VECT) || (v[1].type!=_VECT) )
      setsizeerr();
    vecteur l=vecteur(1,v[1]);
    alg_lvar(v[0],l);
    // v[2] will serve for ordering
    // convert eq to polynomial
    vecteur eq_in(*e2r(v[0],l)._VECTptr);
    vector<polynome> eqp;
    if (!vecteur2vector_polynome(eq_in,l,eqp))
      return vecteur(1,plus_one);
    vector<polynome> eqpr(gbasis(eqp));
    vecteur res;
    vector<polynome>::const_iterator it=eqpr.begin(),itend=eqpr.end();
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(r2e(*it,l));
    return res;
  }
  const string _gbasis_s("gbasis");
  unary_function_unary __gbasis(&_gbasis,_gbasis_s);
  unary_function_ptr at_gbasis (&__gbasis);
  
  // greduce(P,[gbasis],[vars])
  gen _greduce(const gen & args){
    if (args.type!=_VECT)
      return symbolic(at_gbasis,args);
    vecteur & v = *args._VECTptr;
    if (v.size()<3)
      toofewargs("greduce");
    if ( (v[1].type!=_VECT) || (v[2].type!=_VECT) )
      setsizeerr();
    vecteur l=vecteur(1,v[2]);
    alg_lvar(v[0],l);
    alg_lvar(v[1],l);
    // v[3] will serve for ordering
    gen eq(e2r(v[0],l));
    if (eq.type!=_POLY)
      return v[0];
    gen C(eq._POLYptr->constant_term());
    // FIXME: get constant term, substract one to get the correct constant
    eq=eq-C+plus_one;
    vecteur eq_in(*e2r(v[1],l)._VECTptr);
    vector<polynome> eqp;
    if (!vecteur2vector_polynome(eq_in,l,eqp))
      return zero;
    polynome res(reduce(*eq._POLYptr,eqp.begin(),eqp.end()));
    gen C1(res.constant_term());
    return r2e(res/C1-plus_one,l)+C;
  }
  const string _greduce_s("greduce");
  unary_function_unary __greduce(&_greduce,_greduce_s);
  unary_function_ptr at_greduce (&__greduce);
  
#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
