// -*- mode:C++ ; compile-command: "g++ -I.. -g -c subst.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
using namespace std;
#include <stdexcept>
#include <cmath>
#include "subst.h"
#include "symbolic.h"
#include "sym2poly.h"
#include "unary.h"
#include "usual.h"
#include "derive.h"
#include "intg.h"
#include "prog.h"
#include "lin.h"
#include "solve.h"
#include "plot.h"
#include "modpoly.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  void checkanglemode(GIAC_CONTEXT){
    if (!angle_radian(contextptr)) 
      setsizeerr("This function works only in radian mode");
  }

  // bool quote_subst=false;
  vector <unary_function_ptr> merge(const vector <unary_function_ptr>& v,const vector <unary_function_ptr> & w){
    vector <unary_function_ptr> res(v);
    vector <unary_function_ptr>::const_iterator it=w.begin(),itend=w.end();
    for (;it!=itend;++it){
      res.push_back(*it);
    }
    return res;
  }
  
  vector <gen_op> merge(const vector <gen_op>& v,const vector <gen_op> & w){
    vector <gen_op> res(v);
    vector <gen_op>::const_iterator it=w.begin(),itend=w.end();
    for (;it!=itend;++it){
      res.push_back(*it);
    }
    return res;
  }

  // sin, cos, tan in terms of tan(x/2)
  gen sin2tan2(const gen & e){
    gen a=symb_tan(rdiv(e,plus_two));
    return rdiv(plus_two*a,pow(a,2)+1);
  }

  gen cos2tan2(const gen & e){
    gen a=symb_tan(rdiv(e,plus_two));
    return rdiv(1-pow(a,2),pow(a,2)+1);
  }

  gen tan2tan2(const gen & e){
    gen a=symb_tan(rdiv(e,plus_two));
    return rdiv(plus_two*a,1-pow(a,2));
  }

  // hyperbolic trig to exp
  gen sinh2exp(const gen & e){
    gen a=exp(e);
    return rdiv(a-inv(a),plus_two);
  }

  gen cosh2exp(const gen & e){
    gen a=exp(e);
    return rdiv(a+inv(a),plus_two);
  }

  gen tanh2exp(const gen & e){
    gen a=pow(exp(e),2);
    return rdiv(a-plus_one,a+plus_one);
  }

  // trig to exp
  gen degtorad(const gen & g){
    if (angle_radian(0)) // FIXME GIAC_CONTEXT
      return g;
    return g*deg2rad_e;
  }
  gen radtodeg(const gen & g){
    if (angle_radian(0)) // FIXME GIAC_CONTEXT
      return g;
    return g*rad2deg_e;
  }
  gen sin2exp(const gen & e){
    gen a=exp(cst_i*degtorad(e));
    return rdiv(a-inv(a),plus_two*cst_i);
  }
  gen cos2exp(const gen & e){
    gen a=exp(cst_i*degtorad(e));
    return rdiv(a+inv(a),plus_two);
  }
  gen tan2exp(const gen & e){
    gen a=pow(exp(cst_i*degtorad(e)),2);
    return rdiv(a-plus_one,cst_i*(a+plus_one));
  }

  gen exp2sincos(const gen & e){
    gen a=re(e,0),b=im(e,0); // FIXME GIAC_CONTEXT
    return exp(a)*(cos(radtodeg(b),0)+cst_i*sin(radtodeg(b),0));
  }

  gen tantosincos(const gen & e){
    return rdiv(symb_sin(e),symb_cos(e));
  }

  gen tantosincos2(const gen & e){
    return rdiv(symb_sin(2*e),(1+symb_cos(2*e)));
  }

  gen tantocossin2(const gen & e){
    return rdiv((1-symb_cos(2*e)),symb_sin(2*e));
  }

  gen asintoacos(const gen & e){
    if (angle_radian(0)) // FIXME GIAC_CONTEXT
      return cst_pi_over_2-acos(e,0);
    else
      return 90-acos(e,0);
  }

  gen acostoasin(const gen & e){
    if (angle_radian(0)) // FIXME GIAC_CONTEXT
      return cst_pi_over_2-asin(e,0);
    else
      return 90-asin(e,0);
  }

  gen asintoatan(const gen & e){
    return symb_atan(rdiv(e,sqrt(1-pow(e,plus_two))));
  }

  gen atantoasin(const gen & e){
    return symb_asin(rdiv(e,sqrt(1+pow(e,plus_two))));
  }

  gen acostoatan(const gen & e){
    return cst_pi_over_2-asintoatan(e);
  }

  gen atantoacos(const gen & e){
    return _asin2acos(atantoasin(e));
  }

  gen asin2ln(const gen & g_orig){
    gen g=degtorad(g_orig);
    return cst_i*ln(g+sqrt(pow(g,2)-1))+cst_pi_over_2;
  }

  gen acos2ln(const gen & g_orig){
    gen g=degtorad(g_orig);
    return -cst_i*ln(g+sqrt(pow(g,2)-1));
  }

  gen atan2ln(const gen & g_orig){
    gen g=degtorad(g_orig);
    return rdiv(cst_i*ln(rdiv(cst_i+g,cst_i-g)),plus_two);
  }

  // g=[base, exponant]
  gen trigcospow(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    gen & base=v.front();
    gen & expo=v.back();
    if ( (base.type!=_SYMB) || (expo.type!=_INT_) )
      return symbolic(at_pow,g);
    gen tmpcos=symb_cos(base._SYMBptr->feuille);
    int ediv=expo.val/2,emod=expo.val%2;
    if (base._SYMBptr->sommet==at_sin)
      return pow(1-pow(tmpcos,2),ediv)*pow(base,emod);
    if (base._SYMBptr->sommet==at_tan){
      gen tmp=pow(tmpcos,2);
      tmp=rdiv(plus_one,tmp)-plus_one; // 1/ cos^2 -1
      return pow(tmp,ediv)*pow(rdiv(symb_sin(base._SYMBptr->feuille),tmpcos),emod);
    }
    return symbolic(at_pow,g);
  }

  gen trigsinpow(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    gen & base=v.front();
    gen & expo=v.back();
    if ( (base.type!=_SYMB) || (expo.type!=_INT_) )
      return symbolic(at_pow,g);
    gen tmpsin=symb_sin(base._SYMBptr->feuille);
    int ediv=expo.val/2,emod=expo.val%2;
    if (base._SYMBptr->sommet==at_cos)
      return pow(1-pow(tmpsin,2),ediv)*pow(base,emod);
    if (base._SYMBptr->sommet==at_tan){
      gen tmp=pow(tmpsin,2);
      tmp=rdiv(tmp,plus_one-tmp); // sin^2/ (1-sin^2)
      return pow(tmp,ediv)*pow(rdiv(tmpsin,symb_cos(base._SYMBptr->feuille)),emod);
    }
    return symbolic(at_pow,g);
  }

  gen trigtanpow(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v=*g._VECTptr;
    gen & base=v.front();
    gen & expo=v.back();
    if ( (base.type!=_SYMB) || (expo.type!=_INT_) )
      return symbolic(at_pow,g);
    gen tmptan=symb_tan(base._SYMBptr->feuille);
    int ediv=expo.val/2,emod=expo.val%2;
    if (base._SYMBptr->sommet==at_cos)
      return pow(1+pow(tmptan,2),-ediv)*pow(base,emod);
    if (base._SYMBptr->sommet==at_sin){
      gen tmp=pow(tmptan,2);
      tmp=rdiv(tmp,plus_one+tmp); // tan^2/ (1+tan^2)
      return pow(tmp,ediv)*pow(tmptan*symb_cos(base._SYMBptr->feuille),emod);
    }
    return symbolic(at_pow,g);
  }

  gen powtopowexpand(const gen & g){
    if (g.type!=_VECT)
      setsizeerr();
    vecteur & v(*g._VECTptr);
    if (v.size()!=2)
      setsizeerr();
    if (v[1].type!=_SYMB)
      return symbolic(at_pow,g);
    unary_function_ptr &u=v[1]._SYMBptr->sommet;
    if (u==at_neg)
      return inv(powtopowexpand(makevecteur(v[0],v[1]._SYMBptr->feuille)));
    if ( (v[1]._SYMBptr->feuille.type!=_VECT) || ((u!=at_plus) && (u!=at_prod)) )
      return symbolic(at_pow,g);
    vecteur & w=*v[1]._SYMBptr->feuille._VECTptr;
    const_iterateur it=w.begin(),itend=w.end();
    if (u==at_plus){
      gen res(plus_one);
      for (;it!=itend;++it)
	res=res*powtopowexpand(makevecteur(v[0],*it));
      return res;
    }
    if (u==at_prod){
      if (w.size()!=2)
	return symbolic(at_pow,g);
      if (w[0].type==_INT_)
	return pow(powtopowexpand(makevecteur(v[0],w[1])),w[0]);
      if (w[1].type==_INT_)
	return pow(powtopowexpand(makevecteur(v[0],w[0])),w[1]);
    }
    return symbolic(at_pow,g);
  }

  gen exptopower(const gen & g){
    vecteur l(lop(g,at_ln));
    if (l.size()!=1)
      return symbolic(at_exp,g);
    identificateur tmp(" x");
    gen gg=subst(g,l,vecteur(1,tmp)),a,b;
    if (!is_linear_wrt(gg,tmp,a,b))
      return symbolic(at_exp,g);
    return exp(b)*pow(l[0]._SYMBptr->feuille,a);
  }

  // One substitution of an identifier
  vecteur subst(const vecteur & v,const gen & i,const gen & newi,bool quotesubst){
    vecteur w;
    w.reserve(v.size());
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it)
      w.push_back(subst(*it,i,newi,quotesubst));
    return w;
  }

  gen subst_integrate(const gen & e,const gen & i,const gen & newi,bool quotesubst){
    vecteur v=*e._SYMBptr->feuille._VECTptr;
    int s=v.size();
    if ( (s>1) && (v[1]==i) ){
      vecteur l(*_lname(newi)._VECTptr);
      if (l.empty())
	setsizeerr();
      v[1]=l.front();
      v=subst(v,i,newi,quotesubst);
      v[0]=v[0]*derive(newi,l.front());
      if (s>2){
	identificateur t(" t");
	vecteur w=solve(newi-t,*l.front()._IDNTptr,0); // FIXME GIAC_CONTEXT
	if (w.empty())
	  setsizeerr("Unable to solve");
	v[2]=subst(w.front(),t,v[2],quotesubst);
	if (s>3)
	  v[3]=subst(w.front(),t,v[3],quotesubst);
      }
      return symbolic(at_integrate,v);
    }
    return symbolic(at_integrate,subst(v,i,newi,quotesubst));
  }

  gen subst_derive(const gen & e,const gen & i,const gen & newi,bool quotesubst){
    vecteur & v=*e._SYMBptr->feuille._VECTptr;
    int s=v.size();
    if ( (s==2) && (v[1]==i) ){
      vecteur l(*_lname(newi)._VECTptr);
      if (l.empty())
	setsizeerr();
      v[1]=l.front();
      v=subst(v,i,newi,quotesubst);
      return rdiv(symbolic(at_derive,v),derive(newi,l.front()));
    }
    // Warning: ? return e for desolve (is_linear_diffeq)
    return symbolic(at_derive,subst(v,i,newi,quotesubst));
    // return e;
  }

  gen subst(const gen & e,const gen & i,const gen & newi,bool quotesubst){
    if (i.type==_VECT){
      if (newi.type!=_VECT || i._VECTptr->size()!=newi._VECTptr->size())
	setdimerr();
      return subst(e,*i._VECTptr,*newi._VECTptr,quotesubst);
    }
    switch (e.type){
    case _INT_: case _ZINT: case _CPLX: case _DOUBLE_: case _REAL: case _STRNG:
      return e;
    case _IDNT:
      if (e==i)
	return newi;
      else
	return e;
    case _SYMB:
      if (e==i)
	return newi;
      if ( (e._SYMBptr->sommet==at_pow) && (exp((*(e._SYMBptr->feuille._VECTptr))[1]*ln((*(e._SYMBptr->feuille._VECTptr))[0])) == i) )
	return newi;
      if ( (e._SYMBptr->sommet==at_integrate) && (e._SYMBptr->feuille.type==_VECT) && (i.type==_IDNT) )
	return subst_integrate(e,i,newi,quotesubst);
      if ( (e._SYMBptr->sommet==at_derive) && (e._SYMBptr->feuille.type==_VECT) &&(i.type==_IDNT) )
	return subst_derive(e,i,newi,quotesubst);
      if (e._SYMBptr->sommet.quoted){
	gen res= symbolic(e._SYMBptr->sommet,subst(e._SYMBptr->feuille,i,newi,true));
	return res;
      }
      if (quotesubst)
	return symbolic(e._SYMBptr->sommet,subst(e._SYMBptr->feuille,i,newi,true));
      else
	return e._SYMBptr->sommet(subst(e._SYMBptr->feuille,i,newi,false),0); // FIXME GIAC_CONTEXT
    case _VECT:
      return gen(subst(*e._VECTptr,i,newi,quotesubst),e.subtype);
    case _FRAC:
      if (e==i)
	return newi;
      return fraction(subst(e._FRACptr->num,i,newi,quotesubst),subst(e._FRACptr->den,i,newi,quotesubst));
    default:
      settypeerr();
    }
  }

  gen quotesubst(const gen & e,const gen & i,const gen & newi){
    gen res(subst(e,i,newi,true));
    return res;
  }

  sparse_poly1 subst(const sparse_poly1 & v,const gen & i,const gen & newi,bool quotesubst){
    sparse_poly1 p;
    sparse_poly1::const_iterator it=v.begin(),itend=v.end();
    p.reserve(itend-it);
    gen e;
    for (;it!=itend;++it){
      e=recursive_normal(subst(it->coeff,i,newi,quotesubst),0); // ? FIXME GIAC_CONTEXT
      if (!is_zero(e))
	p.push_back(monome(e,it->exponent));
    }
    return p;
  }

  // Multiple substitutions
  vecteur subst(const vecteur & v,const vecteur & i,const vecteur & newi,bool quotesubst){
    if (i.empty())
      return v;
    vecteur w;
    w.reserve(v.size());
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it)
      w.push_back(subst(*it,i,newi,quotesubst));
    return w;
  }

  gen subst(const gen & e,const vecteur & i,const vecteur & newi,bool quotesubst){
    if (i.empty())
      return e;
    int pos;
    switch (e.type){
    case _INT_: case _ZINT: case _DOUBLE_: case _CPLX: case _REAL:
      return e;
    case _IDNT:
      pos=equalposcomp(i,e);
      if (pos)
	return newi[pos-1];
      else
	return e;
    case _SYMB:
      pos=equalposcomp(i,e);
      if (pos)
	return newi[pos-1];
      if (e._SYMBptr->sommet==at_pow){ 
	pos=equalposcomp(i,exp((*(e._SYMBptr->feuille._VECTptr))[1]*ln((*(e._SYMBptr->feuille._VECTptr))[0]))  );
	if (pos)
	  return newi[pos-1];
      }
      if (e._SYMBptr->feuille.type==_VECT){
	gen ef(subst(*e._SYMBptr->feuille._VECTptr,i,newi,quotesubst));
	ef.subtype=e._SYMBptr->feuille.subtype;
	return symbolic(e._SYMBptr->sommet,ef);
      }
      else
	return symbolic(e._SYMBptr->sommet,subst(e._SYMBptr->feuille,i,newi,quotesubst));
    case _VECT:
      return gen(subst(*e._VECTptr,i,newi,quotesubst),e.subtype);
    case _FRAC:
      pos=equalposcomp(i,e);
      if (pos)
	return newi[pos-1];
      return fraction(subst(e._FRACptr->num,i,newi,quotesubst),subst(e._FRACptr->den,i,newi,quotesubst));
    default:
      pos=equalposcomp(i,e);
      if (pos)
	return newi[pos-1];
      return e;
    }
  }


  gen subst(const gen & e,const vector<unary_function_ptr> & v,const vector< gen_op > & w,bool quotesubst){
    if (v.empty())
      return e;
    if (e.type==_VECT){
      const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
      vecteur res;
      res.reserve(itend-it);
      for (;it!=itend;++it)
	res.push_back(subst(*it,v,w,quotesubst));
      return gen(res,e.subtype);
    }
    if (e.type!=_SYMB)
      return e;
    gen arg=subst(e._SYMBptr->feuille,v,w,quotesubst);
    int n=equalposcomp(v,e._SYMBptr->sommet);
    if (!n){
      if (quotesubst){
	gen res=symbolic(e._SYMBptr->sommet,arg);
	res.subtype=e.subtype;
	return res;
      }
      return e._SYMBptr->sommet(arg,0); // FIXME GIAC_CONTEXT
    }
    gen tmp=w[n-1](arg);
    return tmp;
  }

  gen halftan(const gen & e){
    return subst(e,sincostan_v,halftan_v);
  }

  gen _halftan(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_halftan);
    return halftan(args);
  }
  const string _halftan_s("halftan");
  unary_function_unary __halftan(&giac::_halftan,_halftan_s);
  unary_function_ptr at_halftan (&__halftan);

  gen inv_test_exp(const gen & e){
    if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_exp))
      return symbolic(at_exp,-e._SYMBptr->feuille);
    return inv(e);
  }

  gen rewrite_hyper(const gen & e){
    vector<unary_function_ptr> vu(sinhcoshtanh_v);
    vu.push_back(at_inv);
    vector <gen_op> vv(hyp2exp_v);
    vv.push_back(inv_test_exp);
    return subst(e,vu,vv);
  }

  gen hyp2exp(const gen & e){
    return subst(e,sinhcoshtanh_v,hyp2exp_v);
  }
  gen _hyp2exp(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_hyp2exp);
    return hyp2exp(args);
  }
  const string _hyp2exp_s("hyp2exp");
  unary_function_unary __hyp2exp(&giac::_hyp2exp,_hyp2exp_s);
  unary_function_ptr at_hyp2exp (&__hyp2exp);

  gen sincos(const gen & e,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return subst(tan2sincos(e),exp_v,exp2sincos_v);
    else
      return e;
  }
  gen _sincos(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_sincos,contextptr);
    return sincos(args,contextptr);
  }
  const string _sincos_s("sincos");
  unary_function_eval __sincos(&giac::_sincos,_sincos_s);
  unary_function_ptr at_sincos (&__sincos);

  gen trig2exp(const gen & e,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return subst(e,sincostan_v,trig2exp_v);
    else
      return e;
  }
  gen _trig2exp(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_trig2exp,contextptr);
    return trig2exp(args,contextptr);
  }
  const string _trig2exp_s("trig2exp");
  unary_function_eval __trig2exp(&giac::_trig2exp,_trig2exp_s);
  unary_function_ptr at_trig2exp (&__trig2exp);

  gen halftan_hyp2exp(const gen & e){
    return subst(e,sincostansinhcoshtanh_v,halftan_hyp2exp_v);
  }

  gen _halftan_hyp2exp(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_halftan_hyp2exp);
    return halftan_hyp2exp(args);
  }
  const string _halftan_hyp2exp_s("halftan_hyp2exp");
  unary_function_unary __halftan_hyp2exp(&giac::_halftan_hyp2exp,_halftan_hyp2exp_s);
  unary_function_ptr at_halftan_hyp2exp (&__halftan_hyp2exp);

  gen asin2acos(const gen & e){
    return subst(e,asin_v,asin2acos_v);
  }
  gen _asin2acos(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_asin2acos);
    return asin2acos(args);
  }
  const string _asin2acos_s("asin2acos");
  unary_function_unary __asin2acos(&giac::_asin2acos,_asin2acos_s);
  unary_function_ptr at_asin2acos (&__asin2acos);

  gen asin2atan(const gen & e){
    return subst(e,asin_v,asin2atan_v);
  }
  gen _asin2atan(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_asin2atan);
    return asin2atan(args);
  }
  const string _asin2atan_s("asin2atan");
  unary_function_unary __asin2atan(&giac::_asin2atan,_asin2atan_s);
  unary_function_ptr at_asin2atan (&__asin2atan);

  gen acos2asin(const gen & e){
    return subst(e,acos_v,acos2asin_v);
  }
  gen _acos2asin(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,acos2asin);
    return acos2asin(args);
  }
  const string _acos2asin_s("acos2asin");
  unary_function_unary __acos2asin(&giac::_acos2asin,_acos2asin_s);
  unary_function_ptr at_acos2asin (&__acos2asin);

  gen acos2atan(const gen & e){
    return subst(e,acos_v,acos2atan_v);
  }
  gen _acos2atan(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_acos2atan);
    return acos2atan(args);
  }
  const string _acos2atan_s("acos2atan");
  unary_function_unary __acos2atan(&giac::_acos2atan,_acos2atan_s);
  unary_function_ptr at_acos2atan (&__acos2atan);

  gen atan2asin(const gen & e){
    return subst(e,atan_v,atan2asin_v);
  }
  gen _atan2asin(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_atan2asin);
    return atan2asin(args);
  }
  const string _atan2asin_s("atan2asin");
  unary_function_unary __atan2asin(&giac::_atan2asin,_atan2asin_s);
  unary_function_ptr at_atan2asin (&__atan2asin);

  gen atan2acos(const gen & e){
    return subst(e,atan_v,atan2acos_v);
  }
  gen _atan2acos(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_atan2acos);
    return atan2acos(args);
  }
  const string _atan2acos_s("atan2acos");
  unary_function_unary __atan2acos(&giac::_atan2acos,_atan2acos_s);
  unary_function_ptr at_atan2acos (&__atan2acos);

  gen atrig2ln(const gen & e,GIAC_CONTEXT){
    if (angle_radian(contextptr))
      return subst(e,asinacosatan_v,atrig2ln_v);
    else
      return e;
  }
  gen _atrig2ln(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_atrig2ln,contextptr);
    return atrig2ln(args,contextptr);
  }
  const string _atrig2ln_s("atrig2ln");
  unary_function_eval __atrig2ln(&giac::_atrig2ln,_atrig2ln_s);
  unary_function_ptr at_atrig2ln (&__atrig2ln);

  bool is_rational(const gen & g){
    if (is_integer(g))
      return true;
    if (g.type!=_FRAC)
      return false;
    return is_integer(g._FRACptr->num) && is_integer(g._FRACptr->den);
  }
  // if g is a symbolic depending linearly and rationnaly on a ln, 
  // factors out this term before taking the exp
  gen rewrite_strong_exp(const gen & g_orig){
    if (g_orig.type!=_SYMB)
      return exp(g_orig);
    gen g(g_orig),res(plus_one);
    vecteur v(lop(lvar(g),at_ln));
    v.insert(v.begin(),cst_pi);
    int s=v.size();
    identificateur t(" t");
    for (int i=0;i<s;++i){
      gen gt=quotesubst(g,v[i],t);
      gen dg=normal(subst(derive(gt,t),t,zero));
      if (!i)
	dg=dg/cst_i;
      if (is_zero(dg))
	continue;
      if (!is_rational(dg))
	continue;
      g=g-dg*v[i];
      if (!i)
	res=res*exp(cst_i*cst_pi*dg);
      else
	res=res*pow(v[i]._SYMBptr->feuille,dg);
    }
    return res*exp(normal(g));
  }

  // After extracting the cst coeff of g
  // if g is a linear combination of the components of wrt, 
  // this will return the coeffs of the linear comb
  // otherwise it will add g at the end of wrt and return [0...0 1]
  // It assumes that g and the coeff of wrt are multivariate rat. fractions
  vecteur as_linear_combination(const gen & g,vecteur & wrt ){
    vecteur v(wrt);
    v.push_back(g);
    gen d;
    lcmdeno(v,d);
    gen cl=v.back();
    int n=wrt.size();
    vecteur res(n),mat;
    if (cl.type!=_POLY){ // search for a non poly in wrt
      gen gg;
      int i;
      for (i=0;i<n;++i){
	if (v[i].type!=_POLY){
	  gg=rdiv(g,wrt[i]);
	  if (gg.type==_INT_)
	    break;
	  if ( (gg.type==_FRAC) && (gg._FRACptr->num.type==_INT_) && (gg._FRACptr->den.type==_INT_) )
	    break;
	}
      }
      if (i==n){
	wrt.push_back(g);
	res.push_back(plus_one);
      }
      else
	res[i]=gg;
      return res;
    }
    // we are now back to express cl as linear comb with integer coeff
    // but here in multivariate polynomials
    // we build now a linear system and solve it
    polynome & p =*cl._POLYptr;
    vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
    for (;it!=itend;++it){
      const index_m & i=it->index;
      vecteur tmp(n+1);
      tmp.back()=it->value;
      for (int j=0,k;j<n;++j){
	gen & gg=v[j];
	if (gg.type==_POLY){ 
	  if ( (k=gg._POLYptr->position(i)) >=0 )
	    tmp[j]=gg._POLYptr->coord[k].value;
	}
	else {
	  if (is_zero(*i.iptr))
	    tmp[j]=gg;
	}
      }
      mat.push_back(tmp);
    }
    // add monomials that are not inside the last polynomial
    polynome fait(p);
    for (int j=0,k;j<n;++j){
      gen & gg=v[j];
      if (gg.type!=_POLY)
	continue;
      polynome & pj =*gg._POLYptr;
      vector< monomial<gen> >::const_iterator it=pj.coord.begin(),itend=pj.coord.end();
      for (;it!=itend;++it){
	const index_m & i=it->index;
	k=fait.position(i);
	if (k>=0 ) // this monomial is already in the system
	  continue;
	fait=fait+monomial<gen>(plus_one,i);
	// make a new line in the system for this new monomial
	vecteur tmp(n+1);
	tmp.back()=zero;
	for (int J=0,K;J<n;++J){
	  gen & GG=v[J];
	  if (GG.type==_POLY){ 
	    if ( (K=GG._POLYptr->position(i)) >=0 )
	      tmp[J]=GG._POLYptr->coord[K].value;
	  }
	  else {
	    if (is_zero(*i.iptr))
	      tmp[J]=GG;
	  }
	}
	mat.push_back(tmp);
      }
    }
    // take re and im since coeffs are reals
    // FIXME: should have a better algo for rational coeffs
    mat=mergevecteur(*re(mat,0)._VECTptr,*im(mat,0)._VECTptr);
    // find solutions in kernel of mat
    matrice noyau=mker(mat);
    // try each row of noyau that has a non-zero last coeff
    // since the first row coeff is -1 all coeff must be rationals
    int ns=noyau.size();
    for (int i=0;i<ns;++i){
      vecteur & w=*noyau[i]._VECTptr;
      if (is_zero(w.back()))
	continue;
      gen gg;
      lcmdeno(w,gg);
      // check that all coeff are integers
      const_iterateur jt=w.begin(),jtend=w.end();
      for (;jt!=jtend;++jt){
	if (jt->type!=_INT_)
	  break;
      }
      if (jt!=jtend)
	continue;
      // additional check: w.v=0
      gen check=dotvecteur(w,v);
      if (!is_zero(check))
	continue;
      gg=w.back();
      w.pop_back();
      res=divvecteur(w,-gg);
      return res;
    }
    // no solution found
    wrt.push_back(g);
    res.push_back(plus_one);
    return res;
  }

  bool is_unit(const gen & g){
    if (g.type==_INT_)
      return (g.val==1) || (g.val==-1);
    if (g.type==_CPLX)
      return (g._CPLXptr->type==_INT_) && (g._CPLXptr->val==0) && ( (g._CPLXptr+1)->type==_INT_) && ( ( (g._CPLXptr+1)->val==1) || ( (g._CPLXptr+1)->val==-1) );
    if (g.type==_POLY)
      return (g._POLYptr->coord.size()==1) && (is_zero(*g._POLYptr->coord.front().index.iptr)) && is_unit(g._POLYptr->coord.front().value);
    return false;
  }

  // Complete a list of prime together factors primeargs such that
  // a is a product of these factors
  void decompose(const gen &a_orig,vecteur & primeargs,int begin=0){
    if (is_unit(a_orig))
      return;
    gen a(a_orig);
    for (int i=begin;i<primeargs.size();){
      gen g=simplify3(a,primeargs[i]);
      if (is_unit(g)){ // prime together -> go to the next element of primeargs
	++i;
	continue;
      }
      if (is_unit(primeargs[i])){
	primeargs[i]=g;
	continue;
      }
      decompose(g,primeargs,i);
    }
    if (!is_unit(a))
      primeargs.push_back(a);
  }

  // FIXME GIAC_CONTEXT?
  gen branch_evalf(const gen & g){
    vecteur v(*_lname(evalf(g))._VECTptr);
    gen gg(g);
    int s=v.size();
    for (int i=0;i<s;++i)
      gg=limit(gg,*v[i]._IDNTptr,0,1,0);
    return evalf(gg);
  }
  gen expanded_ln(const gen & a_orig,const vecteur & primeargs,const vecteur & lnprimeargs,const vecteur & vars){
    gen res,gg,a=a_orig;
    int k;
    int p=primeargs.size();
    for (int j=0;j<p;++j){
      for (k=0;;++k){
	gen gp=primeargs[j];
	gg=simplify3(a,gp);
	if (is_unit(gg))
	  break;
      }
      if (k)
	res=res+gen(k)*lnprimeargs[j];
      res=res+ln(r2e(gg,vars));
    }
    res=res+ln(r2e(a,vars));
    gg=re(branch_evalf(rdiv(ln(r2e(a_orig,vars))-res,cst_pi_over_2*cst_i)),0); // FIXME GIAC_CONTEXT
    if (gg.type==_DOUBLE_)
      res=res+cst_pi_over_2*cst_i*gen(int(floor(gg._DOUBLE_val+0.5)));
    return res;
  }

  polynome gen2poly(const gen & g,int s){
    if (g.type==_POLY)
      return *g._POLYptr;
    else
      return polynome(g,s);
  }
  // If g has exponential variables, factors out powers of
  // these exponential variables and return the ln of g simplified
  // plus the arg of the exponential vars
  gen simplifylnarg(const gen & g){
    gen res;
    vecteur l(lvar(g));
    int s=l.size();
    fraction f(sym2r(g,l));
    gen nf,df;
    fxnd(f,nf,df);
    polynome n(gen2poly(nf,s)),d(gen2poly(df,s));
    // for every exp variable inside l, look if n or d has a valuation
    // with respect to this degree
    const_iterateur it=l.begin(),itend=l.end();
    for (int i=0;it!=itend;++it,++i){
      if (!it->is_symb_of_sommet(at_exp))
	continue;
      index_t shift_index(s);
      int nv=n.valuation(i);
      shift_index[i]=-nv;
      n=n.shift(shift_index);
      int dv=d.valuation(i);
      shift_index[i]=-dv;
      d=d.shift(shift_index);
      int v=nv-dv;
      res=res+v*it->_SYMBptr->feuille;
    }
    res=res+ln(r2sym(n,l)/r2sym(d,l));
    return res;
  }
  gen simplifylnexp(const gen & g){
    vecteur l(lop(g,at_ln));
    int s=l.size();
    if (!s)
      return g;
    vecteur lsub;
    for (int i=0;i<s;++i)
      lsub.push_back(simplifylnarg(l[i]._SYMBptr->feuille));
    return quotesubst(g,l,lsub);
  }

  void rlvar(const gen &e,vecteur & res,bool alg){
    vecteur v;
    if (alg){
      vecteur tmp=alg_lvar(e);
      // make one list from the matrix
      const_iterateur it=tmp.begin(),itend=tmp.end();
      for (;it!=itend;++it){
	if (it->type==_VECT){
	  const_iterateur jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();
	  for (;jt!=jtend;++jt){
	    if (!equalposcomp(v,*jt))
	      v.push_back(*jt);
	  }
	}
      }
    }
    else
      v=lvar(e);
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it){
      if (!equalposcomp(res,*it)){
	// recursive call
	res.push_back(*it);
	if (it->type==_SYMB) {
	  rlvar(it->_SYMBptr->feuille,res,alg);
	  if (it->_SYMBptr->sommet==at_pow) 
	    rlvar(ln((*(it->_SYMBptr->feuille._VECTptr))[0]),res,alg);
	}
      }
    }
  }

  // recursive (alg) list of var
  vecteur rlvar(const gen &e,bool alg){
    vecteur res;
    rlvar(e,res,alg);
    sort(res.begin(),res.end(),symb_size_less);
    return res;
  }


  gen _pow2exp(const gen & e){
    if (e.type==_VECT){
      const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
      vecteur v;
      v.reserve(itend-it);
      for (;it!=itend;++it)
	v.push_back(_pow2exp(*it));
      return v;
    }
    if (e.type!=_SYMB)
      return e;
    if ( e._SYMBptr->sommet==at_pow && e._SYMBptr->feuille.type==_VECT && e._SYMBptr->feuille._VECTptr->size()==2){ 
      vecteur & v=*e._SYMBptr->feuille._VECTptr;
      if (v[1].type!=_INT_ && v[1].type!=_FRAC)
	return exp(_pow2exp(v[1])*_pow2exp(ln(v[0])));
    }
    return e._SYMBptr->sommet(_pow2exp(e._SYMBptr->feuille),0); // FIXME GIAC_CONTEXT
  }
  const string _pow2exp_s("pow2exp");
  unary_function_unary __pow2exp(&_pow2exp,_pow2exp_s);
  unary_function_ptr at_pow2exp (&__pow2exp,0,true);

  gen pow2expln(const gen & e,const identificateur & x){
    if (e.type==_VECT){
      const_iterateur it=e._VECTptr->begin(),itend=e._VECTptr->end();
      vecteur v;
      v.reserve(itend-it);
      for (;it!=itend;++it)
	v.push_back(pow2expln(*it,x));
      return v;
    }
    if (e.type!=_SYMB)
      return e;
    if (e._SYMBptr->feuille.type==_VECT){
      vecteur & v=*e._SYMBptr->feuille._VECTptr;
      if ((e._SYMBptr->sommet==at_pow)  && ( contains(v[1],x) ||(v[1].type!=_INT_ && contains(v[0],x) ) ) )
	return symb_exp(pow2expln(v[1],x)*symb_ln(pow2expln(v[0],x)));
    }
    return e._SYMBptr->sommet(pow2expln(e._SYMBptr->feuille,x),0); // FIXME GIAC_CONTEXT
  }

  gen pow2expln(const gen & e){
    if (e.type==_VECT)
      return apply(e,pow2expln);
    if (e.type!=_SYMB)
      return e;
    if (e._SYMBptr->feuille.type==_VECT){
      vecteur & v=*e._SYMBptr->feuille._VECTptr;
      if (e._SYMBptr->sommet==at_pow  && v[1].type!=_INT_)
	return symb_exp(pow2expln(v[1])*symb_ln(pow2expln(v[0])));
    }
    return e._SYMBptr->sommet(pow2expln(e._SYMBptr->feuille),0); // FIXME GIAC_CONTEXT
  }

  gen simplifyfactorial(const gen & g){
    vecteur l(lop(g,at_factorial));
    int s=l.size();
    if (s<2)
      return g;
    // look if difference of arguments in l are integers
    vecteur newl(s);
    // newl will contain couples of arg of factorials and integers shift
    newl[0]=makevecteur(l[0]._SYMBptr->feuille,zero);
    for (int i=1;i<s;++i){
      // look in newl for an arg of factorial with integer difference
      gen current=l[i]._SYMBptr->feuille;
      newl[i]=makevecteur(current,zero);
      for (int j=0;j<i;++j){
	if (!is_zero(newl[j][1]))
	  continue;
	gen base=newl[j][0];
	gen difference=current-base;
	difference=simplify(difference); // recursive call
	if (difference.type!=_INT_)
	  continue;
	int k=difference.val;
	if (k>=0){
	  newl[i]=makevecteur(base,difference);
	  break;
	}
	// current is the new factorial basis, change base in newl
	newl[i]=makevecteur(current,zero);
	for (k=0;k<i;++k){
	  vecteur & n=*newl[k]._VECTptr;
	  if (n[0]==base){
	    n[0]=current;
	    n[1]=n[1]-difference;
	  }
	}
	break;
      }
    }
    // now replace in newl the couple by a factorial*a product
    for (int i=0;i<s;++i){
      gen base=newl[i][0];
      int shift=newl[i][1].val;
      gen product=plus_one;
      for (int j=shift;j>0;--j){
	product=product*(base+j);
      }
      newl[i]=product*symbolic(at_factorial,base);
    }
    return subst(g,l,newl);
  }

  gen gammatofactorial(const gen & g){
    return symbolic(at_factorial,g-1);
  }

  gen gamma2factorial(const gen & g){
    return subst(g,gamma_v,gamma2factorial_v);
  }

  gen tsimplify_common(const gen & e,GIAC_CONTEXT){
    gen g=pow2expln(e);
    g=gamma2factorial(g);
    g=simplifyfactorial(g);
    // analyse of args of ln
    g=simplifylnexp(g);
    vecteur l(lop(g,at_ln));
    int s=l.size();
    if (s>1){
      vecteur argln(s);
      for (int i=0;i<s;++i)
	argln[i]=tsimplify(l[i]._SYMBptr->feuille,contextptr);
      // check for common factors between args
      // now we make a vector of prime together values and rewrite
      // each arg as a product of these values + a multiple of 2*pi*i
      // arg <--> [ powers multiple_2*pi ]
      // Initialization
      vecteur vars(lvar(argln));
      for (int i=0;i<s;++i)
	argln[i]=e2r(argln[i],vars);
      gen num,den;
      fxnd(argln[0],num,den);
      vecteur primeargs(1,num);
      decompose(den,primeargs);
      for (int i=1;i<s;++i){
	// check for common factors between argln[i] and primeargs
	fxnd(argln[i],num,den);
	decompose(num,primeargs);
	decompose(den,primeargs);
      }
      // now rewrite ln[argln[i]] as a sum of ln[primeargs[]]
      int p=primeargs.size();
      vecteur lnprimeargs(*apply(r2e(primeargs,vars),at_ln)._VECTptr);
      vecteur newln(s);
      for (int i=0;i<s;++i){
	fxnd(argln[i],num,den);	
	newln[i]=expanded_ln(num,primeargs,lnprimeargs,vars)-expanded_ln(den,primeargs,lnprimeargs,vars);
	gen gg=re(branch_evalf(rdiv(l[i]-newln[i],cst_two_pi*cst_i)),0); // FIXME GIAC_CONTEXT
	if (gg.type==_DOUBLE_)
	  newln[i]=newln[i]+cst_two_pi*cst_i*gen(int(floor(gg._DOUBLE_val+0.5)));
       }
      g=subst(g,l,newln);
    }
    // analyse of arguments of exp:
    l=lop(g,at_exp);
    s=l.size();
    if (!s)
      return recursive_normal(g,0); // FIXME GIAC_CONTEXT
    // recursively simplify inside exp
    vecteur newl(s); // vector of args of the exponential
    for (int i=0;i<s;++i)
      newl[i]=tsimplify(l[i]._SYMBptr->feuille,contextptr);
    // check for linear relations with rational coefficients between args
    // add i*pi and ln to the linear relations checking
    // First convert everything to multivariate fractions
    vecteur vars(1,cst_pi);
    lvar(newl,vars);
    vecteur ln_vars(lop(newl,at_ln));
    vecteur independant(*e2r(ln_vars,vars)._VECTptr);
    int n_ln=independant.size();
    independant.push_back(e2r(cst_i*cst_pi,vars));
    matrice m;
    for (int i=0;i<s;++i){
      m.push_back(as_linear_combination(e2r(newl[i],vars),independant));
    }
    // we have a matrix of rational numbers and a vector "independant"
    // so that newl[i]=i-th line of the matrix dot independant
    // For each column of the matrix we take the lcm of the denominators
    // and multiply the column by the lcm, dividing the corresponding
    // coeff of independant
    // then we use exp[r2e(independant,vars)]^[i-th line] for exp[newl[i]]
    // we do the substitution l by exp[newl] in g
    // and we return normal(g)
    // First make m a rectangular array
    int c=m.back()._VECTptr->size(),r=m.size();
    for (int i=0;i<r;++i){
      int ms=m[i]._VECTptr->size();
      if (ms<c)
	m[i]=mergevecteur(*m[i]._VECTptr,vecteur(c-ms,zero));
    }
    // lcm
    matrice mt=mtran(m);
    gen lcmg;
    for (int i=0;i<c;++i){
      lcmdeno(*mt[i]._VECTptr,lcmg);
      // check for arg of the form ln()/deno
      if (i<n_ln)
	independant[i]=pow(ln_vars[i]._SYMBptr->feuille,inv(lcmg));
      else
	independant[i]=rewrite_strong_exp(r2e(rdiv(independant[i],lcmg),vars));
    }
    m=mtran(mt);
    // compute exp[newl]
    for (int i=0;i<s;++i){
      vecteur & ligne=*m[i]._VECTptr;
      gen res(plus_one);
      for (int j=0;j<c;++j){
	res=res*pow(independant[j],ligne[j]);
      }
      newl[i]=res;
    }
    g=subst(g,l,newl);
    return normal(g); 
  }
  gen tsimplify(const gen & e,GIAC_CONTEXT){
    // replace trig/inv trig expressions with exp/ln
    gen g=trig2exp(e,contextptr);
    g=atrig2ln(g,contextptr);
    return tsimplify_common(g,contextptr);
  }
  gen tsimplify_noexpln(const gen & e,int s1,int s2,GIAC_CONTEXT){
    gen g=e;
    if (s1>1)
      g=trig2exp(e,contextptr);
    if (s2>1)
      g=atrig2ln(g,contextptr);
    return tsimplify_common(g,contextptr);
  }
  gen _tsimplify(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_tsimplify,contextptr);
    return tsimplify(args,contextptr);
  }
  const string _tsimplify_s("tsimplify");
  unary_function_eval __tsimplify(&giac::_tsimplify,_tsimplify_s);
  unary_function_ptr at_tsimplify (&__tsimplify);

  // find symbolic vars in g that have u has sommet
  vecteur lop(const gen & g,const unary_function_ptr & u){
    if (g.type==_SYMB) {
      if (g._SYMBptr->sommet==u) 
	return vecteur(1,g);
      else
	return lop(g._SYMBptr->feuille,u);
    }
    if (g.type!=_VECT)
      return vecteur(0);
    vecteur res;
    const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
    for (;it!=itend;++it)
      res=mergeset(res,lop(*it,u));
    return res;
  }

  // find symbolic vars in g that have u has sommet
  vecteur lop(const gen & g,const vector<unary_function_ptr> & v){
    if (g.type==_SYMB) {
      if (equalposcomp(v,g._SYMBptr->sommet))
	return vecteur(1,g);
      else
	return lop(g._SYMBptr->feuille,v);
    }
    if (g.type!=_VECT)
      return vecteur(0);
    vecteur res;
    const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
    for (;it!=itend;++it)
      res=mergeset(res,lop(*it,v));
    return res;
  }

  gen expln2trig(const gen & g,GIAC_CONTEXT){
    if (g.type==_VECT)
      return apply(g,contextptr,expln2trig);
    if (g.type!=_SYMB)
      return g;
    if (g._SYMBptr->sommet==at_inv){
      // inv[*]=*[inv]
      // inv[exp]=exp[neg]
      const gen & f=g._SYMBptr->feuille;
      if (f.type==_SYMB){ 
	const unary_function_ptr & s=f._SYMBptr->sommet;
	const gen & ff=f._SYMBptr->feuille;
	if (s==at_exp)
	  return expln2trig(exp(-ff),contextptr);
	if (s==at_prod)
	  return _prod(expln2trig(inv(ff),contextptr));
	if (s==at_pow)
	  return pow(expln2trig(inv(ff._VECTptr->front()),contextptr),ff._VECTptr->back());
      }	
      // otherwise multiply by the conjugate
      gen tmp=expln2trig(g._SYMBptr->feuille,contextptr);
      gen retmp=re(tmp,contextptr); 
      gen imtmp=im(tmp,contextptr); 
      return (retmp-cst_i*imtmp)*inv(pow(retmp,2)+pow(imtmp,2));
    }
    if (g._SYMBptr->sommet==at_exp)
      return sincos(g,contextptr);
    gen f=normal(expln2trig(g._SYMBptr->feuille,contextptr));
    if (g._SYMBptr->sommet==at_ln){
      gen gre=re(f,contextptr); 
      gen gim=im(f,contextptr); 
      if (is_zero(gre))
	return ln(pow(gim,2))/2+sign(gim,contextptr)*cst_i*cst_pi_over_2;
      if (is_zero(gim)){
	if (complex_mode(contextptr))
	  return rdiv(ln(pow(gre,2)),plus_two)+cst_i*(plus_one-sign(gre,contextptr))*cst_pi_over_2;
	else
	  return ln(f);
      }
      return rdiv(ln(pow(gre,2)+pow(gim,2)),plus_two)+cst_i*(atan(gim/gre,contextptr)+sign(gim,contextptr)*(plus_one-sign(gre,contextptr))*cst_pi_over_2);
    }
    return symbolic(g._SYMBptr->sommet,f);
  }

  gen gen_feuille(const gen & g){
    if (g.type==_SYMB)
      return g._SYMBptr->feuille;
    return g;
  }

  gen simplify(const gen & e_orig){
    gen e=e_orig;
    vecteur vabs(lop(e,at_abs));
    vecteur vabs2(vabs);
    iterateur it=vabs2.begin(),itend=vabs2.end();
    for (;it!=itend;++it){
      *it=symbolic(at_abs,factor(it->_SYMBptr->feuille));
    }
    e=quotesubst(e,vabs,vabs2);
    vabs=lop(e,at_pow);
    vabs2=vabs;
    it=vabs2.begin(); itend=vabs2.end();
    for (;it!=itend;++it){
      gen tmp=it->_SYMBptr->feuille;
      if (tmp.type==_VECT && tmp._VECTptr->size()==2)
	*it=normal(pow(simplify(tmp._VECTptr->front()),tmp._VECTptr->back()));
    }
    e=recursive_normal(quotesubst(e,vabs,vabs2),0); // FIXME GIAC_CONTEXT
    // Don't touch fractional powers and absolute value anymore
    vabs2=mergevecteur(lop(e,at_abs),lop(e,at_pow));
    int s=vabs2.size();
    vabs.clear();
    for (int i=0;i<s;++i){
      if (!vabs2[i].is_symb_of_sommet(at_pow)){
	vabs.push_back(vabs2[i]);
	continue;
      }
      gen tmp=vabs2[i]._SYMBptr->feuille;
      if (tmp.type==_VECT && tmp._VECTptr->size()==2 && tmp._VECTptr->back().type==_FRAC)
	vabs.push_back(vabs2[i]);
    }
    s=vabs.size();
    vabs2=vecteur(s);
    for (int i=0;i<s;++i){
      vabs2[i]=identificateur(" "+print_INT_(i));
    }
    e=quotesubst(e,vabs,vabs2);
    // check for the presence of trig/atrig functions
    vecteur v1(lop(e,sincostan_v));
    int s1=v1.size(),s2=lop(e,asinacosatan_v).size();
    gen g=tsimplify_noexpln(e,s1,s2,0); // FIXME GIAC_CONTEXT
    g=_exp2pow(g);
    g=quotesubst(g,vabs2,vabs);
    if (s1<=1 && s2<= 1)
      return g;
    // convert back to trig and atrig functions
    g=expln2trig(g,0); // FIXME GIAC_CONTEXT
    if (!complex_mode(0) && !has_i(g)){ // FIXME GIAC_CONTEXT
      if (s1){
	if (v1.front().is_symb_of_sommet(at_sin))
	  return recursive_normal(trigsin(g),0); // FIXME GIAC_CONTEXT
      }
      return recursive_normal(trigcos(g),0); // FIXME GIAC_CONTEXT and below
    }
    gen reg=recursive_normal(re(g,0),0),img=recursive_normal(im(g,0),0);
    if (s1){
      if (v1.front().is_symb_of_sommet(at_sin))
	return normal(trigsin(reg))+cst_i*normal(trigsin(img));
    }
    g=normal(trigcos(reg))+cst_i*normal(trigcos(img));
    return g;
  }
  gen _simplify(const gen & args){
    if (args.type==_VECT)
      return apply(args,_simplify);
    if (is_equal(args))
      return apply_to_equal(args,_simplify);
    return simplify(args);
  }
  const string _simplify_s("simplify");
  unary_function_unary __simplify(&giac::_simplify,_simplify_s);
  unary_function_ptr at_simplify (&__simplify);

  gen trigcos(const gen & e){
    return subst(e,pow_v,trigcos_v);
  }
  gen _trigcos(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_trigcos);
    return normal(trigcos(args));
  }
  const string _trigcos_s("trigcos");
  unary_function_unary __trigcos(&giac::_trigcos,_trigcos_s);
  unary_function_ptr at_trigcos (&__trigcos);

  gen trigsin(const gen & e){
    return subst(e,pow_v,trigsin_v);
  }
  gen _trigsin(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_trigsin);
    return normal(trigsin(args));
  }
  const string _trigsin_s("trigsin");
  unary_function_unary __trigsin(&giac::_trigsin,_trigsin_s);
  unary_function_ptr at_trigsin (&__trigsin);

  gen trigtan(const gen & e){
    return subst(e,pow_v,trigtan_v);
  }
  gen _trigtan(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_trigtan);
    return normal(trigtan(args));
  }
  const string _trigtan_s("trigtan");
  unary_function_unary __trigtan(&giac::_trigtan,_trigtan_s);
  unary_function_ptr at_trigtan (&__trigtan);

  gen tan2sincos(const gen & e){
    return subst(e,tan_v,tan2sincos_v);
  }
  gen _tan2sincos(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_tan2sincos);
    return tan2sincos(args);
  }
  const string _tan2sincos_s("tan2sincos");
  unary_function_unary __tan2sincos(&giac::_tan2sincos,_tan2sincos_s);
  unary_function_ptr at_tan2sincos (&__tan2sincos);

  gen tan2sincos2(const gen & e){
    return subst(e,tan_v,tan2sincos2_v);
  }
  gen _tan2sincos2(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_tan2sincos2);
    return tan2sincos2(args);
  }
  const string _tan2sincos2_s("tan2sincos2");
  unary_function_unary __tan2sincos2(&giac::_tan2sincos2,_tan2sincos2_s);
  unary_function_ptr at_tan2sincos2 (&__tan2sincos2);

  gen tan2cossin2(const gen & e){
    return subst(e,tan_v,tan2cossin2_v);
  }
  gen _tan2cossin2(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_tan2cossin2);
    return tan2cossin2(args);
  }
  const string _tan2cossin2_s("tan2cossin2");
  unary_function_unary __tan2cossin2(&giac::_tan2cossin2,_tan2cossin2_s);
  unary_function_ptr at_tan2cossin2 (&__tan2cossin2);

  gen tcollect(const gen & args,GIAC_CONTEXT){
    checkanglemode(contextptr);
    vecteur v;
    tlin(args,v,contextptr);
    // v= [coeff, sin/cos/1]
    int s=v.size();
    vector<int> deja;
    gen res,argu;
    for (int i=1;i<s;i+=2){
      if (equalposcomp(deja,i))
	continue;
      if (v[i].type!=_SYMB){
	res = res + v[i-1]*v[i];
	continue;
      }
      argu=v[i]._SYMBptr->feuille;
      int j=i+2;
      for (;j<s;j+=2){
	if ( (v[j].type==_SYMB) && (v[j]._SYMBptr->feuille==argu) )
	  break;
      }
      if (j>=s)
	res = res + v[i-1]*v[i];
      else { // found 2 trig terms with the same arg, combine
	deja.push_back(j);
	gen coscoeff,sincoeff;
	if (v[i]._SYMBptr->sommet==at_cos){
	  coscoeff=v[i-1];
	  sincoeff=v[j-1];
	}
	else {
	  coscoeff=v[j-1];
	  sincoeff=v[i-1];
	}
	gen newcoeff=normal(sqrt(pow(coscoeff,2)+pow(sincoeff,2)));
	gen angle=normal(arg(coscoeff+cst_i*sincoeff,contextptr));
	res=res+newcoeff*cos(argu-angle,contextptr);
      }
    }
    return res;
  }
  gen _tcollect(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_tcollect,contextptr);
    return apply(args,contextptr,tcollect);
  }
  const string _tcollect_s("tcollect");
  unary_function_eval __tcollect(&giac::_tcollect,_tcollect_s);
  unary_function_ptr at_tcollect (&__tcollect);

  // -> [ coeffln argln ...]
  vecteur inlncollect(const gen & args){
    if (args.type!=_SYMB)
      return makevecteur(args,e__IDNT_e);
    unary_function_ptr & u=args._SYMBptr->sommet;
    gen g=args._SYMBptr->feuille;
    if (u==at_ln)
      return makevecteur(1,g);
    if (u==at_plus){
      if (g.type!=_VECT)
	return inlncollect(g);
      vecteur rescoeff,resargln;
      const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end(),jt,jtend;
      for (;it!=itend;++it){
	vecteur tmp(inlncollect(*it));
	jt=tmp.begin();
	jtend=tmp.end();
	for (;jt!=jtend;jt+=2){
	  int i;
	  if (i=equalposcomp(resargln,*(jt+1))){
	    rescoeff[i-1]=normal(rescoeff[i-1]+*jt);
	  }
	  else {
	    rescoeff.push_back(*jt);
	    resargln.push_back(*(jt+1));
	  }
	}
      }
      it=rescoeff.begin();
      itend=rescoeff.end();
      jt=resargln.begin();
      vecteur res;
      gen lnint(plus_one);
      res.reserve(2*(itend-it));
      for (;it!=itend;++it,++jt){
	if (is_zero(*it))
	  continue;
	if (it->type==_INT_)
	  lnint=lnint*pow(*jt,it->val);
	else {
	  res.push_back(*it);
	  res.push_back(*jt);
	}
      }
      if (!is_one(lnint)){
	res.push_back(1);
	res.push_back(lnint);
      }
      return res;
    }
    if (u==at_neg){
      vecteur tmp(inlncollect(g));
      iterateur it=tmp.begin(),itend=tmp.end();
      for (;it!=itend;it+=2){
	*it=-*it;
      }
      return tmp;
    }
    if (u==at_prod){
      if (g.type!=_VECT)
	return inlncollect(g);
      // is there a ln in g?
      const_iterateur it=g._VECTptr->begin(),it0=g._VECTptr->begin(),itend=g._VECTptr->end();
      for (;it!=itend;++it){
	if ( (it->type==_SYMB) && (it->_SYMBptr->sommet==at_ln) )
	  break;
      }
      if (it!=itend){
	vecteur v(mergevecteur(vecteur(it0,it),vecteur(it+1,itend)));
	return makevecteur(_prod(v),it->_SYMBptr->feuille);
      }
    }
    return makevecteur(u(_lncollect(g),0),e__IDNT_e); // FIXME GIAC_CONTEXT
  }
  gen lncollect(const gen & args){
    vecteur v(inlncollect(args));
    gen res;
    const_iterateur it=v.begin(),itend=v.end();
    for (;it!=itend;it+=2){
      res = res + (*it) * ln (*(it+1));
    }
    return res;
  }
  gen _lncollect(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_lncollect);
    return apply(args,lncollect);
  }
  const string _lncollect_s("lncollect");
  unary_function_unary __lncollect(&giac::_lncollect,_lncollect_s);
  unary_function_ptr at_lncollect (&__lncollect);

  gen powexpand(const gen & e){
    return subst(e,pow_v,powexpand_v);
  }
  gen _powexpand(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_powexpand);
    return apply(args,powexpand);
  }
  const string _powexpand_s("powexpand");
  unary_function_unary __powexpand(&giac::_powexpand,_powexpand_s);
  unary_function_ptr at_powexpand (&__powexpand);

  gen exp2pow(const gen & e){
    return subst(e,exp_v,exp2power_v);
  }
  gen _exp2pow(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_exp2pow);
    return apply(args,exp2pow);
  }
  const string _exp2pow_s("exp2pow");
  unary_function_unary __exp2pow(&giac::_exp2pow,_exp2pow_s);
  unary_function_ptr at_exp2pow (&__exp2pow);
  
  gen factor_xn(const gen & args,const gen & x){
    vecteur l(1,x);
    lvar(args,l);
    gen temp=e2r(args,l);
    gen n,d;
    fxnd(temp,n,d);
    l.erase(l.begin());
    vecteur nv(gen2vecteur(r2e(polynome2poly1(n,1),l)));
    vecteur dv(gen2vecteur(r2e(polynome2poly1(d,1),l)));
    int ns=nv.size(),ds=dv.size();
    int n1=ns-ds;
    return pow(x,n1)*symb_horner(nv,x,ns-1)/symb_horner(dv,x,ds-1);
  }
  gen factor_xn(const gen & args){
    return factor_xn(args,vx_var);
  }
  gen _factor_xn(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_factor_xn);
    return apply(args,factor_xn);
  }
  const string _factor_xn_s("factor_xn");
  unary_function_unary __factor_xn(&giac::_factor_xn,_factor_xn_s);
  unary_function_ptr at_factor_xn (&__factor_xn);

#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
