// -*- mode:C++ ; compile-command: "g++ -I.. -g -c sym2poly.cc" -*-
#include "first.h"
/*
 *  This file implements several functions that work on univariate and
 *  multivariate polynomials and rational functions.
 *  These functions include polynomial quotient and remainder, GCD and LCM
 *  _VECTutation, factorization and rational function normalization. */

/*
 *  Copyright (C) 2000 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using namespace std;
#include <fstream>
#include <string>
#include <unistd.h> // For reading arguments from file

#include "sym2poly.h"
#include "usual.h"
#include "unary.h"
#include "subst.h"
#include "modpoly.h"
#include "alg_ext.h"
#include "solve.h"
#include "input_parser.h"
#include "ezgcd.h"
#include "prog.h"
#include "ifactor.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  // "instantiate" debugging functions
  void dbgprint(const polynome &p){
    p.dbgprint();
  }

  void dbgprint(const gen & e){
    e.dbgprint();
  }

  vecteur cdr_VECT(const vecteur & l){
    if (l.empty())
      return vecteur(l);
    vecteur::const_iterator it=l.begin(),itend=l.end();
    vecteur res;
    ++it;
    for (;it!=itend;++it)
      res.push_back(*it);
    return vecteur(res);
  }

  //***************************
  // functions relative to lvar
  //***************************
  int equalposcomp(const vecteur & l,const gen & e){
    int n=1;
    for (vecteur::const_iterator it=l.begin();it!=l.end();++it){
      if ((*it)==e)
	return(n);
      else
	n++;
    }
    return(0);
  }

  void addtolvar(const gen & e, vecteur & l){
    if (equalposcomp(l,e))
      return;
    l.push_back(e);
  }

  void lvar(const symbolic & s, vecteur &l){
    if ( (s.sommet==at_plus) || (s.sommet==at_prod)){
      if (s.feuille.type!=_VECT){
	lvar(s.feuille,l);
	return;
      }
      vecteur::iterator it=s.feuille._VECTptr->begin(), itend=s.feuille._VECTptr->end();
      for (;it!=itend;++it)
	lvar(*it,l);
      return;
    }
    if ( (s.sommet==at_neg) || (s.sommet==at_inv) ){
      lvar(s.feuille,l);
      return;
    }
    if ( (s.sommet==at_pow) && ( (*s.feuille._VECTptr)[1].type==_INT_))
      lvar(s.feuille._VECTptr->front(),l);
    else
      addtolvar(s,l);
  }

  void lvar(const vecteur & v,vecteur & l){
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;++it)
      lvar(*it,l);
  }

  void lvar(const gen & e, vecteur & l) {
    switch (e.type){
    case _INT_: case _DOUBLE_: case _ZINT: case _CPLX: case _POLY: case _EXT: case _ROOT: case _USER:
      return;
    case _IDNT:
      if (*e._IDNTptr!=undef)
	addtolvar(e,l);
      return ;
    case _SYMB:
      lvar(*e._SYMBptr,l);
      return ;
    case _VECT:
      lvar(*e._VECTptr,l);
      return ;
    case _FRAC:
      lvar(e._FRACptr->num,l);
      lvar(e._FRACptr->den,l);
      return;
    case _MOD:
      lvar(*e._MODptr,l);
      lvar(*(e._MODptr+1),l);
      return;
    default:
      settypeerr();
    }
  }
  
  void lvar(const sparse_poly1 & p,vecteur & l){
    sparse_poly1::const_iterator it=p.begin(),itend=p.end();
    for (;it!=itend;++it){
      lvar(it->coeff,l);
      // lvar(it->exponent,l);
    }
  }

  vecteur lvar(const gen & e){
    vecteur l;
    lvar(e,l);
    return l;
  }

  gen symb_lvar(const gen & e){
    return symbolic(at_lvar,e);
  }
  gen cklvar(const gen & e){
    vecteur l;
    lvar(e,l);
    return l;
  }
  const string _lvar_s("lvar");
  unary_function_unary __lvar(&giac::cklvar,_lvar_s);
  unary_function_ptr at_lvar (&__lvar);

  bool is_algebraic_EXTension(const gen & e){
    if (e.type==_EXT)
      return true;
    if (e.type!=_SYMB)
      return false;
    if ( (e._SYMBptr->sommet==at_sqrt) || (e._SYMBptr->sommet==at_rootof) )
      return true;
    if ( (e._SYMBptr->sommet==at_pow) && (e._SYMBptr->feuille._VECTptr->back().type==_FRAC) && (e._SYMBptr->feuille._VECTptr->back()._FRACptr->den.type==_INT_) &&(absint(e._SYMBptr->feuille._VECTptr->back()._FRACptr->den.val)<=MAX_ALG_EXT_ORDER_SIZE)  )
      return true;
    return false;
  }

  gen algebraic_argument(const gen & e){
    if (e.type==_EXT)
      return makevecteur(*e._EXTptr,*(e._EXTptr+1));
    assert(e.type==_SYMB);
    if ((e._SYMBptr->sommet==at_sqrt) || (e._SYMBptr->sommet==at_rootof) )
      return e._SYMBptr->feuille;
    if ( (e._SYMBptr->sommet==at_pow) && (e._SYMBptr->feuille._VECTptr->back().type==_FRAC) && (e._SYMBptr->feuille._VECTptr->back()._FRACptr->den.type==_INT_) )
      return e._SYMBptr->feuille._VECTptr->front();
    settypeerr();
  }

  bool equalposmat(const matrice & m,const gen & e,int &i,int & j){
    i=0;
    const_iterateur it=m.begin(),itend=m.end(),jt,jtend;
    for (;it!=itend;++it,++i){
      if (*it==e){
	j=-1;
	return true;
      }
      else {
	assert(it->type==_VECT);
	for (j=0,jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();jt!=jtend;++jt,++j)
	  if (*jt==e)
	    return true;
      }
    }
    return false;
  }

  void addfirstrow(const gen & e,matrice & m){
    if (m.empty()){
      vecteur v(1,e);
      m.push_back(v);
    }
    else {
      assert(m.front().type==_VECT);
      vecteur v(*m.front()._VECTptr);
      v.push_back(e);
      m.front()=v;
    }
  }

  matrice ext_glue_matrices(const matrice & a,const matrice & b){
    if (a.size()>b.size())
      return ext_glue_matrices(b,a);
    if (b.empty() || a.empty() || (a==b))
      return b;
    int i,j;
    matrice res(b.begin()+1,b.end()); // all alg. extensions of b
    // look in a for vars that are not inside res
    matrice a_not_in_res;
    const_iterateur it=a.begin(),itend=a.end(),jt,jtend;
    for (;it!=itend;++it){
      vecteur temp;
      for (jt=it->_VECTptr->begin(),jtend=it->_VECTptr->end();jt!=jtend;++jt){
	if (!equalposmat(res,*jt,i,j))
	  temp.push_back(*jt);
      }
      if (it==a.begin() || (!temp.empty()) )
	a_not_in_res.push_back(temp);
    }
    // look in the first row of b for vars that are not inside a_not_in_res
    jt=b.begin()->_VECTptr->begin(),jtend=b.begin()->_VECTptr->end();
    for (;jt!=jtend;++jt){
      if (!equalposmat(a_not_in_res,*jt,i,j))
	addfirstrow(*jt,a_not_in_res);
    }
    return mergevecteur(a_not_in_res,res);
  }

  // return true if there is at least one algebraic extension
  void alg_lvar(const gen & e,matrice & m){
    vecteur temp;
    lvar(e,temp);
    int i,j;
    // For each variable of temp, 
    // if not alg var look if still inside m else add it to the first line
    // else make a "merge"
    const_iterateur it=temp.begin(),itend=temp.end();
    for (;it!=itend;++it){
      if ( !is_algebraic_EXTension(*it) ){
	if (!equalposmat(m,*it,i,j)){
	  addfirstrow(*it,m);
	}
      }
      else { // *it is an algebraic extension!
	matrice ext_mat;
	vecteur v,vt;
	ext_mat.push_back(v);
	vt=alg_lvar(algebraic_argument(*it));
	if ((!vt.empty()) && (!vt.front()._VECTptr->empty()) )
	  ext_mat=mergevecteur(ext_mat,vt);
	m=ext_glue_matrices(ext_mat,m);
      }
    }
  }

  vecteur alg_lvar(const gen & e){
    vecteur l;
    l.push_back(l); // insure a null line inside the matrix of alg_lvar
    alg_lvar(e,l);
    return l;
  }

  gen symb_algvar(const gen & e){
    return symbolic(at_algvar,e);
  }
  gen ckalgvar(const gen & e){
    vecteur l;
    alg_lvar(e,l);
    return l;
  }
  const string _algvar_s("algvar");
  unary_function_unary __algvar(&giac::ckalgvar,_algvar_s);
  unary_function_ptr at_algvar (&__algvar);

  
  //***********************************************
  // trivial divisors
  //***********************************************
  inline bool is_strictly_smaller(const gen & a,const gen & b){
    return is_strictly_greater(b,a,0); // return (a<b)
  }

  vecteur divisor(const gen & n){
    gen ntemp;
    if (!is_positive(n,0))
      ntemp=-n;
    else
      ntemp=n;
    vector<nfactor> nv(trivial_n_factor(ntemp));
    int k=nv.size();
    vecteur v;
    v.push_back(gen(1));
    for (int j=0;j<k;j++){
      gen current(1);
      int mult=nv[j].mult;
      gen multiplie(nv[j].fact);
      v.reserve(v.size()*(mult+1));
      vecteur::const_iterator itbeg=v.begin();
      vecteur::const_iterator itend=v.end();
      for (int i=0;i<mult;i++){
	current=current*multiplie;
	vecteur::const_iterator it=itbeg;
	// cout << "for " << *it << endl;
	for (;it!=itend;++it){
	  gen temp((*it)*current);
	  // cout << *it << " " << current << " " << temp << endl;
	  v.push_back( temp );
	}
      }
    }
    sort(v.begin(),v.end(),ptr_fun(is_strictly_smaller));
    return v;
  }

  //***********************************************
  // functions relative to fractions for efficiency
  //***********************************************

  fraction pow(const fraction & p,const gen & n){
    assert(n.type==_INT_);
    return pow(p,n.val);
  }

  gen simplify3(gen & n,gen & d){
    if (is_one(n) || is_one(d))
      return plus_one;
    if (n.type==_EXT){
      gen n_EXT=*n._EXTptr;
      gen g=simplify(n_EXT,d);
      n=algebraic_EXTension(n_EXT,*(n._EXTptr+1));
      return g;
    }
    if ((n.type==_POLY) && (d.type==_POLY)){
      polynome * pptr = new polynome(n._POLYptr->dim);
      if (*n.ref_count==1 && *d.ref_count==1){
	simplify(*n._POLYptr,*d._POLYptr,*pptr);
	return pptr;
      }
      polynome np(*n._POLYptr),dp(*d._POLYptr);
      simplify(np,dp,*pptr);
      gen g=pptr;
      n=np;
      d=dp;
      return g;
    }
    if (n.type==_POLY) {
      polynome np(*n._POLYptr);
      gen l;
      vector< monomial<gen> > :: const_iterator it=np.coord.begin(),itend=np.coord.end();
      for (;it!=itend;++it)
	l=gcd(l,it->value);
      gen g=simplify3(l,d);
      np=np/g;
      n=np;
      return g;
    }
    if (d.type==_POLY){
      polynome np(n,d._POLYptr->dim),dp(*d._POLYptr);
      polynome g(np.dim);
      g=simplify(np,dp);
      n=np;
      d=dp;
      return g;
    }
    gen g=gcd(n,d);
    n=n/g; // iquo(n,g);
    d=d/g; // iquo(d,g);
    return g;
  }

  bool has_EXT(const gen & g){
    if (g.type==_EXT)
      return true;
    if (g.type!=_POLY)
      return false;
    polynome & p = *g._POLYptr;
    vector< monomial<gen> >::const_iterator it=p.coord.begin(),itend=p.coord.end();
    for (;it!=itend;++it){
      if (has_EXT(it->value))
	return true;
    }
    return false;
  }

  void _FRACadd(const gen & n1, const gen & d1,const gen & n2, const gen & d2, gen & num, gen & den){
    // cout << n1 << "/" << d1 << "+" << n2 << "/" << d2 << "=";
    if (is_one(d1)){
      num=n1*d2+n2;
      den=d2;
      return;
    }
    if (is_one(d2)){
      num=n2*d1+n1;
      den=d1;
      // cout << num << "/" << den << endl;
      return;
    }
    // n1/d1+n2/d2 with g=gcd(d1,d2), d1=d1g*g, d2=d2g*g is
    // (n1*d2g+n2*d1g)/g * 1/(d1g*d2g)
    gen d1g(d1),d2g(d2);
    den=simplify3(d1g,d2g);
    num=(n1*d2g+n2*d1g);
    simplify3(num,den);
    den=den*d1g*d2g;
  }

  void _FRACmul(const gen & n1, const gen & d1,const gen & n2, const gen & d2, gen & num, gen & den){
    // cout << n1 << "/" << d1 << "*" << n2 << "/" << d2 << "=";
    if (is_one(d1)){
      num=n1;
      den=d2;
      simplify3(num,den);
      num=num*n2;
      // cout << num << "/" << den << endl;
      return;
    }
    if (is_one(d2)){
      num=n2;
      den=d1;
      simplify3(num,den);
      num=num*n1;
      // cout << num << "/" << den << endl;
      return;
    }
    num=n1;
    den=d2;
    simplify3(num,den);
    gen ntemp(n2),dtemp(d1);
    simplify3(ntemp,dtemp);
    num=num*ntemp;
    den=den*dtemp;
    // Further simplifications may occur with _EXT multiplications
    if (has_EXT(ntemp))
      simplify3(num,den);
    // cout << num << "/" << den << endl;
  }

  //**********************************
  // symbolic to tensor
  //**********************************
  bool sym2radd (vecteur::const_iterator debut,vecteur::const_iterator fin,const vecteur &l,const vecteur & lv, const vecteur & lvnum,const vecteur & lvden, int l_size, gen & num, gen & den){
    bool totally_converted=true;
    if (debut+4>fin){
      gen n1,d1,n2,d2;
      num=zero;
      den=plus_one;
      for (;debut!=fin;++debut){
	totally_converted=totally_converted && sym2r(*debut,l,lv,lvnum,lvden,l_size,n1,d1);
	n2=num;
	d2=den;
	_FRACadd(n1,d1,n2,d2,num,den);
      }
    }
    else {
      vecteur::const_iterator milieu=debut+(fin-debut)/2;
      gen n1,d1,n2,d2;
      totally_converted=totally_converted && sym2radd(debut,milieu,l,lv,lvnum,lvden,l_size,n1,d1);
      totally_converted=totally_converted && sym2radd(milieu,fin,l,lv,lvnum,lvden,l_size,n2,d2);
      _FRACadd(n1,d1,n2,d2,num,den);
    }
    return totally_converted;
  }

  bool sym2rmul (vecteur::const_iterator debut,vecteur::const_iterator fin,const vecteur &l, const vecteur & lv, const vecteur & lvnum,const vecteur & lvden,int l_size, gen & num, gen & den){
    bool totally_converted=true;
    // First check for a "normal" monomial
    gen coeff=plus_one;
    if (!l.empty()){ 
      bool embedd = l.front().type==_VECT ;
      vecteur l1;
      if (embedd)
	l1=*l.front()._VECTptr;
      else
	l1=l;
      gen tmp;
      int pui,pos;
      index_t i(l_size);
      for (;debut!=fin;++debut){
	if (debut->type<_IDNT)
	  coeff=coeff*(*debut);
	else {
	  if (debut->is_symb_of_sommet(at_pow) && debut->_SYMBptr->feuille._VECTptr->back().type==_INT_){
	    tmp=debut->_SYMBptr->feuille._VECTptr->front();
	    pui=debut->_SYMBptr->feuille._VECTptr->back().val;
	  }
	  else {
	    tmp=*debut;
	    pui=1;
	  }
	  if ( tmp.type==_IDNT && (pos=equalposcomp(l1,tmp)) && pui>=0){
	    i[pos-1] += pui;
	  }
	  else
	    break;
	}
      }
      if (!is_zero(coeff))
	coeff=polynome(monomial<gen>(coeff,i));
    }
    if (debut+4>fin){
      gen n1,d1,n2,d2;
      num=coeff;
      den=plus_one;
      for (;debut!=fin;++debut){
	totally_converted=totally_converted && sym2r(*debut,l,lv,lvnum,lvden,l_size,n1,d1);
	n2=num;
	d2=den;
	_FRACmul(n1,d1,n2,d2,num,den);
      }
    }
    else {
      vecteur::const_iterator milieu=debut+(fin-debut)/2;
      gen n1,d1,n2,d2;
      totally_converted=totally_converted && sym2rmul(debut,milieu,l,lv,lvnum,lvden,l_size,n1,d1);
      totally_converted=totally_converted && sym2rmul(milieu,fin,l,lv,lvnum,lvden,l_size,n2,d2);
      _FRACmul(n1,d1,n2,d2,num,den);
      simplify3(coeff,den);
      num=num*coeff;
    }
    return totally_converted;
  }

  void sym2rxroot(gen & num,gen & den,int n,int d,const vecteur & l){
    if (is_zero(num))
      return;
    bool sign_changed=false;
    if (d<0){
      n=-n;
      d=-d;
    }
    if (n<0){
      if (num.type==_EXT){
	gen temp(inv_EXT(num)*den);
	fxnd(temp,num,den);
      }
      else 
	swap(num,den);
      num=pow(num,-n)*pow(den,n*(1-d));
      den=pow(den,-n);
    }
    else {
      num=pow(num,n)*pow(den,n*(d-1));
      den=pow(den,n);
    }
    if ( (d%2) && (num.type==_POLY) && (is_positive(-num._POLYptr->coord.front())) ){
      num=-num;
      sign_changed=true;
    }
    // compute number of cst polynomial in num and keep track of dims
    int embeddings=0;
    vector<int> embeddings_s;
    if (is_atomic(num)){
      const_iterateur it=l.begin(),itend=l.end();
      embeddings=itend-it;
      for (int j=0;j<embeddings;++it,++j){
	if (it->type!=_VECT){
	  string s("sym2rxroot error num="+num.print()+" den="+den.print()+" l="+gen(l).print());
	  cerr << s << endl;
	  setsizeerr(s);
	}
	embeddings_s.push_back(it->_VECTptr->size());
      }
    }
    else {
      for (;((num.type==_POLY) && (Tis_constant<gen>(*num._POLYptr)));++embeddings){
	embeddings_s.push_back(num._POLYptr->dim);
	num=num._POLYptr->coord.front().value;
      }
    }
    // make the polynomial X^d - num
    vecteur v(d+1,zero);
    v.front()=plus_one;
    v.back()=-num;
    // check for irreducibility
    polynome p(poly12polynome(v));
    polynome p_content(p.dim);
    factorization f;
    if (!factor(p,p_content,f,true,false,false))
      setsizeerr("Can't check irreducibility extracting rootof");
    // now we choose the factor of lowest degree of the factorization
    int lowest_degree=v.size(),deg;
    factorization::const_iterator f_it=f.begin(),f_itend=f.end();
    for (;f_it!=f_itend;++f_it){
      polynome irr_p(f_it->fact);
      deg=irr_p.lexsorted_degree();
      if (!deg)
	continue;
      if (deg==1){
	v=polynome2poly1(irr_p);
	lowest_degree=1;
	vecteur lv=vecteur(l.begin()+embeddings,l.end()); // vecteur(1,vecteur(0))
	num=rdiv(-v.back(),v.front());
	// cerr << "xroot" << num << endl;
	if (is_positive(r2sym(num,lv),0)) // FIXME GIAC_CONTEXT
	  break;
      }
      if (deg>=lowest_degree)
	continue;
      v=polynome2poly1(irr_p);
      lowest_degree=deg;
    }
    gen tmpden=1; 
    if (lowest_degree>1){
      // here we must check that num is not an extension!!
        if (num.type==_EXT){
            gen a=*(num._EXTptr+1),b;
            vecteur a__VECT;
            if (a.type==_VECT)
                a__VECT=*a._VECTptr;
            else {
                assert( (a.type==_EXT) && ((a._EXTptr+1)->type==_VECT) );
                a__VECT=*((a._EXTptr+1)->_VECTptr);
            }
            int k;
            vecteur new_v=common_minimal_POLY(a__VECT,v,a,b,k);
            *(num._EXTptr+1)=a;
            if (b.type==_FRAC){
                num=b._FRACptr->num;
                tmpden=tmpden*b._FRACptr->den;
            }
            else
                num=b;
        }
        else {
            vecteur w(2);
            w[0]=plus_one;
            num=algebraic_EXTension(w,v);
        }
    }
    // and eventually we embedd num 
    for (;embeddings;--embeddings){
      num=polynome(num,embeddings_s.back());
      tmpden=polynome(tmpden,embeddings_s.back());
      embeddings_s.pop_back();
    }
    if (sign_changed)
      num=-num;
    den=den*tmpden;
  }

  bool sym2r (const symbolic &s,const vecteur &l, const vecteur & lv, const vecteur & lvnum,const vecteur & lvden, int l_size,gen & num,gen & den){
    if (s.sommet==at_plus){
      if (s.feuille.type!=_VECT){
	return sym2r(s.feuille,l,lv,lvnum,lvden,l_size,num,den);
      }
      vecteur::iterator debut=s.feuille._VECTptr->begin();
      vecteur::iterator fin=s.feuille._VECTptr->end();
      return sym2radd(debut,fin,l,lv,lvnum,lvden,l_size,num,den);
    }
    if (s.sommet==at_prod){
      vecteur::iterator debut=s.feuille._VECTptr->begin();
      vecteur::iterator fin=s.feuille._VECTptr->end();
      return sym2rmul(debut,fin,l,lv,lvnum,lvden,l_size,num,den);
    }
    if (s.sommet==at_neg){
      bool totally_converted=sym2r(s.feuille,l,lv,lvnum,lvden,l_size,num,den);
      num=-num;
      return totally_converted;
    }
    if (s.sommet==at_inv){
      bool totally_converted=sym2r(s.feuille,l,lv,lvnum,lvden,l_size,num,den);
      if (is_zero(num)){
	num=undef;
	den=plus_one;
	return true;
      }
      if (num.type==_EXT){
	gen temp(inv_EXT(num)*den);
	fxnd(temp,num,den);
      }
      else {
	if ( (num.type==_POLY) && (num._POLYptr->dim==0) && (!num._POLYptr->coord.empty()) && (num._POLYptr->coord.front().value.type==_EXT) ){
	  gen temp(inv_EXT(num._POLYptr->coord.front().value));
	  if ( (den.type==_POLY) && (den._POLYptr->dim==0) && (!den._POLYptr->coord.empty()) )
	    temp=temp*den._POLYptr->coord.front().value;
	  else
	    temp=temp*den;
	  gen tempnum,tempden;
	  fxnd(temp,tempnum,tempden);
	  polynome tmpnum(0),tmpden(0);
	  tmpnum.coord.push_back(monomial<gen>(tempnum,index_t(0)));
	  tmpden.coord.push_back(monomial<gen>(tempden,index_t(0)));
	  num=tmpnum;
	  if (tempden.type==_POLY)
	    den=tmpden;
	  else
	    den=tempden;
	}
	else
	  swap(num,den);
      }
      return totally_converted;
    }
    if (s.sommet==at_pow){
      if ((*s.feuille._VECTptr)[1].type==_INT_) {
	bool totally_converted=sym2r(s.feuille._VECTptr->front(),l,lv,lvnum,lvden,l_size,num,den);
	int n=(*s.feuille._VECTptr)[1].val;
	if (n<0){
	  if (num.type==_EXT){
	    gen temp(inv_EXT(num)*den);
	    fxnd(temp,num,den);
	  }
	  else 
	    swap(num,den);
	  num=pow(num,-n);
	  den=pow(den,-n);
	}
	else {
	  num=pow(num,n);
	  den=pow(den,n);
	}
	return totally_converted;
      }
      if ((*s.feuille._VECTptr)[1].type==_FRAC) {
	fraction f=*((*s.feuille._VECTptr)[1]._FRACptr);
	if ( (f.num.type==_INT_) && (f.den.type==_INT_)){
	  bool totally_converted=sym2r(s.feuille._VECTptr->front(),l,lv,lvnum,lvden,l_size,num,den);
	  sym2rxroot(num,den,f.num.val,f.den.val,l);
	  return totally_converted;
	}
      }
    }
    if (s.sommet==at_rootof){
        bool totally_converted=sym2r(s.feuille._VECTptr->front(),l,lv,lvnum,lvden,l_size,num,den);
        gen pmin_num,pmin_den;
        totally_converted=totally_converted && sym2r(s.feuille._VECTptr->back(),l,lv,lvnum,lvden,l_size,pmin_num,pmin_den);
        if (!is_one(pmin_den))
            setsizeerr("Minimal poly. in rootof must be fraction free");
	int embeddings=0;
	vector<int> embeddings_s;
	// put out constant polynomials
	if (num.type!=_VECT || pmin_num.type!=_VECT)
	  return totally_converted;
	vecteur vnum=*num._VECTptr,vpmin=*pmin_num._VECTptr;
	int s=l.size();
	for (;embeddings<s;){
	  bool exitmainloop=false;
	  iterateur it=vnum.begin(),itend=vnum.end();
	  int i=0;
	  for (;;++it){
	    if (it==itend){
	      if (i==0){
		++i;
		it=vpmin.begin();
		itend=vpmin.end();
	      }
	      else
		break;
	    }
	    if (is_atomic(*it))
	      continue;
	    if (it->type==_POLY && Tis_constant<gen>(*it->_POLYptr))
	      continue;
	    exitmainloop=true;
	    break;
	  }
	  if (exitmainloop)
	    break;
	  assert(l[embeddings].type==_VECT);
	  embeddings_s.push_back(l[embeddings]._VECTptr->size());
	  ++embeddings;
	  for (it=vnum.begin(),itend=vnum.end(),i=0;;++it){
	    if (it==itend){
	      if (i==0){
		++i;
		it=vpmin.begin();
		itend=vpmin.end();
	      }
	      else
		break;
	    }
	    if (it->type==_POLY)
	      *it=it->_POLYptr->coord.front().value;
	  }
	}
        num=algebraic_EXTension(vnum,vpmin);
	for (;embeddings;--embeddings){
	  num=polynome(num,embeddings_s.back());
	  embeddings_s.pop_back();
	}
        return totally_converted;
    }
    num=s;
    den=plus_one;
    return false;
  }

  bool sym2r (const fraction &f,const vecteur &l, const vecteur & lv, const vecteur & lvnum,const vecteur & lvden, int l_size,gen & num,gen & den){
    gen dent,numt;
    bool totally_converted=sym2r(f.num,l,lv,lvnum,lvden,l_size,num,dent);
    totally_converted=totally_converted && sym2r(f.den,l,lv,lvnum,lvden,l_size,den,numt);
    num=num*numt;
    den=den*dent;
    return totally_converted;
  }

  bool sym2r (const vecteur &v,const vecteur &l, const vecteur & lv, const vecteur & lvnum,const vecteur & lvden, int l_size,gen & num,gen & den){
    den=plus_one;
    if (v.empty()){
      num=zero;
      return true;
    }
    bool totally_converted=true;
    gen lcmdeno=plus_one;
    const_iterateur it=v.begin(),itend=v.end();
    vecteur res,numv;
    res.reserve(2*(itend-it));
    numv.reserve(itend-it);
    for (;it!=itend;++it){
      totally_converted=totally_converted && sym2r(*it,l,lv,lvnum,lvden,l_size,num,den);
      lcmdeno = lcm(lcmdeno,den);
      res.push_back(num);
      res.push_back(den);
    }
    for (it=res.begin(),itend=res.end();it!=itend;){
      num=*it;
      ++it;
      den=*it;
      ++it;
      numv.push_back(num*rdiv(lcmdeno,den));
    }
    den=lcmdeno;
    num=numv;
    return totally_converted;
  }

  bool sym2rmod (const gen * gptr,const vecteur &l, const vecteur & lv, const vecteur & lvnum,const vecteur & lvden, int l_size,gen & num,gen & den){
    gen modnum,modden,modulo;
    bool totally_converted=sym2r(*(gptr+1),l,lv,lvnum,lvden,l_size,modnum,modden);
    modulo=rdiv(modnum,modden);
    totally_converted=totally_converted && sym2r(*gptr,l,lv,lvnum,lvden,l_size,num,den);
    num=makemod(num,modulo);
    den=makemod(den,modulo);
    return totally_converted;
  }

  bool sym2r (const gen &e,const vecteur &l, const vecteur & lv, const vecteur & lvnum,const vecteur & lvden, int l_size,gen & num,gen & den){
    int n;
    switch (e.type ) {
    case _INT_: case _DOUBLE_: case _ZINT: case _CPLX:
      num=e;
      den=plus_one;
      return true; 
    case _IDNT: case _SYMB:
      n=equalposcomp(lv,e);
      if (n && (n<=lvnum.size())){
	num=lvnum[n-1];
	den=lvden[n-1];
	return true;
      }
      if ((!l.empty()) && (l.front().type==_VECT) ){
	int i,j;
	if (equalposmat(l,e,i,j)){
	  num=polynome(monomial<gen>(gen(1),j+1,l[i]._VECTptr->size()));
	  for (int k=i-1;k>=0;--k)
	    num=polynome(monomial<gen>(num,l[k]._VECTptr->size()));
	  den=plus_one;
	  return true;
	}
      }
      else {
	n=equalposcomp(l,e);
	if (n){
	  num=polynome(monomial<gen>(gen(1),n,l_size));
	  den=plus_one;
	  return true;
	}
      }
      if (e.type!=_SYMB){
	num=e;
	den=plus_one;
	return true;
      }
      return sym2r(*e._SYMBptr,l,lv,lvnum,lvden,l_size,num,den);
    case _FRAC:
      return sym2r(*e._FRACptr,l,lv,lvnum,lvden,l_size,num,den);
    case _VECT:
      return sym2r(*e._VECTptr,l,lv,lvnum,lvden,l_size,num,den);
    case _POLY: case _EXT:
      if ((!l.empty()) && (l.front().type==_VECT) ){
	num=e;
	for (int k=l.size()-1;k>=0;--k) // was l_size
	  num=polynome(monomial<gen>(num,l[k]._VECTptr->size()));
	den=plus_one;
      }
      else {
	num=polynome(monomial<gen>(e,l_size));
	den=plus_one;
      }
      return true;
    case _MOD:
      return sym2rmod(e._MODptr,l,lv,lvnum,lvden,l_size,num,den);
    case _USER:
      num=e;
      den=plus_one;
      return true;
    default: 
      settypeerr("sym2r type sym2poly.cc l.919");
    }
  }

  // rewrite embedded fraction inside g as num/den
  void reduce_alg_ext(const gen & g,gen & num,gen & den){
    num=g;
    den=plus_one;
    int embeddings=0;
    vector<int> embeddings_s;
    for (;((num.type==_POLY) && (Tis_constant<gen>(*num._POLYptr)));++embeddings){
      embeddings_s.push_back(num._POLYptr->dim);
      num=num._POLYptr->coord.front().value;
    }
    if (num.type==_EXT)
      num=ext_reduce(num);
    if (num.type==_FRAC){
      den=num._FRACptr->den;
      num=num._FRACptr->num;
    }
    else {
      num=g;
      return;
    }
    for (;embeddings;--embeddings){
      num=polynome(num,embeddings_s.back());
      den=polynome(den,embeddings_s.back());
      embeddings_s.pop_back();
    }    
  }

  void compute_lv_lvnum_lvden(const vecteur & l,vecteur & lv,vecteur & lvnum,vecteur & lvden,bool & totally_converted,int l_size){
    gen num,den;
    // sort by ascending complexity
    iterateur it=lv.begin(),itend=lv.end();
    lvnum.reserve(itend-it);
    lvden.reserve(itend-it);
    sort(it,itend,symb_size_less);    
    // find num/den for each var of lv
    for (;it!=itend;++it){
      totally_converted = totally_converted && sym2r(*it,l,lv,lvnum,lvden,l_size,num,den);
      lvnum.push_back(num);
      lvden.push_back(den);
    }
    it=lv.begin();
    gen res; // create now a common extension for rootofs
    for (int i=0;it!=itend;++it,++i){
      if (it->is_symb_of_sommet(at_pow) || it->is_symb_of_sommet(at_rootof)){
	res=lvnum[i]+res;
      // reduce res (remove embedded fractions)
	reduce_alg_ext(res,num,den);
      }
      res=num;
    }
    // "reduce" lvnum/lvden
    it=lv.begin();
    for (int i=0;it!=itend;++it,++i){
      if (it->is_symb_of_sommet(at_pow) || it->is_symb_of_sommet(at_rootof)){
	reduce_alg_ext(lvnum[i],num,den);
	lvden[i]=lvden[i]*den;
	gen tmp=res+num; 
	// insure that the first elements of lvnum are written with common ext
	reduce_alg_ext(num,tmp,den);
	lvnum[i]=tmp;
	lvden[i]=lvden[i]*den;
      }
    }
  }

  bool sym2r (const gen &e,const vecteur &l, int l_size,gen & num,gen & den){
    if (e.type<_POLY){
      num=e;
      den=plus_one;
      return true; 
    }
    if ( (e.type==_POLY) || (e.type==_EXT)){
      if ((!l.empty()) && (l.front().type==_VECT) ){
	num=e;
	for (int k=l.size()-1;k>=0;--k) // was l.size()
	  num=polynome(monomial<gen>(num,l[k]._VECTptr->size()));
	den=plus_one;
      }
      else {
	num=polynome(monomial<gen>(e,l_size));
	den=plus_one;
      }
      return true;
    }
    bool totally_converted=true;
    vecteur lv,lvnum,lvden;
    lvar(e,lv);
    compute_lv_lvnum_lvden(l,lv,lvnum,lvden,totally_converted,l_size);
    totally_converted =totally_converted && sym2r(e,l,lv,lvnum,lvden,l_size,num,den);
    // If den is a _POLY, multiply den by the _EXT conjugate of it's lcoeff
    // FIXME this should be done recursively if the 1st coeff is a _POLY!
    if (den.type==_POLY && !den._POLYptr->coord.empty() && den._POLYptr->coord.front().value.type==_EXT){
      gen a = ext_reduce(den._POLYptr->coord.front().value);
      if (a.type == _EXT && a._EXTptr->type==_VECT){
	vecteur u,v,d;
	egcd(*(a._EXTptr->_VECTptr),*((a._EXTptr+1)->_VECTptr),0,u,v,d);
	if (d.size()==1){
	  gen aconj=algebraic_EXTension(u,*(a._EXTptr+1));
	  aconj=polynome(aconj,den._POLYptr->coord.front().index.iptr->size());
	  num=aconj*num;
	  den=aconj*den;
	}
      }
    }
    return totally_converted;
  }

  fraction sym2r(const gen & e, const vecteur & l){
    int l_size;
    if (!l.empty() && l.front().type==_VECT)
      l_size=l.front()._VECTptr->size();
    else
      l_size=l.size();
    gen num,den;
    sym2r(e,l,l_size,num,den);
    if (is_positive(-den,0)) // FIXME GIAC_CONTEXT
      return fraction(-num,-den);
    else
      return fraction(num,den);
  }

  void fxnd(const gen & e,gen & num, gen & den){
    if (e.type==_FRAC){
      num=e._FRACptr->num;
      den=e._FRACptr->den;
    }
    else {
      num=e;
      den=plus_one;
    }
  }

  // fraction / x -> fraction of vecteur
  gen e2r(const gen & e,const gen & x){
    vecteur l(1,x);
    lvar(e,l);
    gen r=polynome2poly1(e2r(e,l),1);
    return r2e(r,cdr_VECT(l));
  }

  gen e2r(const gen & e,const vecteur & l){
    if (e.type!=_VECT)
      return sym2r(e,l);
    bool totally_converted=true;
    int l_size;
    if (!l.empty() && l.front().type==_VECT)
      l_size=l.front()._VECTptr->size();
    else
      l_size=l.size();
    gen num,den;
    vecteur lv,lvnum,lvden;
    lvar(e,lv);
    compute_lv_lvnum_lvden(l,lv,lvnum,lvden,totally_converted,l_size);
    vecteur res;
    const_iterateur jt=e._VECTptr->begin(),jtend=e._VECTptr->end();
    for (;jt!=jtend;++jt){
      sym2r(*jt,l,lv,lvnum,lvden,l_size,num,den);
      res.push_back(num/den);
    }
    return res;
  }
  gen _e2r(const gen & args){
    if (args.type!=_VECT)
      return _e2r(makevecteur(args,vx_var));
    if (args._VECTptr->size()!=2)
      setdimerr();
    if (args._VECTptr->back().type==_VECT)
      return e2r(args._VECTptr->front(),*(args._VECTptr->back()._VECTptr));
    vecteur l(1,args._VECTptr->back());
    lvar(args._VECTptr->front(),l);
    gen temp=e2r(args._VECTptr->front(),l);
    l.erase(l.begin());
    gen res;
    res=r2e(polynome2poly1(temp,1),l);
    if (res.type==_FRAC)
      return res;
    if (res.type==_VECT)
      return gen(*res._VECTptr,_POLY1__VECT);
    else
      return gen(vecteur(1,res),_POLY1__VECT);
  }

  const string _e2r_s("e2r");
  symbolic symb_e2r(const gen & arg1,const gen & arg2){
    return symbolic(at_e2r,makevecteur(arg1,arg2));
  }
  unary_function_unary __e2r(&giac::_e2r,_e2r_s);
  unary_function_ptr at_e2r (&__e2r);


  //**********************************
  // tensor to symbolic
  //**********************************

  gen niceprod(const vecteur & res,bool negatif){
    if (res.size()!=1){
      if (negatif)
	return -symbolic(at_prod,res);
      else
	return symbolic(at_prod,res);
    }
    else {
      if (negatif)
	return -res.front();
      else
	return res.front();
    }
  }

  gen r2sym(const gen & e,const index_m & i,const vecteur & l){
    if (is_undef(e))
      return e;
    if (i.iptr->size()!=l.size())
      setsizeerr("sym2poly/r2sym(const gen & e,const index_m & i,const vecteur & l)");
    vecteur::const_iterator l_it=l.begin();
    index_t::const_iterator it=i.iptr->begin(),itend=i.iptr->end();
    vecteur res;
    res.reserve(itend-it+1);
    bool negatif=false;
    if (!is_one(e) || (e.type==_MOD) ){
      if (is_minus_one(e) && (e.type!=_MOD) )
	negatif=true;
      else
	res.push_back(e);
    }
    for (;it!=itend;++it,++l_it){
      if ((*it))
	res.push_back(pow((*l_it),*it));
    }
    if (res.empty())
      return e;
    return niceprod(res,negatif);
  }

  gen r2sym(const polynome & p, const vecteur & l){
    if (p.coord.empty())
      return zero;
    if (p.dim==0){
      return p.constant_term();
    }
    if (is_positive(-p.coord.front()))
      return -r2sym(-p,l);
    monomial_v::const_iterator it=p.coord.begin();
    monomial_v::const_iterator itend=p.coord.end();
    vecteur res;
    res.reserve(itend-it);
    for (;it!=itend;++it){
      res.push_back(r2sym(it->value,it->index,l)) ;
    }
    if (res.size()==1)
      return res.front();
    if (increasing_power(0)) // FIXME GIAC_CONTEXT
      reverse(res.begin(),res.end());
    return symbolic(at_plus,res);
  }

  gen r2sym(const fraction & f, const vecteur & l){
    return rdiv(r2sym(f.num,l),r2sym(f.den,l));
  }

  gen r2sym(const vecteur & v,const vecteur & l){
    const_iterateur it=v.begin(),itend=v.end();
    vecteur res;
    res.reserve(itend-it);
    for (;it!=itend;++it)
      res.push_back(r2sym(*it,l));
    return res;
  }

  gen ckdeg2_rootof(const gen & p,const gen & pmin){
    assert((p.type==_VECT) &&(pmin.type==_VECT));
    if (pmin._VECTptr->size()!=3)
      return symb_rootof(p,pmin);
    assert(p._VECTptr->size()==2);
    vecteur v;
    identificateur x(" x");
    in_solve(symb_horner(*pmin._VECTptr,x),x,v,1,0); // FIXME GIAC_CONTEXT
    return p._VECTptr->front()*v.front()+p._VECTptr->back();
  }

      // check that an _EXT e is root of a second order poly
      // return 0 if not, 2 if pmin(e) is 2nd order, 1 otherwise
  int is_root_of_deg2(const gen & e,vecteur & v){
          // find a,b,c such that a*e*e+b*e+c=0
#ifdef DEBUG_SUPPORT
      assert(e.type==_EXT);
      assert(e._EXTptr->type==_VECT);
#endif
      gen pmin=*(e._EXTptr+1);
      if (min_pol(pmin).size()==3)
          return 2;
      v.clear();
      gen e_square(e*e);
      if (e_square.type!=_EXT){ // b=0, a=1, e*e=-c
          v.push_back(plus_one);
          v.push_back(zero);
          v.push_back(-e_square);
          return 1;
      }
      assert(e_square._EXTptr->type==_VECT);
      int s=e._EXTptr->_VECTptr->size();      
      int s2=e_square._EXTptr->_VECTptr->size();
      if (s!=s2)
          return false;
      gen b=-e_square._EXTptr->_VECTptr->front();
      gen a=e._EXTptr->_VECTptr->front();
      simplify(a,b);
      gen c=a*e_square+b*e;
      if (c.type==_EXT)
          return false;
      v.push_back(a);
      v.push_back(b);
      v.push_back(-c);
      return true;
  }
  
  gen r2sym(const gen & p, const const_iterateur & lt, const const_iterateur & ltend){
    if (p.type==_FRAC)
      return rdiv(r2sym(p._FRACptr->num,lt,ltend),r2sym(p._FRACptr->den,lt,ltend));
    if (p.type==_MOD)
      return makemodquoted(r2sym(*p._MODptr,lt,ltend),r2sym(*(p._MODptr+1),lt,ltend));
    if (p.type==_VECT){
      const_iterateur it=p._VECTptr->begin(),itend=p._VECTptr->end();
      vecteur res;
      res.reserve(itend-it);
      for (;it!=itend;++it)
	res.push_back(r2sym(*it,lt,ltend));
      return res;
    }
    if (p.type==_EXT){
      gen pp=ext_reduce(*(p._EXTptr),*(p._EXTptr+1));
      if (pp.type!=_EXT)
	return r2sym(pp,lt,ltend);
      gen f=min_pol(*(pp._EXTptr+1)); // f is a _VECT
      vecteur v;
      int t=is_root_of_deg2(pp,v);
      if (t==2)
	return ckdeg2_rootof(r2sym(*(pp._EXTptr),lt,ltend),r2sym(f,lt,ltend));
      if (t==1){
	vecteur w;
	identificateur x(" x");
	in_solve(symb_horner(*(r2sym(v,lt,ltend)._VECTptr),x),x,w,1,0); // FIXME GIAC_CONTEXT
	try {
	  vecteur vinit(lt,ltend);
	  if (lt!=ltend && vinit.front().type==_VECT)
	    vinit=*vinit.front()._VECTptr;
	  vecteur vzero=vecteur(vinit.size());
	  gen tmp0=r2sym(*pp._EXTptr,lt,ltend);
	  tmp0=subst(tmp0,vinit,vzero);
	  gen tmp1=r2sym(f,lt,ltend);
	  tmp1=subst(tmp1,vinit,vzero);
	  tmp0=algebraic_EXTension(tmp0,tmp1);
	  tmp0=tmp0.evalf_double();
	  if (tmp0.type<=_CPLX){
	    tmp1=eval(subst(w.front(),vinit,vzero));
	    tmp1=tmp1.evalf_double();
	    gen tmp2=eval(subst(w.back(),vinit,vzero));
	    tmp2=tmp2.evalf_double();
	    if (tmp1.type<=_CPLX && tmp2.type<=_CPLX && abs(tmp2-tmp0,0)._DOUBLE_val<abs(tmp1-tmp0,0)._DOUBLE_val)
	      return w.back();
	  }
	}
	catch (std::runtime_error & e){
	  cerr << "sym2poly exception caught " << e.what() << endl;
	}
	return w.front();
	/* gen tmp0=algebraic_EXTension(r2sym(*pp._EXTptr,lt,ltend),r2sym(f,lt,ltend)).evalf_double();
	if (tmp0.type==_DOUBLE_){
	  gen tmp1=evalf_double(w.front()),tmp2=evalf_double(w.back());
	  if (tmp1.type==_DOUBLE_ && tmp2.type==_DOUBLE_ && fabs((tmp2-tmp0)._DOUBLE_val)<fabs((tmp1-tmp0)._DOUBLE_val))
	    return w.back();
	    }
	    return w.front(); */
      }
      int s=f._VECTptr->size();
      v=vecteur(s,zero);
      v.front()=plus_one;
      v.back()=f._VECTptr->back();
      gen theta;
      if (f==v)
	theta=pow(r2sym(-v.back(),lt,ltend),fraction(1,s-1));
      else
	return symb_rootof(r2sym(*(pp._EXTptr),lt,ltend),r2sym(f,lt,ltend));
      return symb_horner(*(r2sym(*(pp._EXTptr),lt,ltend)._VECTptr),theta);
    }
    if ((p.type!=_POLY) || (lt==ltend))
      return p;
    if (p._POLYptr->coord.empty())
      return zero;
    if (p._POLYptr->dim==0){
      return r2sym(p._POLYptr->coord.front().value,lt+1,ltend);
    }
    if (is_positive(-p,0)) // FIXME GIAC_CONTEXT
      return -r2sym(-p,lt,ltend);
    monomial_v::const_iterator it=p._POLYptr->coord.begin();
    monomial_v::const_iterator itend=p._POLYptr->coord.end();
    if (itend==it)
      return zero;
    vecteur res;
    res.reserve(itend-it);
    for (;it!=itend;++it){
      res.push_back(r2sym(r2sym(it->value,lt+1,ltend),it->index,*(lt->_VECTptr))) ;
    }
    if (res.size()==1)
      return res.front();
    if (increasing_power(0)) // FIXME GIAC_CONTEXT
      reverse(res.begin(),res.end());
    return symbolic(at_plus,res);
  }

  gen r2sym(const gen & p, const vecteur & l){
    if (p.type==_VECT)
      return r2sym(*p._VECTptr,l);
    if ( (!l.empty()) && (l.front().type==_VECT) )
      return r2sym(p,l.begin(),l.end());
    if (p.type==_FRAC)
      return r2sym(*p._FRACptr,l);
    if (p.type==_MOD)
      return makemodquoted(r2sym(*p._MODptr,l),r2sym(*(p._MODptr+1),l));
    if (p.type==_EXT){ 
      gen pp=ext_reduce(*(p._EXTptr),*(p._EXTptr+1));
      if (pp.type!=_EXT)
	return r2sym(pp,l);
      gen f=min_pol(*(pp._EXTptr+1)); // f is a _VECT
      vecteur v;
      int t=is_root_of_deg2(pp,v);
      if (t==2)
	return ckdeg2_rootof(r2sym(*(pp._EXTptr),l),r2sym(f,l));
      if (t==1){
	vecteur w;
	identificateur x(" x");
	in_solve(symb_horner(*(r2sym(v,l)._VECTptr),x),x,w,1,0); // FIXME GIAC_CONTEXT
	return w.front();
      }
      int s=f._VECTptr->size();
      v=vecteur(s,zero);
      v.front()=plus_one;
      v.back()=f._VECTptr->back();
      gen theta;
      if (f==v)
	theta=pow(r2sym(-v.back(),l),fraction(1,s-1));
      else
	return symb_rootof(r2sym(*(pp._EXTptr),l),r2sym(f,l));
      return symb_horner(*(r2sym(*(pp._EXTptr),l)._VECTptr),theta);
    }
    if (p.type==_POLY)
      return r2sym(*p._POLYptr,l);
    return p;
  }

  gen r2e(const gen & p, const vecteur & l){
    return r2sym(p,l);
  }

  // alias vecteur -> polynome / x
  gen r2e(const gen & r,const gen & x){
    if (r.type==_FRAC)
      return fraction(r2e(r._FRACptr->num,x),r2e(r._FRACptr->den,x));
    if (r.type==_VECT)
      return symb_horner(*r._VECTptr,x);
    else
      return r;
  }

  gen _r2e(const gen & args){
      if (args.type!=_VECT)
          return _r2e(makevecteur(args,vx_var));
      if (args._VECTptr->size()!=2)
          return _r2e(makevecteur(args,vx_var));
      if (args._VECTptr->back().type==_VECT)
          return r2e(args._VECTptr->front(),*(args._VECTptr->back()._VECTptr));
      if (args._VECTptr->front().type==_VECT)
          return horner(*(args._VECTptr->front()._VECTptr),args._VECTptr->back());
      settypeerr();
  }

  const string _r2e_s("r2e");
  symbolic symb_r2e(const gen & arg1,const gen & arg2){
    return symbolic(at_r2e,makevecteur(arg1,arg2));
  }
  unary_function_unary __r2e(&giac::_r2e,_r2e_s);
  unary_function_ptr at_r2e (&__r2e);

  // convert factorization to symbolic form 
  gen r2sym(const factorization & vnum,const vecteur & l){
    gen resnum(1);
    factorization::const_iterator it=vnum.begin(),itend=vnum.end();
    for (;it!=itend;++it)
      resnum=resnum*pow(r2sym(gen(it->fact),l),it->mult);
    return resnum;
  }

  // convert pfde_VECT to symbolic form 
  gen r2sym(const vector< pf<gen> > & pfde_VECT,const vecteur & l){
    gen res(0);
    vector< pf<gen> >::const_iterator it=pfde_VECT.begin(),itend=pfde_VECT.end();
    for (;it!=itend;++it){
      res=res+rdiv(r2sym(gen(it->num),l),(r2sym(gen(it->den/pow(it->fact,it->mult)),l)*pow(r2sym(gen(it->fact),l),it->mult)));
    }
    return res;
  }

  //*****************************
  /* Fonctions relatives to ex */
  //*****************************

  void readargs_from_stream(istream & inf,vecteur & args,vecteur & l){
    string to_parse;
    char c;
    for (bool notbackslash=true;;){
      inf.get(c);
      if (!inf)
	break;
      if ( notbackslash || (c!='\n') )
	to_parse +=c;
      else
	to_parse = to_parse.substr(0,to_parse.size()-1);
      notbackslash= (c!='\\');
    }
    gen e(to_parse,l);
    if (e.type==_VECT)
      args=*e._VECTptr;
    else
      args=makevecteur(e);
  }

  gen read1arg_from_stream(istream & inf,vecteur & l){
    string to_parse;
    char c;
    for (bool notbackslash=true;;){
      inf.get(c);
      if (!inf)
	break;
      if ( notbackslash || (c!='\n') )
	to_parse +=c;
      else
	to_parse = to_parse.substr(0,to_parse.size()-1);
      notbackslash= (c!='\\');
    }
    return gen(to_parse,l);
  }

  void readargs(int ARGC, char *ARGV[],const vecteur &l,vecteur & args){
    // first initialize random generator for factorizations
    srand(0);
    vecteur ll(l);
    ll=mergevecteur(l,list_one_letter__IDNT);
    ll.push_back(_IDNT_break);
    ll.push_back(_IDNT_continue);
    //srand(time(NULL));
    string s;
    if (ARGC==1)
      readargs_from_stream(cin,args,ll);
    else {
      if (ARGC==2) {
	if (!secure_run && access(ARGV[1],R_OK)==0){
	  ifstream inf(ARGV[1]);
	  readargs_from_stream(inf,args,ll);
	}
	else {
	  s=string(ARGV[1]);
	  gen e(s,ll);
	  args.push_back(e);
	}
      }
      else {
	vecteur v;
	for (int i=1;i<ARGC;i++){
	  s=string(ARGV[i]);
	  gen e(s,ll);
	  v.push_back(e);
	}
	args.push_back(v);
      }
    }
    if (args.empty())
      settypeerr();
  }

  gen recursive_normal(const gen & e,GIAC_CONTEXT){
    if (e.type==_VECT)
      return apply(e,contextptr,recursive_normal);
    gen e_copy(eval(e,contextptr));
    //gen e_copy(global_eval(e,100));
    if (e_copy.type!=_SYMB && e_copy.type!=_MOD)
      return e_copy;
    if (is_inf(e_copy) || is_undef(e_copy) )
      return e_copy;
    vecteur l=lvar(e_copy);
    vecteur l_subst(l);
    iterateur it=l_subst.begin(),itend=l_subst.end();
    for (;it!=itend;++it){
      if (it->type!=_SYMB)
	continue;
      if (it->_SYMBptr->sommet!=at_pow){
	gen tmp=it->_SYMBptr->feuille;
	tmp=liste2symbolique(symbolique2liste(tmp));
	*it=it->_SYMBptr->sommet(recursive_normal(tmp,contextptr),contextptr);
	continue;
      }
      vecteur l=lvar(it->_SYMBptr->feuille._VECTptr->back());
      int l_size;
      if (!l.empty() && l.front().type==_VECT)
	l_size=l.front()._VECTptr->size();
      else
	l_size=l.size();
      gen f,f_num,f_den;
      f=e2r(it->_SYMBptr->feuille._VECTptr->back(),l);
      fxnd(f,f_num,f_den);
      gen num=r2sym(f_num,l),den=r2sym(f_den,l);
      gen base=recursive_normal(it->_SYMBptr->feuille._VECTptr->front(),contextptr);
      if ( (num.type!=_INT_) || (den.type!=_INT_) )
	*it= pow(base,rdiv(num,den));
      else 
	*it= pow(base, num.val /den.val) *pow(base,rdiv(num.val%den.val,den));
    }
    e_copy=subst(e_copy,l,l_subst);
    // return global_eval(normal(e_copy),100);
    return eval(normal(e_copy),contextptr);
  }
  const string _non_recursive_normal_s("non_recursive_normal");
  symbolic symb_non_recursive_normal(const gen & args){
    return symbolic(at_non_recursive_normal,args);
  }
  unary_function_unary __non_recursive_normal(&giac::normal,_non_recursive_normal_s);
  unary_function_ptr at_non_recursive_normal (&__non_recursive_normal);

  bool has_embedded_fractions(const gen & g){
    if (g.type==_POLY){
      vector< monomial<gen> >::const_iterator it=g._POLYptr->coord.begin(),itend=g._POLYptr->coord.end();
      for (;it!=itend;++it){
	if (has_embedded_fractions(it->value))
	  return true;
      }
      return false;
    }
    if (g.type==_VECT){
      const_iterateur it=g._VECTptr->begin(),itend=g._VECTptr->end();
      for (;it!=itend;++it){
	if (has_embedded_fractions(*it))
	  return true;
      }
      return false;
    }
    if (g.type==_FRAC){
      if (is_one(g._FRACptr->den))
	return has_embedded_fractions(g._FRACptr->num);
      return true;
    }
    return false;
  }

  gen ratnormal(const gen & e){
    // cout << e << endl;
    if (e.type==_VECT)
      return apply(e,ratnormal);
    if (e.type==_FRAC){
      gen n=e._FRACptr->num;
      gen d=e._FRACptr->den;
      simplify(n,d);
      if (is_one(d))
	return n;
      if (is_minus_one(d))
	return -n;
      if (is_zero(d)){
	if (is_zero(n))
	  return undef;
	else
	  return unsigned_inf;
      }
      if (is_zero(n))
	return zero;
      return fraction(n,d);
    }
    if (e.type!=_SYMB && e.type!=_MOD)
      return e;
    if (is_inf(e) || is_undef(e) )
      return e;
    matrice l=lvar(e);
    fraction f=e2r(e,l);
    if (has_embedded_fractions(f.num) || has_embedded_fractions(f.den))
      return normal(r2sym(f,l));
    return r2sym(f,l);
  }

  factorization rsqff(const polynome & p){
    polynome s(lgcd(p));
    factorization f(sqff(p/s));
    if (p.dim==1){
      f.push_back(facteur<polynome>(s,1));
      return f;
    }
    factorization ff(rsqff(s.trunc1()));
    factorization::const_iterator it=ff.begin(),itend=ff.end();
    for (;it!=itend;++it)
      f.push_back(facteur<polynome>(it->fact.untrunc1(),it->mult));
    return f;
  }

  gen normalize_sqrt(const gen & e){
    if (complex_mode(0)) // FIXME GIAC_CONTEXT
      return e;
    // remove multiple factors inside sqrt
    vecteur l0=lop(e,at_pow),lin,lout;
    const_iterateur it=l0.begin(),itend=l0.end();
    for (;it!=itend;++it){
      vecteur & arg=*it->_SYMBptr->feuille._VECTptr;
      gen g=arg[1],expnum,expden;
      if (g.type==_FRAC){
	expnum=g._FRACptr->num;
	expden=g._FRACptr->den;
      }
      else {
	if ( (g.type!=_SYMB) || (g._SYMBptr->sommet!=at_prod) )
	  continue;
	gen & arg1=g._SYMBptr->feuille;
	if (arg1.type!=_VECT)
	  continue;
	vecteur & v=*arg1._VECTptr;
	if ( (v.size()!=2) || (v[1].type!=_SYMB) || (v[1]._SYMBptr->sommet==at_inv) )
	  continue;
	expnum=v[0];
	expden=v[1]._SYMBptr->feuille;
      }
      if ( (expden.type!=_INT_) || (expden.val!=2 ) )
	continue;
      vecteur lv(lvar(arg[0]));
      gen a,num,den;
      a=e2r(arg[0],lv);
      fxnd(a,num,den);
      gen nd=num*den;
      if (nd.type!=_POLY)
	continue;
      lin.push_back(*it);
      factorization f(rsqff(*nd._POLYptr));
      polynome s(plus_one,nd._POLYptr->dim),d(plus_one,s.dim);
      factorization::const_iterator jt=f.begin(),jtend=f.end();
      for (;jt!=jtend;++jt){
	if (jt->mult%2)
	  s=s*jt->fact;
	d=d*pow(jt->fact,jt->mult/2);
      }
      lout.push_back(pow(r2e(s,lv),rdiv(expnum,plus_two))*pow(abs(r2e(d,lv),0),expnum)*pow(abs(r2e(den,lv),0),-expnum)); // FIXME GIAC_CONTEXT
    }
    return subst(e,lin,lout);
  }

  gen normal(const gen & e){
    if (is_equal(e))
      return apply_to_equal(e,normal);
    // cout << e << endl;
    if (e.type==_VECT)
      return apply(e,normal);
    if (e.type==_FRAC){
      gen n=e._FRACptr->num;
      gen d=e._FRACptr->den;
      simplify(n,d);
      if (is_one(d))
	return n;
      if (is_minus_one(d))
	return -n;
      if (is_zero(d)){
	if (is_zero(n))
	  return undef;
	else
	  return unsigned_inf;
      }
      if (is_zero(n))
	return zero;
      return fraction(n,d);
    }
    if (e.type==_MOD){
      gen p=*e._MODptr;
      gen m=*(e._MODptr+1);
      vecteur l(lvar(m));
      if (l.empty()){
	l=lvar(p);
	p=e2r(p,l);
	gen num,den;
	fxnd(p,num,den);
	num=makemod(num,m);
	if (!is_one(den))
	  den=makemod(den,m);
	p=rdiv(num,den);
	return r2e(p,l);
      }
      return _quorem(makevecteur(p,m))[1];
    }
    if (e.type!=_SYMB)
      return e;
    if (e._SYMBptr->sommet==at_equal){
      vecteur & v=*e._SYMBptr->feuille._VECTptr;
      return symbolic(at_equal,makevecteur(normal(v.front()),normal(v.back())));
    }
    if (is_inf(e) || is_undef(e) )
      return e;
    gen ee,tmp;
    matrice l;
    fraction f(0);
    try {
      ee=normalize_sqrt(e);
      l=alg_lvar(ee);
      tmp=e2r(ee,l);
    }
    catch (std::runtime_error & err){
      return e;
    }
    if (tmp.type==_FRAC){
      f.num=tmp._FRACptr->num;
      f.den=tmp._FRACptr->den;
    }
    else
      f=tmp;
    // search for embedded fractions
    if (has_embedded_fractions(f.num) || has_embedded_fractions(f.den))
      return normal(r2sym(f,l));
    if (f.num.type==_POLY && f.den.type<_POLY)
      return r2sym(gen(*f.num._POLYptr/f.den),l);
    return r2sym(f,l);
  }
  const string _normal_s("normal");
  symbolic symb_normal(const gen & args){
    return symbolic(at_normal,args);
  }
  unary_function_eval __normal(&giac::recursive_normal,_normal_s);
  unary_function_ptr at_normal (&__normal);

  gen rationalgcd(const gen & a, const gen & b){
    vecteur l(alg_lvar(a));
    alg_lvar(b,l);
    fraction fa(e2r(a,l)),fb(e2r(b,l));
    if (!is_one(fa.den) || !is_one(fb.den))
      settypeerr();
    return r2sym(gcd(fa.num,fb.num),l);
  }

  gen factor(const polynome & p,const vecteur &l,bool fixed_order,bool with_sqrt){
    // find main var, make permutation on f.num, f.den and l
    //    cout << "Factor " << p << " " << l << endl;
    if (is_one(p))
      return 1;
    if (l.empty() )
      return r2sym(p,l);
    if (!p.dim){
      if (l.front().type==_VECT)
	return r2sym(p,l.begin(),l.end());
      return r2sym(p,l);
    }
    polynome pp(p);
    int nvars=l.size();
    if (l.front().type==_VECT)
        nvars=l.front()._VECTptr->size();
    int deg[nvars];
    int mindeg=pp.lexsorted_degree();
    int posmin=0;
    for (int i=1;i<nvars;i++){
      int d=pp.degree(i);
      deg[i]=d;
      if (d<mindeg){
	posmin=i;
	mindeg=d;
      }
    }
    // move posmin variable at the beginning for p and l
    vecteur lp;
    if (posmin && !fixed_order){
      vecteur lptemp;
      pp.reorder(transposition(0,posmin,nvars));
      vecteur::const_iterator it;
      if (l.front().type==_VECT){
	it=l.front()._VECTptr->begin();
	++it;
	for (int i=1;i<nvars;i++,++it){
	  if (i==posmin){
	    if (lptemp.empty())
	      lptemp.push_back(*it);
	    else
	      lptemp.insert(lptemp.begin(),*it);
	    lptemp.push_back(l.front()._VECTptr->front());
	  }
	  else
	    lptemp.push_back(*it);
	}
	lp=l;
	lp[0]=lptemp;
      }
      else {
	it=l.begin();
	++it;
	for (int i=1;i<nvars;i++,++it){
	  if (i==posmin){
	    if (lp.empty())
	      lp.push_back(*it);
	    else
	      lp.insert(lp.begin(),*it);
	    lp.push_back(l.front());
	  }
	  else
	    lp.push_back(*it);
	}
      }
    }
    else
      lp=l;
    factorization v;
    polynome p_content(pp.dim);
    factor(pp,p_content,v,false,with_sqrt,complex_mode(0)); // FIXME GIAC_CONTEXT
    // factor p_content
    if (pp.dim>1){
      pp=p_content.trunc1();
      vecteur ll;
      if (lp.front().type==_VECT){
	ll=lp;
	ll[0]=cdr_VECT(*(ll[0]._VECTptr));
      }
      else
	ll=cdr_VECT(lp);
      return factor(pp,ll,false,with_sqrt)*r2sym(v,lp);
    }
    gen tmp(p_content);
    if (is_one(tmp))
      return r2sym(v,lp);
    else
      return r2sym(tmp,lp)*r2sym(v,lp);
  }

  gen var_factor(const gen & e,const vecteur & l,bool fixed_order,bool with_sqrt){
    if (e.type!=_POLY)
      return e;
    return factor(*e._POLYptr,l,fixed_order,with_sqrt);
  }

  gen ordered_factor(const gen & e,vecteur & l,bool with_sqrt){
    alg_lvar(e,l);
    gen f_num,f_den,f;
    f=e2r(e,l);
    fxnd(f,f_num,f_den);
    return rdiv(var_factor(f_num,l,true,with_sqrt),var_factor(f_den,l,true,with_sqrt));
  }

  gen factor(const gen & e,const identificateur & x,bool with_sqrt){
    if (e.type==_VECT){
      vecteur w;
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	w.push_back(factor(*it,x,with_sqrt));
      return w;
    }
    vecteur l(1,vecteur(1,x)); // insure x is the main var
    return ordered_factor(e,l,with_sqrt);
  }

  gen factor(const gen & ee,bool with_sqrt){
    if (maple_mode==3 && is_integer(ee))
      return _ifactor(ee);
    gen e(ee);
    if (has_num_coeff(ee))
      e=e.evalf();
    if (e.type==_VECT){
      vecteur w;
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	w.push_back(factor(*it,with_sqrt));
      return w;
    }
    vecteur l;
    alg_lvar(e,l);
    gen f_num,f_den,f;
    f=e2r(e,l);
    fxnd(f,f_num,f_den);
    return rdiv(var_factor(f_num,l,false,with_sqrt),var_factor(f_den,l,false,with_sqrt));
  }

  gen ratfactor(const gen & ee,bool with_sqrt){
    gen e(ee);
    if (has_num_coeff(ee))
      e=e.evalf();
    if (e.type==_VECT){
      vecteur w;
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	w.push_back(ratfactor(*it,with_sqrt));
      return w;
    }
    vecteur l;
    lvar(e,l);
    gen f_num,f_den,f;
    f=e2r(e,l);
    fxnd(f,f_num,f_den);
    return rdiv(var_factor(f_num,l,false,with_sqrt),var_factor(f_den,l,false,with_sqrt));
  }

  gen factor(const gen & ee,const gen & f,bool with_sqrt){
    if (ee.type==_VECT){
      vecteur & v=*ee._VECTptr;
      int s=v.size();
      vecteur res(s);
      for (int i=0;i<s;++i)
	res[i]=factor(v[i],f,with_sqrt);
      return res;
    }
    gen e(ee);
    if (has_num_coeff(ee))
      e=e.evalf();
    if (f.type==_IDNT)
      return factor(e,*f._IDNTptr,with_sqrt);
    if (f.type==_VECT){
      // should check that *f._VECTptr is made only of atomic _SYMB
      return ordered_factor(e,*f._VECTptr,with_sqrt);
    }
    settypeerr();
  }

  const string _factor_s("factor");
  symbolic symb_factor(const gen & args){
    return symbolic(at_factor,args);
  }
  gen factorcollect(const gen & args,bool with_sqrt){
    if (args.type!=_VECT)
      return factor(args,with_sqrt);
    vecteur & v=*args._VECTptr;
    if (v.empty())
      setsizeerr();
    if (v.size()==1)
      return factor(v.front(),with_sqrt);
    if (args.subtype==_SEQ__VECT){
      if (v.size()>2)
	toomanyargs(_factor_s);
      if (v.back().type!=_IDNT){ // FIXME could be improved!
	gen f=v.back();
	if (v.back().type==_VECT)
	  f=symbolic(at_prod,v.back());
	gen res=factor(v.front()*f,with_sqrt);
	if (res.is_symb_of_sommet(at_prod)){
	  gen & resf=res._SYMBptr->feuille;
	  if (resf.type==_VECT){
	    vecteur v=*resf._VECTptr;
	    if (!v.empty()){
	      v.front()=rdiv(v.front(),f);
	      v=*normal(normal(v))._VECTptr;
	      res=_prod(gen(v,_SEQ__VECT));
	      vecteur num,den;
	      prod2frac(res,num,den);
	      gen n=_prod(num),d=_prod(den);
	      v=alg_lvar(d);
	      if (!v.empty() && v.front().type==_VECT && v.front()._VECTptr->empty())
		d=normal(d);
	      return rdiv(n,d);
	    }
	  }
	}
	return rdiv(res,f);
      }
      return factor(v.front(),v.back(),with_sqrt);
    }
    int s=v.size();
    vecteur res(s);
    for (int i=0;i<s;++i)
      res[i]=factor(v[i],with_sqrt);
    return res;
  }
  gen _factor(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_factor,contextptr);
    gen res;
    if (maple_mode==3)
      res=factorcollect(args,lvar(args).size()==1);
    else
      res=factorcollect(args,withsqrt(contextptr));      
    return res;
  }
  unary_function_eval __factor(&giac::_factor,_factor_s);
  unary_function_ptr at_factor (&__factor);

  const string _collect_s("collect");
  gen _collect(const gen & args){
    if (is_equal(args))
      return apply_to_equal(args,_collect);
    gen res=factorcollect(args,false);
    return res;
  }
  unary_function_unary __collect(&giac::_collect,_collect_s);
  unary_function_ptr at_collect (&__collect);

  gen partfrac(const gen & e,const vecteur & l,bool with_sqrt){
    if (e.type==_VECT){
      vecteur w;
      vecteur::const_iterator it=e._VECTptr->begin(),itend=e._VECTptr->end();
      for (;it!=itend;++it)
	w.push_back(partfrac(*it,l,with_sqrt));
      return w;
    }
    int l_size;
    gen xvar;
    if (!l.empty() && l.front().type==_VECT){
      l_size=l.front()._VECTptr->size();
      xvar=l.front();
    }
    else {
      l_size=l.size();
      xvar=l;
    }
    if (!l_size)
      return e;
    else
      xvar=xvar._VECTptr->front();
    gen r=e2r(e,l);
    gen r_num,r_den;
    fxnd(r,r_num,r_den);
    if (r_den.type!=_POLY){
      if (r_num.type==_POLY)
	return rdiv(r2sym(r_num,l),r2sym(r_den,l));
      else 
	return e;
    }
    polynome f_den(*r_den._POLYptr),f_num(l_size);
    if (r_num.type==_POLY)
      f_num=*r_num._POLYptr;
    else
      f_num=polynome(r_num,l_size);
    factorization vden;
    polynome p_content(l_size);
    factor(f_den,p_content,vden,false,with_sqrt,complex_mode(0)); // FIXME GIAC_CONTEXT
    vector< pf<gen> > pfde_VECT;
    polynome ipnum(l_size),ipden(l_size);
    partfrac(f_num,f_den,vden,pfde_VECT,ipnum,ipden);
    gen res=rdiv(r2sym(gen(ipnum),l),r2sym(gen(ipden),l));
    vector< pf<gen> > ::const_iterator it=pfde_VECT.begin(),itend=pfde_VECT.end();
    for (;it!=itend;++it){
      gen reste(r2sym(gen(it->num),l)),deno(r2sym(gen(it->fact),l));
      gen cur_deno(r2sym(gen(it->den/pow(it->fact,it->mult)),l));
      for (int i=0;i<it->mult;++i){
	gen tmp(_quorem(makevecteur(reste,deno,xvar)));
	if (tmp.type!=_VECT)
	  setsizeerr();
	vecteur & vtmp=*tmp._VECTptr;
	reste=vtmp.front();
	res=res+normal(vtmp.back()/cur_deno)/pow(deno,it->mult-i);
      }
    }
    return res; // +r2sym(pfde_VECT,l);
  }

  gen partfrac(const gen & e,const identificateur & x,bool with_sqrt){
    vecteur l;
    l.push_back(x); // insure x is the main var
    l=vecteur(1,l);
    alg_lvar(e,l);
    return partfrac(e,l,with_sqrt);
  }

  gen partfrac(const gen & e,bool with_sqrt){
    vecteur l;
    alg_lvar(e,l);
    return partfrac(e,l,with_sqrt);
  }

  gen partfrac(const gen & e,const gen & f,bool with_sqrt){
    if (f.type==_IDNT)
      return partfrac(e,*f._IDNTptr,with_sqrt);
    if (f.type==_VECT){
      // should check that *f._VECTptr is made only of atomic _SYMB
      return partfrac(e,*f._VECTptr,with_sqrt);
    }
    settypeerr();
  }

  const string _partfrac_s("partfrac");
  symbolic symb_partfrac(const gen & args){
    return symbolic(at_partfrac,args);
  }
  gen _partfrac(const gen & args,GIAC_CONTEXT){
    if (is_equal(args))
      return apply_to_equal(args,_partfrac,contextptr);
    if (args.type!=_VECT)
      return partfrac(args,withsqrt(contextptr));
    if (args._VECTptr->size()>2)
      toomanyargs(_partfrac_s);
    return partfrac(args._VECTptr->front(),args._VECTptr->back(),withsqrt(contextptr));
  }
  unary_function_eval __partfrac(&giac::_partfrac,_partfrac_s);
  unary_function_ptr at_partfrac (&__partfrac);

  const string _resultant_s("resultant");
  symbolic symb_resultant(const gen & args){
    return symbolic(at_resultant,args);
  }
  gen _resultant(const gen & args){
    if (args.type!=_VECT)
      settypeerr();
    vecteur v =*args._VECTptr;
    int s=v.size();
    if (s<2)
      toofewargs(_resultant_s);
    if (s==2) v.push_back(vx_var);
    if (v.size()>3)
      toomanyargs(_resultant_s);
    if (v.back().type==_MOD)
      v.back()=*v.back()._MODptr;
    if (v.back().type!=_IDNT)
      setsizeerr();
    identificateur x=*v.back()._IDNTptr;
    vecteur l;
    l.push_back(x);
    l=vecteur(1,l);
    gen p1=v.front(),p2=v[1];
    alg_lvar(p1,l);
    alg_lvar(p2,l);
    int l_size;
    if (!l.empty() && l.front().type==_VECT)
      l_size=l.front()._VECTptr->size();
    else
      l_size=l.size();
    gen f1,f1_num,f1_den,f2,f2_num,f2_den;
    f1=e2r(p1,l);
    fxnd(f1,f1_num,f1_den);
    f2=e2r(p2,l);
    fxnd(f2,f2_num,f2_den);
    if ( (f1_num.type==_POLY) && (f2_num.type==_POLY))
      return r2sym(gen(resultant(*f1_num._POLYptr,*f2_num._POLYptr)),l);
    return zero;
  }
  unary_function_unary __resultant(&giac::_resultant,_resultant_s);
  unary_function_ptr at_resultant (&__resultant);
  
#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC

