// -*- mode:C++ ; compile-command: "g++ -I.. -g -c tex.cc" -*-
#include "first.h"
/*
 *  Copyright (C) 2002 B. Parisse, Institut Fourier, 38402 St Martin d'Heres
 *  Figure printing adapted from eukleides (c) 2002, Christian Obrecht
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
using namespace std;
#include "gen.h"
#include "gausspol.h"
#include "identificateur.h"
#include "symbolic.h"
#include "poly.h"
#include "usual.h"
#include "tex.h"
#include "prog.h"
#include "rpn.h"
#include "plot.h"

#ifndef NO_NAMESPACE_GIAC
namespace giac {
#endif // ndef NO_NAMESPACE_GIAC

  // dimension of the LaTeX output figures default 12 cm x 12 cm
  double horiz_latex=12.;
  double vert_latex=12.;
  string tex_preamble("\\documentclass{article} \n \\usepackage{pstricks,color} \n \\begin{document} \n");
  string tex_end("\n\\end{document}");
  string mbox_begin("\\mbox{");
  string mbox_end("}");

  string spread2tex(const matrice & m,int formule){
    int l=m.size();
    if (!l)
      return "\\mbox{empty_spread}";
    int c=m.front()._VECTptr->size();
    string s("\\ \\mbox{\\begin{tabular}{|");
    for (int j=0;j<=c;++j)
      s += "r|";
    s += "}\\hline\n & ";
    for (int k=0;;){ // write first line
      string tmp;
      int i=k;
      for(int j=0;;++j){
	tmp=char('A'+i%26-(j!=0))+tmp;
	i=i/26;
	if (!i)
	  break;
      }
      s += tmp;
      ++k;
      if (k==c){
	s += " \\\\\\hline\n";
	break;
      }
      else
	s += " & ";
    }
    for (int i=0;i<l;++i){
      s += print_INT_(i)+" & ";
      for (int j=0;j<c;++j){
	if (formule)
	  s += "$"+gen2tex(m[i][j][formule])+"$" ;
	else {
	  int save_r=printcell_current_row,save_c=printcell_current_col;
	  printcell_current_row=i,printcell_current_col=j;
	  string t(m[i][j][0].print()),tt;
	  int ll=t.size();
	  for (int l=0;l<ll;++l){
	    if (t[l]!='$')
	      tt += t[l];
	    else
	      tt += "\\$";
	  }
	  s += "{\\tt $"+tt+"$}";
	  printcell_current_row=save_r;printcell_current_col=save_c;
	}
	if (j!=c-1)
	  s += " & ";
      }
      if (i!=l-1){
	if (i%5==4)
	  s += " \\\\\\hline";
	else
	  s += " \\\\";
      }
      else
	s += " \\\\\\hline";
      s+='\n';
    }
    s += "\\end{tabular} } ";
    return s;
  }

  string matrix2tex(const matrice & m){
    int l=m.size();
    if (!l)
      return string("()");
    int c=m.front()._VECTptr->size();
    string s("\\left(\\begin{array}{");
    for (int j=0;j<c;++j)
      s += 'c';
    s += "}\n";
    for (int i=0;i<l;++i){
      for (int j=0;j<c;++j){
	  s += gen2tex(m[i][j]) ;
	  if (j!=c-1)
	    s += " & ";
      }
      if (i!=l-1)
	s += " \\\\";
      s+='\n';
    }
    s += "\\end{array}\\right) ";
    return s;
  }

  string _VECT2tex(const vecteur & v,int subtype){
    string s(begin_VECT_string(subtype,true));
    vecteur::const_iterator it=v.begin(),itend=v.end();
    for (;it!=itend;){
      s += gen2tex(*it);
      ++it;
      if (it!=itend)
	s += ',';
    }
    s += end_VECT_string(subtype,true);
    return s;
  }

  string prod_vect2tex(const vecteur & v){
    if (v.empty())
      return "1";
    vecteur::const_iterator it=v.begin(),itend=v.end();
    string s;
    for (;;){
      if ( (it->type==_SYMB) && ( (it->_SYMBptr->sommet==at_plus) || (it->_SYMBptr->sommet==at_neg)) )
	s += string("(")+gen2tex(*it)+string(")");
      else 
	s += gen2tex(*it);
      ++it;
      if (it==itend)
	return s;
      else
	s += "\\* ";
    }
  }

  string translate_underscore(const string & s){
    string res;
    string::const_iterator it=s.begin(),itend=s.end();
    for (;it!=itend;++it){
      switch (*it){
      case '_':
	res += "\\_";
	break;
      case '^':
	res += "\\^";
	break;
      case '\n':
	res += "\\\\\n";
	break;
      case '&':
	res += "\\&";
	break;
      case '{':
	res += "\\{";
	break;
      case '}':
	res += "\\}";
	break;
      case '\\':
	res += "$\\backslash $";
	break;
      default:
	res += *it;
      }
    }
    return res;
  }

  string symbolic2tex(const symbolic & mys){
    if (mys.sommet.ptr->texprint)
      return mys.sommet.ptr->texprint(mys.feuille,mys.sommet.ptr->s);
    if ( (mys.feuille.type==_VECT) && (mys.feuille._VECTptr->empty()) )
      return mbox_begin+translate_underscore(mys.sommet.ptr->print())+string("()")+mbox_end;
    if ( (mys.feuille.type!=_VECT) || (mys.feuille._VECTptr->front().type==_VECT)){
      if ((mys.sommet==at_neg) || (mys.sommet==at_plus)){
	if (mys.feuille.type!=_SYMB) 
	  return translate_underscore(mys.sommet.ptr->print())+gen2tex(mys.feuille) ;
	if (mys.feuille._SYMBptr->sommet==at_inv)
	  return mys.sommet.ptr->print()+gen2tex(mys.feuille) ;
	return translate_underscore(mys.sommet.ptr->print())+string("(") + gen2tex(mys.feuille) +string(")");
      }
      if (mys.sommet==at_inv){
	return string("(") + gen2tex(mys.feuille) +string(")^{-1}");
      }
      return mbox_begin + translate_underscore(mys.sommet.ptr->print()) + mbox_end + "(" + gen2tex(mys.feuille) +")" ;
    }
    string s;
    string opstring( translate_underscore(mys.sommet.ptr->print()) );
    int l=mys.feuille._VECTptr->size();
    if ( mys.sommet==at_plus ){
      for (int i=0;i<l;++i){
	gen e((*(mys.feuille._VECTptr))[i]);
	if ((e.type==_SYMB) && (e._SYMBptr->sommet==at_neg)){
	  s += "-";
	  if (e._SYMBptr->feuille.type!=_CPLX && (e._SYMBptr->feuille.type!=_SYMB || e._SYMBptr->feuille._SYMBptr->sommet==at_inv) )
	    s += gen2tex(e._SYMBptr->feuille) ;
	  else
	    s += "(" + gen2tex(e._SYMBptr->feuille) + ")";
	}
	else {
	  if ( ( (e.type==_INT_) || (e.type==_ZINT) ) && (!is_positive(e,0)) )
	    s += e.print();
	  else {
	    if (i)
	      s += opstring;
	    s += gen2tex(e);
	  }
	}
      } // end_for
      return s;
    }
    if (mys.sommet==at_prod) {
      vecteur num;
      vecteur den;
      for (int i=0;i<l;++i){
	gen e((*(mys.feuille._VECTptr))[i]);
	if ( (e.type==_SYMB) && (e._SYMBptr->sommet==at_inv) )
	  den.push_back(e._SYMBptr->feuille);
	else {
	  if (!den.empty()){
	    s += "\\frac{"+prod_vect2tex(num)+"}{"+prod_vect2tex(den)+"} \\* ";
	    num.clear();
	    den.clear();
	  }
	  num.push_back(e);
	}
      }
      if (den.empty())
	return s+prod_vect2tex(num);
      return s+"\\frac{"+prod_vect2tex(num)+"}{"+prod_vect2tex(den)+'}';
    } // end if sommet_is_prod
    if (mys.sommet==at_pow){
      vecteur & v=*mys.feuille._VECTptr;
      if ( (v.back()==plus_one_half)  )
	return "\\sqrt{"+gen2tex(v.front())+"}";
      if ( v.back()==minus_one_half || v.back()==fraction(minus_one,plus_two) )
	return "\\frac{1}{\\sqrt{"+gen2tex(v.front())+"}}";
      if (v.front().type==_SYMB || is_positive(-v.front(),0) )
	return "("+gen2tex(v.front())+")^{"+gen2tex(v.back())+'}';      
      return gen2tex(v.front())+"^{"+gen2tex(v.back())+'}';
    }
    s = mbox_begin+opstring + mbox_end +'(';
    for (int i=0;;++i){
      s += gen2tex((*(mys.feuille._VECTptr))[i]);
      if (i==l-1)
	return s+')';
      s += ',';
    }
  }

  string double2tex(double d){
    char s[32];
    sprintf(s,"%13.6f",d);
    return s;
  }
  // assume math mode enabled
  string gen2tex(const gen & e){
    switch (e.type){
    case _INT_: case _ZINT: case _REAL:
      return e.print();
    case _DOUBLE_:
      return double2tex(e._DOUBLE_val);
    case _CPLX:
      return e.print();
    case _IDNT:
      if (e==unsigned_inf)
	return "\\infty ";
      if (e==cst_pi)
	return "\\pi ";
      if (e==undef)
	return "\\,\\mbox{undef}\\,";
      return e.print();
    case _SYMB:
      return symbolic2tex(*e._SYMBptr);
    case _VECT:
      if (e.subtype==_SPREAD__VECT)
	return spread2tex(*e._VECTptr);
      if (ckmatrix(*e._VECTptr))
	return matrix2tex(*e._VECTptr);
      else
	return _VECT2tex(*e._VECTptr,e.subtype);
    case _POLY:
      return string("\\mbox{polynome} ");
    case _FRAC:
      return string("\\frac{")+gen2tex(e._FRACptr->num)+"}{"+gen2tex(e._FRACptr->den)+'}';
    case _EXT: case _ROOT:
      return "";
    case _STRNG:
      return "\\mbox{"+translate_underscore(*e._STRNGptr)+"}";
    case _FUNC:
      return "\\mbox{\\tt"+translate_underscore(e.print())+"}";
    case _USER:
      return e._USERptr->texprint();
    case _MOD:
      return gen2tex(*e._MODptr)+"\\%"+gen2tex(*(e._MODptr+1));
    case _POINTER_:
      if (e.subtype==_FL_WIDGET_POINTER &&fl_widget_texprint_function )
	return fl_widget_texprint_function(e._POINTER_val);
      return "Done";
    default:
      settypeerr("Tex convert "+e.print());
    }
  }
  gen _latex(const gen & g){
    if (!secure_run && g.type==_VECT && g.subtype==_SEQ__VECT && g._VECTptr->size()>1 && (*g._VECTptr)[1].type==_STRNG){
      ofstream of((*g._VECTptr)[1]._STRNGptr->c_str());
      of << gen2tex(g._VECTptr->front()) << endl;
      return plus_one;
    }
    return string2gen(gen2tex(g),false);
  }
  const string _latex_s("latex");
  unary_function_unary __latex(&_latex,_latex_s);
  unary_function_ptr at_latex (&__latex);

  const string _TeX_s("TeX");
  unary_function_unary __TeX(&_latex,_TeX_s);
  unary_function_ptr at_TeX (&__TeX);


  string fill_flag(bool flag){
    if (flag)
      return "linestyle=none,fillstyle=solid,";
    else
      return "";
  }

  string line_flag(int flag){
    switch(flag) {
    case _STYLE_FULL : return "";
    case _STYLE_DOTTED : return "[linestyle=dotted]";
    case _STYLE_DASHED : return "[linestyle=dashed]";
    }
  }

  string vector_flag(int flag)
  {
    switch(flag) {
    case _VECTOR__VECT : return "{->}";
      // case BACKARROW : return  "{<-}";
      // case DOUBLEARROW : return "{<->}";
    default : return "";
    }
  }

  string point_flag(int flag){
    switch(flag) {
    case _STYLE_BOX : return "[dotstyle=square*]"; break;
    case _STYLE_CROSS : return "[dotstyle=x]"; break;
    case _STYLE_PLUS : return "[dotstyle=+]"; break;
    default : return "[dotstyle=*]"; break;
    }
  }

  string flcolor2pstrickscolor(int color){
    switch(color) {
    case FL_BLACK: return "black";
    case FL_DARK1: return "darkgray"; 
    case FL_GRAY: return "gray"; 
    case FL_LIGHT1  : return "lightgray"; 
#ifdef HAVE_LIBFLTK
    case _WHITE: return "white"; 
#endif
    case FL_WHITE : return "white"; 
#ifdef HAVE_LIBFLTK
    case _RED: return "red"; 
#endif
    case FL_RED	: return "red"; 
#ifdef HAVE_LIBFLTK
    case _GREEN: return "green"; 
#endif
    case FL_GREEN : return "green"; 
#ifdef HAVE_LIBFLTK
    case _BLUE: return "blue"; 
#endif
    case FL_BLUE : return "blue"; 
#ifdef HAVE_LIBFLTK
    case _CYAN: return "cyan"; 
#endif
    case FL_CYAN : return "cyan"; 
#ifdef HAVE_LIBFLTK
    case _MAGENTA: return "magenta"; 
#endif
    case FL_MAGENTA : return "magenta"; 
#ifdef HAVE_LIBFLTK
    case _YELLOW: return "yellow"; 
#endif
    case FL_YELLOW : return "yellow"; 
    default: return "black"; 
    } 
  }
  // This function is mainly a copy of drawing.c from eukleides by C. Obrecht
  // v is the output history
  bool pnt2tex(FILE * file,const vecteur & v,double & xd,double & yd,bool & drawlegende, int & labelpos,int & couleur){
    drawlegende=true;
    int s=v.size();
    if (!s)
      return false;
    // set color
    int ensemble_attributs=0;
    if (s>=2){
      if (v[1].type==_INT_) 
	ensemble_attributs=v[1].val;
      if (v[1].type==_VECT){
	vecteur & w=*v[1]._VECTptr;
	if (!w.empty() && (w.front().type==_INT_) )
	  ensemble_attributs=w.front().val;
      }
    }
    int width           =(ensemble_attributs & 0x00070000) >> 16; // 3 bits
    int epaisseur_point =(ensemble_attributs & 0x00380000) >> 19; // 3 bits
    int type_line       =(ensemble_attributs & 0x01c00000) >> 22; // 3 bits
    int type_point      =(ensemble_attributs & 0x0e000000) >> 25; // 3 bits
    labelpos        =(ensemble_attributs & 0x30000000) >> 28; // 2 bits
    bool fill_polygon   =(ensemble_attributs & 0x40000000) >> 30;
    couleur         =(ensemble_attributs & 0x0000ffff);
    string col(flcolor2pstrickscolor(couleur));
    fprintf(file,"\\psset{linecolor=%s}\n", col.c_str());
    // draw point/line/parametric
    gen point=v[0],x,y;
    // point is either a symb that evals to a complex or a param curve
    // or a vector of complex points
    // next line should be replaced by postscript (rpn) translation of curve
    // equation if possible
    if ( (point.type==_SYMB) && (point._SYMBptr->sommet==at_curve) && (point._SYMBptr->feuille.type==_VECT) && (point._SYMBptr->feuille._VECTptr->size()) )
      point=point._SYMBptr->feuille._VECTptr->back();
    if ( (point.type==_SYMB) && (point._SYMBptr->sommet==at_cercle)){
      vecteur v=*point._SYMBptr->feuille._VECTptr;
      gen diametre=remove_at_pnt(v[0]);
      gen e1=diametre._VECTptr->front().evalf(),e2=diametre._VECTptr->back().evalf();
      gen centre=rdiv(e1+e2,2.0),rayon=abs(rdiv(e2-e1,2.0),0);
      x=evalf_double(re(centre,0));
      y=evalf_double(im(centre,0));
      if ( (x.type==_DOUBLE_) && (y.type==_DOUBLE_) && (rayon.type==_DOUBLE_) ){
	xd=x._DOUBLE_val;
	yd=y._DOUBLE_val;
	if (v.size()>2){
	  gen theta1=evalf_double(v[1]),theta2=evalf_double(v[2]);
	  bool rad=angle_radian(0);
	  angle_radian(true,0);
	  gen angle=arg(e2-e1,0).evalf_double();
	  angle_radian(rad,0);
	  if (theta1.type==_DOUBLE_ && theta2.type==_DOUBLE_ && angle.type==_DOUBLE_){
	    double t1=theta1._DOUBLE_val,t2=theta2._DOUBLE_val;
	    if (t1>t2){
	      double tmp=t1;
	      t1=t2;
	      t2=tmp;
	    }
	    t1=(t1+angle._DOUBLE_val)*rad2deg_d;
	    t2=(t2+angle._DOUBLE_val)*rad2deg_d;
	    fprintf (file,"\\pswedge[%sfillcolor=%s](%.4f,%.4f){%.4f}{%.4f}{%.4f}\n",fill_flag(fill_polygon).c_str(),col.c_str(),xd,yd,rayon._DOUBLE_val,t1,t2); // was psarc
	    xd=x._DOUBLE_val+rayon._DOUBLE_val*std::cos(t1);
	    yd=y._DOUBLE_val+rayon._DOUBLE_val*std::sin(t1);;
	    return true;
	  }
	}
	fprintf(file,"\\pscircle[%sfillcolor=%s](%.4f,%.4f){%.4f}\n",fill_flag(fill_polygon).c_str(),col.c_str(),xd,yd,rayon._DOUBLE_val);
	xd=x._DOUBLE_val+rayon._DOUBLE_val;
	return true;
      }
      return false;
    } // end cercle
    if (point.type!=_VECT){ // single point
      point=evalf(point);
      x=re(point,0);
      y=im(point,0).evalf();
      if ( (x.type==_DOUBLE_) && (y.type==_DOUBLE_) ){
	xd=x._DOUBLE_val;
	yd=y._DOUBLE_val;
	fprintf(file,"\\psdots%s(%.4f,%.4f)\n", point_flag(type_point).c_str(), xd, yd);
	return true;
      }
      return false;
    }
    vecteur & w=*point._VECTptr;
    int ws=w.size();
    if (ws==2){ 
      gen A=w[0].evalf(),B=w[1].evalf();
      gen Ax=re(A,0),Ay=im(A,0).evalf(),Bx=re(B,0),By=im(B,0).evalf();
      if ( (Ax.type==_DOUBLE_) && (Ay.type==_DOUBLE_) 
	   && (Bx.type==_DOUBLE_) && (By.type==_DOUBLE_) ){
	double ax=Ax._DOUBLE_val,ay=Ay._DOUBLE_val,bx=Bx._DOUBLE_val,by=By._DOUBLE_val;
	xd=(ax+bx)/2.;
	yd=(ay+by)/2.;
 	if (point.subtype==_GROUP__VECT)
	  drawlegende=false;
	double vx=bx-ax,vy=by-ay;
	// check for line/halfline
	if ((point.subtype==_HALFLINE__VECT) || (point.subtype==_LINE__VECT) ){
	  bx=bx+5*vx;
	  by=by+5*vy;
	}
	if (point.subtype==_LINE__VECT){
	  ax=ax-5*vx;
	  ay=ay-5*vy;
	}
	fprintf(file,"\\psline%s(%.4f,%.4f)(%.4f,%.4f)\n",vector_flag(point.subtype).c_str(), ax, ay, bx, by);	
	return true;
      } // end DOUBLE types
      return false;
    } // end w.size()==2
    // multi-line
    string printstring("\\psline[");
    if (fill_polygon)
      printstring="\\pspolygon["+fill_flag(fill_polygon);
    printstring += "fillcolor="+col+"]";
    for (int i=0;i<ws;++i){
      x=w[i].evalf();
      y=im(x,0).evalf();
      x=re(x,0);
      if ( (x.type!=_DOUBLE_) || (y.type!=_DOUBLE_) )
	continue;
      xd=x._DOUBLE_val;
      yd=y._DOUBLE_val;
      printstring += "("+double2tex(xd)+","+double2tex(yd)+")";
    }
    printstring += '\n';
    fprintf(file,printstring.c_str());
    return true;
  }

  int graph2tex(FILE * file,const vecteur & v,double X1,double X2,double Y1,double Y2,double Unit,const char * filename,bool logo){
    double z(0);
    if (filename)
      fprintf(file,"\n%%file %s",filename);
    fprintf(file,"\n%% Generated by xcas\n\\noindent\n");
    fprintf(file,"\\psset{unit=%.4fcm}\n\\psset{linewidth=.5pt}\n\\psset{arrowsize=2pt 4}\n\\pspicture*(%.4f,%.4f)(%.4f,%.4f)\n", Unit, X1, Y1, X2, Y2);
    fprintf(file,"\\psset{linecolor=black}\n");
    if (logo){
      // fprintf(file,"\\psframe[fillstyle=solid,fillcolor=gray](%.4f,%.4f)(%.4f,%.4f)\n",X1,Y1,X2,Y2);
      vector<logo_turtle> w=vecteur2turtlevect(v);
      vector<logo_turtle> * turtleptr=&w;
      int l=turtleptr->size();
      if (l>0){
	double labelsep(0.1/Unit),angle(45);
	fprintf(file,"\\uput{%.4f}[%.4f](%.4f,%.4f){%s%i}\n",labelsep,angle,X2-100,Y2-25, "x ",int(turtleptr->back().x+.5));
	fprintf(file,"\\uput{%.4f}[%.4f](%.4f,%.4f){%s%i}\n",labelsep,angle,X2-100,Y2-48, "y ",int(turtleptr->back().y+.5));
	fprintf(file,"\\uput{%.4f}[%.4f](%.4f,%.4f){%s%i}\n",labelsep,angle,X2-100,Y2-65, "t ",int(turtleptr->back().theta+.5));
	logo_turtle prec =(*turtleptr)[0];
	for (int k=1;k<l;++k){
	  logo_turtle current =(*turtleptr)[k];
	  if (!current.s.empty()){ // Write a string
	    // FIXME fl_font(FL_HELVETICA,current.radius);
	    fprintf(file,"\\uput{%.4f}[%.4f](%.4f,%.4f){%s}\n",labelsep,angle,current.x,current.y, current.s.c_str());
	  }
	  else {
	    if (current.color!=prec.color){
	      string col(flcolor2pstrickscolor(current.color));
	      fprintf(file,"\\psset{linecolor=%s}\n", col.c_str());
	    }
	    if (current.radius>0){
	      int r=current.radius & 0x1ff; // bit 0-8
	      double theta1,theta2;
	      if (current.direct){
		theta1=prec.theta+double((current.radius >> 9) & 0x1ff); // bit 9-17
		theta2=prec.theta+double((current.radius >> 18) & 0x1ff); // bit 18-26
	      }
	      else {
		theta1=prec.theta-double((current.radius >> 9) & 0x1ff); // bit 9-17
		theta2=prec.theta-double((current.radius >> 18) & 0x1ff); // bit 18-26
	      }
	      bool rempli=(current.radius >> 27) & 0x1;
	      double angle;
	      double x,y;
	      angle = M_PI/180*(theta2-90);
	      if (current.direct){
		x=current.x - r*std::cos(angle);
		y=current.y - r*std::sin(angle);
	      }
	      else {
		x=current.x + r*std::cos(angle);
		y=current.y + r*std::sin(angle);
	      }
	      if (current.direct){
		if (rempli){
		  fprintf(file,"\\pswedge*[fillcolor=");
		  fprintf(file,flcolor2pstrickscolor(prec.color).c_str());
		  fprintf(file,"]");
		}
		else
		  fprintf(file,"\\psarc");
		fprintf(file,"(%.4f,%.4f){%.4f}{%.4f}{%.4f}\n",x,y,double(r),theta1-90,theta2-90);
	      }
	      else {
		if (rempli){
		  fprintf(file,"\\pswedge*[fillcolor=");
		  fprintf(file,flcolor2pstrickscolor(prec.color).c_str());
		  fprintf(file,"]");
		}
		else
		  fprintf(file,"\\psarc");
		fprintf(file,"(%.4f,%.4f){%.4f}{%.4f}{%.4f}\n",x,y,double(r),theta2+90,theta1+90);
	      }
	    } // end radius>0
	    else {
	      if (prec.mark && (prec.x!=current.x || prec.y!=current.y) ){
		fprintf(file,"\\psline(%.4f,%.4f)(%.4f,%.4f)\n", prec.x,prec.y,current.x,current.y);	
	      }
	    }
	    if (current.radius<-1 && k+current.radius>=0){
	      // poly-line from (*turtleptr)[k+current.radius] to (*turtleptr)[k]
	      string printstring("\\pspolygon[linestyle=none,fillstyle=solid,fillcolor="+flcolor2pstrickscolor(prec.color)+"]");
	      for (int i=0;i>=current.radius;--i){
		logo_turtle & t=(*turtleptr)[k+i];
		printstring += "("+double2tex(t.x)+","+double2tex(t.y)+")";
	      }
	      printstring += "("+double2tex(current.x)+","+double2tex(current.y)+")\n";
	      fprintf(file,printstring.c_str());
	    }
	  } // end else (non-string turtle record)
	  prec=current;
	} // end for (all turtle records)
	logo_turtle & t = (*turtleptr)[l-1];
	double x=t.x,y=t.y;
	double cost=std::cos(t.theta*deg2rad_d);
	double sint=std::sin(t.theta*deg2rad_d);
	double Dx=t.turtle_length*cost/2;
	double Dy=t.turtle_length*sint/2;
	string col(flcolor2pstrickscolor(prec.color));
	fprintf(file,"\\psset{linecolor=%s}\n", col.c_str());
	if (t.visible){
	  fprintf(file,"\\psline(%.4f,%.4f)(%.4f,%.4f)\n", x+Dy, y-Dx, x-Dy, y+Dx);	
	  // fl_line(deltax+x+Dy,deltay+h()-(y-Dx),deltax+x-Dy,deltay+h()-(y+Dx));
	  if (!t.mark){
	    string col(flcolor2pstrickscolor(t.color+1));
	    fprintf(file,"\\psset{linecolor=%s}\n", col.c_str());
	  }
	  fprintf(file,"\\psline(%.4f,%.4f)(%.4f,%.4f)\n", x+Dy, y-Dx, x+3*Dx, y+3*Dy);	
	  fprintf(file,"\\psline(%.4f,%.4f)(%.4f,%.4f)\n", x-Dy, y+Dx, x+3*Dx, y+3*Dy);	
	} // end if t.visible
      } // end if l>0
    } // end if logo
    else {
      if (show_axes && (Y2>=0) && (Y1<=0) )
	fprintf(file,"\\psline[linestyle=dashed]{->}(%.4f,%.4f)(%.4f,%.4f)\n",X1,z,X2,z);
      if (show_axes && (X2>=0) && (X1<=0) )
	fprintf(file,"\\psline[linestyle=dashed]{->}(%.4f,%.4f)(%.4f,%.4f)\n",z,Y1,z,Y2);
      vecteur w;
      const_iterateur it=v.begin(),itend=v.end();
      for (;it!=itend;++it){
	gen sortie=*it;
	// cerr << "graph2tex " << *it << endl;
	if (sortie.type==_POINTER_ && sortie.subtype==_FL_WIDGET_POINTER && fl_widget_updatepict_function)
	  sortie = fl_widget_updatepict_function(sortie);
	if (sortie.type!=_VECT)
	  w=vecteur(1,sortie);
	else
	  w=*sortie._VECTptr;
	if ( w.empty() || (w.back().type!=_SYMB) || 
	     !equalposcomp(plot_sommets,w.back()._SYMBptr->sommet) )
	  continue;
	const_iterateur jt=w.begin(),jtend=w.end();
	for (;jt!=jtend;++jt){
	  if ( (jt->type!=_SYMB) || (jt->_SYMBptr->sommet!=at_pnt) )
	    continue;
	  gen & g=jt->_SYMBptr->feuille;
	  if (g.type!=_VECT)
	    continue;
	  vecteur & arg=*g._VECTptr;
	  double xd,yd;
	  bool drawlegende;
	  int labelpos,couleur;
	  if (pnt2tex(file,arg,xd,yd,drawlegende,labelpos,couleur) && drawlegende && (arg.size()>2)){
	    // draw legende at xd,yd + 
	    string ss;
	    if (arg[2].type==_STRNG)
	      ss=*arg[2]._STRNGptr;
	    else
	      ss=arg[2].print();
	    ss="\\color{"+flcolor2pstrickscolor(couleur) +"} "+ss;
	    double labelsep(0.1/Unit),angle(45+labelpos*90);
	    fprintf(file,"\\uput{%.4f}[%.4f](%.4f,%.4f){%s}\n",labelsep,angle,xd,yd, ss.c_str());
	  }
	}
      }
    }
    fprintf(file,"\\endpspicture \n\n");
    return 1;
  }

  int graph2tex(const string &s,const vecteur & v,double X1,double X2,double Y1,double Y2,double Unit,bool logo){
    check_secure();
    // int file;
    // file = open (s.c_str(), O_WRONLY);
    // if (file==-1)
    //  return file;
    // int savestdout;
    // dup2(STDOUT_FILENO,savestdout);
    // dup2(file,STDOUT_FILENO);
    // Begin eukleides code with redirected stdout
    FILE * file=fopen(s.c_str(), "w");
    if (!file){
      cerr << "Unable to open file "+s << endl;
      return 0;
    }
    fprintf(file,"\\documentclass{article} \n\\usepackage{pstricks,color} \n\\begin{document} \n");
    graph2tex(file,v,X1,X2,Y1,Y2,Unit,s.c_str(),logo);
    fprintf(file,"\\end{document}\n");
    fclose(file);
    return 1;
    // End eukleides code, restore stdout
    // dup2(savestdout,STDOUT_FILENO);  
  }

  gen _graph2tex(const gen & args){
    int i=erase_pos();
    vecteur hist(history_out.begin()+i,history_out.end());
    return graph2tex(args,hist);
  }

  gen graph2tex(const gen & args,const vecteur & hist){
    if (args.type==_STRNG){
      double horiz_unit=horiz_latex/(gnuplot_xmax-gnuplot_xmin);
      double vert_unit=vert_latex/(gnuplot_ymax-gnuplot_ymin);
      double unit=horiz_unit;
      if (horiz_unit>vert_unit)
	unit=vert_unit;
      return graph2tex(*args._STRNGptr,hist,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,unit);
    }
    if ( (args.type!=_VECT) || (args._VECTptr->size()<2) )
      return symbolic(at_graph2tex,args);
    vecteur & v=*args._VECTptr;
    int vs=v.size();
    if ( (v[0].type!=_STRNG) || (v[1].type!=_DOUBLE_) )
      setsizeerr();
    if ( (vs>2) && (v[2].type==_DOUBLE_) )
      gnuplot_xmin=v[2]._DOUBLE_val;
    if ( (vs>3) && (v[3].type==_DOUBLE_) )
      gnuplot_xmax=v[3]._DOUBLE_val;
    if ( (vs>4) && (v[4].type==_DOUBLE_) )
      gnuplot_ymin=v[4]._DOUBLE_val;
    if ( (vs>5) && (v[5].type==_DOUBLE_) )
      gnuplot_ymax=v[5]._DOUBLE_val;
    return graph2tex(*v[0]._STRNGptr,hist,gnuplot_xmin,gnuplot_xmax,gnuplot_ymin,gnuplot_ymax,v[1]._DOUBLE_val);
  }
  const string _graph2tex_s("graph2tex");
  unary_function_unary __graph2tex(&_graph2tex,_graph2tex_s);
  unary_function_ptr at_graph2tex (&__graph2tex);

  gen _graph3d2tex(const gen & args){
    check_secure();
    if (args.type==_STRNG){
      string & s=*args._STRNGptr;
      bool clrplot;
      int out_handle;
      FILE * gnuplot_out_readstream,* stream = open_gnuplot(clrplot,gnuplot_out_readstream,out_handle);
      latex_replot(stream,s);
      gnuplot_wait(out_handle,gnuplot_out_readstream,5);
      return string2gen(s,false);
    }
    else { // search in history for the last plot command answer == to an int
      int s=min(history_out.size(),history_in.size());
      for (int i=s-1;i>=0;--i){
	if (history_out[i].is_symb_of_sommet(at_pnt) && history_out[i].subtype>=0)
	  return history_out[i].subtype;
	if ( history_out[i].type==_INT_ && history_in[i].type==_SYMB && ( equalposcomp(implicittex_plot_sommets,history_in[i]._SYMBptr->sommet) || equalposcomp(notexprint_plot_sommets,history_in[i]._SYMBptr->sommet) ) )
	  return history_out[i];
      }
      return undef;
    }
  }
  const string _graph3d2tex_s("graph3d2tex");
  unary_function_unary __graph3d2tex(&_graph3d2tex,_graph3d2tex_s);
  unary_function_ptr at_graph3d2tex (&__graph3d2tex);
#ifndef NO_NAMESPACE_GIAC
} // namespace giac
#endif // ndef NO_NAMESPACE_GIAC
